char *RCScll="$Id: cll.c,v 1.2 2000/03/02 16:17:37 lfm Exp $";
/*
** cll.c - Public functions for primitive collection.
**
** Public functions:
** --------------------------------------------------------------------
**
** void CllCreate( CllType type,
**                 Ihandle *iup_cv, void *cd_cv, 
**                 void (*end_action)( Ihandle *iup_cv, int status ),
**                 void (*put)( Ihandle *iup_cv, void *prmtv, double tol ), 
**                 void (*prompt)( Ihandle *iup_cv, char *msg ), 
**                 void (*snap)( Ihandle *iup_cv, double *x, double *y,
**                               double tol, int shift, int control ), 
**                 void (*echo)( Ihandle *iup_cv, double x, double y ) )
**
** type       - type of the primitive to collect                  (IN)
** iup_cv     - IUP handle of canvas where primitive is collected (IN)
** cd_cv      - CD handle of canvas                               (IN)
** end_action - function to be called after finishing prmtv.creat.(IN)
** put        - ptr. to function to put prmtv. after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
** snap       - ptr. to function to snap points during collection (IN)
** echo       - ptr. to function to echo cursor position          (IN)
**
** This function interactively collects from the user a primitive of
** the given type on a given canvas.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal collection ==> status = CLL_NORMAL = 1
**     This is also returned if user quit during collection, after
**     entering the first point.
** (2) Quit collection ==> status = CLL_QUIT = 0
**     This is returned if user quit collection at the begining,
**     before entering the first point. 
** (3) Interupted collection ==> status = CLL_INTERUPTED = -1
**     This is returned when the collection is closed by something
**     else (not by the mouse interaction).
**
** --------------------------------------------------------------------
**
** int  CllKbdCreate( CllType type, Ihandle *iup_cv, 
**                    void (*put)( Ihandle *iup_cv, void *prmtv, double tol ), 
**                    void (*prompt)( Ihandle *iup_cv, char *msg ) )
**
** type       - type of the primitive to collect                  (IN)
** iup_cv     - IUP handle of canvas for client reference         (IN)
** put        - ptr. to function to put prmtv. after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
**
** This function interactively collects from the keyboard a primitive of
** the given type.
** If the collection is successfull the function returns a CLL_NORMAL (1)
** status.  Otherwise, it returns a CLL_QUIT (0) status.
**
** --------------------------------------------------------------------
**
** void CllOpen( void *cllprmtv, void (*display)( void *cllprmtv ) )
**
** cllprmtv   - pointer to primitive being collected              (IN)
** display    - function to display primitive being collected     (IN)
**
** This function registers in the primitive collection local data
** structure a pointer to the current primitive being collected and
** a pointer to a function to display the primitive during collection.
** It was deviced to be called by a typical primitive collection 
** procedure when it starts the collection.
** After finishing collection, the function CllClose should be called
** to reset the local data structure of this module.
**
** --------------------------------------------------------------------
**
** void CllClose( void )
**
** This function resets the local data structure of this module.
**
** --------------------------------------------------------------------
**
** void CllDisplay( void )
**
** If there is a primitive being collected, this function displays 
** the primitive during the collection.
**
** --------------------------------------------------------------------
**
** void CllShiftSnap( double x0, double y0, double *x1, double *y1 )
**
** x0, y0     - fixed point of line                               (IN)
** x1, y1     - snaped point of line                          (IN/OUT)
**
** This functions snaps the end point of a line either to the 
** horizontal axis, to the vertical axis, or to the bi-sector lines
** of the main quadrants.
**
** --------------------------------------------------------------------
**
** Version:      1-000
**
** Created:    12-Apr-94    Luiz F. Martha
**
** Modified:   29-May-95    LFM
**     Version 1: Version to use with CD graphics package.
**     Got rid of GraPnt, replacing it by CllPnt.  CllPoint was
**     redefined as CllPnt: structure containing two fields: x, y.
**     Replaced double by float.
**     Changed prototype of snap and echo functions so that point
**     structure is replaced by x and y coordinates.
**
** Modified:   08-Mar-96    LFM
**     When CD changed, creating cdCreateCanvas, which returns
**     a CD handle for the canvas.
**     Changed CllCreate adding a parameter which is the CD
**     canvas handle.
**     Created CllPlgonCreate function for collection of polygon
**     primitive.
**
** Modified:   15-Oct-96    LFM
**     When created collection of circle, arc, and bezier primitives.
**
** Modified:   28-Jan-97    LFM 
**     Added parameters "shift" and "control" to snap callback function.
**
** Modified:   03-Mar-97    LFM
**     Added keyboard collection function CllKbdCreate.
**
** Modified:   11-Apr-97    LFM
**     Added call to CllClose in function CllCreate.
**     Added function CllShiftSnap.
**
** Modified:   17-Apr-97    LFM
**     Added return status parameter for end action function.
**
** Modified:   02-Mar-2000    LFM
**     Created collection of dimension lines.
**
** Modified:   18-Dec-2002    Marcos C. de Arruda
**     When created collection of spline primitive.
**
** Modified:   21-Apr-2004    William Lira
**     Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
** Modified:   11-Mar-2005    Marcos C. de Arruda
**     When created collection of elypse arc primitive.
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <math.h>

#include "cll.h"
#include "cllpnt.h"
#include "cllline.h"
#include "cllpline.h"
#include "cllcircl.h"
#include "cllarc.h"
#include "cllelparc.h"
#include "cllelipse.h"
#include "cllbezie.h"
#include "clldimln.h"

/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/
/* Current primitive being collected:
 */
static void *prmtv = 0L;

/* Function to display a primitive being collected:
 */
static void (*DspPrmtv)( void *prmtv ) = 0L;

/*
** --------------------------------------------------------------------
** Local functions:
*/

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* ==========================  CllCreate  ========================= */

void CllCreate( CllType type,
                Ihandle *iup_cv, void *cd_cv, 
                void (*end_action)( Ihandle *iup_cv, int status ), 
                void (*put)( Ihandle *iup_cv, void *prmtv, double tol ), 
                void (*prompt)( Ihandle *iup_cv, char *msg ), 
                void (*snap)( Ihandle *iup_cv, double *x, double *y, 
                              double tol, int shift, int control ), 
                void (*echo)( Ihandle *iup_cv, double x, double y ) )
{
 CllClose( );

 switch( type )
 {
  case CLL_PNT:
   CllPntCreate( iup_cv, cd_cv, end_action, put, prompt, snap, echo );
   break;

  case CLL_LINE:
   CllLineCreate( iup_cv, cd_cv, end_action, put, prompt, snap, echo );
   break;

  case CLL_PLINE:
   CllPLineCreate( iup_cv, cd_cv, end_action, put, prompt, snap, echo );
   break;

  case CLL_PLGON:
   CllPlgonCreate( iup_cv, cd_cv, end_action, put, prompt, snap, echo );
   break;

  case CLL_SPLINE:
   CllSplineCreate( iup_cv, cd_cv, end_action, put, prompt, snap, echo );
   break;

  case CLL_CIRCLE:
   CllCircleCreate( iup_cv, cd_cv, end_action, put, prompt, snap, echo );
   break;

  case CLL_ARC:
   CllArcCreate( iup_cv, cd_cv, end_action, put, prompt, snap, echo );
   break;

  case CLL_ELPARC:
   CllElpArcCreate( iup_cv, cd_cv, end_action, put, prompt, snap, echo );
   break;
  
  case CLL_ELIPSE:
   CllElipseCreate( iup_cv, cd_cv, end_action, put, prompt, snap, echo );
   break;
  
  case CLL_BEZIER:
   CllBezierCreate( iup_cv, cd_cv, end_action, put, prompt, snap, echo );
   break;

  case CLL_DIMLINE:
   CllDimLnCreate( iup_cv, cd_cv, end_action, put, prompt, snap, echo );
   break;

  default:
   break;
 }
}


/* ========================  CllKbdCreate  ======================== */

int  CllKbdCreate( CllType type, Ihandle *iup_cv, 
                   void (*put)( Ihandle *iup_cv, void *prmtv, double tol ), 
                   void (*prompt)( Ihandle *iup_cv, char *msg ) )
{
 int  status = 0;

 switch( type )
 {
  case CLL_PNT:
   status = CllKbdPntCreate( iup_cv, put, prompt );
   break;

  case CLL_LINE:
   status = CllKbdLineCreate( iup_cv, put, prompt );
   break;

  case CLL_PLINE:
   status = CllKbdPLineCreate( iup_cv, put, prompt );
   break;

  case CLL_PLGON:
   status = CllKbdPlgonCreate( iup_cv, put, prompt );
   break;

  case CLL_SPLINE:
   status = CllKbdSplineCreate( iup_cv, put, prompt );
   break;

  case CLL_CIRCLE:
   status = CllKbdCircleCreate( iup_cv, put, prompt );
   break;

  case CLL_ARC:
   status = CllKbdArcCreate( iup_cv, put, prompt );
   break;

  case CLL_ELPARC:
   status = CllKbdElpArcCreate( iup_cv, put, prompt );
   break;

  case CLL_ELIPSE:
   status = CllKbdElipseCreate( iup_cv, put, prompt );
   break;

  case CLL_BEZIER:
   status = CllKbdBezierCreate( iup_cv, put, prompt );
   break;

  case CLL_DIMLINE:
   status = CllKbdDimLnCreate( iup_cv, put, prompt );
   break;

  default:
   break;
 }

 return( status );
}


/* ===========================  CllOpen  ========================== */

void CllOpen( void *cllprmtv, void (*display)( void *cllprmtv ) )
{
 prmtv = cllprmtv;
 DspPrmtv = display;
}


/* ==========================  CllClose  ========================== */

void CllClose( void )
{
 prmtv = 0L;
 DspPrmtv = 0L;
}


/* =========================  CllDisplay  ========================= */

void CllDisplay( void )
{
 if( (prmtv != 0L) && (DspPrmtv != 0L) )
  (*DspPrmtv)( prmtv );
}


/* ========================  CllShiftSnap  ======================== */

void CllShiftSnap( double x0, double y0, double *x1, double *y1 )
{
 double  vx, vy, a;

 vx = *x1 - x0;
 vy = *y1 - y0;

 if( (fabs(vx) == 0.0) || (fabs(vy) == 0.0) )
  return;

 if( fabs(vx) > fabs(vy) )
 {
  a = vy / vx;
  if( fabs(a) > 0.75 )
  {
   if( a > 0 )
    *y1 = y0 + (*x1-x0);
   else
    *y1 = y0 - (*x1-x0);
  }
  else
  {
   *y1 = y0;
  }
 }
 else
 {
  a = vx / vy;
  if( fabs(a) > 0.75 )
  {
   if( a > 0 )
    *x1 = x0 + (*y1-y0);
   else
    *x1 = x0 - (*y1-y0);
  }
  else
  {
   *x1 = x0;
  }
 }
}


