char *RCScllarc="$Id: cllarc.c,v 1.9 2001/09/13 14:34:48 william Exp $";
/*
** cllarc.c - Functions to collect circle arcs.
**
** Public functions:
** --------------------------------------------------------------------
**
** void CllArcCreate( Ihandle *iup_cv, void *cd_cv,
**               void (*end_action)( Ihandle *iup_cv, int status ),
**               void (*put)( Ihandle *iup_cv, void *arc, double tol ), 
**               void (*prompt)( Ihandle *iup_cv, char *msg ), 
**               void (*snap)( Ihandle *iup_cv, double *x, double *y,
**                             double tol, int shift, int control ),
**               void (*echo)( Ihandle *iup_cv, double x, double y ) )
**
** iup_cv     - IUP handle of canvas where primitive is collected (IN)
** cd_cv      - CD handle of canvas                               (IN)
** end_action - funct. to be called after finishing arc creation  (IN)
** put        - ptr. to function to put arc after collection      (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
** snap       - ptr. to function to snap points during collection (IN)
** echo       - ptr. to function to echo cursor position          (IN)
**
** This function interactively collects from the user a circle arc 
** on a given canvas.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal collection ==> status = CLL_NORMAL = 1
**     This is also returned if user quit during collection, after
**     entering the first point.
** (2) Quit collection ==> status = CLL_QUIT = 0
**     This is returned if user quit collection at the begining,
**     before entering the first point.
** (3) Interupted collection ==> status = CLL_INTERUPTED = -1
**     This is returned when the collection is closed by something 
**     else (not by the mouse interaction). 
**
** --------------------------------------------------------------------
**
** int  CllKbdArcCreate( Ihandle *iup_cv,
**                    void (*put)( Ihandle *iup_cv, void *arc, double tol ),
**                    void (*prompt)( Ihandle *iup_cv, char *msg ) )
** 
** iup_cv     - IUP handle of canvas for client reference         (IN)
** put        - ptr. to function to put prmtv. after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
**
** This function interactively collects from the keyboard a circle arc.
** If the collection is successfull the function returns a CLL_NORMAL (1)
** status.  Otherwise, it returns a CLL_QUIT (0) status.
**
** --------------------------------------------------------------------
**
** Version:      0-001
**
** Created:    09-Jun-94    Joao Luiz E.C. (Acai)
**
** Modified:   13-Oct-96    LFM
**     Due to change in CD package, creating cdCreateCanvas, which 
**     returns a CD handle for the canvas.  Changed CllArcCreate 
**     adding a parameter which is the CD canvas handle.
**     Modified local snap function such that it calls client
**     snap function only if control key is pressed.  Before
**     this was done in Pck module.
**     Changed such that nothing is done when end_action function
**     in null.  Before the collector reinitialized itself.
**
** Modified:   28-Jan-97    LFM 
**     Added parameters "shift" and "control" to snap callback function.
**
** Modified:   03-Mar-97    LFM
**     Added keyboard collection function CllKbdArcCreate.
**
** Modified:   08-Mar-97    LFM
**     Changed interaction such that arc collection is performed
**     in two (button down) picks.
**     Added angle echo during collection.
**
** Modified:   09-Apr-97    LFM
**     Added call to CllClose in function CllArcCreate to close
**     any previous data structure in Cll super-class.
**
** Modified:   11-Apr-97    LFM
**     Added call to CllShiftSnap.
**
** Modified:   17-Apr-97    LFM
**      Added return status parameter for end action function.
**
** Modified:   23-Apr-97    LFM
**      Considered environment variable IUP_INGLES for prompt and
**      dialog messages.
**
** Modified:   19-Aug-97    LFM
**      Fixed problem of accessing collected object after deleting
**      it for calling end action function.
**
** Modified:   18-Feb-98    LFM
**      Modification to avoid displaying echo messages on the canvas.
**      Instead, echo messages are send to prompt message UI object.
**
** Modified:   19-Jan-2001  LFM
**      When call to function getenv( "IUP_INGLES" ) was replaced 
**      by call to function IupGetLanguage( ).
**      Local string variable iup_english changed to int.
**      Modified IupScanf floating point format.
**
** Modified:   11-Sep-2001  William
**      Substitution of the calls the IupScanf( ) for dialogs defined in the 
**      dlgarc module.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "iup.h"
#include "cd.h"
#include "wd.h"
#include "pck.h"
#include "cll.h"
#include "cllarc.h"
#include "dlgarc.h"


/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/
/*  Arc data structure:
 */
typedef struct _cllarcp  {   /* Collected arc private spec.descrip.: */
        CllArc   a;                 /* collected arc                 */
        int      ctr;               /* if true, center already given */
        int      fst;               /* if true, first arc pt. given  */
        void     *iup_cv;           /* IUP handle to target canvas   */
        void     *cd_cv;            /* cd handle to corresp.canvas   */
        void     (*end_action)      /* end action callback           */
                       ( Ihandle *iup_cv, int status );
        void     (*put)             /* function to put collect. arc  */
                       ( Ihandle *iup_cv, void *arc, double tol );
        void     (*prompt)          /* function to prompt messages   */
                       ( Ihandle *iup_cv, char *msg );
        void     (*snap)            /* function to snap collect. pt. */
                       ( Ihandle *iup_cv, double *x, double *y, 
                                       double tol, int shift, int control );
        void     (*echo)            /* function to echo cursor point */
                       ( Ihandle *iup_cv, double x, double y );
        } CllArcP;

#ifndef LEN
#define LEN(p0,p1) sqrt((p1.x-p0.x)*(p1.x-p0.x)+(p1.y-p0.y)*(p1.y-p0.y))
#endif
#ifndef PI
#define PI           3.141592653589793
#endif
#ifndef RAD
#define RAD(a)       ((a)*(PI/180.0))
#endif
#ifndef DEG
#define DEG(a)       ((a)*(180.0/PI))
#endif

static int  iup_english;
static char *prompt_msg1;
static char *prompt_msg2;
static char *prompt_msg3;

static char *prompt_msg1p = { "Com botao esquerdo do mouse entre "
                              "com centro do arco; "
                              "Botao do meio => Teclado." };
static char *prompt_msg2p = { "Entre com primeiro ponto do arco "
                              "(sentido anti-horario); "
                              "Botao do meio => Teclado." };
static char *prompt_msg3p = { "Entre com segundo ponto do arco; "
                              "Botao do meio => Teclado; "
                              "Botao da direita => Cancela." };
static char *prompt_msg1e = { "With mouse left button enter arc center; "
                              "Mid button => Keyboard." };
static char *prompt_msg2e = { "Enter arc first point "
                              "(counter-clockwise direction); "
                              "Mid button => Keyboard." };
static char *prompt_msg3e = { "Enter arc second point; "
                              "Mid button => Keyboard; "
                              "Right button => Cancel." };

/*
** --------------------------------------------------------------------
** Local functions:
*/

/* ========================  CllArcSnapTo  ======================= */

static int CllArcSnapTo( double xc, double yc, double r, double *x, double *y )
{
 double   s, t;

 s = sqrt( (*x-xc) * (*x-xc) + (*y-yc) * (*y-yc) );
 if( s == 0.0 )
  return( 0 );

 t = r / s;

 *x = xc + t * (*x-xc);
 *y = yc + t * (*y-yc);

 return( 1 );
}


/* ========================  CllArcAngle  ======================== */

static int CllArcAngle( double xc, double yc, double x, double y, double *a )
{
 double   s;

 s = sqrt( (x-xc) * (x-xc) + (y-yc) * (y-yc) );
 if( s == 0.0 )
  return( 0 );

 *a = atan2( y-yc, x-xc );
 *a = DEG( *a );

 return( 1 );
}


/* ==========================  CllArcDel  ======================== */

static void CllArcDel( CllArcP *arc )
{
 if( arc == 0L ) return;

/*  Release memory used for arc.
 */    
 free( arc );
}


/* ========================  CllArcSnap  ========================= */

static void CllArcSnap( PckDesc *pk, double *x, double *y, double tol )
{
/* Restore registered collection spec. from the data field.
 */
 CllArcP  *arc = (CllArcP *)pk->data;

/* If shift key is pressed and center arc point has been given,
 * snap radius to the horizontal axis, to the vertical axis, or
 * the bi-sector lines.
 */
 if( (arc->ctr) && isshift(pk->mousecfg) )
 {
  CllShiftSnap( arc->a.center.x, arc->a.center.y, x, y );
 }

/* Check to see whether client had registered a snap function and 
 * call it passing the given mouse position and parameters.
 */
 if( arc->snap != 0L )
  (*arc->snap )( arc->iup_cv, x, y, tol, 
                 isshift(pk->mousecfg), iscontrol(pk->mousecfg) );

/* If first arc points has already been collected, 
 * snap current point to arc.
 */
 if( (arc->fst) && (arc->a.radius > 0.0) )
 {
  CllArcSnapTo( arc->a.center.x, arc->a.center.y, arc->a.radius, x, y );
 }
}


/* ========================  CllArcEcho  ========================= */
 
static void CllArcEcho( PckDesc *pk, double x, double y )
{
 int    write_mode;
 int    fore_color;
 int    mark_type;
 int    mark_size;
 int    line_style;
 int    line_width;
 double r, d;
 double ang0, ang1;
 CllPnt pt;
 char   echotext[60];

/* Restore registered collection spec. from the data field.
 */
 CllArcP  *arc = (CllArcP *)pk->data;
 
/* Check to see if center has been given.
 * If that is the case, draw a mark at the center in Xor mode.
 */
 if( arc->ctr )
 {
#ifdef _UNIX_
  write_mode = cdWriteMode( CD_XOR );
#else
  write_mode = cdWriteMode( CD_NOT_XOR );
#endif
  fore_color = cdForeground( CD_RED );
  mark_type = cdMarkType( CD_BOX );
  mark_size = cdMarkSize( 4 );

  wdMark( arc->a.center.x, arc->a.center.y );

  if( !arc->fst && CllArcAngle(arc->a.center.x, arc->a.center.y, x, y, &ang0) )
  {
   pt.x = x;  pt.y = y;
   r = LEN( arc->a.center, pt );
   if( iup_english )
    sprintf( echotext, "Radius = %10.3f;   Angle = %7.2f", r, ang0 );
   else
    sprintf( echotext, "Raio = %10.3f;   Angulo = %7.2f", r, ang0 );
   if( arc->prompt != 0L )
    (*arc->prompt)( arc->iup_cv, echotext );
  }

  cdMarkSize( mark_size );
  cdMarkType( mark_type );
  cdForeground( fore_color );
  cdWriteMode( write_mode );
 }

/* Check to see if first point has been given.
 * If that is the case, draw a arc in Xor mode.
 */
 if( (arc->fst) && (arc->a.radius > 0.0) )
 {
  d = 2.0 * arc->a.radius;
  ang0 = arc->a.ang0;

  if( CllArcAngle( arc->a.center.x, arc->a.center.y, x, y, &ang1 ) )
  {
#ifdef _UNIX_
   write_mode = cdWriteMode( CD_XOR );
#else
   write_mode = cdWriteMode( CD_NOT_XOR );
#endif
   fore_color = cdForeground( CD_RED );
   mark_type = cdMarkType( CD_BOX );
   mark_size = cdMarkSize( 4 );
   line_style = cdLineStyle( CD_CONTINUOUS );
   line_width = cdLineWidth( 1 );

   wdArc( arc->a.center.x, arc->a.center.y, d, d, ang0, ang1 );
   wdMark( arc->a.p0.x, arc->a.p0.y );
   if( iup_english )
    sprintf( echotext, "Angle 1 = %7.2f;  Angle 2 = %7.2f", ang0, ang1 );
   else
    sprintf( echotext, "Angulo 1 = %7.2f;  Angulo 2 = %7.2f", ang0, ang1 );
   if( arc->prompt != 0L )
    (*arc->prompt)( arc->iup_cv, echotext );

   cdLineWidth( line_width );
   cdLineStyle( line_style );
   cdMarkSize( mark_size );
   cdMarkType( mark_type );
   cdForeground( fore_color );
   cdWriteMode( write_mode );
  }
 }

/* Check to see whether client had registered an echo function.
 * If that is the case call it passing the given mouse position.
 */
 if( arc->echo != 0L )
  (*arc->echo )( arc->iup_cv, x, y ) ;
}
 

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* ========================  CllArcCreate  ======================= */

static void CllArcCenter( PckDesc * );

void CllArcCreate( Ihandle *iup_cv, void *cd_cv,
              void (*end_action)( Ihandle *iup_cv, int status ),
              void (*put)( Ihandle *iup_cv, void *arc, double tol ),
              void (*prompt)( Ihandle *iup_cv, char *msg ),
              void (*snap)( Ihandle *iup_cv, double *x, double *y, 
                            double tol, int shift, int control ),
              void (*echo)( Ihandle *iup_cv, double x, double y ) )
{
 CllArcP  *arc;
 PckDesc  pk;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  prompt_msg1 = prompt_msg1e;
  prompt_msg2 = prompt_msg2e;
  prompt_msg3 = prompt_msg3e;
 }
 else
 {
  prompt_msg1 = prompt_msg1p;
  prompt_msg2 = prompt_msg2p;
  prompt_msg3 = prompt_msg3p;
 }

 CllClose( );

/* Allocate a arc collection descriptor
 */
 arc = (CllArcP *)malloc( sizeof( CllArcP ) );

/* Initialize collection
 */
 arc->ctr = 0;
 arc->fst = 0;
 arc->iup_cv = iup_cv;
 arc->cd_cv = cd_cv;
 arc->end_action = end_action; 
 arc->put = put; 
 arc->prompt = prompt; 
 arc->snap = snap; 
 arc->echo = echo; 
 arc->a.center.x = arc->a.center.y = 0.0;
 arc->a.p0.x = arc->a.p0.y = 0.0;
 arc->a.p1.x = arc->a.p1.y = 0.0;
 arc->a.radius = 0.0;

/* Setup pick descriptor for getting arc center at button down.  
 */
 if( arc->prompt != 0L )
  (*arc->prompt )( arc->iup_cv, prompt_msg1 );
 pk.iup_cv = arc->iup_cv;
 pk.gra_cv = arc->cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.mode = ButtonDown;
 pk.cursorecho = BoxCursor | CrossCursor;
 pk.button = 0;    /* accept both mouse buttons 1 or 2 */
 pk.toler = 0.0;
 pk.btdwn_callback  = CllArcCenter;
 pk.btup_callback = 0L;
 pk.echofunc = CllArcEcho;
 pk.snapfunc = CllArcSnap;
 pk.data = arc;
 pk.status = PCK_VALID;
 PckSetup( &pk );
}

/* -------------------------  CllArcCenter  ------------------------ */

static void CllArc1stPoint( PckDesc * );

static void CllArcCenter( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllArcP  *arc = (CllArcP *)pk->data;
 void     *iup_cv = arc->iup_cv;
 void     (*end_action)( Ihandle *iup_cv, int status ) = arc->end_action;

/* Check to see if collected arc is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = CLL_QUIT;        break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllArcDel( arc );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Get button down point as center point.
 */
 arc->a.center.x = pk->p0.x;
 arc->a.center.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter the
 * arc center using the keyboard. In this case, prompt the user for
 * center coordinates, and continue collection.
 */
 if( pk->button == 2 )
 {
  float xc, yc;
  xc = arc->a.center.x;
  yc = arc->a.center.y;
  if( DlgCaptureFArc( &xc, &yc ) )
  {
   arc->a.center.x = xc;
   arc->a.center.y = yc;
  }
  else
  {
   CllArcDel( arc );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
   return;
  }
 }

/* Set flag to indicate that center has been selected.
 */
 arc->ctr = 1;

/* Setup pick descriptor to collect first arc point.  Register the 
 * appropriate arc collector function as the callback for pick interaction.
 */
 if( arc->prompt != 0L )
  (*arc->prompt )( arc->iup_cv, prompt_msg2 );
 pk->p0.x = arc->a.center.x;
 pk->p0.y = arc->a.center.y;
 pk->p1.x = pk->p1.y = 0.0;
 pk->mode = ButtonDown;
 pk->cursorecho = BoxCursor | CrossCursor | RubberBandCursor;
 pk->button = 0;    /* accept both mouse buttons 1 or 2 */
 pk->btdwn_callback = CllArc1stPoint;
 pk->btup_callback  = 0L;
 PckSetup( pk );
}

/* -------------------------  CllArc1stPoint  ---------------------- */

static void CllArc2ndPoint( PckDesc * );

static void CllArc1stPoint( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllArcP  *arc = (CllArcP *)pk->data;
 void     *iup_cv = arc->iup_cv;
 void     (*end_action)( Ihandle *iup_cv, int status ) = arc->end_action;

/* Check to see if collected arc is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = CLL_NORMAL;      break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllArcDel( arc );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Get button down point as first arc point.
 */
 arc->a.p0.x = pk->p0.x;
 arc->a.p0.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter first
 * arc point using the keyboard. In this case, prompt the user for
 * point coordinates, and continue collection.
 */
 if( pk->button == 2 )
 {
  float x0, y0;
  x0 = arc->a.p0.x;
  y0 = arc->a.p0.y;
  if( DlgCaptureSArc( &x0, &y0 ) )
  {
   arc->a.p0.x = x0;
   arc->a.p0.y = y0;
  }
  else
  {
   CllArcDel( arc );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
   return;
  }
 }

/* Compute arc radius.
 */
 arc->a.radius = LEN( arc->a.center, arc->a.p0 );

/* Set flag to indicate that first arc point has been selected and
 * compute corresponding angle.
 */
 arc->fst = 1;
 CllArcAngle( arc->a.center.x, arc->a.center.y, 
              arc->a.p0.x, arc->a.p0.y, &arc->a.ang0 );

/* Setup pick descriptor to collect final arc point.  Register the 
 * appropriate arc collector function as the callback for pick interaction.
 */
 if( arc->prompt != 0L )
  (*arc->prompt )( arc->iup_cv, prompt_msg3 );
 pk->p0.x = arc->a.p0.x;
 pk->p0.y = arc->a.p0.y;
 pk->p1.x = pk->p1.y = 0.0;
 pk->mode = ButtonDown;
 pk->cursorecho = BoxCursor | CrossCursor;
 pk->button = 0;    /* accept both mouse buttons 1 or 2 */
 pk->btdwn_callback = CllArc2ndPoint;
 pk->btup_callback  = 0L;
 PckSetup( pk );
}

/* -------------------------  CllArc2ndPoint  ---------------------- */

static void CllArc2ndPoint( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllArcP  *arc = (CllArcP *)pk->data;
 void     *iup_cv = arc->iup_cv;
 void     (*end_action)( Ihandle *iup_cv, int status ) = arc->end_action;

/* Check to see if collected arc is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = CLL_NORMAL;      break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllArcDel( arc );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Get button down point as final arc point.
 */
 arc->a.p1.x = pk->p0.x;
 arc->a.p1.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter final
 * arc point using the keyboard. In this case, prompt the user for
 * all arc point coordinates and finish collection.
 */
 if( pk->button == 2 )
 {
  float xc, yc, x0, y0, x1, y1;
  xc = arc->a.center.x;
  yc = arc->a.center.y;
  x0 = arc->a.p0.x;
  y0 = arc->a.p0.y;
  x1 = arc->a.p1.x;
  y1 = arc->a.p1.y;
  if( DlgCaptureTArc( &xc, &yc, &x0, &y0, &x1, &y1 ) )
  {
   arc->a.center.x = xc;
   arc->a.center.y = xc;
   arc->a.p0.x = x0;
   arc->a.p0.y = y0;
   arc->a.p1.x = x1;
   arc->a.p1.y = y1;
   arc->a.radius = LEN( arc->a.center, arc->a.p0 );
   if( arc->a.radius > 0.0 )
   {
    if( CllArcSnapTo( arc->a.center.x, arc->a.center.y, arc->a.radius, 
                      &arc->a.p1.x, &arc->a.p1.y ) )
    {
     if( LEN( arc->a.p0, arc->a.p1 ) >= pk->toler )
     {
      CllArcAngle( arc->a.center.x, arc->a.center.y, 
                   arc->a.p1.x, arc->a.p1.y, &arc->a.ang1 );
      if( arc->put != 0L )
       (*arc->put )( arc->iup_cv, &arc->a, pk->toler );
     }
    }
   }
   CllArcDel( arc );
   if( end_action ) (*end_action)( iup_cv, CLL_NORMAL );
  }
  else
  {
   CllArcDel( arc );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
  }
  return;
 }

/* Call the client function to put the collected arc, 
 * and finish collection calling end action function.
 */
 if( (arc->a.radius > 0.0) && (LEN(arc->a.p0,arc->a.p1) >= pk->toler) )
 {
  CllArcAngle( arc->a.center.x, arc->a.center.y, 
               arc->a.p1.x, arc->a.p1.y, &arc->a.ang1 );
  if( arc->put != 0L )
   (*arc->put )( arc->iup_cv, &arc->a, pk->toler );
 }
 CllArcDel( arc );
 if( end_action ) (*end_action)( iup_cv, CLL_NORMAL );
}


/* ======================  CllKbdArcCreate  ====================== */

static CllArc  kbdarc;
static double  kbdtol = 0.001;

static char *kbdprompt_msg;

static char *kbdprompt_msgp = { "Digite dados do arco. " };
static char *kbdprompt_msge = { "Use the keyboard to enter arc data. " };

int  CllKbdArcCreate( Ihandle *iup_cv, 
              void (*put)( Ihandle *iup_cv, void *arc, double tol ),
              void (*prompt)( Ihandle *iup_cv, char *msg ) )
{
 CllArc   arc;
 float    xc, yc;
 float    x0, y0, x1, y1;
 float    toler;
 int      status = CLL_QUIT;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  kbdprompt_msg = kbdprompt_msge;
 }
 else
 {
  kbdprompt_msg = kbdprompt_msgp;
 }

/* Initialize collection
 */
 arc = kbdarc;

/* Collect data from user with the keyboard.
 */
 if( prompt != 0L )
  (*prompt)( iup_cv, kbdprompt_msg );

 xc = arc.center.x;
 yc = arc.center.y;
 x0 = arc.p0.x;
 y0 = arc.p0.y;
 x1 = arc.p1.x;
 y1 = arc.p1.y;
 toler = kbdtol;
 if( DlgCaptureArc( &xc, &yc, &x0, &y0, &x1, &y1, &toler ) )
 {
  arc.center.x = xc;
  arc.center.y = yc;
  arc.p0.x = x0;
  arc.p0.y = y0;
  arc.p1.x = x1;
  arc.p1.y = y1;
  arc.radius = LEN( arc.center, arc.p0 );
  if( arc.radius > 0.0 )
  {
   if( CllArcSnapTo( arc.center.x, arc.center.y, arc.radius, 
		     &arc.p1.x, &arc.p1.y ) )
   {
    if( LEN( arc.p0, arc.p1 ) > 0.0 )
    {
     kbdarc = arc;
     kbdtol = toler;
     if( put != 0L )
      (*put)( iup_cv, &arc, kbdtol );
     status = CLL_NORMAL;
    }
   }
  }
 }

 return( status );
}


