char *RCScllbezie="$Id: cllbezie.c,v 1.8 2001/09/13 14:34:48 william Exp $";
/*
** cllbezie.c - Functions to collect a cubic bezier curve.
**
** Public functions:
** --------------------------------------------------------------------
**
** void CllBezierCreate( Ihandle *iup_cv, void *cd_cv,
**               void (*end_action)( Ihandle *iup_cv, int status ),
**               void (*put)( Ihandle *iup_cv, void *bezier, double tol ),
**               void (*prompt)( Ihandle *iup_cv, char *msg ),
**               void (*snap)( Ihandle *iup_cv, double *x, double *y,
**                             double tol, int shift, int control ),
**               void (*echo)( Ihandle *iup_cv, double x, double y ) )
**
** iup_cv     - IUP handle of canvas where primitive is collected (IN)
** cd_cv      - CD handle of canvas                               (IN)
** end_action - function to be called after finishing bez. creat. (IN)
** put        - ptr. to function to put bezier after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
** snap       - ptr. to function to snap points during collection (IN)
** echo       - ptr. to function to echo cursor position          (IN)
**
** This function interactively collects from the user a cubic bezier 
** curve on a given canvas.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal collection ==> status = CLL_NORMAL = 1
**     This is also returned if user quit during collection, after
**     entering the first point.
** (2) Quit collection ==> status = CLL_QUIT = 0
**     This is returned if user quit collection at the begining,
**     before entering the first point.
** (3) Interupted collection ==> status = CLL_INTERUPTED = -1
**     This is returned when the collection is closed by something
**     else (not by the mouse interaction).
**
** --------------------------------------------------------------------
**
** int  CllKbdBezierCreate( Ihandle *iup_cv,
**                    void (*put)( Ihandle *iup_cv, void *bezier, double tol ),
**                    void (*prompt)( Ihandle *iup_cv, char *msg ) )
**
** iup_cv     - IUP handle of canvas for client reference         (IN)
** put        - ptr. to function to put prmtv. after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
**
** This function interactively collects from the keyboard a cubic 
** bezier curve.
** If the collection is successfull the function returns a CLL_NORMAL (1)
** status.  Otherwise, it returns a CLL_QUIT (0) status.
**
** --------------------------------------------------------------------
**
** Version:      0-001
**
** Created:    16-Jun-94    Joao Luiz E. Campos
**
** Modified:   15-Oct-96    LFM
**     Due to change in CD package, creating cdCreateCanvas, which
**     returns a CD handle for the canvas.  Changed CllBezierCreate
**     adding a parameter which is the CD canvas handle.
**     Modified local snap function such that it calls client
**     snap function only if control key is pressed.  Before
**     this was done in Pck module.
**     Changed such that nothing is done when end_action function
**     in null.  Before the collector reinitialized itself.
**
** Modified:   28-Jan-97    LFM 
**     Added parameters "shift" and "control" to snap callback function.
**
** Modified:   03-Mar-97    LFM
**     Added keyboard collection function CllKbdBezierCreate.
**
** Modified:   09-Mar-97    LFM
**     Changed interaction such that initial bezier span line collection 
**     is performed in two (button down) picks.
**     Replaced check for straight curve from distance between 
**     points to pick line test.
**
** Modified:   12-Mar-97    LFM
**     Added control through interpolation points.
**
** Modified:   21-Mar-97    LFM
**     Modified such that it does not redisplay the curve at mouse
**     move if it was not modified.  Created fields "modified" and
**     "displayed" in local data structure.  Added call to functions
**     CllOpen and CllClose.
**
** Modified:   11-Apr-97    LFM
**     Added missing calls to CllClose.
**     Inverted ordering of selection of control and interpolation 
**     points in function CllBezierDef_BtDwnCB (now it gives 
**     preference to control points).
**
** Modified:   11-Apr-97    LFM
**     Added call to CllShiftSnap and added field fst in local
**     collection data structure.
**
** Modified:   17-Apr-97    LFM
**      Added return status parameter for end action function.
**
** Modified:   23-Apr-97    LFM
**      Considered environment variable IUP_INGLES for prompt and
**      dialog messages.
**
** Modified:   19-Aug-97    LFM
**      Fixed problem of accessing collected object after deleting
**      it for calling end action function.
**
** Modified:   19-Jan-2001  LFM
**      When call to function getenv( "IUP_INGLES" ) was replaced 
**      by call to function IupGetLanguage( ).
**      Local string variable iup_english changed to int.
**      Modified IupScanf floating point format.
**
** Modified:   11-Sep-2001  William
**      Substitution of the calls the IupScanf( ) for dialogs defined in the 
**      dlgbez module.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "iup.h"
#include "cd.h"
#include "wd.h"
#include "pck.h"
#include "cll.h"
#include "dlgbez.h"
#include "cllbezie.h"


/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/
/*  Bezier data structure:
 */
typedef struct _cllbezierp {  /* Collected bezier private spec.descrip.:  */
        CllBezier   b;              /* collected bezier                   */
        CllPnt      pi0, pi1;       /* interior interpolation points      */
        int         fst;            /* if true, first curve point given   */
        int         created;        /* flag for created curve             */
        int         modified;       /* flag for modified curve            */
        int         displayed;      /* flag for displayed curve           */
        double      toler;          /* last tolerance value               */
        void        *iup_cv;        /* IUP handle to target canvas        */
        void        *cd_cv;         /* cd handle to corresp.canvas        */
        void       (*end_action)    /* end action callback                */
                       ( Ihandle *iup_cv, int status );
        void       (*put)           /* function to put collected bezier   */
                       ( Ihandle *iup_cv, void *bezier, double tol );
        void       (*prompt)        /* function to prompt messages   */
                       ( Ihandle *iup_cv, char *msg );
        void       (*snap)          /* function to snap collect. pt. */
                       ( Ihandle *iup_cv, double *x, double *y, 
                                       double tol, int shift, int control );
        void       (*echo)          /* function to echo cursor point */
                       ( Ihandle *iup_cv, double x, double y );
        } CllBezierP;

#ifndef LEN
#define LEN(p0,p1) sqrt((p1.x-p0.x)*(p1.x-p0.x)+(p1.y-p0.y)*(p1.y-p0.y))
#endif

static int  iup_english;
static char *prompt_msg1;
static char *prompt_msg2;
static char *prompt_msg3;

static char *prompt_msg1p ={ "Entre ponto inicial com botao esquerdo do mouse; "
                             "Botao do meio => Teclado." };
static char *prompt_msg2p = { "Entre com ponto final; "
                              "Botao do meio => Teclado; "
                              "Botao da direita => Cancela." };
static char *prompt_msg3p = { "Com botao precionado do mouse arraste "
                              "ponto de controle; "
                              "Botao da direita => Termina." };
static char *prompt_msg1e = { "Enter first point with mouse left button; "
                              "Mid button => Keyboard." };
static char *prompt_msg2e = { "Enter end point; "
                              "Mid button => Keyboard; "
                              "Right button => Cancel." };
static char *prompt_msg3e = { "With mouse button pressed drag "
                              "control point; "
                              "Right button => Finish." };

/*
** --------------------------------------------------------------------
** Local functions:
*/
static void  CllBezierDel       ( CllBezierP *bezier );
static void  CllBezierSnap      ( PckDesc *pk,double *x,double *y,double tol );
static void  CllBezierNetToCrv  ( CllBezierP *bezier );
static void  CllBezierCrvToNet  ( CllBezierP *bezier );
static void  CllPickCode        ( double x, double y,
                                  double xmin, double xmax, 
                                  double ymin, double ymax, 
                                  int cod[4] );
static int   CllPickPoint       ( double ptx, double pty, 
                                  double x, double y, double toler );
static int   CllPickLine        ( double x0, double y0, double x1, double y1, 
                                  double x, double y, double toler );
static int   CllStraight        ( CllBezier *bezier, double toler );
static void  CllDrawCurve       ( CllBezier *bezier, double toler );
static void  CllBezierDisplay   ( CllBezierP *bezier, double toler );
static void  CllBezierDisplayXor( CllBezierP *bezier, double toler );
static void  CllBezierRedisplay ( void *b );
static void  CllBezierEcho      ( PckDesc *pk, double echoptx, double echopty );
static void  CllBezierP0CrtEcho ( PckDesc *pk, double echoptx, double echopty );
static void  CllBezierPi0CrtEcho( PckDesc *pk, double echoptx, double echopty );
static void  CllBezierPi1CrtEcho( PckDesc *pk, double echoptx, double echopty );
static void  CllBezierP1CrtEcho ( PckDesc *pk, double echoptx, double echopty );
static void  CllBezierPc0CrtEcho( PckDesc *pk, double echoptx, double echopty );
static void  CllBezierPc1CrtEcho( PckDesc *pk, double echoptx, double echopty );

/* ========================  CllBezierDel  ========================= */

static void CllBezierDel( CllBezierP *bezier )
{
 if( bezier == 0L ) return;

/*  Release memory used for bezier
 */    
 free( bezier );
}

/* ========================  CllBezierSnap  ========================= */

static void CllBezierSnap( PckDesc *pk, double *x, double *y, double tol )
{
/* Restore registered collection spec. from the data field.
 */
 CllBezierP  *bezier = (CllBezierP *)pk->data;

/* If shift key is pressed, the first curve point has been given, 
 * and the bezier curve is not created, snap bezier span line to 
 * the horizontal axis, to the vertical axis, or the bi-sector lines.
 */
 if( (bezier->fst) && (!bezier->created) && isshift(pk->mousecfg) )
 {
  CllShiftSnap( bezier->b.p0.x, bezier->b.p0.y, x, y );
 }

/* Check to see whether client had registered a snap function and 
 * call it passing the given mouse position and parameters.
 */
 if( bezier->snap != 0L )
  (*bezier->snap )( bezier->iup_cv, x, y, tol,
                              isshift(pk->mousecfg), iscontrol(pk->mousecfg) );
}

/* ======================  CllBezierNetToCrv  ====================== */

static void CllBezierNetToCrv( CllBezierP *bezier )
{
 bezier->pi0.x = (    8.0 * bezier->b.p0.x + 12.0 * bezier->b.pc0.x
                   +  6.0 * bezier->b.pc1.x +       bezier->b.p1.x ) / 27.0;
 bezier->pi0.y = (    8.0 * bezier->b.p0.y + 12.0 * bezier->b.pc0.y
                   +  6.0 * bezier->b.pc1.y +       bezier->b.p1.y ) / 27.0;
 bezier->pi1.x = (          bezier->b.p0.x +  6.0 * bezier->b.pc0.x
                   + 12.0 * bezier->b.pc1.x + 8.0 * bezier->b.p1.x ) / 27.0;
 bezier->pi1.y = (          bezier->b.p0.y +  6.0 * bezier->b.pc0.y
                   + 12.0 * bezier->b.pc1.y + 8.0 * bezier->b.p1.y ) / 27.0;
}

/* ======================  CllBezierCrvToNet  ====================== */

static void CllBezierCrvToNet( CllBezierP *bezier )
{
 bezier->b.pc0.x = ( -  5.0 * bezier->b.p0.x + 18.0 * bezier->pi0.x
                     -  9.0 * bezier->pi1.x +   2.0 * bezier->b.p1.x ) / 6.0;
 bezier->b.pc0.y = ( -  5.0 * bezier->b.p0.y + 18.0 * bezier->pi0.y
                     -  9.0 * bezier->pi1.y +   2.0 * bezier->b.p1.y ) / 6.0;
 bezier->b.pc1.x = (    2.0 * bezier->b.p0.x -  9.0 * bezier->pi0.x
                     + 18.0 * bezier->pi1.x -   5.0 * bezier->b.p1.x ) / 6.0;
 bezier->b.pc1.y = (    2.0 * bezier->b.p0.y -  9.0 * bezier->pi0.y
                     + 18.0 * bezier->pi1.y -   5.0 * bezier->b.p1.y ) / 6.0;
}

/* ==========================  CllPickCode  ======================== */

static void CllPickCode( double x, double y,
                         double xmin, double xmax, double ymin, double ymax, 
                         int cod[4] )
{
 cod[0] = x < xmin;
 cod[1] = x > xmax;
 cod[2] = y < ymin;
 cod[3] = y > ymax;
}

/* ==========================  CllPickPoint  ======================= */

static int CllPickPoint( double ptx, double pty, 
                         double x, double y, double toler )
{
 double   xmin, xmax, ymin, ymax;
 int      cod[4];
 int      i;

/* Define atraction window.
 */
 xmin = x - toler;
 xmax = x + toler;
 ymin = y - toler;
 ymax = y + toler;

/* Check to see if given point (x,y) touches target point (ptx,pty)
 * based on given tolerance.
 */
 CllPickCode( ptx, pty, xmin, xmax, ymin, ymax, cod );
 for( i = 0; i < 4; i++ )
  if( cod[i] )
   return( 0 );

 return( 1 );
}

/* ==========================  CllPickLine  ======================== */

static int CllPickLine( double x0, double y0, double x1, double y1, 
                        double x, double y, double toler )
{
 double   xmin, xmax, ymin, ymax;
 int      cod0[4], cod1[4];
 int      i;

/* Define atraction window.
 */
 xmin = x - toler;
 xmax = x + toler;
 ymin = y - toler;
 ymax = y + toler;

/* Check to see if given point (x,y) touches given line (x0,y0)-(x1,y1)
 * based on given tolerance.
 */
 CllPickCode( x1, y1, xmin, xmax, ymin, ymax, cod1 );
 do
 {
  CllPickCode( x0, y0, xmin, xmax, ymin, ymax, cod0 );
  for( i = 0; i < 4; i++ )
   if( cod0[i] && cod1[i] )     /* test no-trivial pick */
    break;
  if( i != 4 )
   break;
  /* move point 0 to window limit */
  if( cod0[0] )
   y0 += (xmin - x0) * (y1 - y0) / (x1 - x0), x0 = xmin;
  else if( cod0[1] )
   y0 += (xmax - x0) * (y1 - y0) / (x1 - x0), x0 = xmax;
  else if( cod0[2] )
   x0 += (ymin - y0) * (x1 - x0) / (y1 - y0), y0 = ymin;
  else if( cod0[3] )
   x0 += (ymax - y0) * (x1 - x0) / (y1 - y0), y0 = ymax;
  else
   return( 1 );
 } while( 1 );

 return( 0 );
}

/* ==========================  CllStraight  ======================== */

static int CllStraight( CllBezier *bezier, double toler )
{
 double   tol = toler * 0.1;
 return( CllPickLine( bezier->p0.x, bezier->p0.y, bezier->p1.x, bezier->p1.y, 
                      bezier->pc0.x, bezier->pc0.y, tol ) && 
         CllPickLine( bezier->p0.x, bezier->p0.y, bezier->p1.x, bezier->p1.y, 
                      bezier->pc1.x, bezier->pc1.y, tol ) );
}

/* =========================  CllDrawCurve  ======================== */

static void CllDrawCurve( CllBezier *bezier, double toler )
{
 if( CllStraight( bezier, toler ) )
 {
  wdLine( bezier->p0.x, bezier->p0.y, bezier->p1.x, bezier->p1.y );
 }
 else
 {
  CllBezier left, right;
  CllPnt    half;

  half.x = bezier->pc0.x + 0.5 * ( bezier->pc1.x - bezier->pc0.x );
  half.y = bezier->pc0.y + 0.5 * ( bezier->pc1.y - bezier->pc0.y );
 
  left.p0    = bezier->p0;
  left.pc0.x = bezier->p0.x + 0.5 * ( bezier->pc0.x - bezier->p0.x );
  left.pc0.y = bezier->p0.y + 0.5 * ( bezier->pc0.y - bezier->p0.y );
  left.pc1.x = left.pc0.x + 0.5 * ( half.x - left.pc0.x );
  left.pc1.y = left.pc0.y + 0.5 * ( half.y - left.pc0.y );
  
  right.p1    = bezier->p1;
  right.pc1.x = bezier->pc1.x + 0.5 * ( bezier->p1.x - bezier->pc1.x );
  right.pc1.y = bezier->pc1.y + 0.5 * ( bezier->p1.y - bezier->pc1.y );
  right.pc0.x = half.x + 0.5 * ( right.pc1.x - half.x );
  right.pc0.y = half.y + 0.5 * ( right.pc1.y - half.y );
 
  left.p1.x = right.p0.x = left.pc1.x + 0.5 * ( right.pc0.x - left.pc1.x );
  left.p1.y = right.p0.y = left.pc1.y + 0.5 * ( right.pc0.y - left.pc1.y );

  CllDrawCurve( &left, toler );
  CllDrawCurve( &right, toler );
 }
}

/* =======================  CllBezierDisplay  ====================== */

static void CllBezierDisplay( CllBezierP *bezier, double toler )
{
/* Draw bezier
 */
 CllDrawCurve( &bezier->b, toler );

/* Draw a mark at each control point
 */
 wdMark( bezier->b.p0.x,  bezier->b.p0.y );
 wdMark( bezier->pi0.x,   bezier->pi0.y );
 wdMark( bezier->pi1.x,   bezier->pi1.y );
 wdMark( bezier->b.p1.x,  bezier->b.p1.y );
 if( (fabs(bezier->b.pc0.x - bezier->pi0.x) > 0.0001) || 
     (fabs(bezier->b.pc0.y - bezier->pi0.y) > 0.0001) )
  wdMark( bezier->b.pc0.x, bezier->b.pc0.y );
 if( (fabs(bezier->b.pc1.x - bezier->pi1.x) > 0.0001) || 
     (fabs(bezier->b.pc1.y - bezier->pi1.y) > 0.0001) )
  wdMark( bezier->b.pc1.x, bezier->b.pc1.y );
}

/* ======================  CllBezierDisplayXor  ==================== */

static void CllBezierDisplayXor( CllBezierP *bezier, double toler )
{
 int    write_mode;
 int    fore_color;
 int    mark_type;
 int    mark_size;
 int    line_style;
 int    line_width;

#ifdef _UNIX_
 write_mode = cdWriteMode( CD_XOR );
#else
 write_mode = cdWriteMode( CD_NOT_XOR );
#endif
 fore_color = cdForeground( CD_RED );
 mark_type = cdMarkType( CD_BOX );
 mark_size = cdMarkSize( 4 );
 line_style = cdLineStyle( CD_CONTINUOUS );
 line_width = cdLineWidth( 1 );

 CllBezierDisplay( bezier, toler );

 cdLineWidth( line_width );
 cdLineStyle( line_style );
 cdMarkSize( mark_size );
 cdMarkType( mark_type );
 cdForeground( fore_color );
 cdWriteMode( write_mode );

 bezier->displayed ^= 1;
}

/* =======================  CllBezierRedisplay  ==================== */

static void CllBezierRedisplay( void *b )
{
 CllBezierP *bezier = (CllBezierP *)b;

/* This is the callback for the collector superclass redisplay.
 * Force the display of a created curve.
 */
 bezier->displayed = 0;

 if( bezier->created )
  CllBezierDisplayXor( bezier, bezier->toler );
}

/* ========================  CllBezierEcho  ======================== */

static void CllBezierEcho( PckDesc *pk, double echoptx, double echopty )
{
/* Restore registered collection spec. from the data field.
 */
 CllBezierP  *bezier = (CllBezierP *)pk->data;

 bezier->toler = pk->toler;

/* Check to see if a curve has been modified.
 * If that is the case, draw the curve in Xor mode.
 */
 if( bezier->created )
 {
  if( bezier->modified || !bezier->displayed )
  {
   CllBezierDisplayXor( bezier, bezier->toler );
  }
 }

/* Check to see whether client had registered an echo function.
 * If that is the case call it passing the given mouse position.
 */
 if( bezier->echo != 0L )
  (*bezier->echo )( bezier->iup_cv, echoptx, echopty );
}

/* =====================  CllBezierP0CrtEcho  ====================== */

static void CllBezierP0CrtEcho( PckDesc *pk, double echoptx, double echopty )
{
 CllBezierP *bezier = (CllBezierP *)pk->data;

 bezier->b.p0.x = echoptx;
 bezier->b.p0.y = echopty;
 CllBezierCrvToNet( bezier );
 bezier->modified = 1;
 CllBezierEcho( pk, echoptx, echopty );
}

/* ====================  CllBezierPi0CrtEcho  ====================== */

static void CllBezierPi0CrtEcho( PckDesc *pk, double echoptx, double echopty )
{
 CllBezierP *bezier = (CllBezierP *)pk->data;

 bezier->pi0.x = echoptx;
 bezier->pi0.y = echopty;
 CllBezierCrvToNet( bezier );
 bezier->modified = 1;
 CllBezierEcho( pk, echoptx, echopty );
}

/* ====================  CllBezierPi1CrtEcho  ====================== */

static void CllBezierPi1CrtEcho( PckDesc *pk, double echoptx, double echopty )
{
 CllBezierP *bezier = (CllBezierP *)pk->data;

 bezier->pi1.x = echoptx;
 bezier->pi1.y = echopty;
 CllBezierCrvToNet( bezier );
 bezier->modified = 1;
 CllBezierEcho( pk, echoptx, echopty );
}

/* =====================  CllBezierP1CrtEcho  ====================== */

static void CllBezierP1CrtEcho( PckDesc *pk, double echoptx, double echopty )
{
 CllBezierP *bezier = (CllBezierP *)pk->data;

 bezier->b.p1.x = echoptx;
 bezier->b.p1.y = echopty;
 CllBezierCrvToNet( bezier );
 bezier->modified = 1;
 CllBezierEcho( pk, echoptx, echopty );
}

/* ====================  CllBezierPc0CrtEcho  ====================== */

static void CllBezierPc0CrtEcho( PckDesc *pk, double echoptx, double echopty )
{
 CllBezierP *bezier = (CllBezierP *)pk->data;

 bezier->b.pc0.x = echoptx;
 bezier->b.pc0.y = echopty;
 CllBezierNetToCrv( bezier );
 bezier->modified = 1;
 CllBezierEcho( pk, echoptx, echopty );
}

/* ====================  CllBezierPc1CrtEcho  ====================== */

static void CllBezierPc1CrtEcho( PckDesc *pk, double echoptx, double echopty )
{
 CllBezierP *bezier = (CllBezierP *)pk->data;

 bezier->b.pc1.x = echoptx;
 bezier->b.pc1.y = echopty;
 CllBezierNetToCrv( bezier );
 bezier->modified = 1;
 CllBezierEcho( pk, echoptx, echopty );
}

/* ====================  CllBezierDef_BtDwnCB  ===================== */

static void CllBezierDef_BtDwnCB( PckDesc *pk )
{
 CllBezierP *bezier = (CllBezierP *)pk->data;

 bezier->toler = pk->toler;

/* Turn off displayed curve.
 */
 if( bezier->displayed )
 {
  CllBezierDisplayXor( bezier, bezier->toler );
 }

 if( CllPickPoint( bezier->b.p0.x, bezier->b.p0.y, 
                   pk->p0.x, pk->p0.y, bezier->toler ) )
  pk->echofunc = CllBezierP0CrtEcho;
 else if( CllPickPoint( bezier->b.p1.x, bezier->b.p1.y, 
                        pk->p0.x, pk->p0.y, bezier->toler ) )
  pk->echofunc = CllBezierP1CrtEcho;
 else if( CllPickPoint( bezier->b.pc0.x, bezier->b.pc0.y, 
                        pk->p0.x, pk->p0.y, bezier->toler ) )
  pk->echofunc = CllBezierPc0CrtEcho;
 else if( CllPickPoint( bezier->b.pc1.x, bezier->b.pc1.y, 
                        pk->p0.x, pk->p0.y, bezier->toler ) )
  pk->echofunc = CllBezierPc1CrtEcho;
 else if( CllPickPoint( bezier->pi0.x, bezier->pi0.y, 
                        pk->p0.x, pk->p0.y, bezier->toler ) )
  pk->echofunc = CllBezierPi0CrtEcho;
 else if( CllPickPoint( bezier->pi1.x, bezier->pi1.y, 
                        pk->p0.x, pk->p0.y, bezier->toler ) )
  pk->echofunc = CllBezierPi1CrtEcho;
 else
 {
  bezier->modified = 0;
  pk->echofunc = CllBezierEcho;
 }
}

/* =====================  CllBezierDef_BtUpCB  ===================== */

static void CllBezierDef_BtUpCB( PckDesc *pk )
{
 CllBezierP *bezier = (CllBezierP *)pk->data;
 void       *iup_cv = bezier->iup_cv;
 void       (*end_action)( Ihandle *iup_cv, int status ) = bezier->end_action;

 bezier->toler = pk->toler;

/* If user gave up reshaping the curve, call the client
 * function to put the collected curve and quit collection.
 * If collection was interupted, just quit collection.
 * In either case call end action function and return.
 */
 if( pk->status != PCK_VALID )
 {
  int status;

  CllClose( );

/* Turn off displayed curve.
 */
  if( bezier->displayed )
  {
   CllBezierDisplayXor( bezier, bezier->toler );
  }
  switch( pk->status )
  {
   default:
   case PCK_QUIT:
    if( bezier->put != 0L )
     (*bezier->put )( bezier->iup_cv, &bezier->b, bezier->toler );
    status = CLL_NORMAL;
    break;
   case PCK_INTERUPTED:
    status = CLL_INTERUPTED;
    break;
  }
  CllBezierDel( bezier );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Keep getting picks from the user.
 */
 bezier->modified = 0;
 if( bezier->prompt != 0L )
  (*bezier->prompt )( bezier->iup_cv, prompt_msg3 ) ;
 pk->button = 1;       /* accept only mouse button 1 */
 pk->echofunc = CllBezierEcho;
 PckSetup( pk );
}

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* ========================  CllBezierCreate  ====================== */

static void CllBezier1stPt( PckDesc * );
static void CllBezierEndPt( PckDesc * );

void CllBezierCreate( Ihandle *iup_cv, void *cd_cv,
              void (*end_action)( Ihandle *iup_cv, int status ),
              void (*put)( Ihandle *iup_cv, void *bez, double tol ),
              void (*prompt)( Ihandle *iup_cv, char *msg ),
              void (*snap)( Ihandle *iup_cv, double *x, double *y,  
                            double tol, int shift, int control ),
              void (*echo)( Ihandle *iup_cv, double x, double y ) )
{
 CllBezierP   *bezier;
 PckDesc      pk;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  prompt_msg1 = prompt_msg1e;
  prompt_msg2 = prompt_msg2e;
  prompt_msg3 = prompt_msg3e;
 }
 else
 {
  prompt_msg1 = prompt_msg1p;
  prompt_msg2 = prompt_msg2p;
  prompt_msg3 = prompt_msg3p;
 }

 CllClose( );

/* Allocate a bezier collection descriptor
 */
 bezier = (CllBezierP *)malloc( sizeof( CllBezierP ) );

/* Initialize collection
 */
 bezier->fst = 0;
 bezier->created = 0;
 bezier->modified = 0;
 bezier->displayed = 0;
 bezier->toler = 0.0;
 bezier->iup_cv = iup_cv; 
 bezier->cd_cv = cd_cv; 
 bezier->end_action = end_action; 
 bezier->put = put; 
 bezier->prompt = prompt; 
 bezier->snap = snap; 
 bezier->echo = echo;
 bezier->b.p0.x = bezier->b.p0.y = 0.0;
 bezier->b.p1.x = bezier->b.p1.y = 0.0;
 bezier->b.pc0.x = bezier->b.pc0.y = 0.0;
 bezier->b.pc1.x = bezier->b.pc1.y = 0.0;

/* Register bezier redisplay function with collection super class.
 */
 CllOpen( bezier, CllBezierRedisplay );

/* Setup pick descriptor.  Register the appropriate bezier collector
 * function as the callback for pick interaction.
 */
 if( bezier->prompt != 0L )
  (*bezier->prompt)( bezier->iup_cv, prompt_msg1 );
 pk.iup_cv = bezier->iup_cv;
 pk.gra_cv = bezier->cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.mode = ButtonDown;
 pk.cursorecho = BoxCursor | CrossCursor;
 pk.button = 0;    /* accept both mouse buttons 1 or 2 */
 pk.toler = 0.0;
 pk.btdwn_callback = CllBezier1stPt;
 pk.btup_callback = 0L;
 pk.echofunc = CllBezierEcho;
 pk.snapfunc = CllBezierSnap;
 pk.data = bezier;
 pk.status = PCK_VALID;
 PckSetup( &pk );
}

/* -------------------------  CllBezier1stPt  ----------------------- */

static void CllBezier1stPt( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllBezierP *bezier = (CllBezierP *)pk->data;
 void       *iup_cv = bezier->iup_cv;
 void       (*end_action)( Ihandle *iup_cv, int status ) = bezier->end_action;

/* Check to see if pick is valid.  If not, delete collection data
 * and call end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = CLL_QUIT;        break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllClose( );
  CllBezierDel( bezier );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Get button down point as first curve point.
 */
 bezier->b.p0.x = pk->p0.x;
 bezier->b.p0.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter the
 * first point using the keyboard. In this case, prompt the user
 * for point coordinates.
 */
 if( pk->button == 2 )
 {
  float x0, y0;
  x0 = bezier->b.p0.x;
  y0 = bezier->b.p0.y;
  if( DlgCapturePBezier( &x0, &y0 ) )
  {
   bezier->b.p0.x = pk->p0.x = x0;
   bezier->b.p0.y = pk->p0.y = y0;
  }
 }

/* Set flag to indicate that first point has been selected.
 */
 bezier->fst = 1;

/* Change cursor echo for rubber band line (first point
 * is initial point of rubber band line) and get pick from user
 * for end curve point.
 */
 if( bezier->prompt != NULL )
  (*bezier->prompt )( bezier->iup_cv, prompt_msg2 ) ;
 pk->cursorecho = BoxCursor | CrossCursor | RubberBandCursor;
 pk->button = 0;       /* accept both mouse buttons 1 or 2 */
 pk->btdwn_callback = CllBezierEndPt;
 PckSetup( pk );
}

/* ---------------------------  CllBezierEndPt  ----------------------- */

static void CllBezierEndPt( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllBezierP *bezier = (CllBezierP *)pk->data;
 void       *iup_cv = bezier->iup_cv;
 void       (*end_action)( Ihandle *iup_cv, int status ) = bezier->end_action;

 bezier->toler = pk->toler;

/* Get button down point as end curve point.
 */
 bezier->b.p1.x = pk->p0.x;
 bezier->b.p1.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter the
 * bezier using the keyboard.  In this case, prompt the user for
 * bezier coordinates and return.
 */
 if( (pk->status == PCK_VALID) && (pk->button == 2) )
 {
  float  p0x, p0y, pi0x, pi0y, pi1x, pi1y, p1x, p1y;

  CllClose( );

  bezier->b.pc0.x = bezier->b.p0.x + ( bezier->b.p1.x - bezier->b.p0.x ) / 3.0;
  bezier->b.pc0.y = bezier->b.p0.y + ( bezier->b.p1.y - bezier->b.p0.y ) / 3.0;
  bezier->b.pc1.x = bezier->b.p0.x + 
                               ( bezier->b.p1.x - bezier->b.p0.x ) / 3.0 * 2.0;
  bezier->b.pc1.y = bezier->b.p0.y + 
                               ( bezier->b.p1.y - bezier->b.p0.y ) / 3.0 * 2.0;
  CllBezierNetToCrv( bezier );
  p0x = bezier->b.p0.x;
  p0y = bezier->b.p0.y;
  pi0x = bezier->pi0.x;
  pi0y = bezier->pi0.y;
  pi1x = bezier->pi1.x;
  pi1y = bezier->pi1.y;
  p1x = bezier->b.p1.x;
  p1y = bezier->b.p1.y;
  if( DlgCaptureLBezier( &p0x,  &p0y,  &pi0x, &pi0y, 
		         &pi1x, &pi1y, &p1x,  &p1y ) )
  {
   bezier->b.p0.x = p0x;
   bezier->b.p0.y = p0y;
   bezier->pi0.x = pi0x;
   bezier->pi0.y = pi0y;
   bezier->pi1.x = pi1x;
   bezier->pi1.y = pi1y;
   bezier->b.p1.x = p1x;
   bezier->b.p1.y = p1y;
   CllBezierCrvToNet( bezier );
   if( LEN( bezier->b.p0, bezier->b.p1 ) >= bezier->toler )
   {
    if( bezier->put != 0L )
     (*bezier->put )( bezier->iup_cv, &bezier->b, bezier->toler );
   }
   CllBezierDel( bezier );
   if( end_action ) (*end_action)( iup_cv, CLL_NORMAL );
  }
  else
  {
   CllBezierDel( bezier );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
  }
  return;
 }

/* Check to see if collected bezier is valid (greater then tolerance).
 * If not valid, just quit the collection, calling end action function.
 */
 if( (pk->status != PCK_VALID) || 
     (LEN(bezier->b.p0,bezier->b.p1) < bezier->toler) )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_VALID:
   case PCK_QUIT:        status = CLL_NORMAL;        break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllClose( );
  CllBezierDel( bezier );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Start with a straight bezier curve.
 * Setup pick descriptor to drag control points.  Register the appropriate
 * control poiny collector functions as the callbacks for pick interaction.
 */
 if( bezier->prompt != 0L )
  (*bezier->prompt)( bezier->iup_cv, prompt_msg3 );
 bezier->b.pc0.x = bezier->b.p0.x + ( bezier->b.p1.x - bezier->b.p0.x ) / 3.0;
 bezier->b.pc0.y = bezier->b.p0.y + ( bezier->b.p1.y - bezier->b.p0.y ) / 3.0;
 bezier->b.pc1.x = bezier->b.p0.x + 
                              ( bezier->b.p1.x - bezier->b.p0.x ) / 3.0 * 2.0;
 bezier->b.pc1.y = bezier->b.p0.y + 
                              ( bezier->b.p1.y - bezier->b.p0.y ) / 3.0 * 2.0;
 CllBezierNetToCrv( bezier );
 bezier->created = 1;
 bezier->modified = 0;
 bezier->displayed = 0;
 pk->cursorecho = BoxCursor | CrossCursor;
 pk->button = 1;       /* accept only mouse button 1 */
 pk->mode = ButtonUp;
 pk->btdwn_callback = CllBezierDef_BtDwnCB;
 pk->btup_callback  = CllBezierDef_BtUpCB;
 PckSetup( pk );
}


/* ======================  CllKbdBezierCreate  ===================== */

static CllBezier  kbdbezier;
static double     kbdtol = 0.001;

static char *kbdprompt_msg;

static char *kbdprompt_msgp = { "Digite dados da curva bezier. " };
static char *kbdprompt_msge = { "Use the keyboard to enter bezier data. " };

int  CllKbdBezierCreate( Ihandle *iup_cv,
              void (*put)( Ihandle *iup_cv, void *bez, double tol ),
              void (*prompt)( Ihandle *iup_cv, char *msg ) )
{
 CllBezierP  bezier;
 float       p0x, p0y, pi0x, pi0y, pi1x, pi1y, p1x, p1y;
 float       toler;
 int         status = CLL_QUIT;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  kbdprompt_msg = kbdprompt_msge;
 }
 else
 {
  kbdprompt_msg = kbdprompt_msgp;
 }

/* Initialize collection
 */
 bezier.b = kbdbezier;

/* Collect data from user with the keyboard.
 */
 if( prompt != 0L )
  (*prompt)( iup_cv, kbdprompt_msg );

 CllBezierNetToCrv( &bezier );
 p0x = bezier.b.p0.x;
 p0y = bezier.b.p0.y;
 pi0x = bezier.pi0.x;
 pi0y = bezier.pi0.y;
 pi1x = bezier.pi1.x;
 pi1y = bezier.pi1.y;
 p1x = bezier.b.p1.x;
 p1y = bezier.b.p1.y;
 toler = kbdtol;
  if( DlgCaptureBezier( &p0x,  &p0y,  &pi0x, &pi0y, 
		        &pi1x, &pi1y, &p1x,  &p1y, &toler ) )
 {
  bezier.b.p0.x = p0x;
  bezier.b.p0.y = p0y;
  bezier.pi0.x = pi0x;
  bezier.pi0.y = pi0y;
  bezier.pi1.x = pi1x;
  bezier.pi1.y = pi1y;
  bezier.b.p1.x = p1x;
  bezier.b.p1.y = p1y;
  CllBezierCrvToNet( &bezier );
  if( LEN( bezier.b.p0, bezier.b.p1 ) > 0.0 )
  {
   kbdbezier = bezier.b;
   kbdtol = toler;
   if( put != 0L )
    (*put)( iup_cv, &bezier.b, kbdtol );
   status = CLL_NORMAL;
  }
 }

 return( status );
}

