/*
** cllelipse.c - Functions to collect ellipses.
**
** Public functions:
** --------------------------------------------------------------------
**
** void CllElipseCreate( Ihandle *iup_cv, void *cd_cv,
**                 void (*end_action)( Ihandle *iup_cv, int status ),
**                 void (*put)( Ihandle *iup_cv, void *elipse, double tol ), 
**                 void (*prompt)( Ihandle *iup_cv, char *msg ), 
**                 void (*snap)( Ihandle *iup_cv, double *x, double *y,
**                               double tol, int shift, int control ),
**                 void (*echo)( Ihandle *iup_cv, double x, double y ) )
**
** iup_cv     - IUP handle of canvas where primitive is collected   (IN)
** cd_cv      - CD handle of canvas                                 (IN)
** end_action - funct. to be called after finishing elipse creation (IN)
** put        - ptr. to function to put elipse after collection     (IN)
** prompt     - ptr. to function to prompt messg. during collect.   (IN)
** snap       - ptr. to function to snap points during collection   (IN)
** echo       - ptr. to function to echo cursor position            (IN)
**
** This function interactively collects from the user an elipse
** on a given canvas.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal collection ==> status = CLL_NORMAL = 1
**     This is also returned if user quit during collection, after
**     entering the first, second or third points.
** (2) Quit collection ==> status = CLL_QUIT = 0
**     This is returned if user quit collection at the begining,
**     before entering the first point.
** (3) Interupted collection ==> status = CLL_INTERUPTED = -1
**     This is returned when the collection is closed by something 
**     else (not by the mouse interaction). 
**
** --------------------------------------------------------------------
**
** int  CllKbdElipseCreate( Ihandle *iup_cv,
**                       void (*put)( Ihandle *iup_cv, void *elipse, 
**                                    double tol ),
**                       void (*prompt)( Ihandle *iup_cv, char *msg ) )
** 
** iup_cv     - IUP handle of canvas for client reference         (IN)
** put        - ptr. to function to put prmtv. after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
**
** This function interactively collects from the keyboard an elipse.
** If the collection is successfull the function returns a CLL_NORMAL (1)
** status.  Otherwise, it returns a CLL_QUIT (0) status.
**
** --------------------------------------------------------------------
**
** Version:      0-001
**
** Created:    18-Mar-2005    Marcos C. de Arruda
**
** Version:      0-002
**
** Modified    17-Mar-2006    Rafael Araujo de Sousa
**
**   Y
**   
**   ^
**   |                                          
**   |                                 ||||||||||||||
**   |                              ||||||||||||||||||| 
**   |                             ||||             |||||
**   |                           ||||               / ||||
**   |                         |||                 /    |||  
**   |                        |||                 /      |||
**   |                      |||                  /        |||
**   |                     |||                  /          |||
**   |                    |||                  /           |||
**   |                   ||| \      lambda    /            |||
**   |                 |||     \       ____  /            |||
**   |                |||        \    /    \/             |||
**   |               |||           \ /     /___          |||
**   |              |||              \    /    \ theta   |||
**   |             |||                 \ /      \ (ang) |||
**   |            |||-------------------|-------|------|||
**   |           |||                    C             |||
**   |          |||                                  |||
**   |         |||                                  |||
**   |        |||                                  |||
**   |        |||                                 |||
**   |       |||                                 |||
**   |       |||                                |||
**   |        |||                               |||
**   |        |||                             |||
**   |         |||                           |||
**   |          |||                        |||
**   |           ||||                    ||| 
**   |             ||||                |||| 
**   |               |||||||||||||||||||
**   |                 |||||||||||||||
**   |
**   |
**   |------------------------------------------------------------> X
**  ORIGEM
**
**    IMPLEMENTAO PARAMTRICA DA ELIPSE 
**    Referncia: http://www.spaceroots.org/documents/ellipse.
**
**    NOTAO:
**    C ->  o centro da elipse com coordenadas (xc,yc),
**    theta -> (ang)  a inclinao que o primeiro semi-eixo forma com o eixo 
**             horizontal do plano Carteziano,
**    lambda ->  o ngulo geomtrico entre um ponto da elipse e o semi-eixo
**              principal (primeiro semi-eixo)
**    a ->  o tamanho do primeiro semi-eixo, 
**    b ->  o tamanho do segundo semi-eixo,
**    eta ->  um ngulo terico (no teria representao na figura, exceto no
**           caso de um crculo).
**
**    RELAES ENTRE AS VARIVEIS:
**
**    r*cos(lambda) = a*cos(eta)  e  r*sin(lambda) = b*sin(eta)  (1)
**
**    Das equaes (1) acima, dividindo uma pela outra, pode-se determinar eta 
**    em funo das outras variveis:
** 
**    eta = arctan2((sin(lambda)/b), (cos(lambda)/a))
**
**    Desta forma para se determinar qualquer ponto da elipse usa-se a seguinte 
**    equao:
**
**    xel = xc + a*cos(theta)*cos(eta) - b*sin(theta)*sin(eta)
**    yel = yc + a*sin(theta)*cos(eta) + b*cos(theta)*sin(eta)
**
** Modified:   22-Mar-2006    Luiz F. Martha
**    Modified functions CllElipseDrawAxis and CllElipse2ndPoint to compute 
**    the secondary half-axis size as the dot product of the vector between 
**    the given point and the center with the secondary half-axis direction.
**    Eliminated point array variable in display function.
**    Made local string array variable static in echo function.
**    Modified function CllEllipseSnap to snap given point to secondary
**    half-axis and, in case of shift key pressed, make ellipse a circle.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "iup.h"
#include "cd.h"
#include "wd.h"
#include "pck.h"
#include "cll.h"
#include "cllelipse.h"
#include "dlgelipse.h"


/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/
/*  Arc Elipse data structure:
 */
typedef struct _cllelipsep  {   /* Collected arc private spec.descrip.:   */
        CllElipse   e;                 /* collected elipse                */
        int         ctr;               /* if true, center already given   */
        int         fst;               /* if true, first pt. given        */
        int         scd;               /* if true, second pt. given       */
        Ihandle     *iup_cv;           /* IUP handle to target canvas     */
        void        *cd_cv;            /* cd handle to corresp.canvas     */
        void        (*end_action)      /* end action callback             */
                          ( Ihandle *iup_cv, int status );
        void        (*put)             /* function to put collect. elipse */
                          ( Ihandle *iup_cv, void *elipse, double tol );
        void        (*prompt)          /* function to prompt messages     */
                          ( Ihandle *iup_cv, char *msg );
        void        (*snap)            /* function to snap collect. pt.   */
                          ( Ihandle *iup_cv, double *x, double *y, 
                                          double tol, int shift, int control );
        void        (*echo)            /* function to echo cursor point   */
                          ( Ihandle *iup_cv, double x, double y );
        } CllElipseP;

#ifndef LEN
#define LEN(p0,p1) sqrt((p1.x-p0.x)*(p1.x-p0.x)+(p1.y-p0.y)*(p1.y-p0.y))
#endif
#ifndef dist
#define dist(x0,y0,x1,y1) sqrt((x1-x0)*(x1-x0)+(y1-y0)*(y1-y0))
#endif
#ifndef PI
#define PI           fabs(acos(-1.))
#endif
#ifndef RAD
#define RAD(a)       ((a)*(PI/180.0))
#endif
#ifndef DEG
#define DEG(a)       ((a)*(180.0/PI))
#endif

static int  iup_english;
static char *prompt_msg1;
static char *prompt_msg2;
static char *prompt_msg3;

static char *prompt_msg1p = { "Com botao esquerdo do mouse entre "
                              "com o centro da elipse; "
                              "Botao do meio => Teclado." };
static char *prompt_msg2p = { "Entre com o ponto que define o primeiro "
                              "semi-eixo da elipse "
                              "Botao do meio => Teclado." };
static char *prompt_msg3p = { "Entre com o ponto que define o segundo " 
                              "semi-eixo da elipse "
                              "Botao do meio => Teclado." };
static char *prompt_msg1e = { "With mouse left button enter elipse center; "
                              "Mid button => Keyboard." };
static char *prompt_msg2e = { "Enter point to define the first "
                              "half-axis of the elipse " };
static char *prompt_msg3e = { "Enter point to define the second "
                              "half-axis of the elipse " };

static double _tol = 1e-5;

/*
** --------------------------------------------------------------------
** Local functions:
*/

static void CllElipseTranslPt       ( CllPnt* pt, double xc, double yc);
static void CllElipseRotPt          ( CllPnt* pt, double ang);
static int CllElipseAngle           ( double xc, double yc, double x, double y,
                                      double *a );
static int CllElipseDrawAxis        ( double xc, double yc, double a, 
                                      double ang, double x, double y,
                                      double *b );
static int CllElipseDrawElipse      ( double xc, double yc, double ang, 
                                      double a, double b );
static void CllElipseDel            ( CllElipseP *elipse );
static void CllElipseSnap           ( PckDesc *pk, double *x, double *y, 
                                      double tol );
static void CllElipseEcho           ( PckDesc *pk, double x, double y );
                                      
/* ========================  CllElipseAngle  ======================== */
/*
** Esta funo calcula o ngulo em relao  horizontal (direo x) da reta
** que passa pelos pontos (xc, yc) e (x, y). Este ngulo necessariamente
** vai estar entre 0 e 360 graus. 
*/
static int CllElipseAngle( double xc, double yc, double x, double y, double *a )
{
 double d;

 d = dist(x, y, xc, yc);
 if( d <= _tol )       /* modificado por Rafael em 09/03/2006 */
  return( 0 );

 *a = atan2( y-yc, x-xc );
 *a = DEG( *a );
 if (*a < 0.0)
  *a += 360.0;

 return( 1 );
}

/* ========================  CllElipseDrawAxis  ======================== */

static int CllElipseDrawAxis( double xc, double yc, double a, double ang,
                              double x, double y, double *b )
{ 
 double x0, y0;     /* primeiro pt que estar no segundo semi-eixo */
 double x0s, y0s;   /* inverso do pt acima */
 int line_style;
 
/* Compute the secondary half-axis size as the dot product of the
 * vector between the given point and the center with the secondary
 * half-axis direction.
 */
 *b = fabs(- (x - xc) * sin(RAD(ang)) + (y - yc) * cos(RAD(ang)));
 if (*b < _tol)
  return( 0 );

 x0  = xc - (*b) * sin(RAD(ang));
 y0  = yc + (*b) * cos(RAD(ang));
 x0s = xc + (*b) * sin(RAD(ang));
 y0s = yc - (*b) * cos(RAD(ang));
 
 line_style = cdLineStyle( CD_DASHED );
 wdLine(x0s, y0s, x0, y0);
 cdLineStyle(line_style);
 return( 1 );
}

/* ========================  CllElipseTranslPt  ======================== */

static void CllElipseTranslPt( CllPnt* pt, double xc, double yc)
{
 pt->x = pt->x + xc;
 pt->y = pt->y + yc;
}

/* ========================  CllElipseRotPt  ======================== */

static void CllElipseRotPt( CllPnt* pt, double ang)
{
 double x;
 double y;
 
 x = pt->x;
 y = pt->y;
 pt->x = x*cos(RAD(ang)) - y*sin(RAD(ang));
 pt->y = x*sin(RAD(ang)) + y*cos(RAD(ang));
}

/* ========================  CllElipseDrawElipse  ======================== */

static int CllElipseDrawElipse( double xc, double yc, double ang, 
                                double a, double b )
{
 int i;           /* contador */
 int n;           /* nmero de pontos que sero colotados */ 
 double d_theta;  /* incremento angular */
 CllPnt v0;       /* ponto inicial do segmento no transformado */
 CllPnt v1;       /* ponto final do segmento no transformado */
 CllPnt v0t;      /* ponto inicial do segmento transformado */
 
 if ((a < _tol) || (b < _tol)) return 0;
 
 n = 200;

/* definio do incremento */
 d_theta = 2*PI / n;

 cdBegin( CD_CLOSED_LINES );

/* ponto de partida para se comear a ligar (desenhar) os pontos */ 
 v0.x = a;
 v0.y = 0.0;
 for (i = 0; i < n; i++)
 {
/* transforma o ponto para a sua posio na elipse deslocada e inclinada */
  v0t = v0;
  CllElipseRotPt( &v0t, ang );
  CllElipseTranslPt( &v0t, xc, yc );
  wdVertex( v0t.x, v0t.y );

/* acha o prximo ponto na elipse de forma adaptativa */
  v1.x = v0.x*cos(d_theta) - (a/b)*v0.y*sin(d_theta);
  v1.y = (b/a)*v0.x*sin(d_theta) + v0.y*cos(d_theta);
  v0 = v1;
 }
 cdEnd( );

 return ( 1 );
}

/* ==========================  CllElipseDel  ======================== */

static void CllElipseDel( CllElipseP *elipse )
{
 if( elipse == 0L ) return;

/*  Release memory used for arc.
 */    
 free( elipse );
}

/* ========================  CllElipseSnap  ========================= */

static void CllElipseSnap( PckDesc *pk, double *x, double *y, double tol )
{
 double b;

/* Restore registered collection spec. from the data field.
 */
 CllElipseP  *elipse = (CllElipseP *)pk->data;

 if( (elipse->ctr) && (!elipse->fst) && isshift(pk->mousecfg) )
 {
/* If shift key is pressed and center arc point has been given,
 * snap radius to the horizontal axis, to the vertical axis, or
 * the bi-sector lines.
 */
  CllShiftSnap( elipse->e.center.x, elipse->e.center.y, x, y );
 }
 else if( (elipse->ctr) && (elipse->fst) && (!elipse->scd) )
 {
/* If center and first points have been given, snap point to the 
 * secondary ellipse axis.  If shift key is pressed, make ellipse
 * be a circle.
 */
  b = - (*x - elipse->e.center.x) * sin(RAD(elipse->e.ang)) 
      + (*y - elipse->e.center.y) * cos(RAD(elipse->e.ang));
  if( isshift(pk->mousecfg) )
  {
   if( b < 0.0 )
    b = -elipse->e.a;
   else
    b =  elipse->e.a;
  }
  *x = elipse->e.center.x - b * sin(RAD(elipse->e.ang));
  *y = elipse->e.center.y + b * cos(RAD(elipse->e.ang));
 }

/* Check to see whether client had registered a snap function and 
 * call it passing the given mouse position and parameters.
 */
 if( elipse->snap != 0L )
  (*elipse->snap )( elipse->iup_cv, x, y, tol, 
                    isshift(pk->mousecfg), iscontrol(pk->mousecfg) );
}

/* ========================  CllElipseEcho  ========================= */
 
static void CllElipseEcho( PckDesc *pk, double x, double y )
{
 int    write_mode;
 int    fore_color;
 int    mark_type;
 int    mark_size;
 int    line_style;
 int    line_width;
 double a, b;
 double ang;
 CllPnt pt;
 static char echotext[128];

/* Restore registered collection spec. from the data field.
 */
 CllElipseP  *elipse = (CllElipseP *)pk->data;
 
#ifdef _UNIX_
 write_mode = cdWriteMode( CD_XOR );
#else
 write_mode = cdWriteMode( CD_NOT_XOR );
#endif
 fore_color = cdForeground( CD_RED );
 mark_type = cdMarkType( CD_BOX );
 mark_size = cdMarkSize( 4 );
 line_style = cdLineStyle( CD_CONTINUOUS );
 line_width = cdLineWidth( 1 );

/* Check to see if center has been given.
 * If that is the case, draw a mark at the center in Xor mode.
 */
 if( elipse->ctr )
 {
  wdMark( elipse->e.center.x, elipse->e.center.y );

  if( !elipse->fst && CllElipseAngle(elipse->e.center.x, elipse->e.center.y,
                                     x, y, &ang) )
  {
   pt.x = x;  pt.y = y;
   a = LEN( elipse->e.center, pt );
   if( iup_english )
    sprintf( echotext, "First half-axis = %10.3f;   Angle = %7.2f", a, ang );
   else
    sprintf( echotext, "Primeiro semi-eixo = %10.3f;   Angulo = %7.2f", 
             a, ang );
   if( elipse->prompt != 0L )
    (*elipse->prompt)( elipse->iup_cv, echotext );
  }
 }

/* Check to see if first point has been given.
 * If that is the case, draw a arc in Xor mode.
 */
 if( (elipse->fst) && (!elipse->scd) && (elipse->e.a > 0.0) )
 {
  ang = elipse->e.ang;
  a = elipse->e.a;
  
  if( CllElipseDrawAxis( elipse->e.center.x, elipse->e.center.y,
                         elipse->e.a, elipse->e.ang, x, y, &b ) )
  {
    CllElipseDrawElipse (elipse->e.center.x, elipse->e.center.y, ang, a, b);
  } 
  if( iup_english )
    sprintf( echotext, "Second half-axis = %10.3f", b );
  else
    sprintf( echotext, "Segundo semi-eixo = %10.3f", b );
  if( elipse->prompt != 0L )
   (*elipse->prompt)( elipse->iup_cv, echotext );
 }

/* Check to see whether client had registered an echo function.
 * If that is the case call it passing the given mouse position.
 */
 if( elipse->echo != 0L )
  (*elipse->echo )( elipse->iup_cv, x, y ) ;

 cdMarkSize( mark_size );
 cdMarkType( mark_type );
 cdForeground( fore_color );
 cdWriteMode( write_mode );
 cdLineWidth(line_width);
 cdLineStyle(line_style);
}
 

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* ========================  CllElipseCreate  ======================= */

static void CllElipseCenter( PckDesc * );

void CllElipseCreate( Ihandle *iup_cv, void *cd_cv,
              void (*end_action)( Ihandle *iup_cv, int status ),
              void (*put)( Ihandle *iup_cv, void *elipse, double tol ),
              void (*prompt)( Ihandle *iup_cv, char *msg ),
              void (*snap)( Ihandle *iup_cv, double *x, double *y, 
                            double tol, int shift, int control ),
              void (*echo)( Ihandle *iup_cv, double x, double y ) )
{
 CllElipseP  *elipse;
 PckDesc  pk;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  prompt_msg1 = prompt_msg1e;
  prompt_msg2 = prompt_msg2e;
  prompt_msg3 = prompt_msg3e;
 }
 else
 {
  prompt_msg1 = prompt_msg1p;
  prompt_msg2 = prompt_msg2p;
  prompt_msg3 = prompt_msg3p;
 }

 CllClose( );

/* Allocate a arc collection descriptor
 */
 elipse = (CllElipseP *)malloc( sizeof( CllElipseP ) );

/* Initialize collection
 */
 elipse->ctr = 0;
 elipse->fst = 0;
 elipse->scd = 0;
 elipse->iup_cv = iup_cv;
 elipse->cd_cv = cd_cv;
 elipse->end_action = end_action; 
 elipse->put = put; 
 elipse->prompt = prompt; 
 elipse->snap = snap; 
 elipse->echo = echo; 
 elipse->e.center.x = elipse->e.center.y = 0.0;
 elipse->e.a = 0.0;
 elipse->e.b = 0.0;
 elipse->e.ang = 0.0;

/* Setup pick descriptor for getting arc center at button down.  
 */
 if( elipse->prompt != 0L )
  (*elipse->prompt )( elipse->iup_cv, prompt_msg1 );
 pk.iup_cv = elipse->iup_cv;
 pk.gra_cv = elipse->cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.mode = ButtonDown;
 pk.cursorecho = BoxCursor | CrossCursor;
 pk.button = 0;    /* accept both mouse buttons 1 or 2 */
 pk.toler = 0.0;
 pk.btdwn_callback  = CllElipseCenter;
 pk.btup_callback = 0L;
 pk.echofunc = CllElipseEcho;
 pk.snapfunc = CllElipseSnap;
 pk.data = elipse;
 pk.status = PCK_VALID;
 PckSetup( &pk );
}

/* -------------------------  CllElipseCenter  ------------------------ */

static void CllElipse1stPoint( PckDesc * );

static void CllElipseCenter( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllElipseP  *elipse = (CllElipseP *)pk->data;
 void     *iup_cv = elipse->iup_cv;
 void     (*end_action)( Ihandle *iup_cv, int status ) = elipse->end_action;

/* Check to see if collected arc is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = CLL_QUIT;        break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllElipseDel( elipse );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Get button down point as center point.
 */
 elipse->e.center.x = pk->p0.x;
 elipse->e.center.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter the
 * arc center using the keyboard. In this case, prompt the user for
 * center coordinates, and continue collection.
 */
 if( pk->button == 2 )
 {
  double xc, yc;
  xc = elipse->e.center.x;
  yc = elipse->e.center.y;
  if( DlgCapture1Elipse( &xc, &yc ) )
  {
   elipse->e.center.x = xc;
   elipse->e.center.y = yc;
  }
  else
  {
   CllElipseDel( elipse );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
   return;
  }
 }

/* Set flag to indicate that center has been selected.
 */
 elipse->ctr = 1;

/* Setup pick descriptor to collect first half-axis and rotation angle.  
 * Register the appropriate arc collector function as the callback for 
 * pick interaction.
 */
 if( elipse->prompt != 0L )
  (*elipse->prompt )( elipse->iup_cv, prompt_msg2 );
 pk->p0.x = elipse->e.center.x;
 pk->p0.y = elipse->e.center.y;
 pk->p1.x = pk->p1.y = 0.0;
 pk->mode = ButtonDown;
 pk->cursorecho = BoxCursor | CrossCursor | RubberBandCursor;
 pk->button = 0;    /* accept both mouse buttons 1 or 2 */
 pk->btdwn_callback = CllElipse1stPoint;
 pk->btup_callback  = 0L;
 PckSetup( pk );
}

/* -------------------------  CllElipse1stPoint  ---------------------- */

static void CllElipse2ndPoint( PckDesc * );

static void CllElipse1stPoint( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllElipseP  *elipse = (CllElipseP *)pk->data;
 void     *iup_cv = elipse->iup_cv;
 void     (*end_action)( Ihandle *iup_cv, int status ) = elipse->end_action;

/* Check to see if collected arc is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = CLL_NORMAL;      break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllElipseDel( elipse );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/*  Get button down point, calculate its distance to the center
 ** and set first half-axis and its angle in relation to the horizontal
    axis (x). 
 */
 elipse->e.a = LEN(elipse->e.center, pk->p0);
 CllElipseAngle(elipse->e.center.x, elipse->e.center.y, 
                pk->p0.x, pk->p0.y, &elipse->e.ang);
                    

/* If middle button was pressed, the user wants to enter first
 * half-axis and the angle between this axis and the X direction 
 * using the keyboard. In this case, prompt the user for
 * point coordinates, and continue collection.
 */
 if( pk->button == 2 )
 {
  double a, ang;
  a = elipse->e.a;
  ang = elipse->e.ang;
  if( DlgCapture2Elipse( &a, &ang ) )
  {
   if ( a > 0.0 )
   {
    elipse->e.a = a;
    elipse->e.ang = ang;
   }
   else
   {
    CllElipseDel( elipse );
    if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
    return;
   }
  }
  else
  {
   CllElipseDel( elipse );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
   return;
  }
 }

/* Set flag to indicate that first half-axis and rotation angle 
 * have been selected.
 */
 elipse->fst = 1;
 
/* Setup pick descriptor to collect second half-axis. Register the 
 * appropriate arc collector function as the callback for pick interaction.
 */
 if( elipse->prompt != 0L )
  (*elipse->prompt )( elipse->iup_cv, prompt_msg3 );
 pk->p0.x = elipse->e.center.x + elipse->e.a * cos(RAD(elipse->e.ang));
 pk->p0.y = elipse->e.center.y + elipse->e.a * sin(RAD(elipse->e.ang));
 pk->p1.x = pk->p1.y = 0.0;
 pk->mode = ButtonDown;
 pk->cursorecho = BoxCursor | CrossCursor;
 pk->button = 0;    /* accept both mouse buttons 1 or 2 */
 pk->btdwn_callback = CllElipse2ndPoint;
 pk->btup_callback  = 0L;
 PckSetup( pk );
}

/* -------------------------  CllElipse2ndPoint  ---------------------- */

static void CllElipse2ndPoint( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllElipseP  *elipse = (CllElipseP *)pk->data;
 void     *iup_cv = elipse->iup_cv;
 void     (*end_action)( Ihandle *iup_cv, int status ) = elipse->end_action;

/* Check to see if collected arc is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = CLL_NORMAL;      break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllElipseDel( elipse );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Compute the secondary half-axis size as the dot product of the
 * vector between the button down point and the center with the secondary
 * half-axis direction.
 */
 elipse->e.b = fabs( -(pk->p0.x-elipse->e.center.x) * sin(RAD(elipse->e.ang)) + 
                      (pk->p0.y-elipse->e.center.y) * cos(RAD(elipse->e.ang)));

/* If middle button was pressed, the user wants to enter the lenght of the 
 * second half-axis using the keyboard. 
 */
 if( pk->button == 2 )
 {
  double xc, yc;
  double a, b;
  double ang;
  
  xc = elipse->e.center.x;
  yc = elipse->e.center.y;
  a = elipse->e.a;
  b = elipse->e.b;
  ang = elipse->e.ang;
  
  if( DlgCapture3Elipse(&xc, &yc, &a, &b, &ang))
  {
   if ( a > 0.0 )
   {
    if( b > 0.0 )
    {
     elipse->e.center.x = xc;
     elipse->e.center.y = yc;
     elipse->e.a = a;
     elipse->e.b = b;
     elipse->e.ang = ang;
    }
   }
  }
  else
  {
   CllElipseDel( elipse );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
   return;
  }
 } 

/* Set flag to indicate that second half-axis has been selected.
 */
 elipse->scd = 1;
 
/* Call the client function to put the collected ellipse, 
 * and finish collection calling end action function.
 */
 if ( (elipse->e.a > 0.0) && (elipse->e.b > 0.0)  )
 {
  if( elipse->put != 0L )
   (*elipse->put )( elipse->iup_cv, &elipse->e, pk->toler );
 }
 CllElipseDel( elipse );
 if( end_action ) (*end_action)( iup_cv, CLL_NORMAL );      
}

/* ======================  CllKbdElipseCreate  ====================== */

static CllElipse  kbdelipse;
static double  kbdtol = 0.001;

static char *kbdprompt_msg;

static char *kbdprompt_msgp = { "Digite dados do arco de elipse. " };
static char *kbdprompt_msge = { "Use the keyboard to enter elipse arc data. " };

int  CllKbdElipseCreate( Ihandle *iup_cv, 
              void (*put)( Ihandle *iup_cv, void *elipse, double tol ),
              void (*prompt)( Ihandle *iup_cv, char *msg ) )
{
 CllElipse elipse;
 double    xc, yc;
 double    toler;
 double    ang;
 double    a, b;
 int       status = CLL_QUIT;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  kbdprompt_msg = kbdprompt_msge;
 }
 else
 {
  kbdprompt_msg = kbdprompt_msgp;
 }

/* Initialize collection
 */
 elipse = kbdelipse;

/* Collect data from user with the keyboard.
 */
 if( prompt != 0L )
  (*prompt)( iup_cv, kbdprompt_msg );

 xc  = elipse.center.x;
 yc  = elipse.center.y;
 ang = elipse.ang;
 a   = elipse.a;
 b   = elipse.b;
 toler = kbdtol;
 if( DlgCaptureElipse( &xc, &yc, &a, &b, &ang, &toler ) )
 {
  elipse.center.x = xc;
  elipse.center.y = yc;
  elipse.ang = ang;
  elipse.a = a;
  elipse.b = b;
  
  if( ( elipse.a > 0.0 ) && ( elipse.b > 0.0 ) )
  {
   kbdelipse = elipse;
   kbdtol = toler;
   if( put != 0L )
    (*put)( iup_cv, &elipse, kbdtol );
   status = CLL_NORMAL;
  }
 }

 return( status );
}

