/*
** cllelparc.c - Functions to collect ellipse arcs.
**
** Public functions:
** --------------------------------------------------------------------
**
** void CllElpArcCreate( Ihandle *iup_cv, void *cd_cv,
**                 void (*end_action)( Ihandle *iup_cv, int status ),
**                 void (*put)( Ihandle *iup_cv, void *elparc, double tol ), 
**                 void (*prompt)( Ihandle *iup_cv, char *msg ), 
**                 void (*snap)( Ihandle *iup_cv, double *x, double *y,
**                               double tol, int shift, int control ),
**                 void (*echo)( Ihandle *iup_cv, double x, double y ) )
**
** iup_cv     - IUP handle of canvas where primitive is collected (IN)
** cd_cv      - CD handle of canvas                               (IN)
** end_action - funct. to be called after finishing arc creation  (IN)
** put        - ptr. to function to put arc after collection      (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
** snap       - ptr. to function to snap points during collection (IN)
** echo       - ptr. to function to echo cursor position          (IN)
**
** This function interactively collects from the user an elipse arc 
** on a given canvas.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal collection ==> status = CLL_NORMAL = 1
**     This is also returned if user quit during collection, after
**     entering the first, second or third points.
** (2) Quit collection ==> status = CLL_QUIT = 0
**     This is returned if user quit collection at the begining,
**     before entering the first point.
** (3) Interupted collection ==> status = CLL_INTERUPTED = -1
**     This is returned when the collection is closed by something 
**     else (not by the mouse interaction). 
**
** --------------------------------------------------------------------
**
** int  CllKbdElpArcCreate( Ihandle *iup_cv,
**                       void (*put)( Ihandle *iup_cv, void *elparc, 
**                                    double tol ),
**                       void (*prompt)( Ihandle *iup_cv, char *msg ) )
** 
** iup_cv     - IUP handle of canvas for client reference         (IN)
** put        - ptr. to function to put prmtv. after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
**
** This function interactively collects from the keyboard an elipse arc.
** If the collection is successfull the function returns a CLL_NORMAL (1)
** status.  Otherwise, it returns a CLL_QUIT (0) status.
**
** --------------------------------------------------------------------
**
** Version:      0-001
**
** Created:    07-Mar-2005    Marcos C. de Arruda
**
** Version:      0-002
**
** Modified    17-Mar-2006    Rafael Araujo de Sousa
**
**   Y
**   
**   ^
**   |                                          
**   |                                 ||||||||||||||
**   |                              ||||||||||||||||||| 
**   |                             |||| |           |||||
**   |                           ||||   |           / ||||
**   |                         |||      |          /    |||  
**   |                        |||       |         /      |||
**   |                      |||         |        /        |||
**   |                     |||          |       /          |||
**   |                    |||           |      /           |||
**   |                   ||| \          | L0  /            |||
**   |                 |||     \        |__  /            |||
**   |                |||        \      |  \/             |||
**   |               |||           \  L1_  /___          |||
**   |              |||              \ /|\/    \ theta   |||
**   |             |||                 \|/      \ (ang) |||
**   |            |||-------------------|-------|------|||
**   |           |||                    C             |||
**   |          |||                                  |||
**   |         |||                                  |||
**   |        |||                                  |||
**   |        |||                                 |||
**   |       |||                                 |||
**   |       |||                                |||
**   |        |||                               |||
**   |        |||                             |||
**   |         |||                           |||
**   |          |||                        |||
**   |           ||||                    ||| 
**   |             ||||                |||| 
**   |               |||||||||||||||||||
**   |                 |||||||||||||||
**   |
**   |
**   |------------------------------------------------------------> X
**  ORIGEM
**
**    IMPLEMENTAO PARAMTRICA DA ELIPSE 
**    Referncia: http://www.spaceroots.org/documents/ellipse.
**
**    NOTAO:
**    C ->  o centro da elipse com coordenadas (xc,yc),
**    theta -> (ang)  a inclinao que o primeiro semi-eixo forma com o eixo 
**             horizontal do plano Carteziano,
**    lambda ->  o ngulo geomtrico entre um ponto da elipse e o semi-eixo
**              principal (primeiro semi-eixo)
**    L0 -> lambda0 ( a inclinao que segmento de reta que une o primeiro 
**          ponto do arco de elipse com o centro da elipse forma com o 
**          primeiro semi-eixo), 
**    L1 -> lambda1 ( a inclinao que segmento de reta que une o segundo 
**          ponto do arco de elipse com o centro da elipse forma com o 
**          primeiro semi-eixo).
**    a ->  o tamanho do primeiro semi-eixo, 
**    b ->  o tamanho do segundo semi-eixo,
**    eta ->  um ngulo terico (no teria representao na figura, exceto no
**           caso de um crculo).
**
**    RELAES ENTRE AS VARIVEIS:
**
**    r*cos(lambda) = a*cos(eta)  e  r*sin(lambda) = b*sin(eta)  (1)
**
**    Das equaes (1) acima, dividindo uma pela outra, pode-se determinar eta 
**    em funo das outras variveis:
** 
**    eta = arctan2((sin(lambda)/b), (cos(lambda)/a))
**
**    Desta forma para se determinar qualquer ponto da elipse usa-se a seguinte 
**    equao:
**
**    xel = xc + a*cos(theta)*cos(eta) - b*sin(theta)*sin(eta)
**    yel = yc + a*sin(theta)*cos(eta) + b*cos(theta)*sin(eta)
**
** Modified:   23-Mar-2006    Luiz F. Martha
**    Modified functions CllElpArcDrawAxis and CllElpArc2ndPoint to compute the
**    secondary half-axis size as the dot product of the vector between the 
**    given point and the center with the secondary half-axis direction.
**    Eliminated point array variable in display functions.
**    Made local string array variable static in echo function.
**    Modified function CllElpArcSnap to snap given point to secondary
**    half-axis and, in case of shift key pressed, make ellipse a circle.
**    Modified function CllElpArcAngle so that it does not add 360 degrees
**    to the angle in case it is negative.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "iup.h"
#include "cd.h"
#include "wd.h"
#include "pck.h"
#include "cll.h"
#include "cllelparc.h"
#include "dlgelparc.h"


/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/
/*  Arc Elipse data structure:
 */
typedef struct _cllelparcp  {   /* Collected arc private spec.descrip.: */
        CllElpArc   e;                 /* collected arc                 */
        int         ctr;               /* if true, center already given */
        int         fst;               /* if true, first arc pt. given  */
        int         scd;               /* if true, second arc pt. given */
        int         thd;               /* if true, third arc pt. given  */
        int         fth;               /* if true, fourth arc pt. given */
        Ihandle     *iup_cv;           /* IUP handle to target canvas   */
        void        *cd_cv;            /* cd handle to corresp.canvas   */
        void        (*end_action)      /* end action callback           */
                          ( Ihandle *iup_cv, int status );
        void        (*put)             /* function to put collect. arc  */
                          ( Ihandle *iup_cv, void *elparc, double tol );
        void        (*prompt)          /* function to prompt messages   */
                          ( Ihandle *iup_cv, char *msg );
        void        (*snap)            /* function to snap collect. pt. */
                          ( Ihandle *iup_cv, double *x, double *y, 
                                          double tol, int shift, int control );
        void        (*echo)            /* function to echo cursor point */
                          ( Ihandle *iup_cv, double x, double y );
        } CllElpArcP;

#ifndef LEN
#define LEN(p0,p1) sqrt((p1.x-p0.x)*(p1.x-p0.x)+(p1.y-p0.y)*(p1.y-p0.y))
#endif
#ifndef dist
#define dist(x0,y0,x1,y1) sqrt((x1-x0)*(x1-x0)+(y1-y0)*(y1-y0))
#endif
#ifndef PI
#define PI           fabs(acos(-1.))
#endif
#ifndef RAD
#define RAD(a)       ((a)*(PI/180.0))
#endif
#ifndef DEG
#define DEG(a)       ((a)*(180.0/PI))
#endif

static int  iup_english;
static char *prompt_msg1;
static char *prompt_msg2;
static char *prompt_msg3;
static char *prompt_msg4;
static char *prompt_msg5;

static char *prompt_msg1p = { "Com botao esquerdo do mouse entre "
                              "com o centro do arco; "
                              "Botao do meio => Teclado." };
static char *prompt_msg2p = { "Entre com o ponto que define o primeiro "
                              "semi-eixo da elipse "
                              "Botao do meio => Teclado." };
static char *prompt_msg3p = { "Entre com o ponto que define o segundo " 
                              "semi-eixo da elipse "
                              "Botao do meio => Teclado." };
static char *prompt_msg4p = { "Entre com o ponto que define o primeiro " 
                              "ponto do arco "
                              "(sentido anti-horario); "
                              "Botao do meio => Teclado." };
static char *prompt_msg5p = { "Entre com o ponto que define o segundo " 
                              "ponto do arco "
                              "Botao do meio => Teclado."
                              "Botao da direita => Cancela." };
static char *prompt_msg1e = { "With mouse left button enter arc center; "
                              "Mid button => Keyboard." };
static char *prompt_msg2e = { "Enter point to define the first "
                              "half-axis of the elipse " };
static char *prompt_msg3e = { "Enter point to define the second "
                              "half-axis of the elipse " };
static char *prompt_msg4e = { "Enter arc first point "
                              "(counter-clockwise direction); "
                              "Mid button => Keyboard." };
static char *prompt_msg5e = { "Enter arc second point; "
                              "Mid button => Keyboard; "
                              "Right button => Cancel." };

static double _tol = 1e-5;

/*
** --------------------------------------------------------------------
** Local functions:
*/
       
static int CllElpArcSTFSnapTo       ( double xc, double yc, double ae, 
                                      double be, double ang, 
                                      double *x, double *y );
static int CllElpArcAngle           ( double xc, double yc, double x, double y,
                                      double *a );
static int CllElpArcDrawAxis        ( double xc, double yc, double a, 
                                      double ang, double x, double y,
                                      double *b );
static int CllElpArcDrawArcElipse   ( double xc, double yc, double ang, 
                                      double a, double b, double ang0, 
                                      double ang1, CllPnt p0);
static int CllElpArcDrawElipse      ( double xc, double yc, double ang, 
                                      double a, double b );
static void CllElpArcDel            ( CllElpArcP *elparc );
static void CllElpArcSnap           ( PckDesc *pk, double *x, double *y, 
                                      double tol );
static void CllElpArcEcho           ( PckDesc *pk, double x, double y );
                                      

/* ========================  CllElpArcSTFSnapTo  ======================= */
/*
** Esta funo determina as coordenadas do ponto de interseo entre uma reta
** que passa pelo pontos pontos (*x, *y) e (xc, yc) e uma elipse com centro em
** (xc, yc), semi-eixos a e b e ngulo do semi-eixo a em relao  horizontal
** ang. 
*/
static int CllElpArcSTFSnapTo( double xc, double yc, double a, double b,
                               double ang, double *x, double *y )
{
 double alpha0;
 double eta0;
 double lambda0;

 if( ! CllElpArcAngle( xc, yc, *x, *y, &alpha0 ) )
  return( 0 );

 lambda0 = alpha0 - ang;
 eta0 = atan2(sin(RAD(lambda0))/b, cos(RAD(lambda0))/a);
 *x = xc + a*cos(RAD(ang))*cos(eta0) - b*sin(RAD(ang))*sin(eta0);
 *y = yc + a*sin(RAD(ang))*cos(eta0) + b*cos(RAD(ang))*sin(eta0);

 return ( 1 );
}

/* ========================  CllElpArcAngle  ======================== */
/*
** Esta funo calcula o ngulo em relao  horizontal (direo x) da reta
** que passa pelos pontos (xc, yc) e (x, y). Este ngulo necessariamente
** vai estar entre 0 e 360 graus. 
*/
static int CllElpArcAngle( double xc, double yc, double x, double y, double *a )
{
 double d;

 d = dist(x, y, xc, yc);
 if( d <= _tol )       /* modificado por Rafael em 15/03/2006 */
  return( 0 );

 *a = atan2( y-yc, x-xc );
 *a = DEG( *a );

 return( 1 );
}

/* ========================  CllElpArcDrawAxis  ======================== */

static int CllElpArcDrawAxis( double xc, double yc, double a, double ang,
                              double x, double y, double *b )
{ 
 double x0, y0;     /* primeiro pt que estar no segundo semi-eixo */
 double x0s, y0s;   /* inverso do pt acima */
 int line_style;
 
/* Compute the secondary half-axis size as the dot product of the
 * vector between the given point and the center with the secondary
 * half-axis direction.
 */
 *b = fabs(- (x - xc) * sin(RAD(ang)) + (y - yc) * cos(RAD(ang)));
 if (*b < _tol)
  return( 0 );

 x0  = xc - (*b) * sin(RAD(ang));
 y0  = yc + (*b) * cos(RAD(ang));
 x0s = xc + (*b) * sin(RAD(ang));
 y0s = yc - (*b) * cos(RAD(ang));
 
 line_style = cdLineStyle( CD_DASHED );
 wdLine(x0s, y0s, x0, y0);
 cdLineStyle(line_style);

 return( 1 );
}

/* ========================  CllElipseTranslPt  ======================== */

static void CllElipseTranslPt( CllPnt* pt, double xc, double yc)
{
 pt->x = pt->x + xc;
 pt->y = pt->y + yc;
}

/* ========================  CllElipseRotPt  ======================== */

static void CllElipseRotPt( CllPnt* pt, double ang)
{
 double x;
 double y;
 
 x = pt->x;
 y = pt->y;
 pt->x = x*cos(RAD(ang)) - y*sin(RAD(ang));
 pt->y = x*sin(RAD(ang)) + y*cos(RAD(ang));
}

/* ========================  CllElpArcDrawArcElipse  ======================== */

static int CllElpArcDrawArcElipse( double xc, double yc, double ang, 
                                   double a, double b, double ang0, double ang1,
                                   CllPnt p0 )
{
 int i;                   /* contador */
 int n;                   /* nmero de pontos que sero colotados */
 double eta0, eta1;       /* ngulos tericos */
 double lambda0, lambda1; /* ngulos de inclinao em relao ao 
                             1 semi-eixo */
 double d_theta;          /* incremento angular */
 double span;
 CllPnt vit;              /* ponto inicial do arco transformado */
 CllPnt vft;              /* ponto final do arco transformado */
 CllPnt v0;               /* ponto inicial do segmento no transformado */
 CllPnt v1;               /* ponto final do segmento no transformado */
 CllPnt v0t;              /* ponto inicial do segmento transformado */

/* coloca o ponto inicial para a origem com seus semi-eixos aos 
   eixos cartesianos */
 CllElipseTranslPt( &p0, -xc, -yc );
 CllElipseRotPt( &p0, -ang );

/* parmetros para o uso da equao paramtrica da elipse */
 lambda0 = ang0 - ang;
 lambda1 = ang1 - ang;
 eta0 = atan2(sin(RAD(lambda0))/b, cos(RAD(lambda0))/a);
 eta1 = atan2(sin(RAD(lambda1))/b, cos(RAD(lambda1))/a); 

/* condies para que os ngulos tericos sejam maiores que 0 e para que 
   eta1 seja maior que eta0 */
 if (eta0 < 0.0)
  eta0 += 2 * PI;
 if (eta1 < 0.0)
  eta1 += 2 * PI; 
 if (eta1 < eta0)
  eta1 += 2 * PI;
  
 span = eta1 - eta0;

/* nmero de pontos que sero coletados */
 n = (int)(200 * span/(2*PI));

 if (n > 2)
 {
/* incremento angular */
  d_theta = span / (n-1);

  cdBegin( CD_OPEN_LINES );

/* posio de incio de coleta */  
  v0 = p0;
  for (i = 0; i < n; i++) 
  {
/* transforma o ponto para a sua posio na elipse deslocada e inclinada */
   v0t = v0;
   CllElipseRotPt( &v0t, ang );
   CllElipseTranslPt( &v0t, xc, yc );
   wdVertex( v0t.x, v0t.y );

/* acha o prximo ponto na elipse de forma adaptativa */
   v1.x = v0.x*cos(d_theta) - (a/b)*v0.y*sin(d_theta);
   v1.y = (b/a)*v0.x*sin(d_theta) + v0.y*cos(d_theta);
   v0 = v1;
  }
  cdEnd( );
 
/* desenha marcas nas extremidades do arco */
  vit = p0;
  CllElipseRotPt( &vit, ang );
  CllElipseTranslPt( &vit, xc, yc );
  wdMark( vit.x, vit.y );
  vft = v0; /* ltimo ponto coletado */
  CllElipseRotPt( &vft, ang );
  CllElipseTranslPt( &vft, xc, yc );
  wdMark( vft.x, vft.y );
 
  return( 1 );
 }

 return( 0 );
}

/* ========================  CllElpArcDrawElipse  ======================== */

static int CllElpArcDrawElipse( double xc, double yc, double ang, 
                                double a, double b )
{
 int i;           /* contador */
 int n;           /* nmero de pontos que sero colotados */
 double d_theta;  /* incremento angular */
 CllPnt v0;       /* ponto inicial do segmento no transformado */
 CllPnt v1;       /* ponto final do segmento no transformado */
 CllPnt v0t;      /* ponto inicial do segmento transformado */

 if ((a < _tol) || (b < _tol)) return 0;

 n = 200;

/* definio do incremento */
 d_theta = 2*PI / n;

 cdBegin( CD_CLOSED_LINES );

/* ponto de partida para se comear a ligar (desenhar) os pontos */
 v0.x = a;
 v0.y = 0.0;
 for (i = 0; i < n; i++)
 {
/* transforma o ponto para a sua posio na elipse deslocada e inclinada */
  v0t = v0;
  CllElipseRotPt( &v0t, ang );
  CllElipseTranslPt( &v0t, xc, yc );
  wdVertex( v0t.x, v0t.y );

/* acha o prximo ponto na elipse de forma adaptativa */
  v1.x = v0.x*cos(d_theta) - (a/b)*v0.y*sin(d_theta);
  v1.y = (b/a)*v0.x*sin(d_theta) + v0.y*cos(d_theta);
  v0 = v1;
 }
 cdEnd( );

 return ( 1 );	
}

/* ==========================  CllElpArcDel  ======================== */

static void CllElpArcDel( CllElpArcP *elparc )
{
 if( elparc == 0L ) return;

/*  Release memory used for arc.
 */    
 free( elparc );
}

/* ========================  CllElpArcSnap  ========================= */

static void CllElpArcSnap( PckDesc *pk, double *x, double *y, double tol )
{
 double b;

/* Restore registered collection spec. from the data field.
 */
 CllElpArcP  *elparc = (CllElpArcP *)pk->data;

 if( (elparc->ctr) && (!elparc->fst) && isshift(pk->mousecfg) )
 {
/* If shift key is pressed and center arc point has been given,
 * snap radius to the horizontal axis, to the vertical axis, or
 * the bi-sector lines.
 */
  CllShiftSnap( elparc->e.center.x, elparc->e.center.y, x, y );
 }
 else if( (elparc->ctr) && (elparc->fst) && (!elparc->scd) && 
          (!elparc->thd) && (!elparc->fth) )
 {
/* If center and first points have been given, snap point to the 
 * secondary ellipse axis.  If shift key is pressed, make ellipse
 * be a circle.
 */ 
  b = - (*x - elparc->e.center.x) * sin(RAD(elparc->e.ang)) 
      + (*y - elparc->e.center.y) * cos(RAD(elparc->e.ang));
  if( isshift(pk->mousecfg) )
  {
   if( b < 0.0 )
    b = -elparc->e.a;
   else
    b =  elparc->e.a;
  }
  *x = elparc->e.center.x - b * sin(RAD(elparc->e.ang));
  *y = elparc->e.center.y + b * cos(RAD(elparc->e.ang));
 }
 else if( (elparc->ctr) && (elparc->fst) && (elparc->scd) && 
          ((!elparc->thd) || (!elparc->fth)) && isshift(pk->mousecfg) )
 {
/* If shift key is pressed and ellipse has been defined,
 * snap radius to the horizontal axis, to the vertical axis, or
 * the bi-sector lines.
 */
  CllShiftSnap( elparc->e.center.x, elparc->e.center.y, x, y );
 }

/* Check to see whether client had registered a snap function and 
 * call it passing the given mouse position and parameters.
 */
 if( elparc->snap != 0L )
  (*elparc->snap )( elparc->iup_cv, x, y, tol, 
                    isshift(pk->mousecfg), iscontrol(pk->mousecfg) );

/* If first arc points has already been collected, 
 * snap current point to other half-axis of the elipse.
 */
 if( (elparc->scd) && (elparc->e.a > 0.0) && (elparc->e.b > 0.0))
 {
  CllElpArcSTFSnapTo( elparc->e.center.x, elparc->e.center.y, elparc->e.a,
                      elparc->e.b, elparc->e.ang, x, y );
 }
}

/* ========================  CllElpArcEcho  ========================= */
 
static void CllElpArcEcho( PckDesc *pk, double x, double y )
{
 int    write_mode;
 int    fore_color;
 int    mark_type;
 int    mark_size;
 int    line_style;
 int    line_width;
 double a, b;
 double ang;
 double ang0, ang1;
 CllPnt pt;
 static char echotext[128];

/* Restore registered collection spec. from the data field.
 */
 CllElpArcP  *elparc = (CllElpArcP *)pk->data;
 
#ifdef _UNIX_
 write_mode = cdWriteMode( CD_XOR );
#else
 write_mode = cdWriteMode( CD_NOT_XOR );
#endif
 fore_color = cdForeground( CD_RED );
 mark_type = cdMarkType( CD_BOX );
 mark_size = cdMarkSize( 4 );
 line_style = cdLineStyle( CD_CONTINUOUS );
 line_width = cdLineWidth( 1 );

/* Check to see if center has been given.
 * If that is the case, draw a mark at the center in Xor mode.
 */
 if( elparc->ctr )
 {
  wdMark( elparc->e.center.x, elparc->e.center.y );

  if( !elparc->fst && CllElpArcAngle(elparc->e.center.x, elparc->e.center.y,
                                     x, y, &ang) )
  {
   pt.x = x;  pt.y = y;
   a = LEN( elparc->e.center, pt );
   if( iup_english )
    sprintf( echotext, "First half-axis = %10.3f;   Angle = %7.2f", a, ang );
   else
    sprintf( echotext, "Primeiro semi-eixo = %10.3f;   Angulo = %7.2f", 
             a, ang );
   if( elparc->prompt != 0L )
    (*elparc->prompt)( elparc->iup_cv, echotext );
  }
 }

/* Check to see if first point has been given.
 * If that is the case, draw a arc in Xor mode.
 */
 if( (elparc->fst) && (!elparc->scd) && (elparc->e.a > 0.0) )
 {
  ang = elparc->e.ang;
  a = elparc->e.a;
  
  if( CllElpArcDrawAxis( elparc->e.center.x, elparc->e.center.y,
                         elparc->e.a, elparc->e.ang, x, y, &b ) )
  {
    CllElpArcDrawElipse (elparc->e.center.x, elparc->e.center.y, ang, a, b);
  } 
  if( iup_english )
   sprintf( echotext, "Second half-axis = %10.3f", b );
  else
   sprintf( echotext, "Segundo semi-eixo = %10.3f", b );
  if( elparc->prompt != 0L )
   (*elparc->prompt)( elparc->iup_cv, echotext );
 }
 else if ( (elparc->scd) && (!elparc->thd) && 
           (elparc->e.a > 0.0) && (elparc->e.b > 0.0) )
 {
  ang = elparc->e.ang;
  a = elparc->e.a;
  b = elparc->e.b;
  
  wdMark(elparc->e.center.x, elparc->e.center.y);
  wdMark(x, y);
  wdLine(elparc->e.center.x, elparc->e.center.y, x, y);
  CllElpArcDrawElipse (elparc->e.center.x, elparc->e.center.y, ang, a, b);
  CllElpArcAngle(elparc->e.center.x, elparc->e.center.y, x, y, &ang0);
  if( iup_english )
   sprintf( echotext, "Point1 = (%.2f, %.2f); Angle1 = %.2f", x, y, ang0 );
  else
   sprintf( echotext, "Ponto1 = (%.2f, %.2f); Angulo1 = %.2f", x, y, ang0 );
  if( elparc->prompt != 0L )
   (*elparc->prompt)( elparc->iup_cv, echotext );
 }          
 else if ( (elparc->thd) && (!elparc->fth) && 
           (elparc->e.a > 0.0) && (elparc->e.b > 0.0) )
 {
  ang = elparc->e.ang;
  ang0 = elparc->e.ang0;
  a = elparc->e.a;
  b = elparc->e.b;
  pt.x = elparc->e.p0.x;
  pt.y = elparc->e.p0.y;
  
  CllElpArcAngle(elparc->e.center.x, elparc->e.center.y, x, y, &ang1);
  if (ang1 < ang0)
   ang1 += 360.0; 
  CllElpArcDrawArcElipse(elparc->e.center.x, elparc->e.center.y, 
                         ang, a, b, ang0, ang1, pt );
  if( iup_english )
  {
   if (ang1 < 360.0)
    sprintf( echotext, "Angle 1 = %7.2f;  Angle 2 = %7.2f", ang0, ang1 );
   else
    sprintf( echotext, "Angle 1 = %7.2f;  Angle 2 = %7.2f", ang0, ang1 );
  }
  else
  {
   if (ang1 < 360.0)
    sprintf( echotext, "Angulo 1 = %7.2f;  Angulo 2 = %7.2f", ang0, ang1 );
   else
    sprintf( echotext, "Angulo 1 = %7.2f;  Angulo 2 = %7.2f", ang0, ang1 );
  }
  if( elparc->prompt != 0L )
    (*elparc->prompt)( elparc->iup_cv, echotext );
 }

/* Check to see whether client had registered an echo function.
 * If that is the case call it passing the given mouse position.
 */
 if( elparc->echo != 0L )
  (*elparc->echo )( elparc->iup_cv, x, y ) ;

 cdMarkSize( mark_size );
 cdMarkType( mark_type );
 cdForeground( fore_color );
 cdWriteMode( write_mode );
 cdLineWidth(line_width);
 cdLineStyle(line_style);
}

 
/*
** --------------------------------------------------------------------
** Public functions:
*/
/* ========================  CllElpArcCreate  ======================= */

static void CllElpArcCenter( PckDesc * );

void CllElpArcCreate( Ihandle *iup_cv, void *cd_cv,
              void (*end_action)( Ihandle *iup_cv, int status ),
              void (*put)( Ihandle *iup_cv, void *elparc, double tol ),
              void (*prompt)( Ihandle *iup_cv, char *msg ),
              void (*snap)( Ihandle *iup_cv, double *x, double *y, 
                            double tol, int shift, int control ),
              void (*echo)( Ihandle *iup_cv, double x, double y ) )
{
 CllElpArcP  *elparc;
 PckDesc  pk;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  prompt_msg1 = prompt_msg1e;
  prompt_msg2 = prompt_msg2e;
  prompt_msg3 = prompt_msg3e;
  prompt_msg4 = prompt_msg4e;
  prompt_msg5 = prompt_msg5e;
 }
 else
 {
  prompt_msg1 = prompt_msg1p;
  prompt_msg2 = prompt_msg2p;
  prompt_msg3 = prompt_msg3p;
  prompt_msg4 = prompt_msg4p;
  prompt_msg5 = prompt_msg5p;
 }

 CllClose( );

/* Allocate a arc collection descriptor
 */
 elparc = (CllElpArcP *)malloc( sizeof( CllElpArcP ) );

/* Initialize collection ( esta coleo so justamente os pontos passados 
 * pelo usurio para a formao do arco de elipse )
 */
 elparc->ctr = 0;
 elparc->fst = 0;
 elparc->scd = 0;
 elparc->thd = 0;
 elparc->fth = 0;
 elparc->iup_cv = iup_cv;
 elparc->cd_cv = cd_cv;
 elparc->end_action = end_action; 
 elparc->put = put; 
 elparc->prompt = prompt; 
 elparc->snap = snap; 
 elparc->echo = echo; 
 elparc->e.center.x = elparc->e.center.y = 0.0;
 elparc->e.p0.x = elparc->e.p0.y = 0.0;
 elparc->e.p1.x = elparc->e.p1.y = 0.0;
 elparc->e.a = 0.0;
 elparc->e.b = 0.0;
 elparc->e.ang = 0.0;
 elparc->e.ang0 = 0.0;
 elparc->e.ang1 = 0.0;

/* Setup pick descriptor for getting arc center at button down.  
 */
 if( elparc->prompt != 0L )
  (*elparc->prompt )( elparc->iup_cv, prompt_msg1 );
 pk.iup_cv = elparc->iup_cv;
 pk.gra_cv = elparc->cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.mode = ButtonDown;
 pk.cursorecho = BoxCursor | CrossCursor;
 pk.button = 0;    /* accept both mouse buttons 1 or 2 */
 pk.toler = 0.0;
 pk.btdwn_callback  = CllElpArcCenter;  /* Aqui a callback pk.btdwn_callback 
                                          passa para a funo CllElpArcCenter */
 pk.btup_callback = 0L;
 pk.echofunc = CllElpArcEcho;
 pk.snapfunc = CllElpArcSnap;
 pk.data = elparc;
 pk.status = PCK_VALID;
 PckSetup( &pk );
}

/* -------------------------  CllElpArcCenter  ------------------------ */

static void CllElpArc1stPoint( PckDesc * );

static void CllElpArcCenter( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllElpArcP  *elparc = (CllElpArcP *)pk->data;
 void     *iup_cv = elparc->iup_cv;
 void     (*end_action)( Ihandle *iup_cv, int status ) = elparc->end_action;

/* Check to see if collected arc is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = CLL_QUIT;        break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllElpArcDel( elparc );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Get button down point as center point.
 */
 elparc->e.center.x = pk->p0.x; /* aqui est se dizendo que o 1 ponto 
                                   clicado no mouse ter sua coord 'x' 
                                   atribuida p a coord 'x' do centro da 
                                   struct a */
 elparc->e.center.y = pk->p0.y; /* dem ao que est escrito acima, apenas 
                                   tendo que trocar 'x' por 'y' */

/* If middle button was pressed, the user wants to enter the
 * arc center using the keyboard. In this case, prompt the user for
 * center coordinates, and continue collection.
 */
 if( pk->button == 2 )
 {
  double xc, yc;
  xc = elparc->e.center.x;
  yc = elparc->e.center.y;
  if( DlgCapture1ElpArc( &xc, &yc ) )
  {
   elparc->e.center.x = xc;
   elparc->e.center.y = yc;
  }
  else
  {
   CllElpArcDel( elparc );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
   return;
  }
 }

/* Set flag to indicate that center has been selected.
 */
 elparc->ctr = 1;

/* Setup pick descriptor to collect first half-axis and rotation angle.  
 * Register the appropriate arc collector function as the callback for 
 * pick interaction.
 */
 if( elparc->prompt != 0L )
  (*elparc->prompt )( elparc->iup_cv, prompt_msg2 );
 pk->p0.x = elparc->e.center.x;
 pk->p0.y = elparc->e.center.y;
 pk->p1.x = pk->p1.y = 0.0;
 pk->mode = ButtonDown;
 pk->cursorecho = BoxCursor | CrossCursor | RubberBandCursor;
 pk->button = 0;    /* accept both mouse buttons 1 or 2 */
 pk->btdwn_callback = CllElpArc1stPoint; /* Aqui a callback 
                                            pk.btdwn_callback passa 
                                            para a funo CllElpArc1stPoint 
                                            (Rafael em 10/03/2006) */
 pk->btup_callback  = 0L;
 PckSetup( pk );
}

/* -------------------------  CllElpArc1stPoint  ---------------------- */

static void CllElpArc2ndPoint( PckDesc * );

static void CllElpArc1stPoint( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllElpArcP  *elparc = (CllElpArcP *)pk->data;
 void     *iup_cv = elparc->iup_cv;
 void     (*end_action)( Ihandle *iup_cv, int status ) = elparc->end_action;

/* Check to see if collected arc is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = CLL_NORMAL;      break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllElpArcDel( elparc );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Get button down point, calculate its distance to the center
 * and set first half-axis and its angle in relation to the horizontal
 * axis (x). 
 */
 elparc->e.a = LEN(elparc->e.center, pk->p0); /* aqui se est preenchendo o 
                                                 campo 'a' da struct a ser 
                                                 igual a dist entre o 
                                                 centro (1) e 2 ponto dado 
                                                 no canvas */
 CllElpArcAngle(elparc->e.center.x, elparc->e.center.y, pk->p0.x, pk->p0.y, 
                &elparc->e.ang);
/* acima se est obtendo o ang que o semi-eixo maior faz com a horizontal */

/* If middle button was pressed, the user wants to enter first
 * half-axis and the angle between this axis and the X direction 
 * using the keyboard. In this case, prompt the user for
 * point coordinates, and continue collection.
 */
 if( pk->button == 2 )
 {
  double a, ang;
  a = elparc->e.a;
  ang = elparc->e.ang;
  if( DlgCapture2ElpArc( &a, &ang ) )
  {
   if (a > 0.0)
   {
    elparc->e.a = a;
    elparc->e.ang = ang;
   }
   else
   {
    CllElpArcDel( elparc );
    if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
    return;
   }
  }
  else
  {
   CllElpArcDel( elparc );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
   return;
  }
 }

/* Set flag to indicate that first half-axis and rotation angle 
 * have been selected.
 */
 elparc->fst = 1;
 
/* Setup pick descriptor to collect second half-axis. Register the 
 * appropriate arc collector function as the callback for pick interaction.
 */
 if( elparc->prompt != 0L )
  (*elparc->prompt )( elparc->iup_cv, prompt_msg3 );
 pk->p0.x = elparc->e.center.x + elparc->e.a * cos(RAD(elparc->e.ang));
 pk->p0.y = elparc->e.center.y + elparc->e.a * sin(RAD(elparc->e.ang));
 pk->p1.x = pk->p1.y = 0.0;
 pk->mode = ButtonDown;
 pk->cursorecho = BoxCursor | CrossCursor;
 pk->button = 0;    /* accept both mouse buttons 1 or 2 */
 pk->btdwn_callback = CllElpArc2ndPoint; /* Aqui a callback 
                                            pk.btdwn_callback passa 
                                            para a funo CllElpArc2ndPoint 
                                            (Rafael em 10/03/2006) */
 pk->btup_callback  = 0L;
 PckSetup( pk );
}

/* -------------------------  CllElpArc2ndPoint  ---------------------- */

static void CllElpArc3rdPoint( PckDesc * );

static void CllElpArc2ndPoint( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllElpArcP  *elparc = (CllElpArcP *)pk->data;
 void     *iup_cv = elparc->iup_cv;
 void     (*end_action)( Ihandle *iup_cv, int status ) = elparc->end_action;

/* Check to see if collected arc is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = CLL_NORMAL;      break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllElpArcDel( elparc );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Compute the secondary half-axis size as the dot product of the
 * vector between the button down point and the center with the secondary
 * half-axis direction.
 */ 
 elparc->e.b = fabs( -(pk->p0.x-elparc->e.center.x) * sin(RAD(elparc->e.ang)) +
                      (pk->p0.y-elparc->e.center.y) * cos(RAD(elparc->e.ang)));

/* If middle button was pressed, the user wants to enter the lenght of the 
 * second half-axis using the keyboard. 
 */
 if( pk->button == 2 )
 {
  double b;
  
  b = elparc->e.b;
  if( DlgCapture3ElpArc(&b))
  {
   if( b > 0.0 )
   { 
    elparc->e.b = b;
   }
   else
   {
    CllElpArcDel( elparc );
    if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
    return;
   }
  }
  else
  {
   CllElpArcDel( elparc );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
   return;
  }
 } 

/* Set flag to indicate that second half-axis has been selected.
 */
 elparc->scd = 1;
 
/* Setup pick descriptor to collect first arc point.  Register the 
 * appropriate arc collector function as the callback for pick interaction.
 */
 if( elparc->prompt != 0L )
  (*elparc->prompt )( elparc->iup_cv, prompt_msg4 );
 pk->p0.x = elparc->e.center.x + elparc->e.b * cos(RAD(90 + elparc->e.ang));
 pk->p0.y = elparc->e.center.y + elparc->e.b * sin(RAD(90 + elparc->e.ang));
 pk->p1.x = pk->p1.y = 0.0;
 pk->mode = ButtonDown;
 pk->cursorecho = BoxCursor | CrossCursor;
 pk->button = 0;    /* accept both mouse buttons 1 or 2 */
 pk->btdwn_callback = CllElpArc3rdPoint; /* Aqui a callback 
                                            pk.btdwn_callback passa 
                                            para a funo CllElpArc3rdPoint
                                            (Rafael em 10/03/2006)*/
 pk->btup_callback  = 0L;
 PckSetup( pk );
}

/* -------------------------  CllElpArc3rdPoint  ---------------------- */

static void CllElpArc4thPoint( PckDesc * );

static void CllElpArc3rdPoint( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllElpArcP  *elparc = (CllElpArcP *)pk->data;
 void     *iup_cv = elparc->iup_cv;
 void     (*end_action)( Ihandle *iup_cv, int status ) = elparc->end_action;

/* Check to see if collected arc is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = CLL_NORMAL;      break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllElpArcDel( elparc );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Get button down point as first arc point. Calculates the first 
 * arc angle (angle between the segment the joins this point and the
 * center of the elipse and the X direction).
 */
 
 elparc->e.p0.x = pk->p0.x;
 elparc->e.p0.y = pk->p0.y;
 
 CllElpArcAngle(elparc->e.center.x, elparc->e.center.y, 
                elparc->e.p0.x, elparc->e.p0.y, &elparc->e.ang0);

/* If middle button was pressed, the user wants to enter first
 * arc point using the keyboard.
 */
 if( pk->button == 2 )
 {
  double x0;
  double y0;
  
  x0 = elparc->e.p0.x;
  y0 = elparc->e.p0.y;
  if( DlgCapture4ElpArc(&x0, &y0))
  {
   CllElpArcSTFSnapTo( elparc->e.center.x, elparc->e.center.y, elparc->e.a, 
                       elparc->e.b, elparc->e.ang, &x0, &y0 );
   elparc->e.p0.x = x0;
   elparc->e.p0.y = y0;
   CllElpArcAngle(elparc->e.center.x, elparc->e.center.y, x0, y0, 
                 &elparc->e.ang0); 
  }
  else
  {
   CllElpArcDel( elparc );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
   return;
  }
 } 

/* Set flag to indicate that first arc point has been selected.
 */
 elparc->thd = 1;
 
/* Setup pick descriptor to collect final arc point.  Register the 
 * appropriate arc collector function as the callback for pick interaction.
 */
 if( elparc->prompt != 0L )
  (*elparc->prompt )( elparc->iup_cv, prompt_msg5 );
 pk->p0.x = elparc->e.p0.x;
 pk->p0.y = elparc->e.p0.y;
 pk->p1.x = pk->p1.y = 0.0;
 pk->mode = ButtonDown;
 pk->cursorecho = BoxCursor | CrossCursor;
 pk->button = 0;    /* accept both mouse buttons 1 or 2 */
 pk->btdwn_callback = CllElpArc4thPoint; /* Aqui a callback 
                                            pk.btdwn_callback passa 
                                            para a funo CllElpArc4thPoint
                                            (Rafael em 10/03/2006) */
 pk->btup_callback  = 0L;
 PckSetup( pk );
}

/* -------------------------  CllElpArc4thPoint  ---------------------- */

static void CllElpArc4thPoint( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllElpArcP  *elparc = (CllElpArcP *)pk->data;
 void     *iup_cv = elparc->iup_cv;
 void     (*end_action)( Ihandle *iup_cv, int status ) = elparc->end_action;

/* Check to see if collected arc is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = CLL_NORMAL;      break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllElpArcDel( elparc );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Get button down point as second arc point. Calculates the second 
 * arc angle (angle between the segment the joins this point and the
 * center of the elipse and the X direction).
 */
 elparc->e.p1.x = pk->p0.x;
 elparc->e.p1.y = pk->p0.y;
 
 CllElpArcAngle(elparc->e.center.x, elparc->e.center.y, 
                elparc->e.p1.x, elparc->e.p1.y, &elparc->e.ang1);

/* If middle button was pressed, the user wants to enter final
 * arc point using the keyboard. In this case, prompt the user for
 * all arc point coordinates and finish collection.
 */
 if( pk->button == 2 )
 {
  double xc, yc;
  CllPnt p0;
  CllPnt p1;
  double a, b;
  double ang;
  
  xc = elparc->e.center.x;
  yc = elparc->e.center.y;
  ang = elparc->e.ang;
  a = elparc->e.a;
  b = elparc->e.b;
  p0.x = elparc->e.p0.x;
  p0.y = elparc->e.p0.y;
  p1.x = elparc->e.p1.x;
  p1.y = elparc->e.p1.y;
  if( DlgCapture5ElpArc(&xc, &yc, &a, &b, &ang, &p0.x, &p0.y, &p1.x, &p1.y))
  {
   elparc->e.center.x = xc;
   elparc->e.center.y = yc;
   elparc->e.ang = ang;
   if (a > 0.0)
   {
    elparc->e.a = a;
    if (b > 0.0)
    {
     elparc->e.b = b;
     CllElpArcSTFSnapTo( xc, yc, a, b, ang, &p1.x, &p1.y );
      
     if (LEN(p0, p1) >= pk->toler)
     {
      elparc->e.p0.x = p0.x;
      elparc->e.p0.y = p0.y;
      elparc->e.p1.x = p1.x;
      elparc->e.p1.y = p1.y;
      
      CllElpArcAngle(elparc->e.center.x, elparc->e.center.y, 
                     p0.x, p0.y, &elparc->e.ang0);
      CllElpArcAngle(elparc->e.center.x, elparc->e.center.y, 
                     p1.x, p1.y, &elparc->e.ang1);
      if (elparc->e.ang1 < elparc->e.ang0)
      {
       elparc->e.ang1 += 360.0;
      }
     } 
    }
   }
  }
  else
  {
   CllElpArcDel( elparc );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
   return;
  }
 } 

/* Set flag to indicate that last arc point has been selected.
 */
 elparc->fth = 1;
 
 if (elparc->e.ang1 < elparc->e.ang0)
 {
  elparc->e.ang1 += 360.0;
 }

/* Call the client function to put the collected arc, 
 * and finish collection calling end action function.
 */
 if ( (elparc->e.a > 0.0) && (elparc->e.b > 0.0) && 
      (LEN(elparc->e.p0, elparc->e.p1) > pk->toler) )
 {
  if( elparc->put != 0L )
   (*elparc->put )( elparc->iup_cv, &elparc->e, pk->toler );
 }
 CllElpArcDel( elparc );
 if( end_action ) (*end_action)( iup_cv, CLL_NORMAL );
}

/* ======================  CllKbdElpArcCreate  ====================== */

static CllElpArc  kbdelparc;
static double  kbdtol = 0.001;

static char *kbdprompt_msg;

static char *kbdprompt_msgp = { "Digite dados do arco de elipse. " };
static char *kbdprompt_msge = { "Use the keyboard to enter elipse arc data. " };

int  CllKbdElpArcCreate( Ihandle *iup_cv, 
              void (*put)( Ihandle *iup_cv, void *elparc, double tol ),
              void (*prompt)( Ihandle *iup_cv, char *msg ) )
{
 CllElpArc elparc;
 double    xc, yc;
 double    x0, y0, x1, y1;
 double    toler;
 double    ang;
 double    a, b;
 int       status = CLL_QUIT;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  kbdprompt_msg = kbdprompt_msge;
 }
 else
 {
  kbdprompt_msg = kbdprompt_msgp;
 }

/* Initialize collection
 */
 elparc = kbdelparc;

/* Collect data from user with the keyboard.
 */
 if( prompt != 0L )
  (*prompt)( iup_cv, kbdprompt_msg );

 xc  = elparc.center.x;
 yc  = elparc.center.y;
 x0  = elparc.p0.x;
 y0  = elparc.p0.y;
 x1  = elparc.p1.x;
 y1  = elparc.p1.y;
 ang = elparc.ang;
 a   = elparc.a;
 b   = elparc.b;
 toler = kbdtol;
 if( DlgCaptureElpArc( &xc, &yc, &a, &b, &ang, &x0, &y0, &x1, &y1, &toler ) )
 {
  elparc.center.x = xc;
  elparc.center.y = yc;
  elparc.p0.x = x0;
  elparc.p0.y = y0;
  elparc.p1.x = x1;
  elparc.p1.y = y1;
  elparc.ang = ang;
  elparc.a = a;
  elparc.b = b;
  
  if( ( elparc.a > 0.0 ) && ( elparc.b > 0.0 ) )
  {
   CllElpArcSTFSnapTo( xc, yc, a, b, ang, &elparc.p0.x, &elparc.p0.y );
   CllElpArcSTFSnapTo( xc, yc, a, b, ang, &elparc.p1.x, &elparc.p1.y );
   CllElpArcAngle(xc, yc, elparc.p0.x, elparc.p0.y, &elparc.ang0);
   CllElpArcAngle(xc, yc, elparc.p1.x, elparc.p1.y, &elparc.ang1);
   if (elparc.ang1 < elparc.ang0)
    elparc.ang1 += 360.0;   
   if( LEN( elparc.p0, elparc.p1 ) > 0.0 )
   {
    kbdelparc = elparc;
    kbdtol = toler;
    if( put != 0L )
     (*put)( iup_cv, &elparc, kbdtol );
    status = CLL_NORMAL;
   }
  }
 }

 return( status );
}

