char *RCScllline="$Id: cllline.c,v 1.7 2001/09/13 14:34:48 william Exp $";
/*
** cllline.c - Functions to collect straight lines.
**
** Public functions:
** --------------------------------------------------------------------
**
** void CllLineCreate( Ihandle *iup_cv, void *cd_cv, 
**               void (*end_action)( Ihandle *iup_cv, int status ),
**               void (*put)( Ihandle *iup_cv, void *line, double tol ), 
**               void (*prompt)( Ihandle *iup_cv, char *msg ), 
**               void (*snap)( Ihandle *iup_cv, double *x, double *y,
**                             double tol, int shift, int control ),
**               void (*echo)( Ihandle *iup_cv, double x, double y ) )
**
** iup_cv     - IUP handle of canvas where primitive is collected (IN)
** cd_cv      - CD handle of canvas                               (IN)
** end_action - function to be called after finishing line creat. (IN)
** put        - ptr. to function to put line after collection     (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
** snap       - ptr. to function to snap points during collection (IN)
** echo       - ptr. to function to echo cursor position          (IN)
**
** This function interactively collects from the user a straight 
** line on a given canvas.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal collection ==> status = CLL_NORMAL = 1
**     This is also returned if user quit during collection, after
**     entering the first point.
** (2) Quit collection ==> status = CLL_QUIT = 0
**     This is returned if user quit collection at the begining,
**     before entering the first point.
** (3) Interupted collection ==> status = CLL_INTERUPTED = -1
**     This is returned when the collection is closed by something
**     else (not by the mouse interaction).
**
** --------------------------------------------------------------------
**
** int  CllKbdLineCreate( Ihandle *iup_cv,
**                    void (*put)( Ihandle *iup_cv, void *line, double tol ),
**                    void (*prompt)( Ihandle *iup_cv, char *msg ) )
**
** iup_cv     - IUP handle of canvas for client reference         (IN)
** put        - ptr. to function to put prmtv. after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
**
** This function interactively collects from the keyboard a straight
** line.
** If the collection is successfull the function returns a CLL_NORMAL (1)
** status.  Otherwise, it returns a CLL_QUIT (0) status.
**
** --------------------------------------------------------------------
**
** Version:      0-001
**
** Created:    25-Mar-94    Luiz F. Martha
**
** Modified:   31-Mar-94    LFM
**     When created Pck package: change old Gra mouse input (pick)
**     calls to Pck calls.
**
** Modified:   12-Apr-94    LFM
**     Eliminated "reinit" parameter in CllLineCreate and added
**     "end_action" parameter.
**
** Modified:   29-May-95    LFM
**     Version 1: Version to use with CD graphics package.
**     Got rid of GraPnt, replacing it by CllPnt.  CllPoint was
**     redefined as CllPnt: structure containing two fields: x, y.
**     Replaced double by float.
**     Changed prototype of snap and echo functions so that point
**     structure is replaced by x and y coordinates.
**     Created local function CllLineSnap due to change in Pck snap
**     function prototype.
**
** Modified:   25-Ago-95    LFM
**     Created prompt messages in  Portuguese.
**
** Modified:   14-Feb-96    LFM
**     Modified local snap function such that it calls client 
**     snap function only if control key is pressed.  Before
**     this was done in Pck module.
**     Changed such that nothing is done when end_action function
**     in null.  Before the collector reinitialized itself.
**
** Modified:   08-Mar-96    LFM
**     When CD changed, creating cdCreateCanvas, which returns
**     a CD handle for the canvas.
**     Changed CllLineCreate adding a parameter which is the CD
**     canvas handle.
**
** Modified:   28-Jan-97    LFM 
**     Added parameters "shift" and "control" to snap callback function.
**
** Modified:   03-Mar-97    LFM
**     Added keyboard collection function CllKbdLineCreate.
**
** Modified:   05-Mar-97    LFM
**     Changed interaction such that line collection is performed
**     in two (button down) picks.
**
** Modified:   09-Apr-97    LFM
**     Added call to CllClose in function CllLineCreate to close
**     any previous data structure in Cll super-class.
**
** Modified:   11-Apr-97    LFM
**     Added call to CllShiftSnap and added field fst in local 
**     collection data structure.
**
** Modified:   17-Apr-97    LFM
**      Added return status parameter for end action function.
**
** Modified:   23-Apr-97    LFM
**      Considered environment variable IUP_INGLES for prompt and 
**      dialog messages.
**
** Modified:   18-Feb-98    LFM
**      Modification to avoid displaying echo messages on the canvas.
**      Instead, echo messages are send to prompt message UI object.
**
** Modified:   19-Jan-2001  LFM
**      When call to function getenv( "IUP_INGLES" ) was replaced 
**      by call to function IupGetLanguage( ).
**      Local string variable iup_english changed to int.
**      Modified IupScanf floating point format.
**
** Modified:   10-Sep-2001  William
**      Substitution of the calls the IupScanf( ) for dialogs defined in the 
**      dlgline module.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "iup.h"
#include "cd.h"
#include "pck.h"
#include "cll.h"
#include "dlgline.h"
#include "cllline.h"


/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/
/*  Line data structure:
 */
typedef struct _clllinep {  /* Collected line private spec.descrip.: */
        CllLine  l;                 /* collected line                */
        int      fst;               /* if true, first line pt. given */
        void     *iup_cv;           /* IUP handle to target canvas   */
        void     *cd_cv;            /* cd handle to corresp.canvas   */
        void     (*end_action)      /* end action callback           */
                       ( Ihandle *iup_cv, int status );
        void     (*put)             /* function to put collect.line  */
                       ( Ihandle *iup_cv, void *line, double tol );
        void     (*prompt)          /* function to prompt messages   */
                       ( Ihandle *iup_cv, char *msg );
        void     (*snap)            /* function to snap collect. pt. */
                       ( Ihandle *iup_cv, double *x, double *y, 
                                       double tol, int shift, int control );
        void     (*echo)            /* function to echo cursor point */
                       ( Ihandle *iup_cv, double x, double y );
        } CllLineP;

#ifndef LEN
#define LEN(p0,p1) sqrt((p1.x-p0.x)*(p1.x-p0.x)+(p1.y-p0.y)*(p1.y-p0.y))
#endif

static int  iup_english;
static char *prompt_msg1;
static char *prompt_msg2;

static char *prompt_msg1p ={ "Entre primeiro ponto com botao esquerdo do mouse;"
                             " Botao do meio => Teclado." };
static char *prompt_msg2p = { "Entre com ponto final; "
                              "Botao do meio => Teclado; "
                              "Botao da direita => Cancela." };
static char *prompt_msg1e = { "Enter first point with mouse left button; "
                              "Mid button => Keyboard." };
static char *prompt_msg2e = { "Enter end point; "
                              "Mid button => Keyboard; "
                              "Right button => Cancel." };


/*
** --------------------------------------------------------------------
** Local functions:
*/

/* ==========================  CllLineDel  ======================== */

static void CllLineDel( CllLineP *line )
{
 if( line == 0L ) return;

/*  Release memory used for line
 */    
 free( line );
}


/* ========================  CllLineSnap  ========================= */

static void CllLineSnap( PckDesc *pk, double *x, double *y, double tol )
{
/* Restore registered collection spec. from the data field.
 */
 CllLineP  *line = (CllLineP *)pk->data;

/* If shift key is pressed and first line point has been given, 
 * snap line to the horizontal axis, to the vertical axis, or 
 * the bi-sector lines.
 */
 if( (line->fst) && isshift(pk->mousecfg) )
 {
  CllShiftSnap( line->l.p0.x, line->l.p0.y, x, y );
 }

/* Check to see whether client had registered a snap function and 
 * call it passing the given mouse position and parameters.
 */
 if( line->snap != 0L )
  (*line->snap )( line->iup_cv, x, y, tol,
                              isshift(pk->mousecfg), iscontrol(pk->mousecfg) );
}


/* ========================  CllLineEcho  ========================= */

static void CllLineEcho( PckDesc *pk, double x, double y )
{
 double len;
 char   echotext[60];

/* Restore registered collection spec. from the data field.
 */
 CllLineP  *line = (CllLineP *)pk->data;

/* If first point was already given, echo line length.
 */
 if( line->fst )
 {
  len = sqrt( (line->l.p0.x-x)*(line->l.p0.x-x) +
              (line->l.p0.y-y)*(line->l.p0.y-y) );

  if( iup_english )
   sprintf( echotext, "Length = %10.3f", len );
  else
   sprintf( echotext, "Comprimento = %10.3f", len );
  if( line->prompt != 0L )
   (*line->prompt)( line->iup_cv, echotext ) ;
 }

/* Check to see whether client had registered an echo function.
 * If that is the case call it passing the given mouse position.
 */
 if( line->echo != 0L )
  (*line->echo )( line->iup_cv, x, y ) ;
}


/*
** --------------------------------------------------------------------
** Public functions:
*/

/* ========================  CllLineCreate  ======================= */

static void CllLine1stPt( PckDesc * );
static void CllLine2ndPt( PckDesc * );

void CllLineCreate( Ihandle *iup_cv, void *cd_cv, 
              void (*end_action)( Ihandle *iup_cv, int status ), 
              void (*put)( Ihandle *iup_cv, void *line, double tol ), 
              void (*prompt)( Ihandle *iup_cv, char *msg ), 
              void (*snap)( Ihandle *iup_cv, double *x, double *y,  
                            double tol, int shift, int control ),
              void (*echo)( Ihandle *iup_cv, double x, double y ) )
{
 CllLineP  *line;
 PckDesc   pk;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  prompt_msg1 = prompt_msg1e;
  prompt_msg2 = prompt_msg2e;
 }
 else
 {
  prompt_msg1 = prompt_msg1p;
  prompt_msg2 = prompt_msg2p;
 }

 CllClose( );

/* Allocate a line collection descriptor
 */
 line = (CllLineP *)malloc( sizeof( CllLineP ) );

/* Initialize collection
 */
 line->fst = 0; 
 line->iup_cv = iup_cv; 
 line->cd_cv = cd_cv; 
 line->end_action = end_action; 
 line->put = put; 
 line->prompt = prompt; 
 line->snap = snap; 
 line->echo = echo;
 line->l.p0.x = line->l.p0.y = 0.0;
 line->l.p1.x = line->l.p1.y = 0.0;

/* Setup pick descriptor.  Register the appropriate line collector
 * function as the callback for pick interaction.
 */
 if( line->prompt != 0L )
  (*line->prompt )( line->iup_cv, prompt_msg1 ) ;
 pk.iup_cv = line->iup_cv;
 pk.gra_cv = line->cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.mode = ButtonDown;
 pk.cursorecho = BoxCursor | CrossCursor;
 pk.button = 0;    /* accept both mouse buttons 1 or 2 */
 pk.toler = 0.0;
 pk.btdwn_callback = CllLine1stPt;
 pk.btup_callback = 0L;
 pk.echofunc = CllLineEcho;
 pk.snapfunc = CllLineSnap;
 pk.data = line;
 pk.status = PCK_VALID;
 PckSetup( &pk );
}

/* ---------------------------  CllLine1stPt  ----------------------- */

static void CllLine1stPt( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllLineP  *line = (CllLineP *)pk->data;
 void      *iup_cv = line->iup_cv;
 void      (*end_action)( Ihandle *iup_cv, int status ) = line->end_action;

/* Check to see if pick is valid.  If not, delete collection data
 * call end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = CLL_QUIT;        break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllLineDel( line );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Get button down point as first line point.
 */
 line->l.p0.x = pk->p0.x;
 line->l.p0.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter the
 * first point using the keyboard. In this case, prompt the user 
 * for point coordinates.
 */
 if( pk->button == 2 )
 {
  float x0, y0;
  x0 = line->l.p0.x;
  y0 = line->l.p0.y;
  if( DlgCaptureFPLine( &x0, &y0 ) )
  {
   line->l.p0.x = pk->p0.x = x0;
   line->l.p0.y = pk->p0.y = y0;
  }
  else
  {
   CllLineDel( line );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
   return;
  }
 }

/* Set flag to indicate that first point has been selected.
 */
 line->fst = 1;

/* Change cursor echo for rubber band line (first point 
 * is initial point of rubber band line) and get pick from user
 * for end line point.
 */
 if( line->prompt != NULL )
  (*line->prompt )( iup_cv, prompt_msg2 ) ;
 pk->cursorecho = BoxCursor | CrossCursor | RubberBandCursor;
 pk->button = 0;       /* accept both mouse buttons 1 or 2 */
 pk->btdwn_callback = CllLine2ndPt;
 PckSetup( pk );
}

/* ---------------------------  CllLine2ndPt  ----------------------- */

static void CllLine2ndPt( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 CllLineP  *line = (CllLineP *)pk->data;
 void      *iup_cv = line->iup_cv;
 void      (*end_action)( Ihandle *iup_cv, int status ) = line->end_action;

/* Get button down point as end line point.
 */
 line->l.p1.x = pk->p0.x;
 line->l.p1.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter the
 * line using the keyboard. In this case, prompt the user for
 * line coordinates and return.
 */
 if( (pk->status == PCK_VALID) && (pk->button == 2) )
 {
  float x0, y0, x1, y1;
  x0 = line->l.p0.x;  y0 = line->l.p0.y;
  x1 = line->l.p1.x;  y1 = line->l.p1.y;
  if( DlgCaptureSPLine( &x0, &y0, &x1, &y1 ) )
  {
   line->l.p0.x = x0;  line->l.p0.y = y0;
   line->l.p1.x = x1;  line->l.p1.y = y1;
   if( LEN(line->l.p0,line->l.p1) >= pk->toler )
   {
    if( line->put != 0L )
     (*line->put )( line->iup_cv, &line->l, pk->toler );
   }
   CllLineDel( line );
   if( end_action ) (*end_action)( iup_cv, CLL_NORMAL );
  }
  else
  {
   CllLineDel( line );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
  }
  return;
 }

/* Check to see if collected line is valid (greater then tolerance).
 * If not valid, just quit the collection, calling end action function.
 */
 if( (pk->status != PCK_VALID) || (LEN(line->l.p0,line->l.p1) < pk->toler) )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_VALID:
   case PCK_QUIT:        status = CLL_NORMAL;      break;
   case PCK_INTERUPTED:  status = CLL_INTERUPTED;  break;
  }
  CllLineDel( line );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Call the client function to put the valid collected line, 
 * and finish collection calling end action function.
 */
 if( line->put != 0L )
  (*line->put )( iup_cv, &line->l, pk->toler );
 CllLineDel( line );
 if( end_action ) (*end_action)( iup_cv, CLL_NORMAL );
}


/* =======================  CllKbdLineCreate  ===================== */

static CllLine  kbdline;
static double   kbdtol = 0.001;

static char *kbdprompt_msg;

static char *kbdprompt_msgp = { "Digite dados da linha. " };
static char *kbdprompt_msge = { "Use the keyboard to enter line data. " };

int  CllKbdLineCreate( Ihandle *iup_cv,
              void (*put)( Ihandle *iup_cv, void *line, double tol ),
              void (*prompt)( Ihandle *iup_cv, char *msg ) )
{
 CllLine   line;
 float     x0, y0, x1, y1;
 float     toler;
 int       status = CLL_QUIT;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  kbdprompt_msg = kbdprompt_msge;
 }
 else
 {
  kbdprompt_msg = kbdprompt_msgp;
 }

/* Collect data from user with the keyboard.
 */
 if( prompt != 0L )
  (*prompt)( iup_cv, kbdprompt_msg );

 x0 = kbdline.p1.x;
 y0 = kbdline.p1.y;
 x1 = kbdline.p1.x;
 y1 = kbdline.p1.y;
 toler = kbdtol;
 if( DlgCaptureLine( &x0, &y0, &x1, &y1, &toler ) )
 {
  line.p0.x = x0;
  line.p0.y = y0;
  line.p1.x = x1;
  line.p1.y = y1;
  if( LEN(line.p0,line.p1) > 0.0 )
  {
   kbdline = line;
   kbdtol = toler;
   if( put != 0L )
    (*put)( iup_cv, &line, kbdtol );
   status = CLL_NORMAL;
  }
 }

 return( status );
}


