/*
** CllSpline.c - Functions to collect splines.
**
** Public functions:
** --------------------------------------------------------------------
**
** void CllSplineCreate( Ihandle *iup_cv, void *cd_cv, 
**                void (*end_action)( Ihandle *iup_cv, int status ),
**                void (*put)( Ihandle *iup_cv, void *spline, double tol ), 
**                void (*prompt)( Ihandle *iup_cv, char *msg ),
**                void (*snap)( Ihandle *iup_cv, double *x, double *y,
**                              double tol, int shift, int control ),
**                void (*echo)( Ihandle *iup_cv, double x, double y ) )
**
** iup_cv     - IUP handle of canvas where primitive is collected   (IN)
** cd_cv      - CD handle of canvas                                 (IN)
** end_action - function to be called after finishing spline creat. (IN)
** put        - ptr. to function to put spline after collection     (IN)
** prompt     - ptr. to function to prompt messg. during collect.   (IN)
** snap       - ptr. to function to snap points during collection   (IN)
** echo       - ptr. to function to echo cursor position            (IN)
**
** This function interactively collects from the user a spline
** on a given canvas.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal collection ==> status = CLL_NORMAL = 1
**     This is also returned if user quit during collection, after
**     entering the first point.
** (2) Quit collection ==> status = CLL_QUIT = 0
**     This is returned if user quit collection at the begining,
**     before entering the first point.
** (3) Interupted collection ==> status = CLL_INTERUPTED = -1
**     This is returned when the collection is closed by something
**     else (not by the mouse interaction).
**
** --------------------------------------------------------------------
** 
** int  CllKbdSplineCreate( Ihandle *iup_cv,
**                    void (*put)( Ihandle *iup_cv, void *spline, double tol ),
**                    void (*prompt)( Ihandle *iup_cv, char *msg ) )
**
** iup_cv     - IUP handle of canvas for client reference         (IN)
** put        - ptr. to function to put prmtv. after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
**
** This function interactively collects from the keyboard a spline.
** If the collection is successfull the function returns a CLL_NORMAL (1)
** status.  Otherwise, it returns a CLL_QUIT (0) status.
**
** --------------------------------------------------------------------
**
** Version:      0-001
**
** Created:    08-Oct-03    Marcos C. de Arruda
**
** Modified:   21-Apr-2004    William Lira
**      Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "iup.h"
#include "cd.h"
#include "wd.h"
#include "pck.h"
#include "cll.h"
#include "cllspline.h"
#include "dlgspline.h"


/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/
/*  Spline data structure:
 */
#define  ALLOC_SIZE      10

typedef struct _CllSplinep { /* Collected spline private spec.descrip.: */
        CllSpline spl;              /* collected spline                 */
        int      allocsize;         /* current allocated size           */
        int      created;           /* flag for created curve           */
        int      modified;          /* flag for modified curve          */
        int      displayed;         /* flag for displayed curve         */
        void     *iup_cv;           /* IUP handle to target canvas      */
        void     *cd_cv;            /* cd handle to corresp.canvas      */
        void     (*end_action)      /* end action callback              */
                       ( Ihandle *iup_cv, int status );
        void     (*put)             /* function to put collect.spline   */
                       ( Ihandle *iup_cv, void *spl, double tol );
        void     (*prompt)          /* function to prompt messages      */
                       ( Ihandle *iup_cv, char *msg );
        void     (*snap)            /* function to snap collect. pt.    */
                       ( Ihandle *iup_cv, double *x, double *y, 
                                       double tol, int shift, int control );
        void     (*echo)            /* function to echo cursor point    */
                       ( Ihandle *iup_cv, double x, double y );
        } CllSplineP;

typedef struct {
 double x;
 double y;
 double w;
} CllPnt3d;

#ifndef LEN
#define LEN(p0,p1) sqrt((p1.x-p0.x)*(p1.x-p0.x)+(p1.y-p0.y)*(p1.y-p0.y))
#endif

#ifndef PI
#define PI  fabs(acos(-1.))
#endif

static int  iup_english;
static char *prompt_msg;
static char *prompt_msg2;

static char *prompt_msgp  = { "Entre ponto (botao esquerdo do mouse); "
                              "Botao do meio => Teclado; "
                            "Botao da direita => Termina insero de pontos." };
static char *prompt_msge  = { "Enter point with mouse left button; "
                              "Mid button => Keyboard; "
                              "Right button => Finish point insertion." };
static char *prompt_msg2e = { "With mouse button pressed drag "
                              "end tangents; "
                              "Right button => Finish." };
static char *prompt_msg2p = { "Com botao precionado do mouse arraste "
                              "as tangentes das extremidades; "
                              "Botao da direita => Termina." };                                                           
static double _dTol;

/*
** --------------------------------------------------------------------
** Local functions:
*/

static void       CllPickCode           ( double x, double y,
                                          double xmin, double xmax, 
                                          double ymin, double ymax, 
                                          int cod[4] );
static int        CllPickPoint          ( double ptx, double pty, 
                                          double x, double y, double toler );
static void       cSpl_Pnt3dConstr      ( CllPnt3d* p );
static void       cSpl_DersBasisFuns    ( CllSpline* at, int n, double u, 
                                          int span, double** ders );                                          
static void       cSpl_DeriveAtH        ( CllSpline* at, double u, int d, 
                                          CllPnt3d* ders );
static void       cSpl_BinomialCoef     ( int nBin, double **Bin );
static void       cSpl_SetEndTan        ( CllSpline *at, CllPnt *D0, 
                                          CllPnt *Dn );                                          
static void       cSpl_Delete           ( CllSplineP* spline );
static void       cSpl_Realloc          ( CllSplineP *spline );
static void       cSpl_Init             ( CllSpline* at, int nQ, int d );
static double     cSpl_ChordLengthParam ( int nQ, CllPnt* Q, int* nub, 
                                          double** ub );
static void       cSpl_Create           ( CllSpline* at, int nQ, CllPnt* Q, 
                                          int nub, double* ub, int d );
static void       cSpl_CreateEndDeriv   ( CllSpline* at, int nQ, CllPnt* Q,
                                          int nub, double* ub, int d, 
                                          CllPnt *D0, CllPnt *Dn );
static void       cSpl_KnotAveraging    ( CllSpline* at, int nuk, double* uk, 
                                          int deg );
static void       cSpl_KnotAverEndDeriv ( CllSpline* at, int nuk, double* uk,
                                          int deg );                                          
static void       cSpl_Resize           ( CllSpline* at, int n, int Deg );
static void       cSpl_ResizeP          ( CllSpline* at, int n );
static void       cSpl_ResizeU          ( CllSpline* at, int n );
static int        cSpl_FindSpan         ( CllSpline* at, double u );
static void       cSpl_BasisFuns        ( CllSpline* at, double u, int i, 
                                          int nN, double* N );
static int        cSpl_Solve            ( int n, double** A, double** B, 
                                          double** X );
static void       cSpl_Decompose        ( int n, double** a, int* pivot );
static void       cSpl_Inverse          ( int n, double** A, int* pivot );
static void       cSpl_Multi            ( int n, double** a, double** b, 
                                          double** prod );
static int        cSpl_Evaluate1Pt      ( CllSpline* at, double u, CllPnt* p );
static int        cSpl_Evaluate2Pt      ( CllSpline* at, double u, CllPnt* p, 
                                          CllPnt* pl );
static void       cSpl_NurbsBasisFuns   ( double u, int span, int deg, 
                                          double* U, double* N );
static CllPnt3d   cSpl_FirstD           ( CllSpline* at, double u, int span );
static CllPnt3d   cSpl_HpointAt         ( CllSpline* at, double u, int span );
static CllPnt     cSpl_FirstDn          ( CllSpline* at, double u, int span );
static double     cSpl_LengthF          ( CllSpline* at, double u, int span );
static void       cSpl_Intccini         ( int nw, double* w );
static double     cSpl_Intcc            ( CllSpline* at, int f, double a, 
                                          double b, double eps, 
                                          int nw, double* w, double* err );
static void       cSpl_Dfct             ( int n, double wr, double wi, 
                                          double *a );
static void       cSpl_Ddct             ( int n, double wr, double wi, 
                                          double *a );
static void       cSpl_Bitrv2           ( int n, double* a );
static void       cSpl_Cdft             ( int n, double wr, double wi, 
                                          double* a );
static void       cSpl_Rdft             ( int n, double wr, double wi, 
                                          double* a );
static void       cSpl_Bitrv            ( int n, double* a );
static int        cSpl_FindMult         ( CllSpline* at, int r );
static double     cSpl_LengthIn         ( CllSpline* at, double us, 
                                          double ue );
static int        isColinear            ( CllPnt p0, CllPnt pm, CllPnt p1 );                                           
static int        cSpl_Straight         ( CllSpline* at, CllPnt p0, 
                                          CllPnt pm, CllPnt p1,
                                          double t0, double tm, double t1 );
static void       cSpl_DrawLine         ( double x0, double y0, double x1, 
                                          double y1 );
static void       cSpl_DrawSplineRecSub ( CllSpline* c, double t0, double tm, 
                                          double t1, CllPnt p0, CllPnt pm, 
                                          CllPnt p1 );
static void       cSpl_TangVec          ( CllSpline* at, CllPnt *p0a, 
                                          CllPnt *p0b, CllPnt *p1a, 
                                          CllPnt *p1b );
static void       cSpl_DrawTangVec      ( CllSpline* at, int clr );
static void       cSpl_Draw             ( CllSpline* c, int clr, int tg );
static void       cSpl_Clr              ( CllSplineP *spl );
static void       cSpl_Dsp              ( void *l );
static void       cSpl_Snap             ( PckDesc *pk, double *x, double *y, 
                                          double tol );
static void       cSpl_Echo             ( PckDesc *pk, double x, double y );                                          
static void       cSpl_DisplayXor       ( CllSplineP *spline );
static void       cSpl_BtDwnCb          ( PckDesc *pk );
static void       cSpl_BtUpCb           ( PckDesc *pk );
static void       cSpl_P0aCrtEcho       ( PckDesc *pk, double echoptx, 
                                          double echopty );
static void       cSpl_P0bCrtEcho       ( PckDesc *pk, double echoptx, 
                                          double echopty );
static void       cSpl_P1aCrtEcho       ( PckDesc *pk, double echoptx, 
                                          double echopty );
static void       cSpl_P1bCrtEcho       ( PckDesc *pk, double echoptx, 
                                          double echopty );
static void       cSpl_Pt               ( PckDesc *pk );
static void       cSpl_Redisplay        ( void *b );
static void       cSpl_CreateGen        ( Ihandle *iup_cv, void *cd_cv,
                         void (*end_action)( Ihandle *iup_cv, int status ),
                         void (*put)( Ihandle *iup_cv, void *spl, double tol ), 
                         void (*prompt)( Ihandle *iup_cv, char *msg ), 
                         void (*snap)( Ihandle *iup_cv, double *x, double *y,  
                         double tol, int shift, int control ),
                         void (*echo)( Ihandle *iup_cv, double x, double y ) );
static int        cSpl_KbdCreateGen ( Ihandle *iup_cv,
                         void (*put)( Ihandle *iup_cv, void *spl, double tol ),
                         void (*prompt)( Ihandle *iup_cv, char *msg ) );

/* ==========================  CllPickCode  ======================== */

static void CllPickCode( double x, double y,
                         double xmin, double xmax, double ymin, double ymax, 
                         int cod[4] )
{
 cod[0] = x < xmin;
 cod[1] = x > xmax;
 cod[2] = y < ymin;
 cod[3] = y > ymax;
} 
 
/* ==========================  CllPickPoint  ======================= */

static int CllPickPoint( double ptx, double pty, 
                         double x, double y, double toler )
{
 double   xmin, xmax, ymin, ymax;
 int      cod[4];
 int      i;

/* Define atraction window.
 */
 xmin = x - toler;
 xmax = x + toler;
 ymin = y - toler;
 ymax = y + toler;

/* Check to see if given point (x,y) touches target point (ptx,pty)
 * based on given tolerance.
 */
 CllPickCode( ptx, pty, xmin, xmax, ymin, ymax, cod );
 for( i = 0; i < 4; i++ )
  if( cod[i] )
   return( 0 );

 return( 1 );
}
                  
/* ========================   cSpl_Pnt3dConstr   ========================== */

static void cSpl_Pnt3dConstr( CllPnt3d* p)
{
 p->x = 0.0;
 p->y = 0.0;
 p->w = 1.0; 
}                  

/* =========================  cSpl_DersBasisFuns  ======================= */

static void cSpl_DersBasisFuns( CllSpline* at, int n, double u, int span, 
                                double** ders )
{
  int i, j, k, r ;
  int rk, pk ;
  int j1, j2 ;
  double d ;
  
  double saved;
  double temp ;

  double*  left ;
  double*  right ;
  
  double** ndu ;
  double** a ;
  
  left = (double*) calloc( 2*(at->dg + 1), sizeof(double) ) ;
  right = &left[at->dg + 1] ;

  ndu = (double **) calloc(at->dg + 1, sizeof(double *) );

  for( i = 0; i < at->dg + 1; i++ )
       ndu[i] = (double *) calloc(at->dg + 1, sizeof(double) );

  ndu[0][0] = 1.0 ;

  for( j = 1; j <= at->dg ; j++ )
  {
    left[j]  = u - at->knot[span+1-j] ;
    right[j] = at->knot[span+j] - u ;
    saved    = 0.0 ;

    for( r = 0; r < j; r++ )
    {
      /* Lower triangle */
      ndu[j][r] = right[r+1]+left[j-r] ;
      temp      = ndu[r][j-1]/ndu[j][r] ;

      /* Upper triangle */
      ndu[r][j] = saved+right[r+1] * temp ;
      saved     = left[j-r] * temp ;
    }

    ndu[j][j] = saved ;
  }

  for( j = at->dg; j >= 0; --j ) ders[0][j] = ndu[j][at->dg] ;

  /* Compute the derivatives */
  a = (double **) calloc(at->dg + 1, sizeof(double *) );

  for( i = 0; i < at->dg + 1; i++ )
       a[i] = (double *) calloc(at->dg + 1, sizeof(double) );

  for( r = 0; r <= at->dg; r++ )
  {
    int s1 = 0 ; 
    int s2 = 1 ; /* alternate rows in array a */

    a[0][0] = 1.0 ;

    /* Compute the kth derivative */
    for( k = 1; k <= n; k++ )
    {      
      d = 0.0 ;

      rk = r-k ; pk = at->dg - k ;

      if( r >= k ) 
      {
        a[s2][0] = a[s1][0]/ndu[pk+1][rk] ;
        d = a[s2][0]*ndu[rk][pk] ;
      }

      if(rk >= -1) j1 = 1 ;
      else         j1 = -rk ;

      if(r-1 <= pk) j2 = k-1 ;
      else          j2 = at->dg - r ;

      for(j = j1; j <= j2; j++)
      {
        a[s2][j] = (a[s1][j]-a[s1][j-1])/ndu[pk+1][rk+j] ;
        d += a[s2][j]*ndu[rk+j][pk] ;
      }

      if(r <= pk)
      {
        a[s2][k] = -a[s1][k-1]/ndu[pk+1][r] ;
        d += a[s2][k]*ndu[r][pk] ;
      }

      ders[k][r] = d ;

      j = s1 ; s1 = s2 ; s2 = j ; /* Switch rows */
    }
  }

  /* Multiply through by the correct factors */
  r = at->dg ;
  for(k = 1; k <= n; k++)
  {
    for(j = at->dg; j >= 0; --j) ders[k][j] *= r ;
    r *= at->dg - k ;
  }

  for( i = 0; i < at->dg + 1; i++ ) 
  {
   free( ndu[i] );
   free( a[i] );
  }
  
  free( a );
  free( ndu );
  free( left );
}

/* =========================  cSpl_DeriveAtH  ======================= */

static void cSpl_DeriveAtH( CllSpline* at, double u, int d, CllPnt3d* ders ) 
{
  int i, j, k;
  int du = d < at->dg ? d : at->dg ;
  int span;
 
  double** derF = (double **) calloc(du+1, sizeof(double *));

  for( i = 0; i < du + 1; i++ )
       derF[i] = (double *) calloc(at->dg + 1, sizeof(double));

  span = cSpl_FindSpan( at, u ) ;

  cSpl_DersBasisFuns(at, du, u, span, derF) ;

  for( k = du; k >= 0; --k )
  {
    ders[k].x = 0 ;
    ders[k].y = 0 ;
    ders[k].w = 0 ;

    for( j = at->dg; j >= 0; --j )
    {
         ders[k].x += derF[k][j] * at->ctrlpts[span - at->dg + j].x ;
         ders[k].y += derF[k][j] * at->ctrlpts[span - at->dg + j].y ;
         ders[k].w += derF[k][j] * 1.0;
    }
  }

  for( i = 0; i < du+1; i++ ) free( derF[i] );
  free( derF );
}

/* =========================  cSpl_BinomialCoef  ======================= */

static void cSpl_BinomialCoef( int nBin, double **Bin )
{
  int n;
  int k;

  /* Setup the first line */
  Bin[0][0] = 1.0 ;

  for(k = nBin - 1; k > 0;--k)
    Bin[0][k] = 0.0 ;

  /* Setup the other lines */
  for(n = 0; n < nBin - 1; n++)
  {
    Bin[n+1][0] = 1.0 ;

    for(k = 1; k < nBin; k++)
    {
      if(n+1 < k)
        Bin[n][k] = 0.0 ;
      else
        Bin[n+1][k] = Bin[n][k] + Bin[n][k-1] ;
    }
  }
}

/* =========================  cSpl_SetEndTan  ======================= */

static void cSpl_SetEndTan( CllSpline *at, CllPnt *D0, CllPnt *Dn )
{
 int nub;
 double *ub;
 
 free(at->knot);
 free(at->ctrlpts);
 at->knot = NULL;
 at->ctrlpts = NULL;
 at->nu = 0;
 at->np = 0;
 cSpl_ChordLengthParam(at->numpts, at->keypts, &nub, &ub ) ;
 cSpl_CreateEndDeriv( at, at->numpts, at->keypts, nub, ub, 
                      at->dg, D0, Dn );
 at->itan.x = D0->x;
 at->itan.y = D0->y;
 at->ftan.x = Dn->x;
 at->ftan.y = Dn->y;
}

/* =========================  cSpl_Delete  ======================= */

static void cSpl_Delete( CllSplineP* spline )
{
  CllSpline *at;	
  
  if (spline == NULL)
   return;
  at = &spline->spl;
   
  if (at)
  {
   if ( at->keypts )     free( at->keypts );
   if (at->numpts > 1)
   {
    if ( at->knot )      free( at->knot );
    if ( at->ctrlpts )   free( at->ctrlpts );
   }
   at->knot      = NULL;
   at->ctrlpts   = NULL;
   at->keypts    = NULL;
   
   free(spline);
   spline = NULL;
  }
}

/* =======================  cSpl_Realloc  ======================= */

static void cSpl_Realloc( CllSplineP *spline )
{
  if( spline->spl.numpts == spline->allocsize )
  {
   spline->allocsize += ALLOC_SIZE;
   spline->spl.keypts = (CllPnt *)realloc(spline->spl.keypts, 
                                          spline->allocsize * sizeof( CllPnt ));
  }
}

/* =========================  cSpl_Init  ======================= */

static void cSpl_Init( CllSpline* at, int nQ, int d )
{
  int     nub;
  double* ub;
  CllPnt  p;

  if (nQ == 2)
  {
   nQ = 3;
   at->keypts[2].x = at->keypts[1].x;
   at->keypts[2].y = at->keypts[1].y;
   at->keypts[1].x = (at->keypts[0].x + at->keypts[2].x) / 2.0;
   at->keypts[1].y = (at->keypts[0].y + at->keypts[2].y) / 2.0;
  }
  
  at->numpts    = nQ;
  at->knot      = NULL;
  at->ctrlpts   = NULL;
  at->nu          = 0;
  at->np          = 0;
  at->dg          = 2;

  cSpl_ChordLengthParam( nQ, at->keypts, &nub, &ub ) ;
  cSpl_Create( at, nQ, at->keypts, nub, ub, d );

  cSpl_Evaluate2Pt(at, 0.0, &p, &at->itan);
  cSpl_Evaluate2Pt(at, 1.0, &p, &at->ftan);
  
  free( ub );
}

/* =========================  cSpl_ChordLengthParam  ======================= */

static double cSpl_ChordLengthParam(int nQ, CllPnt* Q, int* nub, double** ub)
{
  int i ;
  double d = 0.0 ;

  *ub  = (double *)calloc( nQ, sizeof(double) );
  *nub = nQ;

  (*ub)[0] = 0.; 

  for( i = 1; i < *nub; i++ ) d += LEN(Q[i],Q[i-1]) ;

  if( d > 0 )
  {
    for( i = 1; i < *nub; ++i ) (*ub)[i] = (*ub)[i-1] + LEN(Q[i],Q[i-1]) / d;
    (*ub)[*nub - 1] = 1.0 ;
  }
  else
  {
    for( i = 1; i < *nub - 1; ++i ) (*ub)[i] = (double)i/(double)(*nub - 1) ;
    (*ub)[*nub - 1] = 1.0 ;
  }

  return d ;
}

/* =========================  cSpl_Create  ======================= */

static void cSpl_Create( CllSpline* at, int nQ, CllPnt* Q, int nub, 
                         double* ub, int d )
{
  int i ;
  int j ;
  int nN ;
  int span ;
  double* N ;
  double** A ;
  double** qq; 
  double** xx ;

  if( d <= 0 )
      return;

  if( d >= nQ )
      return;

  cSpl_Resize(at,nQ,d) ;

  A = (double **)calloc( nQ, sizeof(double *));

  for( i = 0; i < nQ; i++ ) 
       A[i] = (double *)calloc( nQ, sizeof(double));

  cSpl_KnotAveraging(at, nub, ub, d) ;

  nN = at->dg + 1;
  N = (double *)calloc(at->dg + 1, sizeof(double) );

  for( i = 1; i < nQ - 1; i++ )
  {
    span = cSpl_FindSpan( at, ub[i] );

    cSpl_BasisFuns(at, ub[i], span, nN, N) ;

    for( j = 0; j <= at->dg; j++ ) A[i][span- at->dg + j] = (double)N[j] ;
  }

  A[0][0] = 1.0 ;
  A[nQ - 1][nQ - 1] = 1.0 ;

  /* Init matrix for LSE */
  qq = (double **)calloc( nQ, sizeof(double *));
  xx = (double **)calloc( nQ, sizeof(double *));

  for( i = 0; i < nQ; i++ ) 
  {
       qq[i] = (double *)calloc( 2, sizeof(double));
       xx[i] = (double *)calloc( 2, sizeof(double));
  }

  for( i = 0; i < nQ; i++ )
  {
       qq[i][0] = Q[i].x;
       qq[i][1] = Q[i].y;
  }

  cSpl_Solve(nQ, A, qq, xx) ;

  for( i = 0; i < nQ; i++ )
  {
       at->ctrlpts[i].x = xx[i][0];
       at->ctrlpts[i].y = xx[i][1];
  }

  for( i = 0; i < nQ; i++ )
  {  
   free( A[i] );
   free( qq[i] );  /* NOVO FREE */
   free( xx[i] );  /* NOVO FREE */
  }

  free( A );
  free( N );
  free( qq );      /* NOVO FREE */
  free( xx );      /* NOVO FREE */
}

/* =========================  cSpl_CreateEndDeriv  ======================= */

static void cSpl_CreateEndDeriv( CllSpline* at, int nQ, CllPnt* Q, int nub, 
                                 double* ub, int d, CllPnt *D0, CllPnt *Dn )
{
  int i ;
  int j ;
  int nN ;
  int span ;
  double* N ;
  double** A ;
  double** qq; 
  double** xx ;
  
  if( d <= 0 )
      return;

  if( d >= nQ )
      return;

  cSpl_Resize(at,nQ + 2,d) ;

  A = (double **) calloc( nQ + 2, sizeof(double *));

  for( i = 0; i < nQ + 2; i++ ) 
       A[i] = (double *) calloc( nQ + 2, sizeof(double));

  for( i = 0; i < nQ + 2; i++ ) 
   for( j = 0; j < nQ + 2; j++ )
    A[i][j] = 0.0; 

  cSpl_KnotAverEndDeriv(at, nub, ub, d) ;

  nN = at->dg + 1;
  N = (double *) calloc(at->dg + 1, sizeof(double) );

  for( i = 1; i < nQ - 1; i++ )
  {
    span = cSpl_FindSpan( at, ub[i] );

    cSpl_BasisFuns(at, ub[i], span, nN, N) ;

    for( j = 0; j <= at->dg; j++ ) A[i+1][span- at->dg + j] = (double)N[j] ;
  }

  A[0][0] = 1.0 ;
  A[nQ][nQ+1] = 1.0 ;

  A[1][0] = -1.0;
  A[1][1] = 1.0;
  
  A[nQ+1][nQ] = -1.0;
  A[nQ+1][nQ+1] = 1.0; 

  /* Init matrix for LSE */
  qq = (double **) calloc( nQ + 2, sizeof(double *));
  xx = (double **) calloc( nQ + 2, sizeof(double *));

  for( i = 0; i < nQ + 2; i++ ) 
  {
       qq[i] = (double *) calloc( 2, sizeof(double));
       xx[i] = (double *) calloc( 2, sizeof(double));
  }

  for( i = 0; i < nQ + 2; i++ )
  {
   if (i == 0)
   {
    qq[i][0] = Q[i].x;
    qq[i][1] = Q[i].y;
   }
   else if ( i == 1)
   {
    qq[i][0] = (at->knot[at->dg+1] / at->dg) * D0->x; 
    qq[i][1] = (at->knot[at->dg+1] / at->dg) * D0->y; 
   }  
   else if ((i > 1) && (i < nQ + 1))
   {
    qq[i][0] = Q[i-1].x;
    qq[i][1] = Q[i-1].y;
   }
   else
   {
    qq[i][0] = ((1 - at->knot[at->nu-at->dg-2]) / at->dg) * Dn->x;
    qq[i][1] = ((1 - at->knot[at->nu-at->dg-2]) / at->dg) * Dn->y;
   }
  }

  cSpl_Solve(nQ + 2, A, qq, xx) ;

  for( i = 0; i < nQ + 2; i++ )
  {
       at->ctrlpts[i].x = xx[i][0];
       at->ctrlpts[i].y = xx[i][1];
  }

  for( i = 0; i < nQ + 2; i++ ) 
  {
  free( A[i] );
  free( qq[i] );
  free( xx[i] );
  }
  
  free( A );
  free( N );
  free( qq );
  free( xx );
}

/* =========================  cSpl_KnotAveraging  ======================= */

static void cSpl_KnotAveraging( CllSpline* at, int nuk, double* uk, int deg )
{
  int j ;
  int i ;

  cSpl_ResizeU( at, nuk + deg + 1 ) ;

  for( j = 1; j < nuk - deg; ++j )
  {
    at->knot[j+deg] = 0.0 ;
    for( i = j; i < j + deg; ++i ) at->knot[j+deg] += uk[i] ;
    at->knot[j+deg] /= (double)deg ;
  }

  for( j = 0; j <= deg; ++j )                  at->knot[j] = 0.0 ;
  for( j = at->nu - deg - 1; j < at->nu; ++j ) at->knot[j] = 1.0 ;
}

/* ========================  cSpl_KnotAverEndDeriv  ====================== */

static void cSpl_KnotAverEndDeriv( CllSpline* at, int nuk, double* uk, int deg )
{
  int j ;
  int i ;

  cSpl_ResizeU( at, nuk + deg + 3 ) ;

  for( j = 0; j <= nuk - deg; ++j )
  {
    at->knot[j+deg+1] = 0.0 ;
    for( i = j; i < j + deg; ++i ) at->knot[j+deg+1] += uk[i] ;
    at->knot[j+deg+1] /= (double)deg ;
  }

  for( j = 0; j <= deg; ++j )                  at->knot[j] = 0.0 ;
  for( j = at->nu - deg - 1; j < at->nu; ++j ) at->knot[j] = 1.0 ;
}

/* =========================  cSpl_Resize  ======================= */

static void cSpl_Resize( CllSpline* at, int n, int Deg )
{
  at->dg = Deg;

  cSpl_ResizeP( at, n );
  cSpl_ResizeU( at, n + at->dg + 1 );
}

/* =========================  cSpl_ResizeP  ======================= */
  
static void cSpl_ResizeP( CllSpline* at, int n )
{
  if( n == at->np ) 
      return;
 
  if( at->np > n ) 
  {
      at->np = n; return ;
  } 

  if( at->ctrlpts == NULL )
      at->ctrlpts = (CllPnt *)calloc( n, sizeof(CllPnt) );
  else
      at->ctrlpts = (CllPnt *)realloc( at->ctrlpts, n * sizeof(CllPnt) );

  at->np = n;
}

/* =========================  cSpl_ResizeU  ======================= */

static void cSpl_ResizeU( CllSpline* at, int n )
{
  if( n == at->nu )
  {
      return;
  }

  if( at->nu > n )
  {
      at->nu = n; return ;
  }

  if( at->knot == NULL )
      at->knot = (double *)calloc( n, sizeof(double) );
  else
      at->knot = (double *)realloc( at->knot, n * sizeof(double) );

  at->nu = n;
}

/* =========================  cSpl_FindSpan  ======================= */

static int cSpl_FindSpan( CllSpline* at, double u )
{ 
  int low, high, mid;
  if( u >= at->knot[at->np] )
      return at->np - 1 ;
  
  if( u <= at->knot[at->dg] )
      return at->dg ;

  low  = 0 ;
  high = at->np + 1 ;
  mid  = (low + high)/2 ;

  while( u < at->knot[mid] || u >= at->knot[mid+1] )
  {
    if( u < at->knot[mid] ) high = mid ;
    else                    low  = mid ;

    mid = (low + high)/2 ;
  }

  return mid ;
}

/* =========================  cSpl_BasisFuns  ======================= */

static void cSpl_BasisFuns( CllSpline* at, double u, int i, int nN, double* N )
{

  int j, r;
  double* left  = (double *)calloc(2 * (at->dg + 1), sizeof(double)) ;
  double* right = &left[at->dg+1] ;

  double temp;
  double saved ;

  N[0] = 1.0 ;

  for( j = 1; j <= at->dg ; j++ )
  {
    left[j]  = u - at->knot[i+1-j] ;
    right[j] = at->knot[i+j] - u ;

    saved = 0.0 ;

    for( r = 0 ; r < j; r++ )
    {
      temp  = N[r]/(right[r+1] + left[j-r]) ;
      N[r]  = saved + right[r+1] * temp ;
      saved = left[j-r] * temp ;
    }

    N[j] = saved ;
  }

  free( left );
}

/* =========================  cSpl_Solve  ======================= */

static int cSpl_Solve( int n, double** A, double** B, double** X )
{
  int* pivot = (int *)calloc( n, sizeof(int));

  /* use LU decomposition to solve the problem */
  cSpl_Decompose( n, A, pivot );
  cSpl_Inverse  ( n, A, pivot );

  cSpl_Multi( n, A, B, X ) ;

  free( pivot );

  return 1;
}

/* =========================  cSpl_Decompose  ======================= */

static void cSpl_Decompose( int n, double** a, int* pivot )
{
  int i;
  int j;
  int k;
  int l;
  int kp1;
  double t;
  double q;
  double den;
  double ten;

  int errval = 0;
  int   sign = 1;

  int nm1 = n - 1;

  for( k = 0; k < n; k++ ) pivot[k] = k ;

  if( nm1 >= 1 ) /* non-trivial problem */
  {
      for( k = 0; k < nm1; k++ )
      {
          kp1 = k + 1;

          /* partial pivoting ROW exchanges */
          /* -- search over column */
          ten = fabs( a[k][k] );
          l   = k;

          for( i = kp1; i < n; i++ )
          {
             den = fabs( a[i][k] );

             if( den > ten )
             {
                 ten = den;
                 l = i;
             }
          }

          pivot[k] = l;

          if( a[l][k] != 0.0 )
          {                     /* nonsingular pivot found  */
              if( l != k )      /* interchange needed */
              {
                for( i = k; i < n; i++ )
                {
                    t = a[l][i] ;
                    a[l][i] = a[k][i] ;
                    a[k][i] = t ;
                }

                sign = -sign ;
              }

              q = a[k][k];   /* scale row */

              for( i = kp1; i < n; i++ )
              {
                  t = - a[i][k]/q;
                  a[i][k] = t;
                  for (j = kp1; j < n; j++) a[i][j] += t * a[k][j];
              }
          }
          else          /* pivot singular */
            errval = k;
      }
  }

  pivot[nm1] = nm1;

  if( a[nm1][nm1] == 0.0 )
      errval = nm1;
}

/* =========================  cSpl_Inverse  ======================= */

static void cSpl_Inverse( int n, double** A, int* pivot )
{
  double ten;
  int i, j, k, l, kb, kp1;

  int  nm1 = n - 1;

  /* inverse U  */
  for( k = 0; k < n; k++ )
  {
      A[k][k] = ten = 1.0 / A[k][k] ;

      ten = -ten;

      for( i = 0; i < k; i++ ) A[i][k] *= ten;

      kp1 = k + 1;

      if( nm1 >= kp1 )
      {
         for( j = kp1; j < n; j++ )
         {
              ten = A[k][j] ;

              A[k][j] = 0.0 ;
              for( i = 0; i < kp1; i++ )
                   A[i][j] += ten * A[i][k] ;
         }
      }
  }

  if( nm1 >= 1 )
  {
      double* work = (double *)calloc(n, sizeof(double));

      for( kb = 0; kb < nm1; kb++ )
      {
          k = nm1 - kb - 1;
          kp1 = k + 1;

          for( i = kp1; i < n; i++ )
          {
              work[i] = A[i][k] ;
              A[i][k] = 0.0;
          }

          for( j = kp1; j < n; j++ )
          {
              ten = work[j];
              for (i = 0; i < n; i++)
                A[i][k] += ten * A[i][j] ;
          }

          l = pivot[k];

          if( l != k )
          {
            for (i = 0; i < n; i++)
              {
                ten = A[i][k] ;
                A[i][k] = A[i][l];
                A[i][l] = ten;
              }
          }
      }

      free( work );
  }
}

/* =========================  cSpl_Multi  ======================= */

static void cSpl_Multi( int n, double** a, double** b, double** prod )
{

  int i;
  int j;
  int k;
  int row = n, col = 2, size = n;

  double zero = 0.;

  for(i = row-1; i >= 0; --i)
    for( j = size-1; j >=0; --j)
      if(a[i][j] != zero)
      {
        for(k=col-1;k>=0; --k)
          prod[i][k] += a[i][j]* b[j][k] ;
      }
}

/* ========================  cSpl_Evaluate1Pt  ======================= */
/*
** Given the parametric value of a point on the curve, it returns the cartesian 
** coordinates of that point on the curve
*/
static int cSpl_Evaluate1Pt( CllSpline* at, double u, CllPnt* p )
{
  int i, nN, span;
  static double* Nb = NULL;
  static int  maxNB = 10;
  
  nN = at->dg + 1;
  span = cSpl_FindSpan( at, u ) ;

  if( Nb == NULL )
      Nb = (double *)calloc(maxNB, sizeof(double) );

  if( nN >= maxNB )
  {
      maxNB += nN;
      Nb = (double *)realloc(Nb, maxNB * sizeof(double) );
  }

  cSpl_BasisFuns(at, u, span, nN, Nb) ;

  p->x = 0;
  p->y = 0;

  for( i = at->dg; i >= 0; --i ) 
  {
     p->x += Nb[i] * at->ctrlpts[span - at->dg + i].x ;
     p->y += Nb[i] * at->ctrlpts[span - at->dg + i].y ;
  }

  return (1);
}

/* ========================  cSpl_Evaluate2Pt  ======================= */
/*
** Given the parametric value of a point on the curve, it returns the 
** cartesian coordinates of that point on the curve and the first 
** derivative at that point.
*/
static int cSpl_Evaluate2Pt( CllSpline* at, double u, CllPnt* p, CllPnt* pl )
{
  int i, k;
  int d = 1;
  double** Bin ;
  CllPnt3d v ;
  CllPnt3d* dersW;
  CllPnt3d* ders;
  
  cSpl_Pnt3dConstr(&v);
  
  dersW = (CllPnt3d *) calloc(d+1, sizeof(CllPnt3d)) ;
  for (i = 0; i < d+1; i++)
   cSpl_Pnt3dConstr(&dersW[i]);

  cSpl_DeriveAtH(at, u, d, dersW) ;

  Bin = (double **) calloc(d+1, sizeof(double *));

  for( i = 0; i < d+1; i++ )
       Bin[i] = (double *) calloc(d+1, sizeof(double));

  cSpl_BinomialCoef( d+1, Bin ) ;

  ders = (CllPnt3d *) calloc(d+1, sizeof(CllPnt3d)) ;
  for (i = 0; i < d+1; i++)
   cSpl_Pnt3dConstr(&ders[i]);

  /* Compute the derivative at the parmeter u */
  for(k = 0; k <= d; k++)
  {
    v.x = dersW[k].x ;
    v.y = dersW[k].y ;
    v.w = dersW[k].w ;

    for(i = k ; i > 0 ; --i)
    {
        v.x -= Bin[k][i] * dersW[i].w * ders[k-i].x ;
        v.y -= Bin[k][i] * dersW[i].w * ders[k-i].y ;
        v.w -= Bin[k][i] * dersW[i].w * ders[k-i].w ;
    }

    ders[k] = v ;
    ders[k].x /= dersW[0].w ;
    ders[k].y /= dersW[0].w ;
    ders[k].w /= dersW[0].w ;
  }

  p->x  = ders[0].x;
  p->y  = ders[0].y;
  pl->x = ders[1].x;
  pl->y = ders[1].y;

  for( i = 0; i < d+1; i++ ) free( Bin[i] );

  free( Bin );
  free( dersW );
  free( ders );
  
  return (1);
}

/* ======================   cSpl_NurbsBasisFuns   ======================= */

static void cSpl_NurbsBasisFuns( double u, int span, int deg, 
                                 double* U, double* N )
{
  int j, r;
  double* left  = (double*)malloc( 2*(deg+1)*sizeof(double) ) ;
  double* right = &left[deg+1] ;

  double temp, saved ;

  N[0] = 1.0 ;

  for(j=1; j <= deg ; j++)
  {
    left[j]  = u-U[span+1-j] ;
    right[j] =   U[span+j]-u ;

    saved = 0.0 ;

    for(r=0 ; r<j; r++)
    {
      temp  = N[r]/(right[r+1]+left[j-r]) ;
      N[r]  = saved+right[r+1] * temp ;
      saved = left[j-r] * temp ;
    }

    N[j] = saved ;
  }

  free(left);  /* NOVO FREE */
}

/* ======================   cSpl_FirstD   =========================== */

static CllPnt3d cSpl_FirstD( CllSpline* at, double u, int span )
{
  int i, j ;
  double* N = (double *)calloc(at->dg + 1, sizeof(double));
  CllPnt3d Cd ;
  CllPnt3d Qi ;

  cSpl_NurbsBasisFuns( u, span, at->dg - 1, at->knot, N ) ;

  Cd.x = 0;
  Cd.y = 0;
  Cd.w = 0;

  for(i = at->dg - 1; i >= 0; --i)
  {
    j = span - at->dg + i ;

    Qi.x = (at->ctrlpts[j+1].x - at->ctrlpts[j].x);
    Qi.y = (at->ctrlpts[j+1].y - at->ctrlpts[j].y);
    Qi.w = 0;

    Qi.x *= at->dg / (at->knot[j+at->dg+1] - at->knot[j+1]) ;
    Qi.y *= at->dg / (at->knot[j+at->dg+1] - at->knot[j+1]) ;
    Qi.w *= at->dg / (at->knot[j+at->dg+1] - at->knot[j+1]) ;

    Cd.x += N[i] * Qi.x ;
    Cd.y += N[i] * Qi.y ;
    Cd.w += N[i] * Qi.w ;
  }

  free( N ) ;

  return Cd ;
}

/* ========================   cSpl_HpointAt   ============================ */

static CllPnt3d cSpl_HpointAt( CllSpline* at, double u, int span )
{
  int i;
  int     nN = at->dg + 1;
  double* Nb = (double *)calloc( nN, sizeof(double) );
  CllPnt3d p ;

  cSpl_BasisFuns(at, u, span, nN, Nb) ;

  p.x = 0;
  p.y = 0;
  p.w = 0;

  for(i = at->dg; i >= 0; --i)
  {
      p.x += Nb[i] * at->ctrlpts[span-at->dg+i].x ;
      p.y += Nb[i] * at->ctrlpts[span-at->dg+i].y ;
      p.w += Nb[i] * 1 ;
  }

  free( Nb );

  return p ;
}

/* ========================   cSpl_FirstDn   ============================ */

static CllPnt cSpl_FirstDn( CllSpline* at, double u, int span )
{
  CllPnt3d Cd ;
  CllPnt   pd ;
  double w;

  Cd = cSpl_FirstD(at, u, span) ;

  pd.x =  Cd.x ;
  pd.y =  Cd.y ;

  w = Cd.w ;

  Cd = cSpl_HpointAt(at, u, span) ;

  pd.x -= w * Cd.x/Cd.w ;
  pd.y -= w * Cd.y/Cd.w ;

  pd.x /= Cd.w ;
  pd.y /= Cd.w ;

  return pd ;
}

/* ========================   cSpl_LengthF   ============================ */

static double cSpl_LengthF( CllSpline* at, double u, int span )
{
  CllPnt dd = cSpl_FirstDn( at, u, span ) ;

  double tmp = sqrt( dd.x*dd.x + dd.y*dd.y ) ;
  return tmp ;
}

/* ==========================   cSpl_Intccini   ======================== */

static void cSpl_Intccini( int nw, double* w )
{
  int lenw = nw - 1 ;
  int j;
  int m;

  double cos2 = 0;
  double sin1 = 1;
  double sin2 = 1;
  double hl   = 0.5;

  int k = lenw;
  int l = 2;

  while( l < k - l - 1 )
  {
    w[0] = hl * 0.5;

    for( j = 1; j <= l; j++ ) w[j] = hl / (1 - 4 * j * j);

    w[l] *= 0.5;
    cSpl_Dfct(l, 0.5 * cos2, sin1, w);

    cos2  = sqrt(2 + cos2);
    sin1 /= cos2;
    sin2 /= 2 + cos2;

    w[k] = sin2;
    w[k - 1] = w[0];
    w[k - 2] = w[l];

    k -= 3;
    m  = l;

    while( m > 1 )
    {
      m >>= 1;

      for( j = m; j <= l - m; j += (m << 1) )
      {
	w[k] = w[j];
	k--;
      }
    }

    hl *= 0.5;
    l  *= 2;
  }
}

/* ========================   cSpl_Intcc   ============================ */

static double cSpl_Intcc( CllSpline* at, int f, double a, double b, double eps, 
                          int nw, double* w, double* err )
{
  int j;
  int k;
  int l;
  int lenw = nw - 1 ;

  double erefh, hh, iback, irback, y, fx, errir;
  double eref, ir, i;

  double esf = 10;
  double ba  = 0.5 * (b - a);
  double ss  = 2   * w[lenw];
  double x   = ba  * w[lenw];

  w[0] = 0.5 * cSpl_LengthF(at, a, f);
  w[3] = 0.5 * cSpl_LengthF(at, b, f);
  w[2] = 1.0 * cSpl_LengthF(at, a + x,  f);
  w[4] = 1.0 * cSpl_LengthF(at, b - x,  f);
  w[1] = 1.0 * cSpl_LengthF(at, a + ba, f);

  eref = 0.5 * ( fabs(w[0]) + fabs(w[1]) + fabs(w[2]) + fabs(w[3]) + 
                 fabs(w[4]) );

  w[0] += w[3];
  w[2] += w[4];

  ir = w[0] + w[1] + w[2];
  i  = w[0] * w[lenw - 1] + w[1] * w[lenw - 2] + w[2] * w[lenw - 3];

  erefh = eref * sqrt(eps);
  eref *= eps;

  hh = 0.25;
  l  = 2;
  k  = lenw - 5;

  do {
    iback = i;
    irback = ir;
    x = ba * w[k + 1];
    y = 0;
    i = w[0] * w[k];

    for( j = 1; j <= l; j++ )
    {
      x += y;
      y += ss * (ba - x);
      fx = cSpl_LengthF(at, a + x, f) + cSpl_LengthF(at, b - x, f);
      ir += fx;
      i += w[j] * w[k - j] + fx * w[k - j - l];
      w[j + l] = fx;
    }

    ss  = 2 * w[k + 1];
    *err = esf * l * fabs(i - iback);
    hh *= 0.25;
    errir = hh * fabs(ir - 2 * irback);
    l *= 2;
    k -= l + 2;
  } while( (*err > erefh || errir > eref) && k > 4 * l );

  i *= b - a;

  if( *err > erefh || errir > eref )
      *err *= -fabs(b - a);
  else
      *err = eref * fabs(b - a);

  return i ;
}

/* ========================   cSpl_Dfct   ============================= */

static void cSpl_Dfct( int n, double wr, double wi, double *a )
{
    int j, k, m, mh;
    double xr, xi, an;

    m = n >> 1;

    for( j = 0; j <= m - 1; j++ )
    {
        k = n - j;
        xr    = a[j] + a[k];
        a[j] -= a[k];
        a[k]  = xr;
    }

    an = a[n];

    while( m >= 2 )
    {
        cSpl_Ddct(m, wr, wi, a);
        xr = 1 - 2 * wi * wi;
        wi *= 2 * wr;
        wr = xr;
        cSpl_Bitrv(m, a);
        mh = m >> 1;
        xi = a[m];
        a[m] = a[0];
        a[0] = an - xi;
        an += xi;

        for( j = 1; j <= mh - 1; j++ )
        {
            k = m - j;
            xr = a[m + k];
            xi = a[m + j];
            a[m + j] = a[j];
            a[m + k] = a[k];
            a[j] = xr - xi;
            a[k] = xr + xi;
        }

        xr = a[mh];
        a[mh] = a[m + mh];
        a[m + mh] = xr;
        m = mh;
    }

    xi = a[1];
    a[1] = a[0];
    a[0] = an + xi;
    a[n] = an - xi;
    cSpl_Bitrv(n, a);
}

/* ===========================   cSpl_Ddct   ============================ */

static void cSpl_Ddct( int n, double wr, double wi, double *a )
{
    int j, k, m;
    double wkr, wki, wdr, wdi, ss, xr;

    if( n > 2 )
    {
        wkr = 0.5;
        wki = 0.5;
        wdr = 0.5 * (wr - wi);
        wdi = 0.5 * (wr + wi);

        ss = 2 * wi;
        if (wi < 0)
        {
            xr = a[n - 1];
            for (k = n - 2; k >= 2; k -= 2)
            {
                a[k + 1] = a[k] - a[k - 1];
                a[k] += a[k - 1];
            }

            a[1] = 2 * xr;
            a[0] *= 2;
            cSpl_Rdft(n, 1 - ss * wi, ss * wr, a);
            xr = wdr;
            wdr = wdi;
            wdi = xr;
            ss = -ss;
        }

        m = n >> 1;
        for (k = 1; k <= m - 3; k += 2)
        {
            j = n - k;
            xr = wdi * a[k] - wdr * a[j];
            a[k] = wdr * a[k] + wdi * a[j];
            a[j] = xr;
            wkr -= ss * wdi;
            wki += ss * wdr;
            xr = wki * a[k + 1] - wkr * a[j - 1];
            a[k + 1] = wkr * a[k + 1] + wki * a[j - 1];
            a[j - 1] = xr;
            wdr -= ss * wki;
            wdi += ss * wkr;
        }

        k = m - 1;
        j = n - k;
        xr = wdi * a[k] - wdr * a[j];
        a[k] = wdr * a[k] + wdi * a[j];
        a[j] = xr;
        a[m] *= wki + ss * wdr;
        if (wi >= 0)
        {
            cSpl_Rdft(n, 1 - ss * wi, ss * wr, a);
            xr = a[1];
            for (k = 2; k <= n - 2; k += 2) {
                a[k - 1] = a[k] - a[k + 1];
                a[k] += a[k + 1];
            }
            a[n - 1] = xr;
        }
    }
    else
    {
        if (wi >= 0) {
            xr = 0.5 * (wr + wi) * a[1];
            a[1] = a[0] - xr;
            a[0] += xr;
        } else {
            xr = a[0] - a[1];
            a[0] += a[1];
            a[1] = 0.5 * (wr - wi) * xr;
        }
    }
}

/* ==========================   cSpl_Bitrv2   ============================ */

static void cSpl_Bitrv2(int n, double* a)
{
    int j, j1, k, k1, l, m, m2, n2;
    double xr, xi;

    m = n >> 2;
    m2 = m << 1;
    n2 = n - 2;
    k = 0;
    for (j = 0; j <= m2 - 4; j += 4) {
        if (j < k) {
            xr = a[j];
            xi = a[j + 1];
            a[j] = a[k];
            a[j + 1] = a[k + 1];
            a[k] = xr;
            a[k + 1] = xi;
        } else if (j > k) {
            j1 = n2 - j;
            k1 = n2 - k;
            xr = a[j1];
            xi = a[j1 + 1];
            a[j1] = a[k1];
            a[j1 + 1] = a[k1 + 1];
            a[k1] = xr;
            a[k1 + 1] = xi;
        }
        k1 = m2 + k;
        xr = a[j + 2];
        xi = a[j + 3];
        a[j + 2] = a[k1];
        a[j + 3] = a[k1 + 1];
        a[k1] = xr;
        a[k1 + 1] = xi;
        l = m;
        while (k >= l) {
            k -= l;
            l >>= 1;
        }
        k += l;
    }
}

/* ==========================   cSpl_Cdft   ============================ */

static void cSpl_Cdft(int n, double wr, double wi, double* a)
{
    int i, j, k, l, m;
    double wkr, wki, wdr, wdi, ss, xr, xi;

    m = n;
    while (m > 4) {
        l = m >> 1;
        wkr = 1;
        wki = 0;
        wdr = 1 - 2 * wi * wi;
        wdi = 2 * wi * wr;
        ss = 2 * wdi;
        wr = wdr;
        wi = wdi;
        for (j = 0; j <= n - m; j += m) {
            i = j + l;
            xr = a[j] - a[i];
            xi = a[j + 1] - a[i + 1];
            a[j] += a[i];
            a[j + 1] += a[i + 1];
            a[i] = xr;
            a[i + 1] = xi;
            xr = a[j + 2] - a[i + 2];
            xi = a[j + 3] - a[i + 3];
            a[j + 2] += a[i + 2];
            a[j + 3] += a[i + 3];
            a[i + 2] = wdr * xr - wdi * xi;
            a[i + 3] = wdr * xi + wdi * xr;
        }
        for (k = 4; k <= l - 4; k += 4) {
            wkr -= ss * wdi;
            wki += ss * wdr;
            wdr -= ss * wki;
            wdi += ss * wkr;
            for (j = k; j <= n - m + k; j += m) {
                i = j + l;
                xr = a[j] - a[i];
                xi = a[j + 1] - a[i + 1];
                a[j] += a[i];
                a[j + 1] += a[i + 1];
                a[i] = wkr * xr - wki * xi;
                a[i + 1] = wkr * xi + wki * xr;
                xr = a[j + 2] - a[i + 2];
                xi = a[j + 3] - a[i + 3];
                a[j + 2] += a[i + 2];
                a[j + 3] += a[i + 3];
                a[i + 2] = wdr * xr - wdi * xi;
                a[i + 3] = wdr * xi + wdi * xr;
            }
        }
        m = l;
    }
    if (m > 2) {
        for (j = 0; j <= n - 4; j += 4) {
            xr = a[j] - a[j + 2];
            xi = a[j + 1] - a[j + 3];
            a[j] += a[j + 2];
            a[j + 1] += a[j + 3];
            a[j + 2] = xr;
            a[j + 3] = xi;
        }
    }
    if (n > 4) {
        cSpl_Bitrv2(n, a);
    }
}

/* ==========================   cSpl_Rdft   ============================ */

static void cSpl_Rdft(int n, double wr, double wi, double* a)
{
    int j, k;
    double wkr, wki, wdr, wdi, ss, xr, xi, yr, yi;

    if (n > 4) {
        wkr = 0;
        wki = 0;
        wdr = wi * wi;
        wdi = wi * wr;
        ss = 4 * wdi;
        wr = 1 - 2 * wdr;
        wi = 2 * wdi;
        if (wi >= 0) {
            cSpl_Cdft(n, wr, wi, a);
            xi = a[0] - a[1];
            a[0] += a[1];
            a[1] = xi;
        }
        for (k = (n >> 1) - 4; k >= 4; k -= 4) {
            j = n - k;
            xr = a[k + 2] - a[j - 2];
            xi = a[k + 3] + a[j - 1];
            yr = wdr * xr - wdi * xi;
            yi = wdr * xi + wdi * xr;
            a[k + 2] -= yr;
            a[k + 3] -= yi;
            a[j - 2] += yr;
            a[j - 1] -= yi;
            wkr += ss * wdi;
            wki += ss * (0.5 - wdr);
            xr = a[k] - a[j];
            xi = a[k + 1] + a[j + 1];
            yr = wkr * xr - wki * xi;
            yi = wkr * xi + wki * xr;
            a[k] -= yr;
            a[k + 1] -= yi;
            a[j] += yr;
            a[j + 1] -= yi;
            wdr += ss * wki;
            wdi += ss * (0.5 - wkr);
        }
        j = n - 2;
        xr = a[2] - a[j];
        xi = a[3] + a[j + 1];
        yr = wdr * xr - wdi * xi;
        yi = wdr * xi + wdi * xr;
        a[2] -= yr;
        a[3] -= yi;
        a[j] += yr;
        a[j + 1] -= yi;
        if (wi < 0) {
            a[1] = 0.5 * (a[0] - a[1]);
            a[0] -= a[1];
            cSpl_Cdft(n, wr, wi, a);
        }
    } else {
        if (wi < 0) {
            a[1] = 0.5 * (a[0] - a[1]);
            a[0] -= a[1];
        }
        if (n > 2) {
            xr = a[0] - a[2];
            xi = a[1] - a[3];
            a[0] += a[2];
            a[1] += a[3];
            a[2] = xr;
            a[3] = xi;
        }
        if (wi >= 0) {
            xi = a[0] - a[1];
            a[0] += a[1];
            a[1] = xi;
        }
    }
}

/* ==========================   cSpl_Bitrv   ============================ */

static void cSpl_Bitrv(int n, double* a)
{
    int j, k, l, m, m2, n1;
    double xr;

    if (n > 2) {
        m = n >> 2;
        m2 = m << 1;
        n1 = n - 1;
        k = 0;
        for (j = 0; j <= m2 - 2; j += 2) {
            if (j < k) {
                xr = a[j];
                a[j] = a[k];
                a[k] = xr;
            } else if (j > k) {
                xr = a[n1 - j];
                a[n1 - j] = a[n1 - k];
                a[n1 - k] = xr;
            }
            xr = a[j + 1];
            a[j + 1] = a[m2 + k];
            a[m2 + k] = xr;
            l = m;
            while (k >= l) {
                k -= l;
                l >>= 1;
            }
            k += l;
        }
    }
}

/* ============================ cSpl_FindMult ============================= */

static int cSpl_FindMult( CllSpline* at, int r )
{
  int i;
  int s = 1 ;

  for( i = r; i > at->dg + 1; --i )
  {
    if( at->knot[i] <= at->knot[i-1] )
        s++ ;
    else
        return s ;
  }

  return s ;
}

/* =======================   cSpl_LengthIn   ========================== */

static double cSpl_LengthIn( CllSpline* at, double us, double ue )
{
  double eps = 0.001;
  int    n = 100;
  int    i;

  double l = 0.0 ;
  double err ;

  static double* bufFcn = NULL;

  if( bufFcn == NULL )
      bufFcn = (double *)calloc( n, sizeof(double) );

  cSpl_Intccini( n, bufFcn ) ;

  for( i = at->dg; i < at->np; ++i )
  {
    if( at->knot[i] >= at->knot[i+1] )
        continue ;

    if( i < cSpl_FindSpan( at, us ) )
        continue ;

    if( us >= at->knot[i] && ue <= at->knot[i + cSpl_FindMult(at, i)] )
    {
      l = cSpl_Intcc( at, i, us, ue, eps, n, bufFcn, &err );
      break ;
    }

    if( us >= at->knot[i] )
    {
      l += cSpl_Intcc( at, i, us, at->knot[i+cSpl_FindMult(at, i)], eps, n, 
	               bufFcn, &err ) ;
      continue ;
    }

    if( ue <= at->knot[i+cSpl_FindMult(at, i)] )
    {
      l += cSpl_Intcc( at, i, at->knot[i], ue, eps, n, bufFcn, &err ) ;
      break ;
    }

    l += cSpl_Intcc( at, i, at->knot[i], at->knot[i+cSpl_FindMult(at, i)], eps, 
                     n, bufFcn, &err ) ;
  }

  return l ;
}

/* ============================= isColinear ============================== */

static int isColinear( CllPnt p0, CllPnt pm, CllPnt p1 )
{
  CllPnt u;
  CllPnt v;
  double umod;
  double vmod;
  double ang;
  double tol_ang; 
  
  umod = LEN(p0, p1); 
  u.x = (p0.x - p1.x) / umod;
  u.y = (p0.y - p1.y) / umod;

  vmod = LEN(pm, p1);
  v.x = (pm.x - p1.x) / vmod;
  v.y = (pm.y - p1.y) / vmod;

  ang = acos(u.x * v.x + u.y * v.y);
  tol_ang = PI / 180.0;

  return ( fabs(ang) < tol_ang );
}

/* ============================ cSpl_Straight ============================= */

static int cSpl_Straight( CllSpline* at, CllPnt p0, CllPnt pm, CllPnt p1,
                          double t0, double tm, double t1)
{
  double lu,lv,len,crvlen;
  CllPnt  u,v;

  if( !isColinear( p0, pm, p1 ) )
   return 0;

  u.x = pm.x - p0.x;
  u.y = pm.y - p0.y;
  lu  = sqrt(u.x*u.x + u.y*u.y);

  v.x = p1.x - pm.x;
  v.y = p1.y - pm.y;
  lv  = sqrt(v.x*v.x + v.y*v.y);
  len = lu + lv;
  crvlen = cSpl_LengthIn(at, t0, t1);

  if( fabs(crvlen - len) > _dTol )
   return 0;

  return 1;
}

/* =========================== cSpl_DrawLine ============================== */

static void cSpl_DrawLine(double x0, double y0, double x1, double y1) 
{
  wdLine(x0,y0,x1,y1);
}

/* ======================= cSpl_DrawSplineRecSub ========================== */

static void cSpl_DrawSplineRecSub(CllSpline* c, double t0, double tm, double t1,
                                  CllPnt p0, CllPnt pm, CllPnt p1)
{
  if ((cSpl_Straight(c,p0,pm,p1,t0,tm,t1)) || (c->numpts == 2))
  {
    cSpl_DrawLine(p0.x,p0.y,p1.x,p1.y);
  }
  else
  {  
    double tml,tmr;
    CllPnt pml,pmr;

    /* Evaluate new mid points */
 
    tml = (t0 + tm)/2.0; 
    cSpl_Evaluate1Pt(c, tml, &pml);
 
    tmr = (tm + t1)/2.0; 
    cSpl_Evaluate1Pt(c, tmr, &pmr);
    
    /* Transform left and right splines (recursion) */
 
    cSpl_DrawSplineRecSub(c,t0,tml,tm,p0,pml,pm);
    cSpl_DrawSplineRecSub(c,tm,tmr,t1,pm,pmr,p1);
  }
}

/* ============================== cSpl_TangVec ============================== */

/* Calculate the unit tangent vector in spline initial and final points. */

static void cSpl_TangVec( CllSpline* at, CllPnt *p0a, CllPnt *p0b,
                          CllPnt *p1a, CllPnt *p1b )
{
  double len;
  CllPnt pc0, pc1;
  CllPnt t0, t1;
  double nder0, nder1;
  double xmin, xmax;

  wdGetWindow(&xmin, &xmax, NULL, NULL);
  
  cSpl_Evaluate2Pt(at, 0.0, &pc0, &t0);
  cSpl_Evaluate2Pt(at, 1.0, &pc1, &t1);
  
  len = 0.05 * (xmax - xmin);
  
  nder0 = sqrt(t0.x * t0.x + t0.y * t0.y); 
  nder1 = sqrt(t1.x * t1.x + t1.y * t1.y); 
  
  t0.x /= nder0;
  t0.y /= nder0;
  t1.x /= nder1;
  t1.y /= nder1;
  
  p0a->x = pc0.x - len * t0.x;
  p0a->y = pc0.y - len * t0.y;
  p0b->x = pc0.x + len * t0.x;
  p0b->y = pc0.y + len * t0.y;
  p1a->x = pc1.x - len * t1.x;
  p1a->y = pc1.y - len * t1.y;
  p1b->x = pc1.x + len * t1.x;
  p1b->y = pc1.y + len * t1.y;
}

/* =========================== cSpl_DrawTangVec ============================ */

static void cSpl_DrawTangVec(CllSpline* at, int clr)
{
  CllPnt p0a, p0b, p1a, p1b;
  long int forecolor;
  int mark_type, mark_size; 
  
  cSpl_TangVec(at, &p0a, &p0b, &p1a, &p1b);
  
  if (!clr)
   forecolor = cdForeground(CD_BLUE);
  mark_type = cdMarkType( CD_BOX );
  mark_size = cdMarkSize( 4 );
  
  wdLine(p0a.x, p0a.y, p0b.x, p0b.y);
  wdLine(p1a.x, p1a.y, p1b.x, p1b.y);
  wdMark(p0a.x, p0a.y);
  wdMark(p0b.x, p0b.y);
  wdMark(p1a.x, p1a.y);
  wdMark(p1b.x, p1b.y);
  
  if (!clr)
   cdForeground(forecolor);
  cdMarkType(mark_type);
  cdMarkSize(mark_size);
}

/* ============================== cSpl_Draw =============================== */

static void cSpl_Draw(CllSpline* c, int clr, int tg)
{
  double t0,t1,tm;
  CllPnt p0,p1,pm;
  double crvlen;

  /* Evaluate initial data */

  t0 = c->knot[0];
  t1 = c->knot[c->nu-1];
  tm = (t0+t1)/2.0;

  p0 = c->keypts[0];
  p1 = c->keypts[c->numpts-1];
  cSpl_Evaluate1Pt(c, tm, &pm);

  crvlen = cSpl_LengthIn( c, t0, t1 );

  _dTol = 0.0001 * crvlen;

  /* Draw the spline using recursion */

  cSpl_DrawSplineRecSub(c,t0,tm,t1,p0,pm,p1);

  if (tg)
   cSpl_DrawTangVec(c, clr);
}

/* =========================  cSpl_Clr  ========================= */

static void cSpl_Clr( CllSplineP *spline )
{
 long int  fore_color;
 long int  back_color;
 
 if( spline->spl.numpts < 2 ) return;

 back_color = cdBackground( CD_BLACK );
 fore_color = cdForeground( back_color );
 cSpl_Draw(&spline->spl, 1, 0);
 cdForeground( fore_color );
 cdBackground( back_color );

 spline->displayed = 0;
}

/* =========================  cSpl_Dsp  ========================= */

static void cSpl_Dsp( void *l )
{
 CllSplineP *spline = (CllSplineP *)l;
 long int  fore_color;
 
 if( spline->spl.numpts < 2 ) return; 
 
 cSpl_Init(&spline->spl, spline->spl.numpts, 2);

 fore_color = cdForeground( CD_RED );
 cSpl_Draw(&spline->spl, 0, 0);

 cdForeground( fore_color );

 spline->displayed = 1;
}

/* ========================  cSpl_Snap  ========================= */
 
static void cSpl_Snap( PckDesc *pk, double *x, double *y, double tol )
{
/* Restore registered collection spec. from the data field.
 */
 CllSplineP  *spline = (CllSplineP *)pk->data;
 
/* If shift key is pressed and first spline point has been given,
 * snap current line segment to the horizontal axis, to the vertical 
 * axis, or the bi-sector lines.
 */
 if( (spline->spl.numpts > 0) && isshift(pk->mousecfg) )
 {
  CllShiftSnap( spline->spl.keypts[spline->spl.numpts-1].x, 
                spline->spl.keypts[spline->spl.numpts-1].y, x, y );
 }

/* Check to see whether client had registered a snap function and 
 * call it passing the given mouse position and parameters.
 */
 if( spline->snap != 0L )
  (*spline->snap )( spline->iup_cv, x, y, tol,
                    isshift(pk->mousecfg), iscontrol(pk->mousecfg) );
}
 

/* ========================  cSpl_Echo  ========================= */
 
static void cSpl_Echo( PckDesc *pk, double x, double y )
{
 double len;
 char   echotext[60];

/* Restore registered collection spec. from the data field.
 */
 CllSplineP  *spline = (CllSplineP *)pk->data;
 
/* If first point was already given, echo last spline segment length.
 * Check to see if this is a polygon collection.
 * If that is the case, draws the spline closing off the 
 * current polyline in xor mode.
 */
 if( spline->spl.numpts > 0 )
 {
  len = sqrt( (spline->spl.keypts[spline->spl.numpts-1].x-x)*
              (spline->spl.keypts[spline->spl.numpts-1].x-x) +
              (spline->spl.keypts[spline->spl.numpts-1].y-y) *
              (spline->spl.keypts[spline->spl.numpts-1].y-y) );

  if( spline->created )
  {
   if( spline->modified || !spline->displayed )
   {
    cSpl_DisplayXor( spline );
   }
  }

  if(!spline->created)
  {
   if( iup_english )
    sprintf( echotext, "Segment length = %10.3f", len );
   else
    sprintf( echotext, "Comprimento do segmento = %10.3f", len );
   if( spline->prompt != NULL )
    (*spline->prompt)( spline->iup_cv, echotext ) ;
  }
 }

/* Check to see whether client had registered an echo function.
 * If that is the case call it passing the given mouse position.
 */
 if( spline->echo != NULL )
  (*spline->echo )( spline->iup_cv, x, y ) ;
}
 
/* ======================  cSpl_DisplayXor  ==================== */

static void cSpl_DisplayXor( CllSplineP *spline )
{
 int    write_mode;
 int    fore_color;
 int    mark_type;
 int    mark_size;
 int    line_style;
 int    line_width;

#ifdef _UNIX_
 write_mode = cdWriteMode( CD_XOR );
#else
 write_mode = cdWriteMode( CD_NOT_XOR );
#endif
 fore_color = cdForeground( CD_RED );
 mark_type = cdMarkType( CD_BOX );
 mark_size = cdMarkSize( 4 );
 line_style = cdLineStyle( CD_CONTINUOUS );
 line_width = cdLineWidth( 1 );
 
 if (spline->spl.numpts > 1)
  cSpl_Draw( &spline->spl, 0, 1 );

 cdLineWidth( line_width );
 cdLineStyle( line_style );
 cdMarkSize( mark_size );
 cdMarkType( mark_type );
 cdForeground( fore_color );
 cdWriteMode( write_mode );

 spline->displayed ^= 1;
} 

/* ====================  cSpl_BtDwnCb  ====================== */

static void cSpl_BtDwnCb( PckDesc *pk )
{
 CllPnt p0a, p0b, p1a, p1b;
 CllSplineP *spline = (CllSplineP *)pk->data;
 void (*end_action)( Ihandle *iup_cv, int status ) = spline->end_action;

 if ( spline->displayed )
  cSpl_DisplayXor( spline );

 if( pk->button == 2 )
 {
  /* If middle button was pressed, the user wants to enter the
   * spline end tangents using the keyboard.
  */
  int status;
  CllPnt pi, pli;
  CllPnt pf, plf;
  CllPnt fder, lder;
  float fderx, fdery;
  float lderx, ldery;
   
  cSpl_Evaluate2Pt( &spline->spl, 0.0, &pi, &pli );
  cSpl_Evaluate2Pt( &spline->spl, 1.0, &pf, &plf );
   
  fderx = pli.x;
  fdery = pli.y;
  lderx = plf.x;
  ldery = plf.y;
  
  if (DlgCaptureDerSpline(&fderx, &fdery, &lderx, &ldery))
  {
   fder.x = (double)fderx;
   fder.y = (double)fdery;
   lder.x = (double)lderx;
   lder.y = (double)ldery;
  
   cSpl_Clr( spline );
   
   cSpl_SetEndTan(&spline->spl, &fder, &lder);
  
   if( spline->put != 0L )
    (*spline->put )( spline->iup_cv, &spline->spl, pk->toler );
   status = CLL_NORMAL;
   cSpl_Delete( spline );
   if( end_action ) (*end_action)( spline->iup_cv, status );
   return;
  }
 }
 
 cSpl_TangVec(&spline->spl, &p0a, &p0b, &p1a, &p1b);

 if( CllPickPoint( p0a.x, p0a.y, pk->p0.x, pk->p0.y, pk->toler ) )
  pk->echofunc = cSpl_P0aCrtEcho;
 else if( CllPickPoint( p0b.x, p0b.y, pk->p0.x, pk->p0.y, pk->toler ) )
  pk->echofunc = cSpl_P0bCrtEcho;
 else if( CllPickPoint( p1a.x, p1a.y, pk->p0.x, pk->p0.y, pk->toler ) )
  pk->echofunc = cSpl_P1aCrtEcho;
 else if( CllPickPoint( p1b.x, p1b.y, pk->p0.x, pk->p0.y, pk->toler ) )
  pk->echofunc = cSpl_P1bCrtEcho;
 else
 { 
  spline->modified = 0;
  pk->echofunc = cSpl_Echo;
 }
} 


/* ====================  cSpl_BtUpCb  ====================== */

static void cSpl_BtUpCb( PckDesc *pk )
{
 CllSplineP *spline = (CllSplineP *)pk->data;
 void       *iup_cv = spline->iup_cv;
 void       (*end_action)( Ihandle *iup_cv, int status ) = spline->end_action;

/* If user gave up reshaping the curve, call the client
 * function to put the collected curve and quit collection.
 * If collection was interupted, just quit collection.
 * In either case call end action function and return.
 */
 if( pk->status != PCK_VALID )
 {
  int status;

  CllClose( );
  
  if ( spline->displayed )
   cSpl_DisplayXor( spline );
   
  switch( pk->status )
  {
   default:
   case PCK_QUIT:
    if( spline->put != 0L )
     (*spline->put )( spline->iup_cv, &spline->spl, pk->toler );
    status = CLL_NORMAL;
    break;
   case PCK_INTERUPTED:
    status = CLL_INTERUPTED;
    break;
  }
  cSpl_Delete( spline );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Keep getting picks from the user.
 */
 
 spline->modified = 0;
 
 if( spline->prompt != 0L )
  (*spline->prompt )( spline->iup_cv, prompt_msg2 ) ;
  
 pk->button = 1;       /* accept only mouse button 1 */
 pk->echofunc = cSpl_Echo;
 PckSetup( pk );
}	 
  
 
/* ====================  cSpl_P0aCrtEcho  ====================== */

static void cSpl_P0aCrtEcho( PckDesc *pk, double echoptx, double echopty )
{
 CllPnt vtan, p, pl;	
 CllSplineP *spline = (CllSplineP *)pk->data;
 
 vtan.x = (spline->spl.keypts[0].x - echoptx) * 10.0;  
 vtan.y = (spline->spl.keypts[0].y - echopty) * 10.0;  
 cSpl_Evaluate2Pt(&spline->spl, 1.0, &p, &pl );
 cSpl_SetEndTan(&spline->spl, &vtan, &pl);
 spline->modified = 1;
 cSpl_Echo( pk, echoptx, echopty );
}  
 
/* ====================  cSpl_P0bCrtEcho  ====================== */

static void cSpl_P0bCrtEcho( PckDesc *pk, double echoptx, double echopty )
{
 CllPnt vtan, p, pl;		
 CllSplineP *spline = (CllSplineP *)pk->data;
 
 vtan.x = (echoptx - spline->spl.keypts[0].x) * 10.0;
 vtan.y = (echopty - spline->spl.keypts[0].y) * 10.0;
 cSpl_Evaluate2Pt(&spline->spl, 1.0, &p, &pl );
 cSpl_SetEndTan(&spline->spl, &vtan, &pl);
 spline->modified = 1;
 cSpl_Echo( pk, echoptx, echopty );
}

/* ====================  cSpl_P1aCrtEcho  ====================== */

static void cSpl_P1aCrtEcho( PckDesc *pk, double echoptx, double echopty )
{
 CllPnt vtan, p, pl;		
 CllSplineP *spline = (CllSplineP *)pk->data;

 vtan.x = (spline->spl.keypts[spline->spl.numpts - 1].x - echoptx) * 10.0;  
 vtan.y = (spline->spl.keypts[spline->spl.numpts - 1].y - echopty) * 10.0;  
 cSpl_Evaluate2Pt(&spline->spl, 0.0, &p, &pl );
 cSpl_SetEndTan(&spline->spl, &pl, &vtan);
 spline->modified = 1;
 cSpl_Echo( pk, echoptx, echopty );
}

/* ====================  cSpl_P1bCrtEcho  ====================== */

static void cSpl_P1bCrtEcho( PckDesc *pk, double echoptx, double echopty )
{
 CllPnt vtan, p, pl;		
 CllSplineP *spline = (CllSplineP *)pk->data;

 vtan.x = (echoptx - spline->spl.keypts[spline->spl.numpts - 1].x) * 10.0;  
 vtan.y = (echopty - spline->spl.keypts[spline->spl.numpts - 1].y) * 10.0;  
 cSpl_Evaluate2Pt(&spline->spl, 0.0, &p, &pl );
 cSpl_SetEndTan(&spline->spl, &pl, &vtan);
 spline->modified = 1;
 cSpl_Echo( pk, echoptx, echopty );
} 
 
/* ===========================  cSpl_Pt  ======================== */

static void cSpl_Pt( PckDesc *pk )
{
 static int flag = 1;
 int i;
 CllPnt aux[4]; 	
 CllSplineP *spline = (CllSplineP *)pk->data;
 void       *iup_cv = spline->iup_cv;
 void      (*end_action)( Ihandle *iup_cv, int status ) = spline->end_action;

/* If user gave up adding points to spline, check to see
 * if collected spline is valid.  If it is, call the client 
 * function to put the collected spline.
 * If collection was interupted, just quit collection.
 * In either case call end action function and return.
 */
 if( pk->status != PCK_VALID )
 {
  int status;

  flag = 1;
  
  if( (pk->status != PCK_QUIT) )
  { 
   CllClose( );
   cSpl_Clr( spline );
  }
  switch( pk->status )
  {
   default:
   case PCK_QUIT:
    if (spline->spl.numpts == 0)
    { 
     CllClose( );
     status = CLL_QUIT;
     cSpl_Delete( spline );
     if( end_action ) (*end_action)( iup_cv, status );
     return;
    }
    else if (spline->spl.numpts > 1)
    {
     if( spline->prompt != 0L )
      (*spline->prompt)( spline->iup_cv, prompt_msg2 );
     spline->created = 1;
     spline->modified = 0;
     spline->displayed = 0;
     pk->button = 0;  /* accept both mouse buttons 1 or 2 */
     pk->mode = ButtonUp;
     pk->btdwn_callback = cSpl_BtDwnCb;
     pk->btup_callback = cSpl_BtUpCb;
     pk->cursorecho = BoxCursor | CrossCursor;
     PckSetup( pk );
     cSpl_Clr(spline);
     return; 
    }
    break;
   case PCK_INTERUPTED:
    status = CLL_INTERUPTED;
    break;
  }
  cSpl_Delete( spline );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Reallocate space for spline if necessary.
 */
 cSpl_Realloc( spline );

 if( pk->button == 2 )
 {
/* If middle button was pressed, the user wants to enter the
 * spline point using the keyboard. In this case, prompt the user for
 * point coordinates.
 */
  float x, y;
  x = pk->p1.x;  y = pk->p1.y;
  if( DlgCaptureXSpline( &x, &y ) )
  {
   pk->p1.x = x;
   pk->p1.y = y;
  }
  else
  {
   CllClose( );
   if( spline->spl.numpts > 1 )
    cSpl_Clr( spline );
   cSpl_Delete( spline );
   if( end_action ) (*end_action)( iup_cv, CLL_QUIT );
   return;
  }
 }

/* If last point is valid (either first point or last segment 
 * greater than tolerance), add collected point to spline, and increment 
 * number of collected points.
 */ 
 if( ( spline->spl.numpts == 0 ) || 
     ( LEN(pk->p1,spline->spl.keypts[spline->spl.numpts-1]) >= pk->toler ) )
 {
  cSpl_Clr( spline );
  spline->spl.keypts[spline->spl.numpts].x = pk->p1.x;
  spline->spl.keypts[spline->spl.numpts].y = pk->p1.y;
  spline->spl.numpts++;
  
  if (spline->spl.numpts > 2)
  {
    if (spline->spl.knot)    free( spline->spl.knot );
    if (spline->spl.ctrlpts) free( spline->spl.ctrlpts );
    
    spline->spl.knot    = NULL;
    spline->spl.ctrlpts = NULL;
  }
 }

/* Display collected spline segment.
 */
 if( spline->spl.numpts > 1 )
 { 	
  if ((spline->spl.numpts == 4)&&(flag == 1))
  {
   for (i = 0; i < spline->spl.numpts; i++)
   {
    aux[i].x = spline->spl.keypts[i].x;
    aux[i].y = spline->spl.keypts[i].y;   
   }
   spline->spl.numpts = 3;
   spline->spl.keypts[0].x = aux[0].x;
   spline->spl.keypts[0].y = aux[0].y;
   spline->spl.keypts[1].x = aux[2].x;
   spline->spl.keypts[1].y = aux[2].y;
   spline->spl.keypts[2].x = aux[3].x;
   spline->spl.keypts[2].y = aux[3].y;
   flag = 0;
  }
  cSpl_Dsp(spline);
 }

/* Keep getting picks from the user.
 */
 if( spline->prompt != NULL )
  (*spline->prompt )( spline->iup_cv, prompt_msg ) ;
 pk->button = 0;       /* accept both mouse buttons 1 or 2 */
 if( spline->spl.numpts > 0 )
 {
  pk->p0.x = spline->spl.keypts[spline->spl.numpts-1].x;
  pk->p0.y = spline->spl.keypts[spline->spl.numpts-1].y;
  pk->cursorecho = RubberBandCursor | BoxCursor | CrossCursor;
 }
 pk->p1.x = pk->p1.y = 0.0;
 PckSetup( pk );
}

/* =======================  cSpl_Redisplay  ==================== */

static void cSpl_Redisplay( void *b )
{
 CllSplineP *spline = (CllSplineP *)b;

/* This is the callback for the collector superclass redisplay.
 * Force the display of a created curve.
 */
 spline->displayed = 0;

 if( spline->created )
  cSpl_DisplayXor( spline );
}

/* =======================  cSpl_CreateGen  ===================== */

static void cSpl_CreateGen(  
              Ihandle *iup_cv, void *cd_cv,
              void (*end_action)( Ihandle *iup_cv, int status ),
              void (*put)( Ihandle *iup_cv, void *spline, double tol ), 
              void (*prompt)( Ihandle *iup_cv, char *msg ), 
              void (*snap)( Ihandle *iup_cv, double *x, double *y,  
                            double tol, int shift, int control ),
              void (*echo)( Ihandle *iup_cv, double x, double y ) )
{
 CllSplineP  *spline;
 PckDesc    pk;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  prompt_msg = prompt_msge;
  prompt_msg2 = prompt_msg2e;
 }
 else
 {
  prompt_msg = prompt_msgp;
  prompt_msg2 = prompt_msg2p;
 }

 CllClose( );

/* Allocate a new spline descriptor
 */
 spline = (CllSplineP *)malloc( sizeof( CllSplineP ) );

/* Initialize collection
 */
 spline->spl.numpts = 0;
 spline->allocsize = ALLOC_SIZE;
 spline->created = 0;
 spline->modified = 0;
 spline->displayed = 0;
 spline->spl.keypts = (CllPnt *)calloc( spline->allocsize, sizeof( CllPnt ) );
 spline->iup_cv = iup_cv;
 spline->cd_cv = cd_cv;
 spline->end_action = end_action;
 spline->put = put;
 spline->prompt = prompt;
 spline->snap = snap;
 spline->echo = echo;

/* Register spline display function with collection super class.
 */
 CllOpen( spline, cSpl_Redisplay );

/* Setup pick descriptor.  Register the appropriate spline collector
 * function as the callback for pick interaction.
 */
 if( spline->prompt != NULL )
  (*spline->prompt )( spline->iup_cv, prompt_msg ) ;
 pk.iup_cv = spline->iup_cv;
 pk.gra_cv = spline->cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.mode = ButtonDown;
 pk.cursorecho = BoxCursor | CrossCursor;
 pk.button = 0;    /* accept both mouse buttons 1 or 2 */
 pk.toler = 0.0;
 pk.btdwn_callback = cSpl_Pt;
 pk.btup_callback = NULL;
 pk.echofunc = cSpl_Echo;
 pk.snapfunc = cSpl_Snap;
 pk.data = spline;
 pk.status = PCK_VALID;
 PckSetup( &pk );
}


/* ======================  cSpl_KbdCreateGen  =================== */

static int     kbdnpts;
static CllPnt  kbdpnt;
static double  kbdtol = 0.001;

static char *kbdprompt_msg;

static char *kbdprompt_msgp = { "Digite dados da linha spline. " };
static char *kbdprompt_msge = { "Use the keyboard to enter spline data. " };

static int  cSpl_KbdCreateGen( Ihandle *iup_cv,
              void (*put)( Ihandle *iup_cv, void *spline, double tol ),
              void (*prompt)( Ihandle *iup_cv, char *msg ) )
{
 CllSpline spline;
 CllPnt   pi, pf, pli, plf;
 CllPnt   fder, lder;
 int      npts;
 float    x, y;
 float    fderx, fdery, lderx, ldery;
 float    toler;
 int      i, dg;
 int      status = CLL_QUIT;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  kbdprompt_msg = kbdprompt_msge;
 }
 else
 {
  kbdprompt_msg = kbdprompt_msgp;
 }

/* Initialize collection
 */
 npts = kbdnpts;
 toler = kbdtol;

/* Collect data from user with the keyboard.
 */
 if( prompt != 0L )
  (*prompt)( iup_cv, kbdprompt_msg );

 if( DlgCaptureNSpline( &npts, &toler ) )
 {
  kbdtol = toler;
  if( (npts > 1) && (toler >= 0.0) )
  { 
   spline.numpts = 0;
   spline.keypts = (CllPnt *)calloc( ALLOC_SIZE, sizeof( CllPnt ) );
   
   for( i=0; i<npts; i++ )
   {
    x = kbdpnt.x;
    y = kbdpnt.y;
    if( DlgCaptureXSpline( &x, &y ) )
    {
     kbdpnt.x = x;
     kbdpnt.y = y;

     spline.keypts[spline.numpts++] = kbdpnt;
    }
    else
    {
     status = CLL_INTERUPTED;
     return( status );
    }
   }
   
   dg = 2;   
  
   cSpl_Init(&spline, spline.numpts, dg);
   
   cSpl_Evaluate2Pt( &spline, 0.0, &pi, &pli );
   cSpl_Evaluate2Pt( &spline, 1.0, &pf, &plf );
   
   fderx = pli.x;
   fdery = pli.y;
   lderx = plf.x;
   ldery = plf.y;
   
   if (DlgCaptureDerSpline(&fderx, &fdery, &lderx, &ldery))
   {
    fder.x = (double)fderx;
    fder.y = (double)fdery;
    lder.x = (double)lderx;
    lder.y = (double)ldery;
   
    cSpl_SetEndTan(&spline, &fder, &lder);

    if( spline.numpts > 1 )
    {
     kbdnpts = spline.numpts;
     if( put != 0L )
      (*put)( iup_cv, &spline, kbdtol );
     status = CLL_NORMAL;
    }
    
    free(spline.knot);
    free(spline.ctrlpts);
    free(spline.keypts );
   }
   else
   {
    status = CLL_INTERUPTED;
    return( status );
   }
  }
 }

 return( status );
}


/*
** --------------------------------------------------------------------
** Public functions:
*/

/* ========================  CllSplineCreate  ======================= */

void CllSplineCreate( Ihandle *iup_cv, void *cd_cv,
              void (*end_action)( Ihandle *iup_cv, int status ),
              void (*put)( Ihandle *iup_cv, void *spline, double tol ), 
              void (*prompt)( Ihandle *iup_cv, char *msg ), 
              void (*snap)( Ihandle *iup_cv, double *x, double *y,  
                            double tol, int shift, int control ),
              void (*echo)( Ihandle *iup_cv, double x, double y ) )
{
 cSpl_CreateGen( iup_cv, cd_cv, end_action, put, prompt, snap, echo );
}


/* ======================  CllKbdSplineCreate  ====================== */

int  CllKbdSplineCreate( Ihandle *iup_cv, 
              void (*put)( Ihandle *iup_cv, void *spline, double tol ), 
              void (*prompt)( Ihandle *iup_cv, char *msg ) )
{
 return( cSpl_KbdCreateGen( iup_cv, put, prompt ) );
}

