
/*
** dlgdimln.c - Functions that establish the pre-defined dialogs 
**              of the library to collect dimension lines.
**
**
** Public functions:
** --------------------------------------------------------------------
**
** int DlgCaptureDimLn( float *x1, float *y1, float *x2, float *y2, 
**                      float *xref, float *yref, float *toler )
**
** x1    - X coordinate of first dimension line point.     (IN/OUT)
** y1    - Y coordinate of first dimension line point.     (IN/OUT)
** x2    - X coordinate of second dimension line point.    (IN/OUT)
** y2    - Y coordinate of second dimension line point.    (IN/OUT)
** xref  - X coordinate of reference dimension line point. (IN/OUT)
** yref  - Y coordinate of reference dimension line point. (IN/OUT)
** toler - tolerance used.                                 (IN/OUT)
**
** This function creates a dialog to capture the parameters of
** a dimension line.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureFDimLn( float *x1, float *y1 )
**
** x1    - X coordinate of first dimension line point.     (IN/OUT)
** y1    - Y coordinate of first dimension line point.     (IN/OUT)
**
** This function creates a dialog to capture the coordinates of the 
** first point of a dimension line.
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureSDimLn( float *x2, float *y2 )
**
** x2    - X coordinate of second dimension line point.    (IN/OUT)
** y2    - Y coordinate of second dimension line point.    (IN/OUT)
**
** This function creates a dialog to capture the coordinates of the 
** second point of a dimension line.
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureTDimLn( float *x1, float *y1, float *x2, float *y2, 
**                       float *xref, float *yref )
**
** x1    - X coordinate of first dimension line point.     (IN/OUT)
** y1    - Y coordinate of first dimension line point.     (IN/OUT)
** x2    - X coordinate of second dimension line point.    (IN/OUT)
** y2    - Y coordinate of second dimension line point.    (IN/OUT)
** xref  - X coordinate of reference dimension line point. (IN/OUT)
** yref  - Y coordinate of reference dimension line point. (IN/OUT)
**
** This function creates a dialog to capture the parameters of
** a dimension line.
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** Version:    1-000
**
** Created:    08-Aug-02    Marcos C. de Arruda.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced include of header file iuptabs.h by iupcontrols.h.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "iup.h"
#include "iupmask.h"
#include "iupcontrols.h"

#include "dlgdimln.h"

static char* format = "%.6f";
static int   d_posx = IUP_CENTER;
static int   d_posy = IUP_CENTER;

/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/

/* Data structure to store point dialog */

typedef struct _dimlndlg
{

  char *pointlabel0;
  char *pointlabel1;
  char *pointlabel2;
  char *pointlabel3;
  char *pointlabel4;
  char *pointlabel5;
  char *pointlabel6;
  char *pointlabel7;
  char *pointbuttn1;
  char *pointbuttn2;

  Ihandle *dialog;

} DimLnDlg;


static void  dlgInitDimLn      ( DimLnDlg *dial );
static void* dlgCreateDimLn    ( void );
static void  dlgValueToDimLn   ( float x1,  float y1, float x2,  float y2, 
                                 float xref,  float yref, float tol );
static int   dlgValidateDimLn  ( void );
static void  dlgValueFromDimLn ( float *x1,  float *y1, float *x2,  float *y2, 
                                 float *xref,  float *yref, float *tol );

static void  dlgInitFDimLn      ( DimLnDlg *dial );
static void* dlgCreateFDimLn    ( void );
static void  dlgValueToFDimLn   ( float x1, float y1 );
static int   dlgValidateFDimLn  ( void );
static void  dlgValueFromFDimLn ( float *x1, float *y1 );

static void  dlgInitSDimLn      ( DimLnDlg *dial );
static void* dlgCreateSDimLn    ( void );
static void  dlgValueToSDimLn   ( float x2, float y2 );
static int   dlgValidateSDimLn  ( void );
static void  dlgValueFromSDimLn ( float *x2, float *y2 );

static void  dlgInitTDimLn      ( DimLnDlg *dial );
static void* dlgCreateTDimLn    ( void );
static void  dlgValueToTDimLn   ( float x1,  float y1, float x2,  float y2, 
                                  float xref,  float yref );
static int   dlgValidateTDimLn  ( void );
static void  dlgValueFromTDimLn ( float *x1,  float *y1, float *x2,  float *y2, 
                                  float *xref,  float *yref );

static int  callbackDimLnDialogOk      ( Ihandle *bt );
static int  callbackDimLnDialogCancel  ( Ihandle *bt );
static int  callbackDimLnDialogCloseCB ( Ihandle *bt );

static int  callbackFDimLnDialogOk     ( Ihandle *bt );
static int  callbackFDimLnDialogCancel ( Ihandle *bt );
static int  callbackFDimLnDialogCloseCB( Ihandle *bt );

static int  callbackSDimLnDialogOk     ( Ihandle *bt );
static int  callbackSDimLnDialogCancel ( Ihandle *bt );
static int  callbackSDimLnDialogCloseCB( Ihandle *bt );

static int  callbackTDimLnDialogOk     ( Ihandle *bt );
static int  callbackTDimLnDialogCancel ( Ihandle *bt );
static int  callbackTDimLnDialogCloseCB( Ihandle *bt );

static DimLnDlg  dimln;
static DimLnDlg fdimln;
static DimLnDlg sdimln;
static DimLnDlg tdimln;

static char *size_butt = {"55x12"};
static char *size_text = {"65"};
static char *font_dial = {"TIMES_NORMAL_12"};

static char *llabel0e = { "Enter with dimension line parameters: " };
static char *llabel01e = { "Enter with first dimension line point:" };
static char *llabel02e = { "Enter with second dimension line point:" };
static char *llabel1e = { "Point 1 X:  " };
static char *llabel2e = { "Point 1 Y:  " };
static char *llabel3e = { "Point 2 X:  " };
static char *llabel4e = { "Point 2 Y:  " };
static char *llabel5e = { "Ref.Pt. X:  " };
static char *llabel6e = { "Ref.Pt. Y:  " };
static char *llabel7e = { "Tolerance:  " };
static char *lbuttn1e = { " OK " };
static char *lbuttn2e = { " Cancel " };

static char *llabel0p = { "Entre com os parmetros da linha de cota:" };
static char *llabel01p = { "Entre com o primeiro ponto da linha de cota:" };
static char *llabel02p = { "Entre com o segundo ponto da linha de cota:" };
static char *llabel1p = { "Ponto 1 X:  " };
static char *llabel2p = { "Ponto 1 Y:  " };
static char *llabel3p = { "Ponto 2 X:  " };
static char *llabel4p = { "Ponto 2 Y:  " };
static char *llabel5p = { "Pt.Ref. X:  " };
static char *llabel6p = { "Pt.Ref. Y:  " };
static char *llabel7p = { "Toleranc.:  " };
static char *lbuttn1p = { "Confirma" };
static char *lbuttn2p = { "Cancela " };

/*
** --------------------------------------------------------------------
** Local functions:
*/

/* =======================  dlgInitDimLn  ======================= */

static void dlgInitDimLn ( DimLnDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel0e; 
   dial->pointlabel1 = llabel1e; 
   dial->pointlabel2 = llabel2e; 
   dial->pointlabel3 = llabel3e; 
   dial->pointlabel4 = llabel4e; 
   dial->pointlabel5 = llabel5e;
   dial->pointlabel6 = llabel6e;
   dial->pointlabel7 = llabel7e;
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel0p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointlabel3 = llabel3p; 
   dial->pointlabel4 = llabel4p;    
   dial->pointlabel5 = llabel5p;
   dial->pointlabel6 = llabel6p;
   dial->pointlabel7 = llabel7p;
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* =======================  dlgCreateDimLn  ======================= */

static void* dlgCreateDimLn ( void )
{
  if ( dimln.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( dimln.pointbuttn1, "call_dimln_ok" );
    Ihandle *butt2 = IupButton ( dimln.pointbuttn2, "call_dimln_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;
    Ihandle *labl4 = NULL;	
    Ihandle *labl5 = NULL;
    Ihandle *labl6 = NULL;
    Ihandle *labl7 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;
    Ihandle *text4 = NULL;
    Ihandle *text5 = NULL;
    Ihandle *text6 = NULL;
    Ihandle *text7 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;
    Ihandle *h4 = NULL;
    Ihandle *h5 = NULL;
    Ihandle *h6 = NULL;
    Ihandle *h7 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( dimln.pointlabel1 );
    labl2 = IupLabel  ( dimln.pointlabel2 );
    labl3 = IupLabel  ( dimln.pointlabel3 );
    labl4 = IupLabel  ( dimln.pointlabel4 );
    labl5 = IupLabel  ( dimln.pointlabel5 );
    labl6 = IupLabel  ( dimln.pointlabel6 );
    labl7 = IupLabel  ( dimln.pointlabel7 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );
    text4 = IupText   ( NULL );
    text5 = IupText   ( NULL );
    text6 = IupText   ( NULL );
    text7 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);
    h4 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl4,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text4,IupFill(),NULL), 
		  IupFill(), NULL);

    h5 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl5,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text5,IupFill(),NULL),
		  IupFill(), NULL);
    h6 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl6,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text6,IupFill(),NULL),
		  IupFill(), NULL);
    h7 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl7,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text7,IupFill(),NULL),
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox (
	        IupFill(),
	        h1,
	        IupFill(),
		h2,
	        IupFill(),
		h3,
	        IupFill(),
		h4,
	        IupFill(),
		h5,
	        IupFill(),
		h6,
	        IupFill(),
		h7,
	        IupFill(),
		NULL
	      )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );	
    IupSetAttribute ( text5, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text6, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text7, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );	
    IupSetAttribute ( text5, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text6, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text7, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( text5, "SIZE", size_text );
    IupSetAttribute ( text6, "SIZE", size_text );
    IupSetAttribute ( text7, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );
    IupSetAttribute ( labl5, "SIZE", size_text );
    IupSetAttribute ( labl6, "SIZE", size_text );
    IupSetAttribute ( labl7, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  dimln.pointlabel0 );

    /* sets handles */
    IupSetHandle("x1_dimln1_handle",   text1);
    IupSetHandle("y1_dimln1_handle",   text2);
    IupSetHandle("x2_dimln1_handle",   text3);    
    IupSetHandle("y2_dimln1_handle",   text4);
    IupSetHandle("xref_dimln1_handle", text5);
    IupSetHandle("yref_dimln1_handle", text6);
    IupSetHandle("tl_dimln1_handle",   text7);
    iupmaskSet(IupGetHandle("x1_dimln1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y1_dimln1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("x2_dimln1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y2_dimln1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("xref_dimln1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("yref_dimln1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("tl_dimln1_handle"), IUPMASK_UFLOAT, 0, 1);

    /* creates dialog */
    dimln.dialog = IupDialog (
                      IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_dimln_ok", (Icallback)callbackDimLnDialogOk );
    IupSetFunction ( "call_dimln_cn", (Icallback)callbackDimLnDialogCancel );
    IupSetFunction ( "call_dimln_close",(Icallback)callbackDimLnDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( dimln.dialog, "GAP",    "3" );
    IupSetAttribute ( dimln.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( dimln.dialog, "MAXBOX", "NO");
    IupSetAttribute ( dimln.dialog, "RESIZE", "NO");
    IupSetAttribute ( dimln.dialog, "CLOSE_CB", "call_dimln_close");
#ifdef _UNIX_
    IupSetAttribute ( dimln.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( dimln.dialog, "TITLE",  "DimLn" );

    IupMap ( dimln.dialog );
  }

  return dimln.dialog;
}

/* ======================  dlgValueToDimLn  ======================= */

static void dlgValueToDimLn ( float x1, float y1, float x2, float y2,
                              float xref, float yref, float tol )
{
 static char cx1[50];
 static char cy1[50];
 static char cx2[50];
 static char cy2[50];
 static char cxref[50];
 static char cyref[50];
 static char ctl[50];

 if ( dimln.dialog == NULL ) return;

 sprintf(cx1, format, x1);
 sprintf(cy1, format, y1);
 sprintf(cx2, format, x2); 
 sprintf(cy2, format, y2);
 sprintf(cxref, format, xref);
 sprintf(cyref, format, yref);
 sprintf(ctl, format, tol);

 IupSetAttribute (IupGetHandle("x1_dimln1_handle"),   "VALUE", cx1);
 IupSetAttribute (IupGetHandle("y1_dimln1_handle"),   "VALUE", cy1);
 IupSetAttribute (IupGetHandle("x2_dimln1_handle"),   "VALUE", cx2);
 IupSetAttribute (IupGetHandle("y2_dimln1_handle"),   "VALUE", cy2);
 IupSetAttribute (IupGetHandle("xref_dimln1_handle"), "VALUE", cxref);
 IupSetAttribute (IupGetHandle("yref_dimln1_handle"), "VALUE", cyref);
 IupSetAttribute (IupGetHandle("tl_dimln1_handle"),   "VALUE", ctl);
}

/* =====================  dlgValueFromDimLn  ====================== */

static void dlgValueFromDimLn ( float *x1, float *y1, float *x2, float *y2, 
                                float *xref, float *yref, float *tol )
{
  if ( dimln.dialog == NULL ) return;

  /* normal case */
  *x1   = IupGetFloat (IupGetHandle("x1_dimln1_handle"),   "VALUE");
  *y1   = IupGetFloat (IupGetHandle("y1_dimln1_handle"),   "VALUE");
  *x2   = IupGetFloat (IupGetHandle("x2_dimln1_handle"),   "VALUE");
  *y2   = IupGetFloat (IupGetHandle("y2_dimln1_handle"),   "VALUE");
  *xref = IupGetFloat (IupGetHandle("xref_dimln1_handle"), "VALUE");
  *yref = IupGetFloat (IupGetHandle("yref_dimln1_handle"), "VALUE");
  *tol  = IupGetFloat (IupGetHandle("tl_dimln1_handle"),   "VALUE");
  
  if (*tol == 0.0)
   *tol = 0.001;
}

/* =====================  dlgValidateDimLn  ======================= */

static int dlgValidateDimLn ( void )
{
 char *x1;
 char *y1;
 char *x2;
 char *y2;
 char *xref;
 char *yref;
 char *tol;

 if ( dimln.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(dimln.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 /* normal case */
 x1   = IupGetAttribute (IupGetHandle("x1_dimln1_handle"),   "VALUE");
 y1   = IupGetAttribute (IupGetHandle("y1_dimln1_handle"),   "VALUE");
 x2   = IupGetAttribute (IupGetHandle("x2_dimln1_handle"),   "VALUE");
 y2   = IupGetAttribute (IupGetHandle("y2_dimln1_handle"),   "VALUE");
 xref = IupGetAttribute (IupGetHandle("xref_dimln1_handle"), "VALUE");
 yref = IupGetAttribute (IupGetHandle("xref_dimln1_handle"), "VALUE");
 tol  = IupGetAttribute (IupGetHandle("tl_dimln1_handle"),   "VALUE");

 return 1;
}

/* ====================  dlgInitFDimLn  ==================== */

static void dlgInitFDimLn ( DimLnDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel01e;
   dial->pointlabel1 = llabel1e;
   dial->pointlabel2 = llabel2e;
   dial->pointbuttn1 = lbuttn1e;
   dial->pointbuttn2 = lbuttn2e;
 }
 else
 {
   dial->pointlabel0 = llabel01p;
   dial->pointlabel1 = llabel1p;
   dial->pointlabel2 = llabel2p;
   dial->pointbuttn1 = lbuttn1p;
   dial->pointbuttn2 = lbuttn2p;
 }
}

/* ====================  dlgCreateFDimLn  =================== */

static void* dlgCreateFDimLn ( void )
{
  if ( fdimln.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( fdimln.pointbuttn1, "call_fdimln_ok" );
    Ihandle *butt2 = IupButton ( fdimln.pointbuttn2, "call_fdimln_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( fdimln.pointlabel1 );
    labl2 = IupLabel  ( fdimln.pointlabel2 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  fdimln.pointlabel0 );

    /* sets handles */
    IupSetHandle("x1_dimln3_handle", text1);
    IupSetHandle("y1_dimln3_handle", text2);
    iupmaskSet(IupGetHandle("x1_dimln3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y1_dimln3_handle"), IUPMASK_FLOAT, 0, 1);

    /* creates dialog */
    fdimln.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_fdimln_ok", (Icallback)callbackFDimLnDialogOk );
    IupSetFunction ( "call_fdimln_cn", (Icallback)callbackFDimLnDialogCancel );
    IupSetFunction ( "call_fdimln_close",(Icallback)callbackFDimLnDialogCloseCB );
    
    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( fdimln.dialog, "GAP",    "3" );
    IupSetAttribute ( fdimln.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( fdimln.dialog, "MAXBOX", "NO");
    IupSetAttribute ( fdimln.dialog, "RESIZE", "NO");
    IupSetAttribute ( fdimln.dialog, "CLOSE_CB", "call_fdimln_close");
#ifdef _UNIX_
    IupSetAttribute ( fdimln.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( fdimln.dialog, "TITLE",  "DimLn" );

    IupMap ( fdimln.dialog );
  }

  return fdimln.dialog;
}

/* ===================  dlgValueToFDimLn  =================== */

static void dlgValueToFDimLn ( float x1, float y1 )
{
 static char cx1[50];
 static char cy1[50];

 if ( fdimln.dialog == NULL ) return;

 sprintf(cx1, format, x1);
 sprintf(cy1, format, y1);

 IupSetAttribute (IupGetHandle("x1_dimln3_handle"), "VALUE", cx1);
 IupSetAttribute (IupGetHandle("y1_dimln3_handle"), "VALUE", cy1);
}

/* =================  dlgValueFromFDimLn  =================== */

static void dlgValueFromFDimLn ( float *x1, float *y1 )
{
  if ( fdimln.dialog == NULL ) return;

  *x1  = IupGetFloat (IupGetHandle("x1_dimln3_handle"), "VALUE");
  *y1  = IupGetFloat (IupGetHandle("y1_dimln3_handle"), "VALUE");
}

/* ==================  dlgValidateFDimLn  =================== */

static int dlgValidateFDimLn ( void )
{
 char *x1;
 char *y1;

 if ( fdimln.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(fdimln.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 x1 = IupGetAttribute (IupGetHandle("x1_dimln3_handle"),   "VALUE");
 y1 = IupGetAttribute (IupGetHandle("y1_dimln3_handle"),   "VALUE");

 return 1;
}

/* ====================  dlgInitSDimLn  ==================== */

static void dlgInitSDimLn ( DimLnDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel02e;
   dial->pointlabel1 = llabel3e;
   dial->pointlabel2 = llabel4e;
   dial->pointbuttn1 = lbuttn1e;
   dial->pointbuttn2 = lbuttn2e;
 }
 else
 {
   dial->pointlabel0 = llabel02p;
   dial->pointlabel1 = llabel3p;
   dial->pointlabel2 = llabel4p;
   dial->pointbuttn1 = lbuttn1p;
   dial->pointbuttn2 = lbuttn2p;
 }
}

/* ====================  dlgCreateSDimLn  =================== */

static void* dlgCreateSDimLn ( void )
{
  if ( sdimln.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( sdimln.pointbuttn1, "call_sdimln_ok" );
    Ihandle *butt2 = IupButton ( sdimln.pointbuttn2, "call_sdimln_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( sdimln.pointlabel1 );
    labl2 = IupLabel  ( sdimln.pointlabel2 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl1,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text1,IupFill(),NULL),
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl2,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text2,IupFill(),NULL),
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  sdimln.pointlabel0 );

    /* sets handles */
    IupSetHandle("x2_dimln3_handle", text1);
    IupSetHandle("y2_dimln3_handle", text2);
    iupmaskSet(IupGetHandle("x2_dimln3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y2_dimln3_handle"), IUPMASK_FLOAT, 0, 1);

    /* creates dialog */
    sdimln.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_sdimln_ok", (Icallback)callbackSDimLnDialogOk );
    IupSetFunction ( "call_sdimln_cn", (Icallback)callbackSDimLnDialogCancel );
    IupSetFunction ( "call_sdimln_close",(Icallback)callbackSDimLnDialogCloseCB ); 
    
    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( sdimln.dialog, "GAP",    "3" );
    IupSetAttribute ( sdimln.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( sdimln.dialog, "MAXBOX", "NO");
    IupSetAttribute ( sdimln.dialog, "RESIZE", "NO");
    IupSetAttribute ( sdimln.dialog, "CLOSE_CB", "call_sdimln_close");
#ifdef _UNIX_
    IupSetAttribute ( sdimln.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( sdimln.dialog, "TITLE",  "DimLn" );

    IupMap ( sdimln.dialog );
  }

  return sdimln.dialog;
}

/* ===================  dlgValueToSDimLn  =================== */

static void dlgValueToSDimLn ( float x2, float y2 )
{
 static char cx2[50];
 static char cy2[50];

 if ( fdimln.dialog == NULL ) return;

 sprintf(cx2, format, x2);
 sprintf(cy2, format, y2);

 IupSetAttribute (IupGetHandle("x2_dimln3_handle"), "VALUE", cx2);
 IupSetAttribute (IupGetHandle("y2_dimln3_handle"), "VALUE", cy2);
}

/* =================  dlgValueFromSDimLn  =================== */

static void dlgValueFromSDimLn ( float *x2, float *y2 )
{
  if ( sdimln.dialog == NULL ) return;

  *x2  = IupGetFloat (IupGetHandle("x2_dimln3_handle"), "VALUE");
  *y2  = IupGetFloat (IupGetHandle("y2_dimln3_handle"), "VALUE");
}

/* ==================  dlgValidateSDimLn  =================== */

static int dlgValidateSDimLn ( void )
{
 char *x2;
 char *y2;

 if ( sdimln.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(sdimln.dialog, "CONFIRM"), "1") != 0 )
    return 0;

 x2 = IupGetAttribute (IupGetHandle("x2_dimln3_handle"),   "VALUE");
 y2 = IupGetAttribute (IupGetHandle("y2_dimln3_handle"),   "VALUE");

 return 1;
}

/* ====================  dlgInitTDimLn  ==================== */

static void dlgInitTDimLn ( DimLnDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel0e;
   dial->pointlabel1 = llabel1e;
   dial->pointlabel2 = llabel2e;
   dial->pointlabel3 = llabel3e;
   dial->pointlabel4 = llabel4e;
   dial->pointlabel5 = llabel5e;
   dial->pointlabel6 = llabel6e;
   dial->pointbuttn1 = lbuttn1e;
   dial->pointbuttn2 = lbuttn2e;
 }
 else
 {
   dial->pointlabel0 = llabel0p;
   dial->pointlabel1 = llabel1p;
   dial->pointlabel2 = llabel2p;
   dial->pointlabel3 = llabel3p; 
   dial->pointlabel4 = llabel4p;
   dial->pointlabel5 = llabel5p;
   dial->pointlabel6 = llabel6p;
   dial->pointbuttn1 = lbuttn1p;
   dial->pointbuttn2 = lbuttn2p;
 }
}

/* ====================  dlgCreateTDimLn  =================== */

static void* dlgCreateTDimLn ( void )
{
  if ( tdimln.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( tdimln.pointbuttn1, "call_tdimln_ok" );
    Ihandle *butt2 = IupButton ( tdimln.pointbuttn2, "call_tdimln_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;
    Ihandle *labl4 = NULL;
    Ihandle *labl5 = NULL;
    Ihandle *labl6 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;
    Ihandle *text4 = NULL;
    Ihandle *text5 = NULL;
    Ihandle *text6 = NULL; 

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;
    Ihandle *h4 = NULL;
    Ihandle *h5 = NULL;
    Ihandle *h6 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( tdimln.pointlabel1 );
    labl2 = IupLabel  ( tdimln.pointlabel2 );
    labl3 = IupLabel  ( tdimln.pointlabel3 );
    labl4 = IupLabel  ( tdimln.pointlabel4 );
    labl5 = IupLabel  ( tdimln.pointlabel5 );
    labl6 = IupLabel  ( tdimln.pointlabel6 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );
    text4 = IupText   ( NULL );
    text5 = IupText   ( NULL );
    text6 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);

    h4 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl4,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text4,IupFill(),NULL),
		  IupFill(), NULL);

    h5 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl5,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text5,IupFill(),NULL),
		  IupFill(), NULL);

    h6 = IupHbox (IupFill(),
                  IupVbox(IupFill(),labl6,IupFill(),NULL),
		  IupFill(),
		  IupVbox(IupFill(),text6,IupFill(),NULL),
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), 
	               h1, 
		       IupFill(), 
		       h2, 
		       IupFill(), 
		       h3, 
		       IupFill(),
		       h4, 
		       IupFill(),
		       h5, 
		       IupFill(),
		       h6,
		       IupFill(), NULL )
		);

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text5, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text6, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text5, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text6, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( text5, "SIZE", size_text );
    IupSetAttribute ( text6, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );
    IupSetAttribute ( labl5, "SIZE", size_text );
    IupSetAttribute ( labl6, "SIZE", size_text ); 

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  tdimln.pointlabel0);

    /* sets handles */
    IupSetHandle("x1_dimln2_handle",   text1);
    IupSetHandle("y1_dimln2_handle",   text2);
    IupSetHandle("x2_dimln2_handle",   text3);    
    IupSetHandle("y2_dimln2_handle",   text4);
    IupSetHandle("xref_dimln2_handle", text5);
    IupSetHandle("yref_dimln2_handle", text6);
    iupmaskSet(IupGetHandle("x1_dimln2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y1_dimln2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("x2_dimln2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y2_dimln2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("xref_dimln2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("yref_dimln2_handle"), IUPMASK_FLOAT, 0, 1);

    /* creates dialog */
    tdimln.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_tdimln_ok", (Icallback)callbackTDimLnDialogOk );
    IupSetFunction ( "call_tdimln_cn", (Icallback)callbackTDimLnDialogCancel );
    IupSetFunction ( "call_tdimln_close",(Icallback)callbackTDimLnDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( tdimln.dialog, "GAP",    "3" );
    IupSetAttribute ( tdimln.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( tdimln.dialog, "MAXBOX", "NO");
    IupSetAttribute ( tdimln.dialog, "RESIZE", "NO");
    IupSetAttribute ( tdimln.dialog, "CLOSE_CB", "call_tdimln_close");
#ifdef _UNIX_
    IupSetAttribute ( tdimln.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( tdimln.dialog, "TITLE",  "DimLn" );

    IupMap ( tdimln.dialog );
  }

  return tdimln.dialog;
}

/* ===================  dlgValueToTDimLn  =================== */

static void dlgValueToTDimLn ( float x1, float y1, float x2, float y2,
                               float xref, float yref )
{
 static char cx1[50];
 static char cy1[50];
 static char cx2[50];
 static char cy2[50];
 static char cxref[50];
 static char cyref[50];

 if ( tdimln.dialog == NULL ) return;

 sprintf(cx1, format, x1);
 sprintf(cy1, format, y1);
 sprintf(cx2, format, x2); 
 sprintf(cy2, format, y2);
 sprintf(cxref, format, xref);
 sprintf(cyref, format, yref);

 IupSetAttribute (IupGetHandle("x1_dimln2_handle"),   "VALUE", cx1);
 IupSetAttribute (IupGetHandle("y1_dimln2_handle"),   "VALUE", cy1);
 IupSetAttribute (IupGetHandle("x2_dimln2_handle"),   "VALUE", cx2);
 IupSetAttribute (IupGetHandle("y2_dimln2_handle"),   "VALUE", cy2);
 IupSetAttribute (IupGetHandle("xref_dimln2_handle"), "VALUE", cxref);
 IupSetAttribute (IupGetHandle("yref_dimln2_handle"), "VALUE", cyref);
}

/* =================  dlgValueFromTDimLn  =================== */

static void dlgValueFromTDimLn ( float *x1, float *y1, float *x2, float *y2, 
                                 float *xref, float *yref )
{
  if ( tdimln.dialog == NULL ) return;

  *x1   = IupGetFloat (IupGetHandle("x1_dimln2_handle"),   "VALUE");
  *y1   = IupGetFloat (IupGetHandle("y1_dimln2_handle"),   "VALUE");
  *x2   = IupGetFloat (IupGetHandle("x2_dimln2_handle"),   "VALUE");
  *y2   = IupGetFloat (IupGetHandle("y2_dimln2_handle"),   "VALUE");
  *xref = IupGetFloat (IupGetHandle("xref_dimln2_handle"), "VALUE");
  *yref = IupGetFloat (IupGetHandle("yref_dimln2_handle"), "VALUE");
}

/* ==================  dlgValidateTDimLn  =================== */

static int dlgValidateTDimLn ( void )
{
 char *x1;
 char *y1;
 char *x2;
 char *y2;
 char *xref;
 char *yref;

 if ( tdimln.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(tdimln.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 /* normal case */
 x1   = IupGetAttribute (IupGetHandle("x1_dimln2_handle"),   "VALUE");
 y1   = IupGetAttribute (IupGetHandle("y1_dimln2_handle"),   "VALUE");
 x2   = IupGetAttribute (IupGetHandle("x2_dimln2_handle"),   "VALUE");
 y2   = IupGetAttribute (IupGetHandle("y2_dimln2_handle"),   "VALUE");
 xref = IupGetAttribute (IupGetHandle("xref_dimln1_handle"), "VALUE");
 yref = IupGetAttribute (IupGetHandle("xref_dimln1_handle"), "VALUE");

 return 1;
}

/* =========================  callbackDimLnDialogOk  ======================== */

static int callbackDimLnDialogOk ( Ihandle *bt )
{
 IupSetAttribute(dimln.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* =======================  callbackDimLnDialogCancel  ====================== */

static int callbackDimLnDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(dimln.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackDimLnDialogCloseCB  ==================== */

static int callbackDimLnDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(dimln.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackFDimLnDialogOk  ====================== */

static int callbackFDimLnDialogOk ( Ihandle *bt )
{
 IupSetAttribute(fdimln.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackFDimLnDialogCancel  ==================== */

static int callbackFDimLnDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(fdimln.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackFDimLnDialogCloseCB  ==================== */

static int callbackFDimLnDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(fdimln.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackSDimLnDialogOk  ====================== */

static int callbackSDimLnDialogOk ( Ihandle *bt )
{
 IupSetAttribute(sdimln.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackSDimLnDialogCancel  ==================== */

static int callbackSDimLnDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(sdimln.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackSDimLnDialogCloseCB  ==================== */

static int callbackSDimLnDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(sdimln.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackTDimLnDialogOk  ====================== */

static int callbackTDimLnDialogOk ( Ihandle *bt )
{
 IupSetAttribute(tdimln.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackTDimLnDialogCancel  ==================== */

static int callbackTDimLnDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(tdimln.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackTDimLnDialogCloseCB  ==================== */

static int callbackTDimLnDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(tdimln.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* =========================  DlgCaptureDimLn  ======================== */

int DlgCaptureDimLn ( float *x1, float *y1, float *x2, float *y2,
                      float *xref, float *yref, float *tol )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitDimLn( &dimln );

  /* create dialog to capture point data. */
  d = dlgCreateDimLn();

  /* sets point data to dialog. */
  dlgValueToDimLn ( *x1, *y1, *x2, *y2, *xref, *yref, *tol );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateDimLn ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromDimLn ( x1, y1, x2, y2, xref, yref, tol );

  return 1;
}


/* ======================  DlgCaptureFDimLn  ===================== */

int DlgCaptureFDimLn ( float *x1, float *y1 )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitFDimLn( &fdimln );

  /* create dialog to capture point data. */
  d = dlgCreateFDimLn();

  /* sets point data to dialog. */
  dlgValueToFDimLn ( *x1, *y1 );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateFDimLn ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromFDimLn ( x1, y1 );

  return 1;
}

/* ======================  DlgCaptureSDimLn  ===================== */

int DlgCaptureSDimLn ( float *x2, float *y2 )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitSDimLn( &sdimln );

  /* create dialog to capture point data. */
  d = dlgCreateSDimLn();

  /* sets point data to dialog. */
  dlgValueToSDimLn ( *x2, *y2 );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateSDimLn ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromSDimLn ( x2, y2 );

  return 1;
}

/* ======================  DlgCaptureTDimLn  ===================== */

int DlgCaptureTDimLn ( float *x1, float *y1, float *x2, float *y2,
                       float *xref, float *yref )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitTDimLn( &tdimln );

  /* create dialog to capture point data. */
  d = dlgCreateTDimLn();

  /* sets point data to dialog. */
  dlgValueToTDimLn ( *x1, *y1, *x2, *y2, *xref, *yref );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateTDimLn ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromTDimLn ( x1, y1, x2, y2, xref, yref );

  return 1;
}

