/*
** dlgelipse.c - Functions that establish the pre-defined dialogs 
**               of the library to capture ellipse arcs.
**
**
** Public functions:
** --------------------------------------------------------------------
**
** int DlgCaptureElipse ( double *xc, double *yc, double *a, double *b, 
**                        double *ang, double *tol )
**
** xc  - coordinate (X direction) of center.           (IN/OUT)
** yc  - coordinate (Y direction) of center.           (IN/OUT)
** a   - first half-axis of elipse.                    (IN/OUT)
** b   - second half-axis of elipse.                   (IN/OUT)
** ang - angle between the half-axis with length defined by the 'a' variable
**       and the horizontal axis (X direction).        (IN/OUT)
**
** This function creates a dialog to capture the parameters of an ellipse arc. 
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCapture1Elipse( double *xc, double *yc )
**
** xc  - coordinate (X direction) of center.       (IN/OUT)
** yc  - coordinate (Y direction) of center.       (IN/OUT)
**
** This function creates a dialog to capture the parameters of an ellipse arc.
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCapture2Elipse( double *a, double *ang )
**
** a   - first half-axis of elipse.              (IN/OUT)
** ang - angle between the half-axis with length defined by the 'a' variable
**       and the horizontal axis (X direction).  (IN/OUT)
**
** This function creates a dialog to capture the parameters of an ellipse arc.
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCapture3Elipse( double *xc, double *yc, double *a, double *b, 
**                        double *ang )
**
** xc  - coordinate (X direction) of center.           (IN/OUT)
** yc  - coordinate (Y direction) of center.           (IN/OUT)
** a   - first half-axis of elipse.                    (IN/OUT)
** b   - second half-axis of elipse.                   (IN/OUT)
** ang - angle between the half-axis with length defined by the 'a' variable
**       and the horizontal axis (X direction).        (IN/OUT)
**
** This function creates a dialog to capture the parameters of an ellipse arc.
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
**
** Version:    1-000
**
** Created:    18-Mar-05    Marcos C. de Arruda.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "iup.h"
#include "iupmask.h"
#include "iupcontrols.h"

#include "dlgelipse.h"


static char* format = "%.6f";
static int   d_posx = IUP_CENTER;
static int   d_posy = IUP_CENTER;

/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/

/* Data structure */

typedef struct _elipsedlg
{

  char *pointlabel0;
  char *pointlabel1;
  char *pointlabel2;
  char *pointlabel3;
  char *pointlabel4;
  char *pointlabel5;
  char *pointlabel6;
  char *pointlabel7;
  char *pointlabel8;
  char *pointlabel9;
  char *pointlabel10;
  char *pointlabel11;
  char *pointlabel12;
  char *pointlabel13;
  char *pointlabel14;
  char *pointlabel15;
  char *pointbuttn1;
  char *pointbuttn2;

  Ihandle *dialog;

} ElipseDlg;


static void  dlgInitElipse      ( ElipseDlg *dial );
static void* dlgCreateElipse    ( void );
static void  dlgValueToElipse   ( double xc,  double yc, double a, double b,
                                  double ang, double tol );
static int   dlgValidateElipse  ( void );
static void  dlgValueFromElipse ( double* xc, double* yc, double* a, 
                                  double* b, double* ang, double* tol );

static void  dlgInit1Elipse      ( ElipseDlg *dial );
static void* dlgCreate1Elipse    ( void );
static void  dlgValueTo1Elipse   ( double xc, double yc );
static int   dlgValidate1Elipse  ( void );
static void  dlgValueFrom1Elipse ( double *xc, double *yc );

static void  dlgInit2Elipse      ( ElipseDlg *dial );
static void* dlgCreate2Elipse    ( void );
static void  dlgValueTo2Elipse   ( double a, double ang );
static int   dlgValidate2Elipse  ( void );
static void  dlgValueFrom2Elipse ( double *a, double *ang );

static void  dlgInit3Elipse      ( ElipseDlg *dial );
static void* dlgCreate3Elipse    ( void );
static void  dlgValueTo3Elipse   ( double xc,  double yc, double a, double b,
                                   double ang );
static int   dlgValidate3Elipse  ( void );
static void  dlgValueFrom3Elipse ( double* xc, double* yc, double* a, 
                                   double* b, double* ang );


static int  callbackElipseDialogOk       ( Ihandle *bt );
static int  callbackElipseDialogCancel   ( Ihandle *bt );
static int  callbackElipseDialogCloseCB  ( Ihandle *bt );

static int  callback1ElipseDialogOk      ( Ihandle *bt );
static int  callback1ElipseDialogCancel  ( Ihandle *bt );
static int  callback1ElipseDialogCloseCB ( Ihandle *bt );

static int  callback2ElipseDialogOk      ( Ihandle *bt );
static int  callback2ElipseDialogCancel  ( Ihandle *bt );
static int  callback2ElipseDialogCloseCB ( Ihandle *bt );

static int  callback3ElipseDialogOk      ( Ihandle *bt );
static int  callback3ElipseDialogCancel  ( Ihandle *bt );
static int  callback3ElipseDialogCloseCB ( Ihandle *bt );

static ElipseDlg elipse;
static ElipseDlg d1elipse;
static ElipseDlg d2elipse;
static ElipseDlg d3elipse;

static char *size_butt = {"55x12"};
static char *size_text = {"110"};
static char *font_dial = {"TIMES_NORMAL_12"};

static char *llabel0e = { "Enter with ellipse parameters: " };
static char *llabel1e = { "Center X:  " };
static char *llabel2e = { "Center Y:  " };
static char *llabel3e = { "First half-axis (a):  " };
static char *llabel4e = { "Rotation angle (lambda):  " };
static char *llabel5e = { "Second half-axis (b):  " };
static char *llabel6e = { "Tolerance: " };
static char *lbuttn1e = { " OK " };
static char *lbuttn2e = { " Cancel " };

static char *llabel7e  = { "Enter with center coordinates:" };
static char *llabel8e  = { "Enter with 'a' and 'lambda'"};

static char *llabel0p  = { "Entre com os parametros do arco de elipse: " };
static char *llabel1p  = { "Centro X:  " };
static char *llabel2p  = { "Centro Y:  " };
static char *llabel3p  = { "Primeiro semi-eixo (a):  " };
static char *llabel4p  = { "Angulo de rotacao (lambda):  " };
static char *llabel5p  = { "Segundo semi-eixo (b):  " };
static char *llabel6p = { "Tolerancia: " };
static char *lbuttn1p = { " Confirma " };
static char *lbuttn2p = { " Cancela " };

static char *llabel7p = { "Entre com as coordenadas do centro:" };
static char *llabel8p = { "Entre com o valor de 'a' e 'lambda'"};

/*
** --------------------------------------------------------------------
** Local functions:
*/

/* =======================  dlgInitElipse  ======================= */

static void dlgInitElipse ( ElipseDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0  = llabel0e; 
   dial->pointlabel1  = llabel1e; 
   dial->pointlabel2  = llabel2e; 
   dial->pointlabel3  = llabel3e; 
   dial->pointlabel4  = llabel4e; 
   dial->pointlabel5  = llabel5e; 
   dial->pointlabel6  = llabel6e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0  = llabel0p; 
   dial->pointlabel1  = llabel1p; 
   dial->pointlabel2  = llabel2p; 
   dial->pointlabel3  = llabel3p; 
   dial->pointlabel4  = llabel4p; 
   dial->pointlabel5  = llabel5p; 
   dial->pointlabel6  = llabel6p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* =======================  dlgCreateElipse  ======================= */

static void* dlgCreateElipse ( void )
{
  if ( elipse.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( elipse.pointbuttn1, "call_elipse_ok" );
    Ihandle *butt2 = IupButton ( elipse.pointbuttn2, "call_elipse_cn" );

    Ihandle *labl1  = NULL;
    Ihandle *labl2  = NULL;
    Ihandle *labl3  = NULL;
    Ihandle *labl4  = NULL;
    Ihandle *labl5  = NULL;
    Ihandle *labl6  = NULL;

    Ihandle *text1  = NULL;
    Ihandle *text2  = NULL;
    Ihandle *text3  = NULL;
    Ihandle *text4  = NULL;
    Ihandle *text5  = NULL;
    Ihandle *text6  = NULL;

    Ihandle *h1  = NULL;
    Ihandle *h2  = NULL;
    Ihandle *h3  = NULL;
    Ihandle *h4  = NULL;
    Ihandle *h5  = NULL;
    Ihandle *h6  = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1  = IupLabel  ( elipse.pointlabel1 );
    labl2  = IupLabel  ( elipse.pointlabel2 );
    labl3  = IupLabel  ( elipse.pointlabel3 );
    labl4  = IupLabel  ( elipse.pointlabel4 );
    labl5  = IupLabel  ( elipse.pointlabel5 );
    labl6  = IupLabel  ( elipse.pointlabel6 );
    text1  = IupText   ( NULL );
    text2  = IupText   ( NULL );
    text3  = IupText   ( NULL );
    text4  = IupText   ( NULL );
    text5  = IupText   ( NULL );
    text6  = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);
    h4 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl4,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text4,IupFill(),NULL), 
		  IupFill(), NULL);
    h5 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl5,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text5,IupFill(),NULL), 
		  IupFill(), NULL);
    h6 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl6,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text6,IupFill(),NULL), 
		  IupFill(), NULL);
      
    t1 = IupFrame (
	     IupVbox (
	        IupFill(),
	        h1,
	        IupFill(),
		h2,
	        IupFill(),
		h3,
	        IupFill(),
		h4,
	        IupFill(),
		h5,
	        IupFill(),
		h6,
	        IupFill(),
		NULL
	      )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text5, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text6, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text5, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text6, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( text5, "SIZE", size_text );
    IupSetAttribute ( text6, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );
    IupSetAttribute ( labl5, "SIZE", size_text );
    IupSetAttribute ( labl6, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  elipse.pointlabel0 );

    /* sets handles */
    IupSetHandle("xc_elp1_handle",  text1);
    IupSetHandle("yc_elp1_handle",  text2);
    IupSetHandle("a_elp1_handle",   text3);
    IupSetHandle("ang_elp1_handle", text4);
    IupSetHandle("b_elp1_handle",   text5);
    IupSetHandle("tol_elp1_handle", text6);
    iupmaskSet(IupGetHandle("xc_elp1_handle" ), IUPMASK_FLOAT ,  0, 1);
    iupmaskSet(IupGetHandle("yc_elp1_handle" ), IUPMASK_FLOAT ,  0, 1);
    iupmaskSet(IupGetHandle("a_elp1_handle"  ), IUPMASK_UFLOAT , 0, 1);
    iupmaskSet(IupGetHandle("ang_elp1_handle"), IUPMASK_FLOAT ,  0, 1);
    iupmaskSet(IupGetHandle("b_elp1_handle"  ), IUPMASK_UFLOAT , 0, 1);
    iupmaskSet(IupGetHandle("tol_elp1_handle"), IUPMASK_UFLOAT , 0, 1);

    /* creates dialog */
    elipse.dialog = IupDialog (
                      IupFrame (
                        IupVbox (
		          IupFill(),
		          t1,
		          IupFill(),
			  IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),
                                  NULL),
		          IupFill(),
			  NULL
		        )
		      )
                    );

    /* sets callbacks */
    IupSetFunction ( "call_elipse_ok", (Icallback)callbackElipseDialogOk );
    IupSetFunction ( "call_elipse_cn", (Icallback)callbackElipseDialogCancel );
    IupSetFunction ( "call_elipse_close", 
                                      (Icallback)callbackElipseDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( elipse.dialog, "GAP",    "3" );
    IupSetAttribute ( elipse.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( elipse.dialog, "MAXBOX", "NO");
    IupSetAttribute ( elipse.dialog, "RESIZE", "NO");
    IupSetAttribute ( elipse.dialog, "CLOSE_CB", "call_elipse_close");
#ifdef _UNIX_
    IupSetAttribute ( elipse.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( elipse.dialog, "TITLE",  "Elipse" );

    IupMap ( elipse.dialog );
  }

  return elipse.dialog;
}

/* ======================  dlgValueToElipse  ======================= */

static void dlgValueToElipse ( double xc,  double yc, double a, double b, 
                               double ang, double tol )
{
 char exc[50];
 char eyc[50];
 char ea[50];
 char eb[50];
 char eang[50];
 char etol[50];

 if ( elipse.dialog == NULL ) return;

 sprintf(exc,  format, xc);
 sprintf(eyc,  format, yc);
 sprintf(ea,   format, a);
 sprintf(eb,   format, b);
 sprintf(eang, format, ang);
 sprintf(etol, format, tol);

 IupSetAttribute (IupGetHandle("xc_elp1_handle"),  "VALUE", exc);
 IupSetAttribute (IupGetHandle("yc_elp1_handle"),  "VALUE", eyc);
 IupSetAttribute (IupGetHandle("a_elp1_handle"),   "VALUE", ea);
 IupSetAttribute (IupGetHandle("b_elp1_handle"),   "VALUE", eb);
 IupSetAttribute (IupGetHandle("ang_elp1_handle"), "VALUE", eang);
 IupSetAttribute (IupGetHandle("tol_elp1_handle"), "VALUE", etol);

}

/* =====================  dlgValueFromElipse  ====================== */

static void dlgValueFromElipse ( double* xc, double* yc, double* a,
				 double* b, double* ang, double* tol  )
{
  if ( elipse.dialog == NULL ) return;

  /* normal case */
  *xc  = IupGetFloat (IupGetHandle("xc_elp1_handle"), "VALUE");
  *yc  = IupGetFloat (IupGetHandle("yc_elp1_handle"), "VALUE");
  *a   = IupGetFloat (IupGetHandle("a_elp1_handle"), "VALUE");
  *b   = IupGetFloat (IupGetHandle("b_elp1_handle"), "VALUE");
  *ang = IupGetFloat (IupGetHandle("ang_elp1_handle"), "VALUE");
  *tol = IupGetFloat (IupGetHandle("tol_elp1_handle"), "VALUE");
  
  if(*tol == 0.0)
  *tol = 0.001;
}

/* =====================  dlgValidateElipse  ======================= */

static int dlgValidateElipse ( void )
{
 char *xc;
 char *yc;
 char *a;
 char *b;
 char *ang;
 char *tol;

 if ( elipse.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(elipse.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 /* normal case */
 xc  = IupGetAttribute (IupGetHandle("xc_elp1_handle"),  "VALUE");
 yc  = IupGetAttribute (IupGetHandle("yc_elp1_handle"),  "VALUE");
 a   = IupGetAttribute (IupGetHandle("a_elp1_handle"),   "VALUE");
 b   = IupGetAttribute (IupGetHandle("b_elp1_handle"),   "VALUE");
 ang = IupGetAttribute (IupGetHandle("ang_elp1_handle"), "VALUE");
 tol = IupGetAttribute (IupGetHandle("tol_elp1_handle"), "VALUE");
    
 return 1;
}   

/* ====================  dlgInit1Elipse  ==================== */

static void dlgInit1Elipse ( ElipseDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel7e; 
   dial->pointlabel1 = llabel1e; 
   dial->pointlabel2 = llabel2e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel7p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* ====================  dlgCreate1Elipse  =================== */

static void* dlgCreate1Elipse ( void )
{
  if ( d1elipse.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( d1elipse.pointbuttn1, "call_d1elipse_ok" );
    Ihandle *butt2 = IupButton ( d1elipse.pointbuttn2, "call_d1elipse_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( d1elipse.pointlabel1 );
    labl2 = IupLabel  ( d1elipse.pointlabel2 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  d1elipse.pointlabel0 );

    /* sets handles */
    IupSetHandle("xc_elp2_handle", text1);
    IupSetHandle("yc_elp2_handle", text2);
    iupmaskSet(IupGetHandle("xc_elp2_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("yc_elp2_handle"), IUPMASK_FLOAT , 0, 1);

    /* creates dialog */
    d1elipse.dialog = IupDialog (
                        IupFrame (
		          IupVbox (
		            IupFill(),
		            t1,
		            IupFill(),
			    IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),
                                    NULL),
		            IupFill(),
			    NULL
		          )
		        )
                      );

    /* sets callbacks */
    IupSetFunction ( "call_d1elipse_ok", (Icallback)callback1ElipseDialogOk );
    IupSetFunction ( "call_d1elipse_cn", 
                                     (Icallback)callback1ElipseDialogCancel );
    IupSetFunction ( "call_d1elipse_close", 
                                     (Icallback)callback1ElipseDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( d1elipse.dialog, "GAP",    "3" );
    IupSetAttribute ( d1elipse.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( d1elipse.dialog, "MAXBOX", "NO");
    IupSetAttribute ( d1elipse.dialog, "RESIZE", "NO");
    IupSetAttribute ( d1elipse.dialog, "CLOSE_CB", "call_d1elipse_close");
#ifdef _UNIX_
    IupSetAttribute ( d1elipse.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( d1elipse.dialog, "TITLE",  "Ellipse Arc" );

    IupMap ( d1elipse.dialog );
  }

  return d1elipse.dialog;
}

/* ===================  dlgValueTo1Elipse  =================== */

static void dlgValueTo1Elipse ( double xc, double yc )
{
 char exc[50];
 char eyc[50];

 if ( d1elipse.dialog == NULL ) return;

 sprintf(exc, format, xc);
 sprintf(eyc, format, yc);

 IupSetAttribute (IupGetHandle("xc_elp2_handle"), "VALUE", exc);
 IupSetAttribute (IupGetHandle("yc_elp2_handle"), "VALUE", eyc);
}

/* =================  dlgValueFrom1Elipse  =================== */

static void dlgValueFrom1Elipse ( double *xc, double *yc )
{
  if ( d1elipse.dialog == NULL ) return;
 
  *xc = IupGetFloat (IupGetHandle("xc_elp2_handle"), "VALUE");
  *yc = IupGetFloat (IupGetHandle("yc_elp2_handle"), "VALUE");
}

/* ==================  dlgValidate1Elipse  =================== */

static int dlgValidate1Elipse ( void )
{
 char *exc;
 char *eyc;

 if ( d1elipse.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(d1elipse.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 exc = IupGetAttribute (IupGetHandle("xc_elp2_handle"), "VALUE");
 eyc = IupGetAttribute (IupGetHandle("yc_elp2_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInit2Elipse  ==================== */

static void dlgInit2Elipse ( ElipseDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel8e; 
   dial->pointlabel1 = llabel3e; 
   dial->pointlabel2 = llabel4e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel8p; 
   dial->pointlabel1 = llabel3p; 
   dial->pointlabel2 = llabel4p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* ====================  dlgCreate2Elipse  =================== */

static void* dlgCreate2Elipse ( void )
{
  if ( d2elipse.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( d2elipse.pointbuttn1, "call_d2elipse_ok" );
    Ihandle *butt2 = IupButton ( d2elipse.pointbuttn2, "call_d2elipse_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( d2elipse.pointlabel1 );
    labl2 = IupLabel  ( d2elipse.pointlabel2 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  d2elipse.pointlabel0);

    /* sets handles */
    IupSetHandle("a_elp3_handle", text1);
    IupSetHandle("ang_elp3_handle", text2);
    iupmaskSet(IupGetHandle("a_elp3_handle"), IUPMASK_UFLOAT , 0, 1);
    iupmaskSet(IupGetHandle("ang_elp3_handle"), IUPMASK_FLOAT , 0, 1);

    /* creates dialog */
    d2elipse.dialog = IupDialog (
                        IupFrame (
	                  IupVbox (
		            IupFill(),
		            t1,
		            IupFill(),
			    IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),
                                    NULL),
		            IupFill(),
			    NULL
		          )
		        )
                      );

    /* sets callbacks */
    IupSetFunction ( "call_d2elipse_ok", (Icallback)callback2ElipseDialogOk );
    IupSetFunction ( "call_d2elipse_cn", 
                                      (Icallback)callback2ElipseDialogCancel );
    IupSetFunction ( "call_d2elipse_close", 
                                      (Icallback)callback2ElipseDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( d2elipse.dialog, "GAP",    "3" );
    IupSetAttribute ( d2elipse.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( d2elipse.dialog, "MAXBOX", "NO");
    IupSetAttribute ( d2elipse.dialog, "RESIZE", "NO");
    IupSetAttribute ( d2elipse.dialog, "CLOSE_CB", "call_d2elipse_close");
#ifdef _UNIX_
    IupSetAttribute ( d2elipse.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( d2elipse.dialog, "TITLE",  "Ellipse Arc" );

    IupMap ( d2elipse.dialog );
  }

  return d2elipse.dialog;
}

/* ===================  dlgValueTo2Elipse  =================== */

static void dlgValueTo2Elipse ( double a, double ang )
{
 char ea[50];
 char eang[50];

 if ( d2elipse.dialog == NULL ) return;

 sprintf(ea, format, a);
 sprintf(eang, format, ang);

 IupSetAttribute (IupGetHandle("a_elp3_handle"), "VALUE", ea);
 IupSetAttribute (IupGetHandle("ang_elp3_handle"), "VALUE", eang);
}

/* =================  dlgValueFrom2Elipse  =================== */

static void dlgValueFrom2Elipse ( double *a, double *ang )
{
  if ( d2elipse.dialog == NULL ) return;

  *a = IupGetFloat (IupGetHandle("a_elp3_handle"), "VALUE");
  *ang = IupGetFloat (IupGetHandle("ang_elp3_handle"), "VALUE");
}

/* ==================  dlgValidate2Elipse  =================== */

static int dlgValidate2Elipse ( void )
{
 char *a;
 char *ang;

 if ( d2elipse.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(d2elipse.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 a = IupGetAttribute (IupGetHandle("a_elp3_handle"), "VALUE");
 ang = IupGetAttribute (IupGetHandle("ang_elp3_handle"), "VALUE");

 return 1;
}

/* =======================  dlgInit3Elipse  ======================= */

static void dlgInit3Elipse ( ElipseDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0  = llabel0e; 
   dial->pointlabel1  = llabel1e; 
   dial->pointlabel2  = llabel2e; 
   dial->pointlabel3  = llabel3e; 
   dial->pointlabel4  = llabel4e; 
   dial->pointlabel5  = llabel5e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0  = llabel0p; 
   dial->pointlabel1  = llabel1p; 
   dial->pointlabel2  = llabel2p; 
   dial->pointlabel3  = llabel3p; 
   dial->pointlabel4  = llabel4p; 
   dial->pointlabel5  = llabel5p;
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* =======================  dlgCreate3Elipse  ======================= */

static void* dlgCreate3Elipse ( void )
{
  if ( d3elipse.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( d3elipse.pointbuttn1, "call_d3elipse_ok" );
    Ihandle *butt2 = IupButton ( d3elipse.pointbuttn2, "call_d3elipse_cn" );

    Ihandle *labl1  = NULL;
    Ihandle *labl2  = NULL;
    Ihandle *labl3  = NULL;
    Ihandle *labl4  = NULL;
    Ihandle *labl5  = NULL;

    Ihandle *text1  = NULL;
    Ihandle *text2  = NULL;
    Ihandle *text3  = NULL;
    Ihandle *text4  = NULL;
    Ihandle *text5  = NULL;

    Ihandle *h1  = NULL;
    Ihandle *h2  = NULL;
    Ihandle *h3  = NULL;
    Ihandle *h4  = NULL;
    Ihandle *h5  = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1  = IupLabel  ( d3elipse.pointlabel1 );
    labl2  = IupLabel  ( d3elipse.pointlabel2 );
    labl3  = IupLabel  ( d3elipse.pointlabel3 );
    labl4  = IupLabel  ( d3elipse.pointlabel4 );
    labl5  = IupLabel  ( d3elipse.pointlabel5 );
    text1  = IupText   ( NULL );
    text2  = IupText   ( NULL );
    text3  = IupText   ( NULL );
    text4  = IupText   ( NULL );
    text5  = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);
    h4 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl4,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text4,IupFill(),NULL), 
		  IupFill(), NULL);
    h5 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl5,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text5,IupFill(),NULL), 
		  IupFill(), NULL);
    
    t1 = IupFrame (
	     IupVbox (
	        IupFill(),
	        h1,
	        IupFill(),
		h2,
	        IupFill(),
		h3,
	        IupFill(),
		h4,
	        IupFill(),
		h5,
	        IupFill(),
		NULL
	      )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text5, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text5, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( text5, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );
    IupSetAttribute ( labl5, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  d3elipse.pointlabel0 );

    /* sets handles */
    IupSetHandle("xc_elp4_handle",  text1);
    IupSetHandle("yc_elp4_handle",  text2);
    IupSetHandle("a_elp4_handle",   text3);
    IupSetHandle("ang_elp4_handle", text4);
    IupSetHandle("b_elp4_handle",   text5);
   
    iupmaskSet(IupGetHandle("xc_elp4_handle" ), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("yc_elp4_handle" ), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("a_elp4_handle"  ), IUPMASK_UFLOAT , 0, 1);
    iupmaskSet(IupGetHandle("ang_elp4_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("b_elp4_handle"  ), IUPMASK_UFLOAT , 0, 1);

    /* creates dialog */
    d3elipse.dialog = IupDialog (
                        IupFrame (
                          IupVbox (
		                    IupFill(),
		                    t1,
		                    IupFill(),
			            IupHbox(IupFill(),butt1,IupFill(),
                                            butt2,IupFill(),NULL),
		                    IupFill(), NULL)
		                )
                      );

    /* sets callbacks */
    IupSetFunction ( "call_d3elipse_ok", (Icallback)callback3ElipseDialogOk );
    IupSetFunction ( "call_d3elipse_cn", 
                                      (Icallback)callback3ElipseDialogCancel );
    IupSetFunction ( "call_d3elipse_close", 
                                     (Icallback)callback3ElipseDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( d3elipse.dialog, "GAP",    "3" );
    IupSetAttribute ( d3elipse.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( d3elipse.dialog, "MAXBOX", "NO");
    IupSetAttribute ( d3elipse.dialog, "RESIZE", "NO");
    IupSetAttribute ( d3elipse.dialog, "CLOSE_CB", "call_d3elipse_close");
#ifdef _UNIX_
    IupSetAttribute ( d3elipse.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( d3elipse.dialog, "TITLE",  "Ellipse Arc" );

    IupMap ( d3elipse.dialog );
  }

  return d3elipse.dialog;
}

/* ======================  dlgValueTo3Elipse  ======================= */

static void dlgValueTo3Elipse ( double xc,  double yc, double a, double b,
                                double ang )
{
 char exc[50];
 char eyc[50];
 char ea[50];
 char eb[50];
 char eang[50];

 if ( d3elipse.dialog == NULL ) return;

 sprintf(exc,  format, xc);
 sprintf(eyc,  format, yc);
 sprintf(ea,   format, a);
 sprintf(eb,   format, b);
 sprintf(eang, format, ang);

 IupSetAttribute (IupGetHandle("xc_elp4_handle"),  "VALUE", exc);
 IupSetAttribute (IupGetHandle("yc_elp4_handle"),  "VALUE", eyc);
 IupSetAttribute (IupGetHandle("a_elp4_handle"),   "VALUE", ea);
 IupSetAttribute (IupGetHandle("b_elp4_handle"),   "VALUE", eb);
 IupSetAttribute (IupGetHandle("ang_elp4_handle"), "VALUE", eang);
}

/* =====================  dlgValueFrom3Elipse  ====================== */

static void dlgValueFrom3Elipse ( double* xc,  double* yc, double* a, 
                                  double* b, double* ang )
{
  if ( d3elipse.dialog == NULL ) return;

  /* normal case */
  *xc  = IupGetFloat (IupGetHandle("xc_elp4_handle"),  "VALUE");
  *yc  = IupGetFloat (IupGetHandle("yc_elp4_handle"),  "VALUE");
  *a   = IupGetFloat (IupGetHandle("a_elp4_handle"),   "VALUE");
  *b   = IupGetFloat (IupGetHandle("b_elp4_handle"),   "VALUE");
  *ang = IupGetFloat (IupGetHandle("ang_elp4_handle"), "VALUE");
}

/* =====================  dlgValidate3Elipse  ======================= */

static int dlgValidate3Elipse ( void )
{
 char *xc;
 char *yc;
 char *a;
 char *b;
 char *ang;

 if ( d3elipse.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(d3elipse.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 /* normal case */
 xc  = IupGetAttribute (IupGetHandle("xc_elp4_handle"),  "VALUE");
 yc  = IupGetAttribute (IupGetHandle("yc_elp4_handle"),  "VALUE");
 a   = IupGetAttribute (IupGetHandle("a_elp4_handle"),   "VALUE");
 b   = IupGetAttribute (IupGetHandle("b_elp4_handle"),   "VALUE");
 ang = IupGetAttribute (IupGetHandle("ang_elp4_handle"), "VALUE");
    
 return 1;
}   

/* ========================  callbackElipseDialogOk  ======================== */

static int callbackElipseDialogOk ( Ihandle *bt )
{
 IupSetAttribute(elipse.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ======================  callbackElipseDialogCancel  ====================== */

static int callbackElipseDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(elipse.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackElipseDialogCloseCB  ==================== */

static int callbackElipseDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(elipse.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callback1ElipseDialogOk  ====================== */

static int callback1ElipseDialogOk ( Ihandle *bt )
{
 IupSetAttribute(d1elipse.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callback1ElipseDialogCancel  ==================== */

static int callback1ElipseDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(d1elipse.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ====================  callback1ElipseDialogCloseCB  ==================== */

static int callback1ElipseDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(d1elipse.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackS2ElipseDialogOk  ====================== */

static int callback2ElipseDialogOk ( Ihandle *bt )
{
 IupSetAttribute(d2elipse.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callback2ElipseDialogCancel  ==================== */

static int callback2ElipseDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(d2elipse.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ====================  callback2ElipseDialogCloseCB  =================== */

static int callback2ElipseDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(d2elipse.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackS3ElipseDialogOk  ====================== */

static int callback3ElipseDialogOk ( Ihandle *bt )
{
 IupSetAttribute(d3elipse.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callback3ElipseDialogCancel  ==================== */

static int callback3ElipseDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(d3elipse.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ====================  callback3ElipseDialogCloseCB  =================== */

static int callback3ElipseDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(d3elipse.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* =========================  DlgCaptureElipse  ======================== */

int DlgCaptureElipse ( double *xc, double *yc, double *a, double *b, 
                       double *ang, double* tol )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitElipse( &elipse );

  /* create dialog to capture point data. */
  d = dlgCreateElipse();

  /* sets point data to dialog. */
  dlgValueToElipse ( *xc, *yc, *a, *b, *ang, *tol );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateElipse ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromElipse ( xc, yc, a, b, ang, tol );

  return 1;
}

/* ======================  DlgCapture1Elipse  ===================== */

int DlgCapture1Elipse ( double *xc, double *yc )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInit1Elipse( &d1elipse );

  /* create dialog to capture point data. */
  d = dlgCreate1Elipse();

  /* sets point data to dialog. */
  dlgValueTo1Elipse ( *xc, *yc );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidate1Elipse ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFrom1Elipse ( xc, yc );

  return 1;
}

/* ======================  DlgCapture2Elipse  ===================== */

int DlgCapture2Elipse ( double *a, double *ang )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInit2Elipse( &d2elipse );

  /* create dialog to capture point data. */
  d = dlgCreate2Elipse();

  /* sets point data to dialog. */
  dlgValueTo2Elipse ( *a, *ang );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidate2Elipse ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFrom2Elipse ( a, ang );

  return 1;
}

/* ======================  DlgCapture3Elipse  ===================== */

int DlgCapture3Elipse ( double *xc, double *yc, double *a, double *b, 
                        double *ang )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInit3Elipse( &d3elipse );

  /* create dialog to capture point data. */
  d = dlgCreate3Elipse();

  /* sets point data to dialog. */
  dlgValueTo3Elipse ( *xc, *yc, *a, *b, *ang );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidate3Elipse ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFrom3Elipse ( xc, yc, a, b, ang );

  return 1;
}

