/*
** dlgelparc.c - Functions that establish the pre-defined dialogs 
**               of the library to capture ellipse arcs.
**
**
** Public functions:
** --------------------------------------------------------------------
**
** int DlgCaptureElpArc ( double *xc, double *yc, double *a, double *b, 
**                        double *ang, double *x0, double *y0, double *x1, 
**                        double *y1, double *tol )
**
** xc  - coordinate (X direction) of center.           (IN/OUT)
** yc  - coordinate (Y direction) of center.           (IN/OUT)
** a   - first half-axis of elipse.                    (IN/OUT)
** b   - second half-axis of elipse.                   (IN/OUT)
** ang - angle between the half-axis with length defined by the 'a' variable
**       and the horizontal axis (X direction).        (IN/OUT)
** x0 - coordinate (X direction) of first arc point.   (IN/OUT)
** y0 - coordinate (X direction) of first arc point.   (IN/OUT)
** x1 - coordinate (X direction) of second arc point.  (IN/OUT)
** y1 - coordinate (X direction) of second arc point.  (IN/OUT)
**
** This function creates a dialog to capture the parameters of an ellipse arc. 
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCapture1ElpArc( double *xc, double *yc )
**
** xc  - coordinate (X direction) of center.       (IN/OUT)
** yc  - coordinate (Y direction) of center.       (IN/OUT)
**
** This function creates a dialog to capture the parameters of an ellipse arc.
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCapture2ElpArc( double *a, double *ang )
**
** a   - first half-axis of elipse.              (IN/OUT)
** ang - angle between the half-axis with length defined by the 'a' variable
**       and the horizontal axis (X direction).  (IN/OUT)
**
** This function creates a dialog to capture the parameters of an ellipse arc.
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCapture3ElpArc( double *b )
**
** b - second half-axis of elipse. (IN/OUT)
**
** This function creates a dialog to capture the parameters of an ellipse arc.
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCapture4ElpArc( double *x0, double *y0 )
**
** x0 - coordinate (X direction) of first arc point.   (IN/OUT)
** y0 - coordinate (X direction) of first arc point.   (IN/OUT)
**
** This function creates a dialog to capture the parameters of an ellipse arc.
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCapture5ElpArc( double *xc, double *yc, double *a, double *b, 
**                        double *ang, double *p0x, double *p0y, 
**                        double *p1x, double *p1y )
**
** xc  - coordinate (X direction) of center.           (IN/OUT)
** yc  - coordinate (Y direction) of center.           (IN/OUT)
** a   - first half-axis of elipse.                    (IN/OUT)
** b   - second half-axis of elipse.                   (IN/OUT)
** ang - angle between the half-axis with length defined by the 'a' variable
**       and the horizontal axis (X direction).        (IN/OUT)
** x0 - coordinate (X direction) of first arc point.   (IN/OUT)
** y0 - coordinate (X direction) of first arc point.   (IN/OUT)
** x1 - coordinate (X direction) of second arc point.  (IN/OUT)
** y1 - coordinate (X direction) of second arc point.  (IN/OUT)
**
** This function creates a dialog to capture the parameters of an ellipse arc.
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
**
** Version:    1-000
**
** Created:    11-Mar-05    Marcos C. de Arruda.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "iup.h"
#include "iupmask.h"
#include "iupcontrols.h"

#include "dlgelparc.h"


static char* format = "%.6f";
static int   d_posx = IUP_CENTER;
static int   d_posy = IUP_CENTER;

/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/

/* Data structure */

typedef struct _elparcdlg
{

  char *pointlabel0;
  char *pointlabel1;
  char *pointlabel2;
  char *pointlabel3;
  char *pointlabel4;
  char *pointlabel5;
  char *pointlabel6;
  char *pointlabel7;
  char *pointlabel8;
  char *pointlabel9;
  char *pointlabel10;
  char *pointlabel11;
  char *pointlabel12;
  char *pointlabel13;
  char *pointlabel14;
  char *pointlabel15;
  char *pointbuttn1;
  char *pointbuttn2;

  Ihandle *dialog;

} ElpArcDlg;


static void  dlgInitElpArc      ( ElpArcDlg *dial );
static void* dlgCreateElpArc    ( void );
static void  dlgValueToElpArc   ( double xc,  double yc, double a, double b, 
                                  double ang, double p0x, double p0y,
                                  double p1x, double p1y, double tol );
static int   dlgValidateElpArc  ( void );
static void  dlgValueFromElpArc ( double* xc, double* yc, double* a, 
                                  double* b, double* ang, double* p0x, 
                                  double* p0y, double* p1x, double* p1y, 
                                  double* tol );

static void  dlgInit1ElpArc      ( ElpArcDlg *dial );
static void* dlgCreate1ElpArc    ( void );
static void  dlgValueTo1ElpArc   ( double xc, double yc );
static int   dlgValidate1ElpArc  ( void );
static void  dlgValueFrom1ElpArc ( double *xc, double *yc );

static void  dlgInit2ElpArc      ( ElpArcDlg *dial );
static void* dlgCreate2ElpArc    ( void );
static void  dlgValueTo2ElpArc   ( double a, double ang );
static int   dlgValidate2ElpArc  ( void );
static void  dlgValueFrom2ElpArc ( double *a, double *ang );

static void  dlgInit3ElpArc      ( ElpArcDlg *dial );
static void* dlgCreate3ElpArc    ( void );
static void  dlgValueTo3ElpArc   ( double b );
static int   dlgValidate3ElpArc  ( void );
static void  dlgValueFrom3ElpArc ( double *b );

static void  dlgInit4ElpArc      ( ElpArcDlg *dial );
static void* dlgCreate4ElpArc    ( void );
static void  dlgValueTo4ElpArc   ( double p0x, double p0y );
static int   dlgValidate4ElpArc  ( void );
static void  dlgValueFrom4ElpArc ( double *p0x, double *p0y );

static void  dlgInit5ElpArc      ( ElpArcDlg *dial );
static void* dlgCreate5ElpArc    ( void );
static void  dlgValueTo5ElpArc   ( double xc,  double yc, double a, double b,
                                   double ang, double p0x, double p0y,
                                   double p1x, double p1y );
static int   dlgValidate5ElpArc  ( void );
static void  dlgValueFrom5ElpArc ( double* xc, double* yc, double* a, 
                                   double* b, double* ang, double* p0x, 
                                   double* p0y, double* p1x, double* p1y );


static int  callbackElpArcDialogOk       ( Ihandle *bt );
static int  callbackElpArcDialogCancel   ( Ihandle *bt );
static int  callbackElpArcDialogCloseCB  ( Ihandle *bt );

static int  callback1ElpArcDialogOk      ( Ihandle *bt );
static int  callback1ElpArcDialogCancel  ( Ihandle *bt );
static int  callback1ElpArcDialogCloseCB ( Ihandle *bt );

static int  callback2ElpArcDialogOk      ( Ihandle *bt );
static int  callback2ElpArcDialogCancel  ( Ihandle *bt );
static int  callback2ElpArcDialogCloseCB ( Ihandle *bt );

static int  callback3ElpArcDialogOk      ( Ihandle *bt );
static int  callback3ElpArcDialogCancel  ( Ihandle *bt );
static int  callback3ElpArcDialogCloseCB ( Ihandle *bt );

static int  callback4ElpArcDialogOk      ( Ihandle *bt );
static int  callback4ElpArcDialogCancel  ( Ihandle *bt );
static int  callback4ElpArcDialogCloseCB ( Ihandle *bt );

static int  callback5ElpArcDialogOk      ( Ihandle *bt );
static int  callback5ElpArcDialogCancel  ( Ihandle *bt );
static int  callback5ElpArcDialogCloseCB ( Ihandle *bt );

static ElpArcDlg elparc;
static ElpArcDlg d1elparc;
static ElpArcDlg d2elparc;
static ElpArcDlg d3elparc;
static ElpArcDlg d4elparc;
static ElpArcDlg d5elparc;

static char *size_butt = {"55x12"};
static char *size_text = {"110"};
static char *font_dial = {"TIMES_NORMAL_12"};

static char *llabel0e = { "Enter with ellipse arc parameters: " };
static char *llabel1e = { "Center X:  " };
static char *llabel2e = { "Center Y:  " };
static char *llabel3e = { "First half-axis (a):  " };
static char *llabel4e = { "Rotation angle (lambda):  " };
static char *llabel5e = { "Second half-axis (b):  " };
static char *llabel6e = { "Point1 X:  " };
static char *llabel7e = { "Point1 Y:  " };
static char *llabel8e = { "Point2 X:  " };
static char *llabel9e = { "Point2 Y:  " };
static char *llabel10e = { "Tolerance: " };
static char *lbuttn1e = { " OK " };
static char *lbuttn2e = { " Cancel " };

static char *llabel11e  = { "Enter with center coordinates:" };
static char *llabel12e  = { "Enter with 'a' and 'lambda'"};
static char *llabel13e  = { "Enter with 'b'"};
static char *llabel14e  = { "Enter with first point coordinates:" };

static char *llabel0p  = { "Entre com os parametros do arco de elipse: " };
static char *llabel1p  = { "Centro X:  " };
static char *llabel2p  = { "Centro Y:  " };
static char *llabel3p  = { "Primeiro semi-eixo (a):  " };
static char *llabel4p  = { "Angulo de rotacao (lambda):  " };
static char *llabel5p  = { "Segundo semi-eixo (b):  " };
static char *llabel6p  = { "Ponto1 X:  " };
static char *llabel7p  = { "Ponto1 Y:  " };
static char *llabel8p  = { "Ponto2 X:  " };
static char *llabel9p  = { "Ponto2 Y:  " };
static char *llabel10p = { "Tolerancia: " };
static char *lbuttn1p = { " Confirma " };
static char *lbuttn2p = { " Cancela " };

static char *llabel11p = { "Entre com as coordenadas do centro:" };
static char *llabel12p = { "Entre com o valor de 'a' e 'lambda'"};
static char *llabel13p = { "Entre com o valor de 'b'"};
static char *llabel14p = { "Entre com as coordenadas do primeiro ponto "  
                           "do arco:"};

/*
** --------------------------------------------------------------------
** Local functions:
*/

/* =======================  dlgInitElpArc  ======================= */

static void dlgInitElpArc ( ElpArcDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0  = llabel0e; 
   dial->pointlabel1  = llabel1e; 
   dial->pointlabel2  = llabel2e; 
   dial->pointlabel3  = llabel3e; 
   dial->pointlabel4  = llabel4e; 
   dial->pointlabel5  = llabel5e; 
   dial->pointlabel6  = llabel6e; 
   dial->pointlabel7  = llabel7e; 
   dial->pointlabel8  = llabel8e; 
   dial->pointlabel9  = llabel9e; 
   dial->pointlabel10 = llabel10e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0  = llabel0p; 
   dial->pointlabel1  = llabel1p; 
   dial->pointlabel2  = llabel2p; 
   dial->pointlabel3  = llabel3p; 
   dial->pointlabel4  = llabel4p; 
   dial->pointlabel5  = llabel5p; 
   dial->pointlabel6  = llabel6p; 
   dial->pointlabel7  = llabel7p;
   dial->pointlabel8  = llabel8p; 
   dial->pointlabel9  = llabel9p; 
   dial->pointlabel10 = llabel10p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* =======================  dlgCreateElpArc  ======================= */

static void* dlgCreateElpArc ( void )
{
  if ( elparc.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( elparc.pointbuttn1, "call_elparc_ok" );
    Ihandle *butt2 = IupButton ( elparc.pointbuttn2, "call_elparc_cn" );

    Ihandle *labl1  = NULL;
    Ihandle *labl2  = NULL;
    Ihandle *labl3  = NULL;
    Ihandle *labl4  = NULL;
    Ihandle *labl5  = NULL;
    Ihandle *labl6  = NULL;
    Ihandle *labl7  = NULL;
    Ihandle *labl8  = NULL;
    Ihandle *labl9  = NULL;
    Ihandle *labl10 = NULL;

    Ihandle *text1  = NULL;
    Ihandle *text2  = NULL;
    Ihandle *text3  = NULL;
    Ihandle *text4  = NULL;
    Ihandle *text5  = NULL;
    Ihandle *text6  = NULL;
    Ihandle *text7  = NULL;
    Ihandle *text8  = NULL;
    Ihandle *text9  = NULL;
    Ihandle *text10 = NULL;

    Ihandle *h1  = NULL;
    Ihandle *h2  = NULL;
    Ihandle *h3  = NULL;
    Ihandle *h4  = NULL;
    Ihandle *h5  = NULL;
    Ihandle *h6  = NULL;
    Ihandle *h7  = NULL;
    Ihandle *h8  = NULL;
    Ihandle *h9  = NULL;
    Ihandle *h10 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1  = IupLabel  ( elparc.pointlabel1 );
    labl2  = IupLabel  ( elparc.pointlabel2 );
    labl3  = IupLabel  ( elparc.pointlabel3 );
    labl4  = IupLabel  ( elparc.pointlabel4 );
    labl5  = IupLabel  ( elparc.pointlabel5 );
    labl6  = IupLabel  ( elparc.pointlabel6 );
    labl7  = IupLabel  ( elparc.pointlabel7 );
    labl8  = IupLabel  ( elparc.pointlabel8 );
    labl9  = IupLabel  ( elparc.pointlabel9 );
    labl10 = IupLabel  ( elparc.pointlabel10 );
    text1  = IupText   ( NULL );
    text2  = IupText   ( NULL );
    text3  = IupText   ( NULL );
    text4  = IupText   ( NULL );
    text5  = IupText   ( NULL );
    text6  = IupText   ( NULL );
    text7  = IupText   ( NULL );
    text8  = IupText   ( NULL );
    text9  = IupText   ( NULL );
    text10 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);
    h4 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl4,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text4,IupFill(),NULL), 
		  IupFill(), NULL);
    h5 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl5,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text5,IupFill(),NULL), 
		  IupFill(), NULL);
    h6 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl6,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text6,IupFill(),NULL), 
		  IupFill(), NULL);
    h7 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl7,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text7,IupFill(),NULL), 
		  IupFill(), NULL);
    h8 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl8,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text8,IupFill(),NULL), 
		  IupFill(), NULL);
    h9 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl9,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text9,IupFill(),NULL), 
		  IupFill(), NULL);
    h10 = IupHbox (IupFill(), 
                   IupVbox(IupFill(),labl10,IupFill(),NULL), 
		   IupFill(), 
		   IupVbox(IupFill(),text10,IupFill(),NULL), 
		   IupFill(), NULL);		  		  

    t1 = IupFrame (
	     IupVbox (
	        IupFill(),
	        h1,
	        IupFill(),
		h2,
	        IupFill(),
		h3,
	        IupFill(),
		h4,
	        IupFill(),
		h5,
	        IupFill(),
		h6,
	        IupFill(),
		h7,
	        IupFill(),
		h8,
	        IupFill(),
	        h9,
	        IupFill(),
	        h10,
	        IupFill(),
		NULL
	      )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text5, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text6, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text7, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text8, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text9, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text10, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text5, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text6, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text7, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text8, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text9, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text10, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( text5, "SIZE", size_text );
    IupSetAttribute ( text6, "SIZE", size_text );
    IupSetAttribute ( text7, "SIZE", size_text );
    IupSetAttribute ( text8, "SIZE", size_text );
    IupSetAttribute ( text9, "SIZE", size_text );
    IupSetAttribute ( text10, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );
    IupSetAttribute ( labl5, "SIZE", size_text );
    IupSetAttribute ( labl6, "SIZE", size_text );
    IupSetAttribute ( labl7, "SIZE", size_text );
    IupSetAttribute ( labl8, "SIZE", size_text );
    IupSetAttribute ( labl9, "SIZE", size_text );
    IupSetAttribute ( labl10, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  elparc.pointlabel0 );

    /* sets handles */
    IupSetHandle("xc_arc1_handle",  text1);
    IupSetHandle("yc_arc1_handle",  text2);
    IupSetHandle("a_arc1_handle",   text3);
    IupSetHandle("ang_arc1_handle", text4);
    IupSetHandle("b_arc1_handle",   text5);
    IupSetHandle("p0x_arc1_handle", text6);
    IupSetHandle("p0y_arc1_handle", text7);
    IupSetHandle("p1x_arc1_handle", text8);
    IupSetHandle("p1y_arc1_handle", text9);
    IupSetHandle("tol_arc1_handle", text10);
    iupmaskSet(IupGetHandle("xc_arc1_handle" ), IUPMASK_FLOAT ,  0, 1);
    iupmaskSet(IupGetHandle("yc_arc1_handle" ), IUPMASK_FLOAT ,  0, 1);
    iupmaskSet(IupGetHandle("a_arc1_handle"  ), IUPMASK_UFLOAT , 0, 1);
    iupmaskSet(IupGetHandle("ang_arc1_handle"), IUPMASK_FLOAT ,  0, 1);
    iupmaskSet(IupGetHandle("b_arc1_handle"  ), IUPMASK_UFLOAT , 0, 1);
    iupmaskSet(IupGetHandle("p0x_arc1_handle"), IUPMASK_FLOAT ,  0, 1);
    iupmaskSet(IupGetHandle("p0y_arc1_handle"), IUPMASK_FLOAT ,  0, 1);
    iupmaskSet(IupGetHandle("p1x_arc1_handle"), IUPMASK_FLOAT ,  0, 1);
    iupmaskSet(IupGetHandle("p1y_arc1_handle"), IUPMASK_FLOAT ,  0, 1);
    iupmaskSet(IupGetHandle("tol_arc1_handle"), IUPMASK_UFLOAT , 0, 1);

    /* creates dialog */
    elparc.dialog = IupDialog (
                      IupFrame (
                        IupVbox (
		          IupFill(),
		          t1,
		          IupFill(),
			  IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),
                                  NULL),
		          IupFill(),
			  NULL
		        )
		      )
                    );

    /* sets callbacks */
    IupSetFunction ( "call_elparc_ok", (Icallback)callbackElpArcDialogOk );
    IupSetFunction ( "call_elparc_cn", (Icallback)callbackElpArcDialogCancel );
    IupSetFunction ( "call_elparc_close", 
                                      (Icallback)callbackElpArcDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( elparc.dialog, "GAP",    "3" );
    IupSetAttribute ( elparc.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( elparc.dialog, "MAXBOX", "NO");
    IupSetAttribute ( elparc.dialog, "RESIZE", "NO");
    IupSetAttribute ( elparc.dialog, "CLOSE_CB", "call_elparc_close");
#ifdef _UNIX_
    IupSetAttribute ( elparc.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( elparc.dialog, "TITLE",  "Ellipse Arc" );

    IupMap ( elparc.dialog );
  }

  return elparc.dialog;
}

/* ======================  dlgValueToElpArc  ======================= */

static void dlgValueToElpArc ( double xc,  double yc, double a, double b, 
                               double ang, double p0x, double p0y,
                               double p1x, double p1y, double tol )
{
 char exc[50];
 char eyc[50];
 char ea[50];
 char eb[50];
 char eang[50];
 char ep0x[50];
 char ep0y[50];
 char ep1x[50];
 char ep1y[50];    
 char etol[50];

 if ( elparc.dialog == NULL ) return;

 sprintf(exc,  format, xc);
 sprintf(eyc,  format, yc);
 sprintf(ea,   format, a);
 sprintf(eb,   format, b);
 sprintf(eang, format, ang);
 sprintf(ep0x, format, p0x);
 sprintf(ep0y, format, p0y);
 sprintf(ep1x, format, p1x);
 sprintf(ep1y, format, p1y);
 sprintf(etol, format, tol);

 IupSetAttribute (IupGetHandle("xc_arc1_handle"),  "VALUE", exc);
 IupSetAttribute (IupGetHandle("yc_arc1_handle"),  "VALUE", eyc);
 IupSetAttribute (IupGetHandle("a_arc1_handle"),   "VALUE", ea);
 IupSetAttribute (IupGetHandle("b_arc1_handle"),   "VALUE", eb);
 IupSetAttribute (IupGetHandle("ang_arc1_handle"), "VALUE", eang);
 IupSetAttribute (IupGetHandle("p0x_arc1_handle"), "VALUE", ep0x);
 IupSetAttribute (IupGetHandle("p0y_arc1_handle"), "VALUE", ep0y);
 IupSetAttribute (IupGetHandle("p1x_arc1_handle"), "VALUE", ep1x);
 IupSetAttribute (IupGetHandle("p1y_arc1_handle"), "VALUE", ep1y);
 IupSetAttribute (IupGetHandle("tol_arc1_handle"), "VALUE", etol);

}

/* =====================  dlgValueFromElpArc  ====================== */

static void dlgValueFromElpArc ( double* xc, double* yc, double* a,
				 double* b, double* ang, double* p0x,
				 double* p0y, double* p1x, double* p1y,
				 double* tol  )
{
  if ( elparc.dialog == NULL ) return;

  /* normal case */
  *xc  = IupGetFloat (IupGetHandle("xc_arc1_handle"), "VALUE");
  *yc  = IupGetFloat (IupGetHandle("yc_arc1_handle"), "VALUE");
  *a   = IupGetFloat (IupGetHandle("a_arc1_handle"), "VALUE");
  *b   = IupGetFloat (IupGetHandle("b_arc1_handle"), "VALUE");
  *ang = IupGetFloat (IupGetHandle("ang_arc1_handle"), "VALUE");
  *p0x = IupGetFloat (IupGetHandle("p0x_arc1_handle"), "VALUE");
  *p0y = IupGetFloat (IupGetHandle("p0y_arc1_handle"), "VALUE");
  *p1x = IupGetFloat (IupGetHandle("p1x_arc1_handle"), "VALUE");
  *p1y = IupGetFloat (IupGetHandle("p1y_arc1_handle"), "VALUE");
  *tol = IupGetFloat (IupGetHandle("tol_arc1_handle"), "VALUE");
  
  if(*tol == 0.0)
  *tol = 0.001;
}

/* =====================  dlgValidateElpArc  ======================= */

static int dlgValidateElpArc ( void )
{
 char *xc;
 char *yc;
 char *a;
 char *b;
 char *ang;
 char *p0x;
 char *p0y;
 char *p1x;
 char *p1y;
 char *tol;

 if ( elparc.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(elparc.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 /* normal case */
 xc  = IupGetAttribute (IupGetHandle("xc_arc1_handle"),  "VALUE");
 yc  = IupGetAttribute (IupGetHandle("yc_arc1_handle"),  "VALUE");
 a   = IupGetAttribute (IupGetHandle("a_arc1_handle"),   "VALUE");
 b   = IupGetAttribute (IupGetHandle("b_arc1_handle"),   "VALUE");
 ang = IupGetAttribute (IupGetHandle("ang_arc1_handle"), "VALUE");
 p0x = IupGetAttribute (IupGetHandle("p0x_arc1_handle"), "VALUE");
 p0y = IupGetAttribute (IupGetHandle("p0y_arc1_handle"), "VALUE");
 p1x = IupGetAttribute (IupGetHandle("p1x_arc1_handle"), "VALUE");
 p1y = IupGetAttribute (IupGetHandle("p1y_arc1_handle"), "VALUE");
 tol = IupGetAttribute (IupGetHandle("tol_arc1_handle"), "VALUE");
    
 return 1;
}   

/* ====================  dlgInit1ElpArc  ==================== */

static void dlgInit1ElpArc ( ElpArcDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel11e; 
   dial->pointlabel1 = llabel1e; 
   dial->pointlabel2 = llabel2e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel11p; 
   dial->pointlabel1 = llabel1p; 
   dial->pointlabel2 = llabel2p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* ====================  dlgCreate1ElpArc  =================== */

static void* dlgCreate1ElpArc ( void )
{
  if ( d1elparc.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( d1elparc.pointbuttn1, "call_d1elparc_ok" );
    Ihandle *butt2 = IupButton ( d1elparc.pointbuttn2, "call_d1elparc_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( d1elparc.pointlabel1 );
    labl2 = IupLabel  ( d1elparc.pointlabel2 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  d1elparc.pointlabel0 );

    /* sets handles */
    IupSetHandle("xc_arc2_handle", text1);
    IupSetHandle("yc_arc2_handle", text2);
    iupmaskSet(IupGetHandle("xc_arc2_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("yc_arc2_handle"), IUPMASK_FLOAT , 0, 1);

    /* creates dialog */
    d1elparc.dialog = IupDialog (
                        IupFrame (
		          IupVbox (
		            IupFill(),
		            t1,
		            IupFill(),
			    IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),
                                    NULL),
		            IupFill(),
			    NULL
		          )
		        )
                      );

    /* sets callbacks */
    IupSetFunction ( "call_d1elparc_ok", (Icallback)callback1ElpArcDialogOk );
    IupSetFunction ( "call_d1elparc_cn", 
                                      (Icallback)callback1ElpArcDialogCancel );
    IupSetFunction ( "call_d1elparc_close", 
                                     (Icallback)callback1ElpArcDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( d1elparc.dialog, "GAP",    "3" );
    IupSetAttribute ( d1elparc.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( d1elparc.dialog, "MAXBOX", "NO");
    IupSetAttribute ( d1elparc.dialog, "RESIZE", "NO");
    IupSetAttribute ( d1elparc.dialog, "CLOSE_CB", "call_d1elparc_close");
#ifdef _UNIX_
    IupSetAttribute ( d1elparc.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( d1elparc.dialog, "TITLE",  "Ellipse Arc" );

    IupMap ( d1elparc.dialog );
  }

  return d1elparc.dialog;
}

/* ===================  dlgValueTo1ElpArc  =================== */

static void dlgValueTo1ElpArc ( double xc, double yc )
{
 char exc[50];
 char eyc[50];

 if ( d1elparc.dialog == NULL ) return;

 sprintf(exc, format, xc);
 sprintf(eyc, format, yc);

 IupSetAttribute (IupGetHandle("xc_arc2_handle"), "VALUE", exc);
 IupSetAttribute (IupGetHandle("yc_arc2_handle"), "VALUE", eyc);
}

/* =================  dlgValueFrom1ElpArc  =================== */

static void dlgValueFrom1ElpArc ( double *xc, double *yc )
{
  if ( d1elparc.dialog == NULL ) return;
 
  *xc = IupGetFloat (IupGetHandle("xc_arc2_handle"), "VALUE");
  *yc = IupGetFloat (IupGetHandle("yc_arc2_handle"), "VALUE");
}

/* ==================  dlgValidate1ElpArc  =================== */

static int dlgValidate1ElpArc ( void )
{
 char *exc;
 char *eyc;

 if ( d1elparc.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(d1elparc.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 exc = IupGetAttribute (IupGetHandle("xc_arc2_handle"), "VALUE");
 eyc = IupGetAttribute (IupGetHandle("yc_arc2_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInit2ElpArc  ==================== */

static void dlgInit2ElpArc ( ElpArcDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel12e; 
   dial->pointlabel1 = llabel3e; 
   dial->pointlabel2 = llabel4e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel12p; 
   dial->pointlabel1 = llabel3p; 
   dial->pointlabel2 = llabel4p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* ====================  dlgCreate2ElpArc  =================== */

static void* dlgCreate2ElpArc ( void )
{
  if ( d2elparc.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( d2elparc.pointbuttn1, "call_d2elparc_ok" );
    Ihandle *butt2 = IupButton ( d2elparc.pointbuttn2, "call_d2elparc_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( d2elparc.pointlabel1 );
    labl2 = IupLabel  ( d2elparc.pointlabel2 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  d2elparc.pointlabel0);

    /* sets handles */
    IupSetHandle("a_arc3_handle", text1);
    IupSetHandle("ang_arc3_handle", text2);
    iupmaskSet(IupGetHandle("a_arc3_handle"), IUPMASK_UFLOAT , 0, 1);
    iupmaskSet(IupGetHandle("ang_arc3_handle"), IUPMASK_FLOAT , 0, 1);

    /* creates dialog */
    d2elparc.dialog = IupDialog (
                        IupFrame (
	                  IupVbox (
		            IupFill(),
		            t1,
		            IupFill(),
			    IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),
                                    NULL),
		            IupFill(),
			    NULL
		          )
		        )
                      );

    /* sets callbacks */
    IupSetFunction ( "call_d2elparc_ok", (Icallback)callback2ElpArcDialogOk );
    IupSetFunction ( "call_d2elparc_cn", 
                                      (Icallback)callback2ElpArcDialogCancel );
    IupSetFunction ( "call_d2elparc_close", 
                                      (Icallback)callback2ElpArcDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( d2elparc.dialog, "GAP",    "3" );
    IupSetAttribute ( d2elparc.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( d2elparc.dialog, "MAXBOX", "NO");
    IupSetAttribute ( d2elparc.dialog, "RESIZE", "NO");
    IupSetAttribute ( d2elparc.dialog, "CLOSE_CB", "call_d2elparc_close");
#ifdef _UNIX_
    IupSetAttribute ( d2elparc.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( d2elparc.dialog, "TITLE",  "Ellipse Arc" );

    IupMap ( d2elparc.dialog );
  }

  return d2elparc.dialog;
}

/* ===================  dlgValueTo2ElpArc  =================== */

static void dlgValueTo2ElpArc ( double a, double ang )
{
 char ea[50];
 char eang[50];

 if ( d2elparc.dialog == NULL ) return;

 sprintf(ea, format, a);
 sprintf(eang, format, ang);

 IupSetAttribute (IupGetHandle("a_arc3_handle"), "VALUE", ea);
 IupSetAttribute (IupGetHandle("ang_arc3_handle"), "VALUE", eang);
}

/* =================  dlgValueFrom2ElpArc  =================== */

static void dlgValueFrom2ElpArc ( double *a, double *ang )
{
  if ( d2elparc.dialog == NULL ) return;

  *a = IupGetFloat (IupGetHandle("a_arc3_handle"), "VALUE");
  *ang = IupGetFloat (IupGetHandle("ang_arc3_handle"), "VALUE");
}

/* ==================  dlgValidate2ElpArc  =================== */

static int dlgValidate2ElpArc ( void )
{
 char *a;
 char *ang;

 if ( d2elparc.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(d2elparc.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 a = IupGetAttribute (IupGetHandle("a_arc3_handle"), "VALUE");
 ang = IupGetAttribute (IupGetHandle("ang_arc3_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInit3ElpArc  ==================== */

static void dlgInit3ElpArc ( ElpArcDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel13e; 
   dial->pointlabel1 = llabel5e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel13p; 
   dial->pointlabel1 = llabel5p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* ====================  dlgCreate3ElpArc  =================== */

static void* dlgCreate3ElpArc ( void )
{
  if ( d3elparc.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( d3elparc.pointbuttn1, "call_d3elparc_ok" );
    Ihandle *butt2 = IupButton ( d3elparc.pointbuttn2, "call_d3elparc_cn" );

    Ihandle *labl1 = NULL;

    Ihandle *text1 = NULL;

    Ihandle *h1 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( d3elparc.pointlabel1 );
    text1 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  d3elparc.pointlabel0);

    /* sets handles */
    IupSetHandle("b_arc4_handle", text1);
    iupmaskSet(IupGetHandle("b_arc4_handle"), IUPMASK_UFLOAT , 0, 1);

    /* creates dialog */
    d3elparc.dialog = IupDialog (
                        IupFrame (
	                  IupVbox (
		            IupFill(),
		            t1,
		            IupFill(),
			    IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),
                                    NULL),
		            IupFill(),
			    NULL
		          )
		        )
                      );

    /* sets callbacks */
    IupSetFunction ( "call_d3elparc_ok", (Icallback)callback3ElpArcDialogOk );
    IupSetFunction ( "call_d3elparc_cn", 
                                      (Icallback)callback3ElpArcDialogCancel );
    IupSetFunction ( "call_d3elparc_close", 
                                      (Icallback)callback3ElpArcDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( d3elparc.dialog, "GAP",    "3" );
    IupSetAttribute ( d3elparc.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( d3elparc.dialog, "MAXBOX", "NO");
    IupSetAttribute ( d3elparc.dialog, "RESIZE", "NO");
    IupSetAttribute ( d3elparc.dialog, "CLOSE_CB", "call_d3elparc_close");
#ifdef _UNIX_
    IupSetAttribute ( d3elparc.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( d3elparc.dialog, "TITLE",  "Ellipse Arc" );

    IupMap ( d3elparc.dialog );
  }

  return d3elparc.dialog;
}

/* ===================  dlgValueTo3ElpArc  =================== */

static void dlgValueTo3ElpArc ( double b )
{
 char  eb[50];
 
 if ( d3elparc.dialog == NULL ) return;

 sprintf(eb, format, b);
 
 IupSetAttribute (IupGetHandle("b_arc4_handle"), "VALUE", eb);
}

/* =================  dlgValueFrom3ElpArc  =================== */

static void dlgValueFrom3ElpArc ( double *b )
{
  if ( d3elparc.dialog == NULL ) return;

  *b  = IupGetFloat (IupGetHandle("b_arc4_handle"), "VALUE");
}

/* ==================  dlgValidate3ElpArc  =================== */

static int dlgValidate3ElpArc ( void )
{
 char *b;

 if ( d3elparc.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(d3elparc.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 b = IupGetAttribute (IupGetHandle("b_arc4_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInit4ElpArc  ==================== */

static void dlgInit4ElpArc ( ElpArcDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0 = llabel14e; 
   dial->pointlabel1 = llabel6e; 
   dial->pointlabel2 = llabel7e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0 = llabel14p; 
   dial->pointlabel1 = llabel6p; 
   dial->pointlabel2 = llabel7p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* ====================  dlgCreate4ElpArc  =================== */

static void* dlgCreate4ElpArc ( void )
{
  if ( d4elparc.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( d4elparc.pointbuttn1, "call_d4elparc_ok" );
    Ihandle *butt2 = IupButton ( d4elparc.pointbuttn2, "call_d4elparc_cn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( d4elparc.pointlabel1 );
    labl2 = IupLabel  ( d4elparc.pointlabel2 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  d4elparc.pointlabel0);

    /* sets handles */
    IupSetHandle("p0x_arc5_handle", text1);
    IupSetHandle("p0y_arc5_handle", text2);
    iupmaskSet(IupGetHandle("p0x_arc5_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("p0y_arc5_handle"), IUPMASK_FLOAT , 0, 1);

    /* creates dialog */
    d4elparc.dialog = IupDialog (
                        IupFrame (
	                  IupVbox (
		            IupFill(),
		            t1,
		            IupFill(),
			    IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),
                                    NULL),
		            IupFill(),
			    NULL
		          )
		        )
                      );

    /* sets callbacks */
    IupSetFunction ( "call_d4elparc_ok", (Icallback)callback4ElpArcDialogOk );
    IupSetFunction ( "call_d4elparc_cn", 
                                      (Icallback)callback4ElpArcDialogCancel );
    IupSetFunction ( "call_d4elparc_close", 
                                      (Icallback)callback4ElpArcDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( d4elparc.dialog, "GAP",    "3" );
    IupSetAttribute ( d4elparc.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( d4elparc.dialog, "MAXBOX", "NO");
    IupSetAttribute ( d4elparc.dialog, "RESIZE", "NO");
    IupSetAttribute ( d4elparc.dialog, "CLOSE_CB", "call_d4elparc_close");
#ifdef _UNIX_
    IupSetAttribute ( d4elparc.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( d4elparc.dialog, "TITLE",  "Ellipse Arc" );

    IupMap ( d4elparc.dialog );
  }

  return d4elparc.dialog;
}

/* ===================  dlgValueTo4ElpArc  =================== */

static void dlgValueTo4ElpArc ( double p0x, double p0y )
{
 char ep0x[50];
 char ep0y[50];

 if ( d4elparc.dialog == NULL ) return;

 sprintf(ep0x, format, p0x);
 sprintf(ep0y, format, p0y);

 IupSetAttribute (IupGetHandle("p0x_arc5_handle"), "VALUE", ep0x);
 IupSetAttribute (IupGetHandle("p0y_arc5_handle"), "VALUE", ep0y);
}

/* =================  dlgValueFrom4ElpArc  =================== */

static void dlgValueFrom4ElpArc ( double *p0x, double *p0y )
{
  if ( d4elparc.dialog == NULL ) return;

  *p0x = IupGetFloat (IupGetHandle("p0x_arc5_handle"), "VALUE");
  *p0y = IupGetFloat (IupGetHandle("p0y_arc5_handle"), "VALUE");
}

/* ==================  dlgValidate4ElpArc  =================== */

static int dlgValidate4ElpArc ( void )
{
 char *p0x;
 char *p0y;

 if ( d4elparc.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(d4elparc.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 p0x = IupGetAttribute (IupGetHandle("p0x_arc5_handle"), "VALUE");
 p0y = IupGetAttribute (IupGetHandle("p0y_arc5_handle"), "VALUE");

 return 1;
}

/* =======================  dlgInit5ElpArc  ======================= */

static void dlgInit5ElpArc ( ElpArcDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->pointlabel0  = llabel0e; 
   dial->pointlabel1  = llabel1e; 
   dial->pointlabel2  = llabel2e; 
   dial->pointlabel3  = llabel3e; 
   dial->pointlabel4  = llabel4e; 
   dial->pointlabel5  = llabel5e; 
   dial->pointlabel6  = llabel6e; 
   dial->pointlabel7  = llabel7e; 
   dial->pointlabel8  = llabel8e; 
   dial->pointlabel9  = llabel9e; 
   dial->pointbuttn1 = lbuttn1e; 
   dial->pointbuttn2 = lbuttn2e; 
 }
 else
 {
   dial->pointlabel0  = llabel0p; 
   dial->pointlabel1  = llabel1p; 
   dial->pointlabel2  = llabel2p; 
   dial->pointlabel3  = llabel3p; 
   dial->pointlabel4  = llabel4p; 
   dial->pointlabel5  = llabel5p; 
   dial->pointlabel6  = llabel6p; 
   dial->pointlabel7  = llabel7p;
   dial->pointlabel8  = llabel8p; 
   dial->pointlabel9  = llabel9p; 
   dial->pointbuttn1 = lbuttn1p; 
   dial->pointbuttn2 = lbuttn2p; 
 }
}

/* =======================  dlgCreate5ElpArc  ======================= */

static void* dlgCreate5ElpArc ( void )
{
  if ( d5elparc.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( d5elparc.pointbuttn1, "call_d5elparc_ok" );
    Ihandle *butt2 = IupButton ( d5elparc.pointbuttn2, "call_d5elparc_cn" );

    Ihandle *labl1  = NULL;
    Ihandle *labl2  = NULL;
    Ihandle *labl3  = NULL;
    Ihandle *labl4  = NULL;
    Ihandle *labl5  = NULL;
    Ihandle *labl6  = NULL;
    Ihandle *labl7  = NULL;
    Ihandle *labl8  = NULL;
    Ihandle *labl9  = NULL;

    Ihandle *text1  = NULL;
    Ihandle *text2  = NULL;
    Ihandle *text3  = NULL;
    Ihandle *text4  = NULL;
    Ihandle *text5  = NULL;
    Ihandle *text6  = NULL;
    Ihandle *text7  = NULL;
    Ihandle *text8  = NULL;
    Ihandle *text9  = NULL;

    Ihandle *h1  = NULL;
    Ihandle *h2  = NULL;
    Ihandle *h3  = NULL;
    Ihandle *h4  = NULL;
    Ihandle *h5  = NULL;
    Ihandle *h6  = NULL;
    Ihandle *h7  = NULL;
    Ihandle *h8  = NULL;
    Ihandle *h9  = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1  = IupLabel  ( d5elparc.pointlabel1 );
    labl2  = IupLabel  ( d5elparc.pointlabel2 );
    labl3  = IupLabel  ( d5elparc.pointlabel3 );
    labl4  = IupLabel  ( d5elparc.pointlabel4 );
    labl5  = IupLabel  ( d5elparc.pointlabel5 );
    labl6  = IupLabel  ( d5elparc.pointlabel6 );
    labl7  = IupLabel  ( d5elparc.pointlabel7 );
    labl8  = IupLabel  ( d5elparc.pointlabel8 );
    labl9  = IupLabel  ( d5elparc.pointlabel9 );
    text1  = IupText   ( NULL );
    text2  = IupText   ( NULL );
    text3  = IupText   ( NULL );
    text4  = IupText   ( NULL );
    text5  = IupText   ( NULL );
    text6  = IupText   ( NULL );
    text7  = IupText   ( NULL );
    text8  = IupText   ( NULL );
    text9  = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);
    h4 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl4,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text4,IupFill(),NULL), 
		  IupFill(), NULL);
    h5 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl5,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text5,IupFill(),NULL), 
		  IupFill(), NULL);
    h6 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl6,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text6,IupFill(),NULL), 
		  IupFill(), NULL);
    h7 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl7,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text7,IupFill(),NULL), 
		  IupFill(), NULL);
    h8 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl8,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text8,IupFill(),NULL), 
		  IupFill(), NULL);
    h9 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl9,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text9,IupFill(),NULL), 
		  IupFill(), NULL);  

    t1 = IupFrame (
	     IupVbox (
	        IupFill(),
	        h1,
	        IupFill(),
		h2,
	        IupFill(),
		h3,
	        IupFill(),
		h4,
	        IupFill(),
		h5,
	        IupFill(),
		h6,
	        IupFill(),
		h7,
	        IupFill(),
		h8,
	        IupFill(),
	        h9,
	        IupFill(),
		NULL
	      )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text5, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text6, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text7, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text8, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text9, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text5, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text6, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text7, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text8, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text9, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( text5, "SIZE", size_text );
    IupSetAttribute ( text6, "SIZE", size_text );
    IupSetAttribute ( text7, "SIZE", size_text );
    IupSetAttribute ( text8, "SIZE", size_text );
    IupSetAttribute ( text9, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );
    IupSetAttribute ( labl5, "SIZE", size_text );
    IupSetAttribute ( labl6, "SIZE", size_text );
    IupSetAttribute ( labl7, "SIZE", size_text );
    IupSetAttribute ( labl8, "SIZE", size_text );
    IupSetAttribute ( labl9, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE", "180" );
    IupSetAttribute ( t1, "TITLE",  d5elparc.pointlabel0 );

    /* sets handles */
    IupSetHandle("xc_arc6_handle",  text1);
    IupSetHandle("yc_arc6_handle",  text2);
    IupSetHandle("a_arc6_handle",   text3);
    IupSetHandle("ang_arc6_handle", text4);
    IupSetHandle("b_arc6_handle",   text5);
    IupSetHandle("p0x_arc6_handle", text6);
    IupSetHandle("p0y_arc6_handle", text7);
    IupSetHandle("p1x_arc6_handle", text8);
    IupSetHandle("p1y_arc6_handle", text9);
   
    iupmaskSet(IupGetHandle("xc_arc6_handle" ), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("yc_arc6_handle" ), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("a_arc6_handle"  ), IUPMASK_UFLOAT , 0, 1);
    iupmaskSet(IupGetHandle("ang_arc6_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("b_arc6_handle"  ), IUPMASK_UFLOAT , 0, 1);
    iupmaskSet(IupGetHandle("p0x_arc6_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("p0y_arc6_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("p1x_arc6_handle"), IUPMASK_FLOAT , 0, 1);
    iupmaskSet(IupGetHandle("p1y_arc6_handle"), IUPMASK_FLOAT , 0, 1);

    /* creates dialog */
    d5elparc.dialog = IupDialog (
                        IupFrame (
                          IupVbox (
		                    IupFill(),
		                    t1,
		                    IupFill(),
			            IupHbox(IupFill(),butt1,IupFill(),butt2,
                                            IupFill(),NULL),
		                    IupFill(), NULL )
		                )
                      );

    /* sets callbacks */
    IupSetFunction ( "call_d5elparc_ok", (Icallback)callback5ElpArcDialogOk );
    IupSetFunction ( "call_d5elparc_cn", 
                                      (Icallback)callback5ElpArcDialogCancel );
    IupSetFunction ( "call_d5elparc_close", 
                                      (Icallback)callback5ElpArcDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( d5elparc.dialog, "GAP",    "3" );
    IupSetAttribute ( d5elparc.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( d5elparc.dialog, "MAXBOX", "NO");
    IupSetAttribute ( d5elparc.dialog, "RESIZE", "NO");
    IupSetAttribute ( d5elparc.dialog, "CLOSE_CB", "call_d5elparc_close");
#ifdef _UNIX_
    IupSetAttribute ( d5elparc.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( d5elparc.dialog, "TITLE",  "Ellipse Arc" );

    IupMap ( d5elparc.dialog );
  }

  return d5elparc.dialog;
}

/* ======================  dlgValueTo5ElpArc  ======================= */

static void dlgValueTo5ElpArc ( double xc,  double yc, double a, double b,
                                double ang, double p0x, double p0y,
                                double p1x, double p1y )
{
 char exc[50];
 char eyc[50];
 char ea[50];
 char eb[50];
 char eang[50];
 char ep0x[50];
 char ep0y[50];
 char ep1x[50];
 char ep1y[50];    

 if ( d5elparc.dialog == NULL ) return;

 sprintf(exc,  format, xc);
 sprintf(eyc,  format, yc);
 sprintf(ea,   format, a);
 sprintf(eb,   format, b);
 sprintf(eang, format, ang);
 sprintf(ep0x, format, p0x);
 sprintf(ep0y, format, p0y);
 sprintf(ep1x, format, p1x);
 sprintf(ep1y, format, p1y);

 IupSetAttribute (IupGetHandle("xc_arc6_handle"),  "VALUE", exc);
 IupSetAttribute (IupGetHandle("yc_arc6_handle"),  "VALUE", eyc);
 IupSetAttribute (IupGetHandle("a_arc6_handle"),   "VALUE", ea);
 IupSetAttribute (IupGetHandle("b_arc6_handle"),   "VALUE", eb);
 IupSetAttribute (IupGetHandle("ang_arc6_handle"), "VALUE", eang);
 IupSetAttribute (IupGetHandle("p0x_arc6_handle"), "VALUE", ep0x);
 IupSetAttribute (IupGetHandle("p0y_arc6_handle"), "VALUE", ep0y);
 IupSetAttribute (IupGetHandle("p1x_arc6_handle"), "VALUE", ep1x);
 IupSetAttribute (IupGetHandle("p1y_arc6_handle"), "VALUE", ep1y);
}

/* =====================  dlgValueFrom5ElpArc  ====================== */

static void dlgValueFrom5ElpArc ( double* xc,  double* yc, double* a, 
                                  double* b, double* ang, double* p0x, 
                                  double* p0y, double* p1x, double* p1y )
{
  if ( d5elparc.dialog == NULL ) return;

  /* normal case */
  *xc  = IupGetFloat (IupGetHandle("xc_arc6_handle"),  "VALUE");
  *yc  = IupGetFloat (IupGetHandle("yc_arc6_handle"),  "VALUE");
  *a   = IupGetFloat (IupGetHandle("a_arc6_handle"),   "VALUE");
  *b   = IupGetFloat (IupGetHandle("b_arc6_handle"),   "VALUE");
  *ang = IupGetFloat (IupGetHandle("ang_arc6_handle"), "VALUE");
  *p0x = IupGetFloat (IupGetHandle("p0x_arc6_handle"), "VALUE");
  *p0y = IupGetFloat (IupGetHandle("p0y_arc6_handle"), "VALUE");
  *p1x = IupGetFloat (IupGetHandle("p1x_arc6_handle"), "VALUE");
  *p1y = IupGetFloat (IupGetHandle("p1y_arc6_handle"), "VALUE");
}

/* =====================  dlgValidate5ElpArc  ======================= */

static int dlgValidate5ElpArc ( void )
{
 char *xc;
 char *yc;
 char *a;
 char *b;
 char *ang;
 char *p0x;
 char *p0y;
 char *p1x;
 char *p1y;

 if ( d5elparc.dialog == NULL ) return 0;

 if ( strcmp(IupGetAttribute(d5elparc.dialog, "CONFIRM"), "1") != 0 ) 
    return 0;

 /* normal case */
 xc  = IupGetAttribute (IupGetHandle("xc_arc6_handle"),  "VALUE");
 yc  = IupGetAttribute (IupGetHandle("yc_arc6_handle"),  "VALUE");
 a   = IupGetAttribute (IupGetHandle("a_arc6_handle"),   "VALUE");
 b   = IupGetAttribute (IupGetHandle("b_arc6_handle"),   "VALUE");
 ang = IupGetAttribute (IupGetHandle("ang_arc6_handle"), "VALUE");
 p0x = IupGetAttribute (IupGetHandle("p0x_arc6_handle"), "VALUE");
 p0y = IupGetAttribute (IupGetHandle("p0y_arc6_handle"), "VALUE");
 p1x = IupGetAttribute (IupGetHandle("p1x_arc6_handle"), "VALUE");
 p1y = IupGetAttribute (IupGetHandle("p1y_arc6_handle"), "VALUE");
    
 return 1;
}   

/* ========================  callbackElpArcDialogOk  ======================== */

static int callbackElpArcDialogOk ( Ihandle *bt )
{
 IupSetAttribute(elparc.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ======================  callbackElpArcDialogCancel  ====================== */

static int callbackElpArcDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(elparc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackElpArcDialogCloseCB  ==================== */

static int callbackElpArcDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(elparc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callback1ElpArcDialogOk  ====================== */

static int callback1ElpArcDialogOk ( Ihandle *bt )
{
 IupSetAttribute(d1elparc.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callback1ElpArcDialogCancel  ==================== */

static int callback1ElpArcDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(d1elparc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ====================  callback1ElpArcDialogCloseCB  ==================== */

static int callback1ElpArcDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(d1elparc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackS2ElpArcDialogOk  ====================== */

static int callback2ElpArcDialogOk ( Ihandle *bt )
{
 IupSetAttribute(d2elparc.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callback2ElpArcDialogCancel  ==================== */

static int callback2ElpArcDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(d2elparc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ====================  callback2ElpArcDialogCloseCB  =================== */

static int callback2ElpArcDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(d2elparc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackS3ElpArcDialogOk  ====================== */

static int callback3ElpArcDialogOk ( Ihandle *bt )
{
 IupSetAttribute(d3elparc.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callback3ElpArcDialogCancel  ==================== */

static int callback3ElpArcDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(d3elparc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ====================  callback3ElpArcDialogCloseCB  =================== */

static int callback3ElpArcDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(d3elparc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackS4ElpArcDialogOk  ====================== */

static int callback4ElpArcDialogOk ( Ihandle *bt )
{
 IupSetAttribute(d4elparc.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callback4ElpArcDialogCancel  ==================== */

static int callback4ElpArcDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(d4elparc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ====================  callback4ElpArcDialogCloseCB  =================== */

static int callback4ElpArcDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(d4elparc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackS5ElpArcDialogOk  ====================== */

static int callback5ElpArcDialogOk ( Ihandle *bt )
{
 IupSetAttribute(d5elparc.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callback5ElpArcDialogCancel  ==================== */

static int callback5ElpArcDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(d5elparc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ====================  callback5ElpArcDialogCloseCB  =================== */

static int callback5ElpArcDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(d5elparc.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* =========================  DlgCaptureElpArc  ======================== */

int DlgCaptureElpArc ( double *xc, double *yc, double *a, double *b, 
                       double *ang, double *p0x, double *p0y, double *p1x, 
                       double *p1y, double *tol )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInitElpArc( &elparc );

  /* create dialog to capture point data. */
  d = dlgCreateElpArc();

  /* sets point data to dialog. */
  dlgValueToElpArc ( *xc, *yc, *a, *b, *ang, *p0x, *p0y, *p1x, *p1y, *tol );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidateElpArc ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFromElpArc ( xc, yc, a, b, ang, p0x, p0y, p1x, p1y, tol );

  return 1;
}

/* ======================  DlgCapture1ElpArc  ===================== */

int DlgCapture1ElpArc ( double *xc, double *yc )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInit1ElpArc( &d1elparc );

  /* create dialog to capture point data. */
  d = dlgCreate1ElpArc();

  /* sets point data to dialog. */
  dlgValueTo1ElpArc ( *xc, *yc );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidate1ElpArc ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFrom1ElpArc ( xc, yc );

  return 1;
}

/* ======================  DlgCapture2ElpArc  ===================== */

int DlgCapture2ElpArc ( double *a, double *ang )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInit2ElpArc( &d2elparc );

  /* create dialog to capture point data. */
  d = dlgCreate2ElpArc();

  /* sets point data to dialog. */
  dlgValueTo2ElpArc ( *a, *ang );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidate2ElpArc ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFrom2ElpArc ( a, ang );

  return 1;
}

/* ======================  DlgCapture3ElpArc  ===================== */

int DlgCapture3ElpArc ( double *b )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInit3ElpArc( &d3elparc );

  /* create dialog to capture point data. */
  d = dlgCreate3ElpArc();

  /* sets point data to dialog. */
  dlgValueTo3ElpArc ( *b );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidate3ElpArc ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFrom3ElpArc ( b );

  return 1;
}

/* ======================  DlgCapture4ElpArc  ===================== */

int DlgCapture4ElpArc ( double *p0x, double *p0y )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInit4ElpArc( &d4elparc );

  /* create dialog to capture point data. */
  d = dlgCreate4ElpArc();

  /* sets point data to dialog. */
  dlgValueTo4ElpArc ( *p0x, *p0y );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidate4ElpArc ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFrom4ElpArc ( p0x, p0y );

  return 1;
}

/* ======================  DlgCapture5ElpArc  ===================== */

int DlgCapture5ElpArc ( double *xc, double *yc, double *a, double *b, 
                        double *ang, double *p0x, double *p0y, 
                        double *p1x, double *p1y )
{
  Ihandle *d;

  /* initializes point dialog. */
  dlgInit5ElpArc( &d5elparc );

  /* create dialog to capture point data. */
  d = dlgCreate5ElpArc();

  /* sets point data to dialog. */
  dlgValueTo5ElpArc ( *xc, *yc, *a, *b, *ang, *p0x, *p0y, *p1x, *p1y );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks point data. */
  if ( dlgValidate5ElpArc ( ) == 0 ) return 0;

  /* get point data from dialog. */
  dlgValueFrom5ElpArc ( xc, yc, a, b, ang, p0x, p0y, p1x, p1y );

  return 1;
}

