
/*
** dlgline.c - Functions that establish the pre-defined dialogs 
**             of the library.
**
**
** Public functions:
** --------------------------------------------------------------------
**
** int DlgCaptureLine( float *x0, float *y0, 
**                     float *x1, float *y1, float *tol )
**
** x0  - first coordinate (X direction) of line.       (IN/OUT)
** y0  - first coordinate (Y direction) of line.       (IN/OUT)
** x1  - second coordinate (X direction) of line.      (IN/OUT)
** y1  - second coordinate (Y direction) of line.      (IN/OUT)
** tol - tolerance used.                               (IN/OUT)
**
** This function creates a dialog to capture the parameters of a line. 
** These parameters can be the initial and final rectangular coordinates 
** or the initial rectangular coordinates and the increments belong of 
** Cartesian axis.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureFPLine( float *x0, float *y0 )
**
** x0  - first coordinate (X direction) of line.       (IN/OUT)
** y0  - first coordinate (Y direction) of line.       (IN/OUT)
**
** This function creates a dialog to capture the first parameters of a line. 
** It does not use tolerance.
**
** --------------------------------------------------------------------
**
** --------------------------------------------------------------------
**
** int DlgCaptureSPLine( float *x0, float *y0, 
**                                float *x1, float *y1 )
**
** x0  - first coordinate (X direction) of line.       (IN/OUT)
** y0  - first coordinate (Y direction) of line.       (IN/OUT)
** x1  - second coordinate (X direction) of line.      (IN/OUT)
** y1  - second coordinate (Y direction) of line.      (IN/OUT)
**
** This function creates a dialog to capture the second parameters of a line. 
** These parameters can be the initial and final rectangular coordinates 
** or the initial rectangular coordinates and the increments belong of 
** Cartesian axis. It does not use tolerance.
**
** --------------------------------------------------------------------
**
** Version:    1-000
**
** Created:    10-Sep-01    William W. M. Lira.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced include of header file iuptabs.h by iupcontrols.h.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "iup.h"
#include "iupmask.h"
#include "iupcontrols.h"

#include "dlgline.h"

static char* format = "%.6f";
static int   d_posx = IUP_CENTER;
static int   d_posy = IUP_CENTER;

/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/

/* Data structure to store line dialog */

typedef struct _linedlg
{

  char *linelabel0;
  char *linelabel1;
  char *linelabel2;
  char *linelabel3;
  char *linelabel4;
  char *linelabel5;
  char *linelabel6;
  char *linelabel7;
  char *linebuttn1;
  char *linebuttn2;
  char *linetabs1;
  char *linetabs2;

  Ihandle *dialog;

} LineDlg;


static void  dlgInitLine      ( LineDlg *dial );
static void* dlgCreateLine    ( void );
static void  dlgValueToLine   ( float x0, float y0,
				float x1, float y1, float tol );
static int   dlgValidateLine  ( void );
static void  dlgValueFromLine ( float *x0, float *y0,
				float *x1, float *y1, float *tol );

static void  dlgInitAbsoluLine      ( LineDlg *dial );
static void* dlgCreateAbsoluLine    ( void );
static void  dlgValueToAbsoluLine   ( float x0, float y0,
				      float x1, float y1, float tol );
static int   dlgValidateAbsoluLine  ( void );
static void  dlgValueFromAbsoluLine ( float *x0, float *y0,
				      float *x1, float *y1, float *tol );

static void  dlgInitIncremLine      ( LineDlg *dial );
static void* dlgCreateIncremLine    ( void );
static void  dlgValueToIncremLine   ( float x0, float y0,
				      float x1, float y1, float tol );
static int   dlgValidateIncremLine  ( void );
static void  dlgValueFromIncremLine ( float *x0, float *y0,
				      float *x1, float *y1, float *tol );

static void  dlgInitFPLine      ( LineDlg *dial );
static void* dlgCreateFPLine    ( void );
static void  dlgValueToFPLine   ( float x0, float y0 );
static int   dlgValidateFPLine  ( void );
static void  dlgValueFromFPLine ( float *x0, float *y0 );

static void  dlgInitSPLine      ( LineDlg *dial );
static void* dlgCreateSPLine    ( void );
static void  dlgValueToSPLine   ( float x0, float y0, float x1, float y1 );
static int   dlgValidateSPLine  ( void );
static void  dlgValueFromSPLine ( float *x0, float *y0, float *x1, float *y1 );

static void  dlgInitAbsoluSPLine     ( LineDlg *dial );
static void* dlgCreateAbsoluSPLine   ( void );
static void  dlgValueToAbsoluSPLine  ( float x0, float y0,
				       float x1, float y1 );
static int   dlgValidateAbsoluSPLine ( void );
static void  dlgValueFromAbsoluSPLine( float *x0, float *y0,
				       float *x1, float *y1 );

static void  dlgInitIncremSPLine     ( LineDlg *dial );
static void* dlgCreateIncremSPLine   ( void );
static void  dlgValueToIncremSPLine  ( float x0, float y0,
				       float x1, float y1 );
static int   dlgValidateIncremSPLine ( void );
static void  dlgValueFromIncremSPLine( float *x0, float *y0,
				       float *x1, float *y1 );

static int  callbackLineDialogOk     ( Ihandle *bt );
static int  callbackLineDialogCancel ( Ihandle *bt );
static int  callbackLineDialogCloseCB( Ihandle *bt );

static int  callbackFPLineDialogOk     ( Ihandle *bt );
static int  callbackFPLineDialogCancel ( Ihandle *bt );
static int  callbackFPLineDialogCloseCB ( Ihandle *bt );

static int  callbackSPLineDialogOk     ( Ihandle *bt );
static int  callbackSPLineDialogCancel ( Ihandle *bt );
static int  callbackSPLineDialogCloseCB ( Ihandle *bt );

static LineDlg  line;
static LineDlg fline;
static LineDlg sline;

static char *size_butt = {"55x12"};
static char *size_text = {"55"};
static char *font_dial = {"TIMES_NORMAL_12"};

static char *llabel0e = { "Enter with line coordinates:" };
static char *lbuttn1e = { " OK " };
static char *lbuttn2e = { " Cancel " };
static char *ltabs1e  = { "Absolute" };
static char *ltabs2e  = { "Incremental" };

static char *llabel0p = { "Entre com as coordenadas da linha:" };
static char *lbuttn1p = { "Confirma" };
static char *lbuttn2p = { "Cancela " };
static char *ltabs1p  = { "Absoluto" };
static char *ltabs2p  = { "Incremental" };

static char *llabel1e = { "Point 1 X: " };
static char *llabel2e = { "Point 1 Y: " };
static char *llabel3e = { "Point 2 X: " };
static char *llabel4e = { "Point 2 Y: " };
static char *llabel5e = { "Tolerance: " };
   
static char *llabel1p = { "Ponto 1 X: " };
static char *llabel2p = { "Ponto 1 Y: " };
static char *llabel3p = { "Ponto 2 X: " };
static char *llabel4p = { "Ponto 2 Y: " };
static char *llabel5p = { "Toleranc.: " };

static char *llabel6 = { "Incr.  DX: " };
static char *llabel7 = { "Incr.  DY: " };

/*
** --------------------------------------------------------------------
** Local functions:
*/

/* =======================  dlgInitLine  ======================= */

static void dlgInitLine ( LineDlg *dial )
{
 dlgInitAbsoluLine( dial );
 dlgInitIncremLine( dial );

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->linelabel0 = llabel0e; 
   dial->linebuttn1 = lbuttn1e; 
   dial->linebuttn2 = lbuttn2e; 
   dial->linetabs1  = ltabs1e; 
   dial->linetabs2  = ltabs2e; 
 }
 else
 {
   dial->linelabel0 = llabel0p; 
   dial->linebuttn1 = lbuttn1p; 
   dial->linebuttn2 = lbuttn2p; 
   dial->linetabs1  = ltabs1p; 
   dial->linetabs2  = ltabs2p; 
 }
}

/* =======================  dlgCreateLine  ======================= */

static void* dlgCreateLine ( void )
{
  if ( line.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( line.linebuttn1, "call_ok" );
    Ihandle *butt2 = IupButton ( line.linebuttn2, "call_cn" );

    Ihandle *t1 = dlgCreateAbsoluLine();
    Ihandle *t2 = dlgCreateIncremLine();

    Ihandle *tb = IupTabs ( t1, t2, NULL );

    /* creates dialog */
    line.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        tb,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_ok", (Icallback)callbackLineDialogOk );
    IupSetFunction ( "call_cn", (Icallback)callbackLineDialogCancel );
    IupSetFunction ( "call_close",(Icallback)callbackLineDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( line.dialog, "GAP",    "3" );
    IupSetAttribute ( line.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( line.dialog, "MAXBOX", "NO");
    IupSetAttribute ( line.dialog, "RESIZE", "NO");
    IupSetAttribute ( line.dialog, "CLOSE_CB", "call_close");
#ifdef _UNIX_
    IupSetAttribute ( line.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( line.dialog, "TITLE",  "Line" );

    IupSetAttribute ( tb, "VALUE", line.linetabs1 );
    IupSetAttribute ( tb, ICTL_TABTYPE, ICTL_LEFT );

    /* sets handles */
    IupSetHandle("tb_handle", tb);

    IupMap ( line.dialog );
  }

  return line.dialog;
}

/* ======================  dlgValueToLine  ======================= */

static void dlgValueToLine ( float x0, float y0,
			     float x1, float y1, float tol )
{
  dlgValueToAbsoluLine(x0, y0, x1, y1, tol);
  dlgValueToIncremLine(x0, y0, x1, y1, tol);
}

/* =====================  dlgValueFromLine  ====================== */

static void dlgValueFromLine ( float *x0, float *y0,
			       float *x1, float *y1, float *tol )
{
  Ihandle *tb = IupGetHandle("tb_handle");

  if ( line.dialog == NULL ) return;

  if ( strcmp(IupGetAttribute(tb,"VALUE"), line.linetabs1) == 0 )
  {
    dlgValueFromAbsoluLine(x0, y0, x1, y1, tol);
  }
  else
  {
    dlgValueFromIncremLine(x0, y0, x1, y1, tol);
  }
}

/* =====================  dlgValidateLine  ======================= */

static int dlgValidateLine ( void )
{
  Ihandle *tb = IupGetHandle ( "tb_handle");

  if ( strcmp(IupGetAttribute(line.dialog, "CONFIRM"), "1") != 0 ) 
     return 0;

  if ( strcmp(IupGetAttribute(tb,"VALUE"), line.linetabs1) == 0 )
  {
    return dlgValidateAbsoluLine( );
  }
  else
  {
    return dlgValidateIncremLine( );
  }

  return 0;
}

/* =====================  dlgInitAbsoluLine  ===================== */

static void dlgInitAbsoluLine ( LineDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->linelabel1 = llabel1e; 
   dial->linelabel2 = llabel2e; 
   dial->linelabel3 = llabel3e; 
   dial->linelabel4 = llabel4e; 
   dial->linelabel5 = llabel5e; 
 }
 else
 {
   dial->linelabel1 = llabel1p; 
   dial->linelabel2 = llabel2p; 
   dial->linelabel3 = llabel3p; 
   dial->linelabel4 = llabel4p; 
   dial->linelabel5 = llabel5p; 
 }
}

/* ====================  dlgCreateAbsoluLine  ===================== */

static void* dlgCreateAbsoluLine ( void )
{
  if ( line.dialog == NULL )
  {
    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;
    Ihandle *labl4 = NULL;
    Ihandle *labl5 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;
    Ihandle *text4 = NULL;
    Ihandle *text5 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;
    Ihandle *h4 = NULL;
    Ihandle *h5 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( line.linelabel1 );
    labl2 = IupLabel  ( line.linelabel2 );
    labl3 = IupLabel  ( line.linelabel3 );
    labl4 = IupLabel  ( line.linelabel4 );
    labl5 = IupLabel  ( line.linelabel5 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );
    text4 = IupText   ( NULL );
    text5 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);
    h4 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl4,IupFill(),NULL), 
		  IupFill(), IupVbox(IupFill(),text4,IupFill(),NULL), 
		  IupFill(), NULL);
    h5 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl5,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text5,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox (
	        IupFill(),
	        h1,
	        IupFill(),
		h2,
	        IupFill(),
		h3,
	        IupFill(),
		h4,
	        IupFill(),
		h5,
	        IupFill(),
		NULL
	      )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text5, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text5, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( text5, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );
    IupSetAttribute ( labl5, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE",   "150" );
    IupSetAttribute ( t1, "TABTITLE", line.linetabs1 );
    IupSetAttribute ( t1, "TITLE",    line.linelabel0);

    /* sets handles */
    IupSetHandle("x0_line1_handle", text1);
    IupSetHandle("y0_line1_handle", text2);
    IupSetHandle("x1_line1_handle", text3);
    IupSetHandle("y1_line1_handle", text4);
    IupSetHandle("tl_line1_handle", text5);
    iupmaskSet(IupGetHandle("x0_line1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y0_line1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("x1_line1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y1_line1_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("tl_line1_handle"), IUPMASK_UFLOAT, 0, 1);

    IupSetHandle(line.linetabs1, t1);

    return t1;
  }

  return NULL;
}

/* ===================  dlgValueToAbsoluLine  ===================== */

static void dlgValueToAbsoluLine ( float x0, float y0,
				   float x1, float y1, float tol )
{
 char cx0[50];
 char cy0[50];
 char cx1[50];
 char cy1[50];
 char ctl[50];

 if ( line.dialog == NULL ) return;

 sprintf(cx0, format, x0);
 sprintf(cy0, format, y0);
 sprintf(cx1, format, x1);
 sprintf(cy1, format, y1);
 sprintf(ctl, format, tol);

 IupSetAttribute (IupGetHandle("x0_line1_handle"), "VALUE", cx0);
 IupSetAttribute (IupGetHandle("y0_line1_handle"), "VALUE", cy0);
 IupSetAttribute (IupGetHandle("x1_line1_handle"), "VALUE", cx1);
 IupSetAttribute (IupGetHandle("y1_line1_handle"), "VALUE", cy1);
 IupSetAttribute (IupGetHandle("tl_line1_handle"), "VALUE", ctl);
}

/* ==================  dlgValueFromAbsoluLine  ==================== */

static void dlgValueFromAbsoluLine ( float *x0, float *y0,
				     float *x1, float *y1, float *tol )
{
 if ( line.dialog == NULL ) return;

 /* normal case */ 
 *x0  = IupGetFloat (IupGetHandle("x0_line1_handle"), "VALUE");
 *y0  = IupGetFloat (IupGetHandle("y0_line1_handle"), "VALUE");
 *x1  = IupGetFloat (IupGetHandle("x1_line1_handle"), "VALUE");
 *y1  = IupGetFloat (IupGetHandle("y1_line1_handle"), "VALUE");
 *tol = IupGetFloat (IupGetHandle("tl_line1_handle"), "VALUE");
 
 if(*tol == 0.0)
  *tol = 0.001;
}

/* ==================  dlgValidateAbsoluLine  ===================== */

static int dlgValidateAbsoluLine ( void )
{
 char *x0;
 char *x1;
 char *y0;
 char *y1;
 char *tl;

 if ( line.dialog == NULL ) return 0;

 /* normal case */
 x0 = IupGetAttribute (IupGetHandle("x0_line1_handle"), "VALUE");
 y0 = IupGetAttribute (IupGetHandle("y0_line1_handle"), "VALUE");
 x1 = IupGetAttribute (IupGetHandle("x1_line1_handle"), "VALUE");
 y1 = IupGetAttribute (IupGetHandle("y1_line1_handle"), "VALUE");
 tl = IupGetAttribute (IupGetHandle("tl_line1_handle"), "VALUE");

 return 1;
}

/* =====================  dlgInitIncremLine  ===================== */

static void dlgInitIncremLine ( LineDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->linelabel1 = llabel1e; 
   dial->linelabel2 = llabel2e; 
   dial->linelabel6 = llabel6; 
   dial->linelabel7 = llabel7; 
   dial->linelabel5 = llabel5e; 
 }
 else
 {
   dial->linelabel1 = llabel1p; 
   dial->linelabel2 = llabel2p; 
   dial->linelabel6 = llabel6; 
   dial->linelabel7 = llabel7; 
   dial->linelabel5 = llabel5p; 
 }
}

/* ====================  dlgCreateIncremLine  ===================== */

static void* dlgCreateIncremLine ( void )
{
  if ( line.dialog == NULL )
  {
    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;
    Ihandle *labl4 = NULL;
    Ihandle *labl5 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;
    Ihandle *text4 = NULL;
    Ihandle *text5 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;
    Ihandle *h4 = NULL;
    Ihandle *h5 = NULL;

    Ihandle *t2 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( line.linelabel1 );
    labl2 = IupLabel  ( line.linelabel2 );
    labl3 = IupLabel  ( line.linelabel6 );
    labl4 = IupLabel  ( line.linelabel7 );
    labl5 = IupLabel  ( line.linelabel5 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );
    text4 = IupText   ( NULL );
    text5 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);
    h4 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl4,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text4,IupFill(),NULL), 
		  IupFill(), NULL);
    h5 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl5,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text5,IupFill(),NULL), 
		  IupFill(), NULL);

    t2 = IupFrame (
	     IupVbox (
	        IupFill(),
	        h1,
	        IupFill(),
		h2,
	        IupFill(),
		h3,
	        IupFill(),
		h4,
	        IupFill(),
		h5,
	        IupFill(),
		NULL
	      )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text5, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text5, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( text5, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );
    IupSetAttribute ( labl5, "SIZE", size_text );

    IupSetAttribute ( t2, "GAP", "1" );
    IupSetAttribute ( t2, "MARGIN", "1x1" );
    IupSetAttribute ( t2, "SIZE",   "150" );
    IupSetAttribute ( t2, "TABTITLE", line.linetabs2 );
    IupSetAttribute ( t2, "TITLE",    line.linelabel0);

    /* sets handles */
    IupSetHandle("x0_line2_handle", text1);
    IupSetHandle("y0_line2_handle", text2);
    IupSetHandle("x1_line2_handle", text3);
    IupSetHandle("y1_line2_handle", text4);
    IupSetHandle("tl_line2_handle", text5);
    iupmaskSet(IupGetHandle("x0_line2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y0_line2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("x1_line2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y1_line2_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("tl_line2_handle"), IUPMASK_UFLOAT, 0, 1);

    IupSetHandle(line.linetabs2, t2);

    return t2;
  }

  return NULL;
}

/* ===================  dlgValueToIncremLine  ===================== */

static void dlgValueToIncremLine ( float x0, float y0, 
                                   float x1, float y1, float tol )
{
 char cx0[50];
 char cy0[50];
 char cx1[50];
 char cy1[50];
 char ctl[50];

 if ( line.dialog == NULL ) return;

 sprintf(cx0, format, x0);
 sprintf(cy0, format, y0);
 sprintf(cx1, format, x1 - x0);
 sprintf(cy1, format, y1 - y0);
 sprintf(ctl, format, tol);

 IupSetAttribute (IupGetHandle("x0_line2_handle"), "VALUE", cx0);
 IupSetAttribute (IupGetHandle("y0_line2_handle"), "VALUE", cy0);
 IupSetAttribute (IupGetHandle("x1_line2_handle"), "VALUE", cx1);
 IupSetAttribute (IupGetHandle("y1_line2_handle"), "VALUE", cy1);
 IupSetAttribute (IupGetHandle("tl_line2_handle"), "VALUE", ctl);
}

/* ==================  dlgValueFromIncremLine  ==================== */

static void dlgValueFromIncremLine ( float *x0, float *y0,
				     float *x1, float *y1, float *tol )
{
 if ( line.dialog == NULL ) return;

 /* incremental case */
 *x0  = IupGetFloat (IupGetHandle("x0_line2_handle"), "VALUE");
 *y0  = IupGetFloat (IupGetHandle("y0_line2_handle"), "VALUE");
 *x1  = IupGetFloat (IupGetHandle("x1_line2_handle"), "VALUE");
 *y1  = IupGetFloat (IupGetHandle("y1_line2_handle"), "VALUE");
 *tol = IupGetFloat (IupGetHandle("tl_line2_handle"), "VALUE");
 
 if(*tol == 0.0)
  *tol = 0.001;

 *x1 += *x0;
 *y1 += *y0;
}

/* ===================  dlgValidateIncremLine  ===================== */

static int dlgValidateIncremLine ( void )
{
 char *x0;
 char *x1;
 char *y0;
 char *y1;
 char *tl;

 if ( line.dialog == NULL ) return 0;

 /* normal case */
 x0 = IupGetAttribute (IupGetHandle("x0_line2_handle"), "VALUE");
 y0 = IupGetAttribute (IupGetHandle("y0_line2_handle"), "VALUE");
 x1 = IupGetAttribute (IupGetHandle("x1_line2_handle"), "VALUE");
 y1 = IupGetAttribute (IupGetHandle("y1_line2_handle"), "VALUE");
 tl = IupGetAttribute (IupGetHandle("tl_line2_handle"), "VALUE");

 return 1;
}

/* ====================  dlgInitFPLine  ==================== */

static void dlgInitFPLine ( LineDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->linelabel0 = llabel0e; 
   dial->linelabel1 = llabel1e; 
   dial->linelabel2 = llabel2e; 
   dial->linebuttn1 = lbuttn1e; 
   dial->linebuttn2 = lbuttn2e; 
 }
 else
 {
   dial->linelabel0 = llabel0p; 
   dial->linelabel1 = llabel1p; 
   dial->linelabel2 = llabel2p; 
   dial->linebuttn1 = lbuttn1p; 
   dial->linebuttn2 = lbuttn2p; 
 }
}

/* ====================  dlgCreateFPLine  =================== */

static void* dlgCreateFPLine ( void )
{
  if ( fline.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( fline.linebuttn1, "call_fok" );
    Ihandle *butt2 = IupButton ( fline.linebuttn2, "call_fcn" );

    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( fline.linelabel1 );
    labl2 = IupLabel  ( fline.linelabel2 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox ( IupFill(), h1, IupFill(), h2, IupFill(), NULL )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE",   "150" );
    IupSetAttribute ( t1, "TITLE",  fline.linelabel0 );

    /* sets handles */
    IupSetHandle("x0_line3_handle", text1);
    IupSetHandle("y0_line3_handle", text2);
    iupmaskSet(IupGetHandle("x0_line3_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y0_line3_handle"), IUPMASK_FLOAT, 0, 1);

    /* creates dialog */
    fline.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        t1,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_fok", (Icallback)callbackFPLineDialogOk );
    IupSetFunction ( "call_fcn", (Icallback)callbackFPLineDialogCancel );
    IupSetFunction ( "call_fclose",(Icallback)callbackFPLineDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( fline.dialog, "GAP",    "3" );
    IupSetAttribute ( fline.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( fline.dialog, "MAXBOX", "NO");
    IupSetAttribute ( fline.dialog, "RESIZE", "NO");
    IupSetAttribute ( fline.dialog, "CLOSE_CB", "call_fclose");
#ifdef _UNIX_
    IupSetAttribute ( fline.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( fline.dialog, "TITLE",  "Line" );

    IupMap ( fline.dialog );
  }

  return fline.dialog;
}

/* ===================  dlgValueToFPLine  =================== */

static void dlgValueToFPLine ( float x0, float y0 )
{
 char cx0[50];
 char cy0[50];

 if ( fline.dialog == NULL ) return;

 sprintf(cx0, format, x0);
 sprintf(cy0, format, y0);

 IupSetAttribute (IupGetHandle("x0_line3_handle"), "VALUE", cx0);
 IupSetAttribute (IupGetHandle("y0_line3_handle"), "VALUE", cy0);
}

/* =================  dlgValueFromFPLine  =================== */

static void dlgValueFromFPLine ( float *x0, float *y0 )
{
  if ( fline.dialog == NULL ) return;
  
  *x0  = IupGetFloat (IupGetHandle("x0_line3_handle"), "VALUE");
  *y0  = IupGetFloat (IupGetHandle("y0_line3_handle"), "VALUE");
}

/* ==================  dlgValidateFPLine  =================== */

static int dlgValidateFPLine ( void )
{
 char *x0;
 char *y0;

  if ( strcmp(IupGetAttribute(fline.dialog, "CONFIRM"), "1") != 0 ) 
     return 0;

 if ( fline.dialog == NULL ) return 0;

 /* normal case */
 x0 = IupGetAttribute (IupGetHandle("x0_line3_handle"), "VALUE");
 y0 = IupGetAttribute (IupGetHandle("y0_line3_handle"), "VALUE");

 return 1;
}

/* =======================  dlgInitSPLine  ======================= */

static void dlgInitSPLine ( LineDlg *dial )
{
 dlgInitAbsoluSPLine( dial );
 dlgInitIncremSPLine( dial );

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->linelabel0 = llabel0e; 
   dial->linebuttn1 = lbuttn1e; 
   dial->linebuttn2 = lbuttn2e; 
   dial->linetabs1  = ltabs1e; 
   dial->linetabs2  = ltabs2e; 
 }
 else
 {
   dial->linelabel0 = llabel0p; 
   dial->linebuttn1 = lbuttn1p; 
   dial->linebuttn2 = lbuttn2p; 
   dial->linetabs1  = ltabs1p; 
   dial->linetabs2  = ltabs2p; 
 }
}

/* =======================  dlgCreateSPLine  ======================= */

static void* dlgCreateSPLine ( void )
{
  if ( sline.dialog == NULL )
  {
    Ihandle *butt1 = IupButton ( sline.linebuttn1, "call_sok" );
    Ihandle *butt2 = IupButton ( sline.linebuttn2, "call_scn" );

    Ihandle *t1 = dlgCreateAbsoluSPLine();
    Ihandle *t2 = dlgCreateIncremSPLine();

    Ihandle *tb = IupTabs ( t1, t2, NULL );

    /* creates dialog */
    sline.dialog = IupDialog (
                    IupFrame (
		      IupVbox (
		        IupFill(),
		        tb,
		        IupFill(),
			IupHbox(IupFill(),butt1,IupFill(),butt2,IupFill(),NULL),
		        IupFill(),
			NULL
		      )
		    )
                  );

    /* sets callbacks */
    IupSetFunction ( "call_sok", (Icallback)callbackSPLineDialogOk );
    IupSetFunction ( "call_scn", (Icallback)callbackSPLineDialogCancel );
    IupSetFunction ( "call_sclose",(Icallback)callbackSPLineDialogCloseCB );

    /* sets attributes */
    IupSetAttribute ( butt1, "SIZE", size_butt );
    IupSetAttribute ( butt2, "SIZE", size_butt );

    IupSetAttribute ( sline.dialog, "GAP",    "3" );
    IupSetAttribute ( sline.dialog, "MARGIN", "3x3" );
    IupSetAttribute ( sline.dialog, "MAXBOX", "NO");
    IupSetAttribute ( sline.dialog, "RESIZE", "NO");
    IupSetAttribute ( sline.dialog, "CLOSE_CB", "call_sclose");
#ifdef _UNIX_
    IupSetAttribute ( sline.dialog, "FONT",   font_dial );
#endif
    IupSetAttribute ( sline.dialog, "TITLE",  "Line" );

    IupSetAttribute ( tb, "VALUE", sline.linetabs1 );
    IupSetAttribute ( tb, ICTL_TABTYPE, ICTL_LEFT );

    /* sets handles */
    IupSetHandle("stb_handle", tb);

    IupMap ( sline.dialog );
  }

  return sline.dialog;
}

/* ======================  dlgValueToSPLine  ======================= */

static void dlgValueToSPLine ( float x0, float y0, float x1, float y1 )
{
  dlgValueToAbsoluSPLine(x0, y0, x1, y1);
  dlgValueToIncremSPLine(x0, y0, x1, y1);
}

/* =====================  dlgValueFromSPLine  ====================== */

static void dlgValueFromSPLine ( float *x0, float *y0, float *x1, float *y1 )
{
  Ihandle *tb = IupGetHandle("stb_handle");

  if ( sline.dialog == NULL ) return;

  if ( strcmp(IupGetAttribute(tb,"VALUE"), sline.linetabs1) == 0 )
  {
    dlgValueFromAbsoluSPLine(x0, y0, x1, y1);
  }
  else
  {
    dlgValueFromIncremSPLine(x0, y0, x1, y1);
  }
}

/* =====================  dlgValidateSPLine  ======================= */

static int dlgValidateSPLine ( void )
{
  Ihandle *tb = IupGetHandle ( "stb_handle");

  if ( strcmp(IupGetAttribute(sline.dialog, "CONFIRM"), "1") != 0 ) 
     return 0;

  if ( strcmp(IupGetAttribute(tb,"VALUE"), sline.linetabs1) == 0 )
  {
    return dlgValidateAbsoluSPLine( );
  }
  else
  {
    return dlgValidateIncremSPLine( );
  }

  return 0;
}

/* =====================  dlgInitAbsoluSPLine  ===================== */

static void dlgInitAbsoluSPLine ( LineDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->linelabel1 = llabel1e; 
   dial->linelabel2 = llabel2e; 
   dial->linelabel3 = llabel3e; 
   dial->linelabel4 = llabel4e; 
 }
 else
 {
   dial->linelabel1 = llabel1p; 
   dial->linelabel2 = llabel2p; 
   dial->linelabel3 = llabel3p; 
   dial->linelabel4 = llabel4p; 
 }
}

/* ====================  dlgCreateAbsoluSPLine  ===================== */

static void* dlgCreateAbsoluSPLine ( void )
{
  if ( sline.dialog == NULL )
  {
    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;
    Ihandle *labl4 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;
    Ihandle *text4 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;
    Ihandle *h4 = NULL;

    Ihandle *t1 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( sline.linelabel1 );
    labl2 = IupLabel  ( sline.linelabel2 );
    labl3 = IupLabel  ( sline.linelabel3 );
    labl4 = IupLabel  ( sline.linelabel4 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );
    text4 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);
    h4 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl4,IupFill(),NULL), 
		  IupFill(), IupVbox(IupFill(),text4,IupFill(),NULL), 
		  IupFill(), NULL);

    t1 = IupFrame (
	     IupVbox (
	        IupFill(),
	        h1,
	        IupFill(),
		h2,
	        IupFill(),
		h3,
	        IupFill(),
		h4,
	        IupFill(),
		NULL
	      )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );

    IupSetAttribute ( t1, "GAP", "1" );
    IupSetAttribute ( t1, "MARGIN", "1x1" );
    IupSetAttribute ( t1, "SIZE",   "150" );
    IupSetAttribute ( t1, "TABTITLE", sline.linetabs1 );
    IupSetAttribute ( t1, "TITLE",    sline.linelabel0);

    /* sets handles */
    IupSetHandle("x0_line4_handle", text1);
    IupSetHandle("y0_line4_handle", text2);
    IupSetHandle("x1_line4_handle", text3);
    IupSetHandle("y1_line4_handle", text4);
    iupmaskSet(IupGetHandle("x0_line4_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y0_line4_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("x1_line4_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y1_line4_handle"), IUPMASK_FLOAT, 0, 1); 

    IupSetHandle(sline.linetabs1, t1);

    return t1;
  }

  return NULL;
}

/* ===================  dlgValueToAbsoluSPLine  ===================== */

static void dlgValueToAbsoluSPLine ( float x0, float y0,
				     float x1, float y1 )
{
 char cx0[50];
 char cy0[50];
 char cx1[50];
 char cy1[50];

 if ( sline.dialog == NULL ) return;

 sprintf(cx0, format, x0);
 sprintf(cy0, format, y0);
 sprintf(cx1, format, x1);
 sprintf(cy1, format, y1);

 IupSetAttribute (IupGetHandle("x0_line4_handle"), "VALUE", cx0);
 IupSetAttribute (IupGetHandle("y0_line4_handle"), "VALUE", cy0);
 IupSetAttribute (IupGetHandle("x1_line4_handle"), "VALUE", cx1);
 IupSetAttribute (IupGetHandle("y1_line4_handle"), "VALUE", cy1);
}

/* ==================  dlgValueFromAbsoluSPLine  ==================== */

static void dlgValueFromAbsoluSPLine ( float *x0, float *y0,
				       float *x1, float *y1 )
{
 if ( sline.dialog == NULL ) return;

 /* normal case */
 *x0  = IupGetFloat (IupGetHandle("x0_line4_handle"), "VALUE");
 *y0  = IupGetFloat (IupGetHandle("y0_line4_handle"), "VALUE");
 *x1  = IupGetFloat (IupGetHandle("x1_line4_handle"), "VALUE");
 *y1  = IupGetFloat (IupGetHandle("y1_line4_handle"), "VALUE");
}

/* ==================  dlgValidateAbsoluSPLine  ===================== */

static int dlgValidateAbsoluSPLine ( void )
{
 char *x0;
 char *x1;
 char *y0;
 char *y1;

 if ( sline.dialog == NULL ) return 0;

 /* normal case */
 x0 = IupGetAttribute (IupGetHandle("x0_line4_handle"), "VALUE");
 y0 = IupGetAttribute (IupGetHandle("y0_line4_handle"), "VALUE");
 x1 = IupGetAttribute (IupGetHandle("x1_line4_handle"), "VALUE");
 y1 = IupGetAttribute (IupGetHandle("y1_line4_handle"), "VALUE");

 return 1;
}

/* =====================  dlgInitIncremSPLine  ===================== */

static void dlgInitIncremSPLine ( LineDlg *dial )
{
 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
 {
   dial->linelabel1 = llabel1e; 
   dial->linelabel2 = llabel2e; 
   dial->linelabel6 = llabel6; 
   dial->linelabel7 = llabel7; 
 }
 else
 {
   dial->linelabel1 = llabel1p; 
   dial->linelabel2 = llabel2p; 
   dial->linelabel6 = llabel6; 
   dial->linelabel7 = llabel7; 
 }
}

/* ====================  dlgCreateIncremSPLine  ===================== */

static void* dlgCreateIncremSPLine ( void )
{
  if ( sline.dialog == NULL )
  {
    Ihandle *labl1 = NULL;
    Ihandle *labl2 = NULL;
    Ihandle *labl3 = NULL;
    Ihandle *labl4 = NULL;

    Ihandle *text1 = NULL;
    Ihandle *text2 = NULL;
    Ihandle *text3 = NULL;
    Ihandle *text4 = NULL;

    Ihandle *h1 = NULL;
    Ihandle *h2 = NULL;
    Ihandle *h3 = NULL;
    Ihandle *h4 = NULL;

    Ihandle *t2 = NULL;

    /* creates interface elements */
    labl1 = IupLabel  ( sline.linelabel1 );
    labl2 = IupLabel  ( sline.linelabel2 );
    labl3 = IupLabel  ( sline.linelabel6 );
    labl4 = IupLabel  ( sline.linelabel7 );
    text1 = IupText   ( NULL );
    text2 = IupText   ( NULL );
    text3 = IupText   ( NULL );
    text4 = IupText   ( NULL );

    h1 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl1,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text1,IupFill(),NULL), 
		  IupFill(), NULL);
    h2 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl2,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text2,IupFill(),NULL), 
		  IupFill(), NULL);
    h3 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl3,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text3,IupFill(),NULL), 
		  IupFill(), NULL);
    h4 = IupHbox (IupFill(), 
                  IupVbox(IupFill(),labl4,IupFill(),NULL), 
		  IupFill(), 
		  IupVbox(IupFill(),text4,IupFill(),NULL), 
		  IupFill(), NULL);

    t2 = IupFrame (
	     IupVbox (
	        IupFill(),
	        h1,
	        IupFill(),
		h2,
	        IupFill(),
		h3,
	        IupFill(),
		h4,
	        IupFill(),
		NULL
	      )
	    );

    /* sets attributes */
    IupSetAttribute ( text1, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text2, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text3, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text4, "FGCOLOR", "000 000 000" );
    IupSetAttribute ( text1, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text2, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text3, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text4, "BGCOLOR", "255 255 255" );
    IupSetAttribute ( text1, "SIZE", size_text );
    IupSetAttribute ( text2, "SIZE", size_text );
    IupSetAttribute ( text3, "SIZE", size_text );
    IupSetAttribute ( text4, "SIZE", size_text );
    IupSetAttribute ( labl1, "SIZE", size_text );
    IupSetAttribute ( labl2, "SIZE", size_text );
    IupSetAttribute ( labl3, "SIZE", size_text );
    IupSetAttribute ( labl4, "SIZE", size_text );

    IupSetAttribute ( t2, "GAP", "1" );
    IupSetAttribute ( t2, "MARGIN", "1x1" );
    IupSetAttribute ( t2, "SIZE",   "150" );
    IupSetAttribute ( t2, "TABTITLE", sline.linetabs2 );
    IupSetAttribute ( t2, "TITLE",    sline.linelabel0);

    /* sets handles */
    IupSetHandle("x0_line5_handle", text1);
    IupSetHandle("y0_line5_handle", text2);
    IupSetHandle("x1_line5_handle", text3);
    IupSetHandle("y1_line5_handle", text4);
    iupmaskSet(IupGetHandle("x0_line5_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y0_line5_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("x1_line5_handle"), IUPMASK_FLOAT, 0, 1);
    iupmaskSet(IupGetHandle("y1_line5_handle"), IUPMASK_FLOAT, 0, 1);

    IupSetHandle(sline.linetabs2, t2);

    return t2;
  }

  return NULL;
}

/* ===================  dlgValueToIncremSPLine  ===================== */

static void dlgValueToIncremSPLine ( float x0, float y0, 
                                     float x1, float y1 )
{
 char cx0[50];
 char cy0[50];
 char cx1[50];
 char cy1[50];

 if ( sline.dialog == NULL ) return;

 sprintf(cx0, format, x0);
 sprintf(cy0, format, y0);
 sprintf(cx1, format, x1 - x0);
 sprintf(cy1, format, y1 - y0);

 IupSetAttribute (IupGetHandle("x0_line5_handle"), "VALUE", cx0);
 IupSetAttribute (IupGetHandle("y0_line5_handle"), "VALUE", cy0);
 IupSetAttribute (IupGetHandle("x1_line5_handle"), "VALUE", cx1);
 IupSetAttribute (IupGetHandle("y1_line5_handle"), "VALUE", cy1);
}

/* ==================  dlgValueFromIncremSPLine  ==================== */

static void dlgValueFromIncremSPLine ( float *x0, float *y0,
				       float *x1, float *y1 )
{
 if ( sline.dialog == NULL ) return;

 /* incremental case */
 *x0  = IupGetFloat (IupGetHandle("x0_line5_handle"), "VALUE");
 *y0  = IupGetFloat (IupGetHandle("y0_line5_handle"), "VALUE");
 *x1  = IupGetFloat (IupGetHandle("x1_line5_handle"), "VALUE");
 *y1  = IupGetFloat (IupGetHandle("y1_line5_handle"), "VALUE");

 *x1 += *x0;
 *y1 += *y0;
}

/* ===================  dlgValidateIncremSPLine  ===================== */

static int dlgValidateIncremSPLine ( void )
{
 char *x0;
 char *x1;
 char *y0;
 char *y1;

 if ( sline.dialog == NULL ) return 0;

 /* normal case */
 x0 = IupGetAttribute (IupGetHandle("x0_line5_handle"), "VALUE");
 y0 = IupGetAttribute (IupGetHandle("y0_line5_handle"), "VALUE");
 x1 = IupGetAttribute (IupGetHandle("x1_line5_handle"), "VALUE");
 y1 = IupGetAttribute (IupGetHandle("y1_line5_handle"), "VALUE");

 return 1;
}


/* =========================  callbackLineDialogOk  ======================== */

static int callbackLineDialogOk ( Ihandle *bt )
{
 IupSetAttribute(line.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* =======================  callbackLineDialogCancel  ====================== */

static int callbackLineDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(line.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackLineDialogCloseCB  ====================== */

static int callbackLineDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(line.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackFPLineDialogOk  ====================== */

static int callbackFPLineDialogOk ( Ihandle *bt )
{
 IupSetAttribute(fline.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackFPLineDialogCancel  ==================== */

static int callbackFPLineDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(fline.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackFPLineDialogCloseCB  ====================== */

static int callbackFPLineDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(fline.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* ======================  callbackSPLineDialogOk  ====================== */

static int callbackSPLineDialogOk ( Ihandle *bt )
{
 IupSetAttribute(sline.dialog, "CONFIRM", "1");
 return IUP_CLOSE;
}

/* ====================  callbackSPLineDialogCancel  ==================== */

static int callbackSPLineDialogCancel ( Ihandle *bt )
{
 IupSetAttribute(sline.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/* =======================  callbackSPLineDialogCloseCB  ====================== */

static int callbackSPLineDialogCloseCB ( Ihandle *bt )
{
 IupSetAttribute(sline.dialog, "CONFIRM", "0");
 return IUP_CLOSE;
}

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* =========================  DlgCaptureLine  ======================== */

int DlgCaptureLine ( float *x0, float *y0, 
                     float *x1, float *y1, float *tol )
{
  Ihandle *d;

  /* initializes line dialog. */
  dlgInitLine( &line );

  /* create dialog to capture line data. */
  d = dlgCreateLine();

  /* sets line data to dialog. */
  dlgValueToLine ( *x0, *y0, *x1, *y1, *tol );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks line data. */
  if ( dlgValidateLine ( ) == 0 ) return 0;

  /* get line data from dialog. */
  dlgValueFromLine ( x0, y0, x1, y1, tol );

  return 1;
}

/* ======================  DlgCaptureFPLine  ===================== */

int DlgCaptureFPLine ( float *x0, float *y0 )
{
  Ihandle *d;

  /* initializes line dialog. */
  dlgInitFPLine( &fline );

  /* create dialog to capture line data. */
  d = dlgCreateFPLine();

  /* sets line data to dialog. */
  dlgValueToFPLine ( *x0, *y0 );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks line data. */
  if ( dlgValidateFPLine ( ) == 0 ) return 0;

  /* get line data from dialog. */
  dlgValueFromFPLine ( x0, y0 );

  return 1;
}

/* =====================  DlgCaptureSPLine  ===================== */

int DlgCaptureSPLine ( float *x0, float *y0, float *x1, float *y1 )
{
  Ihandle *d;

  /* initializes line dialog. */
  dlgInitSPLine( &sline );

  /* create dialog to capture line data. */
  d = dlgCreateSPLine();

  /* sets line data to dialog. */
  dlgValueToSPLine ( *x0, *y0, *x1, *y1 );

  IupPopup (d, d_posx, d_posy);

  d_posx = IupGetInt( d, IUP_X );
  d_posy = IupGetInt( d, IUP_Y );

  /* checks line data. */
  if ( dlgValidateSPLine ( ) == 0 ) return 0;

  /* get line data from dialog. */
  dlgValueFromSPLine ( x0, y0, x1, y1 );

  return 1;
}

