char *RCSdrag="$Id: drag.c,v 1.2 2001/01/19 13:19:35 lfm Exp $";
/*
** ---------------------------------------------------------------
**
** drag.c - Generic drag functions for primitives.
**
** ---------------------------------------------------------------
** Purpose:
** Allows generic drag of client entities on a canvas window.
**
** ---------------------------------------------------------------
** Description:
**
** This module drives the interactive dialog with the user to
** provide for generic drag of client entities on a canvas
** window.
**
** Each canvas is associated with a drag specification
** descriptor.  It is possible to have a number of these
** descriptors.  To use this package, clients should first call
** function DragOpen to create a descriptor.  This function
** requires two arguments: one is the IUP canvas handle and the
** other is the CD canvas handle (see IUP and CD documentation).
** The IUP canvas handle will be used as an identifier for the
** viewing specification descriptor.  All calls of functions of
** this module have the IUP canvas handle as the first parameter
** (argument). This will set the target viewing descriptor as the
** current active one.
**
** A drag operation consists basically in two main services:
** button down action should call select registered function
** to select entities and button up action should make the 
** drag in the selected primitive using put registered function.
**
** Clients of this module need to implement and register with
** the Drag package five functions:
**   void (*selpk)( Ihandle *iup_cv, double x, double y, double tol,
**                                int shift, int control )
**    Drag select client function based on a given pick point.  
**
**   void (*put)( Ihandle *iup_cv, double x0, double y0,
**                              double x1, double y1, double tol )
**    Drag put client function given a direction.
**
**   void (*prompt)( Ihandle *iup_cv, char *msg )
**    Drag prompt message client function.
**
**   void (*snap)( Ihandle *iup_cv, double *x, double *y, double tol )
**    Drag snap client function.
**
**   void (*echo)( Ihandle *iup_cv, double x0, double y0,
**                               double x1, double y1 )
**    Drag echo client function given a direction.
**
** To register these functions, the client must call the 
** DragRegFunc function, passing the pointers to its functions.
**
** ---------------------------------------------------------------
** Public functions:
** ---------------------------------------------------------------
**
** int    DragOpen( Ihandle *iup_cv, void *cd_cv )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**   cd_cv     - CD handle to target canvas                   (in)
**
** This function initializes a drag specification descriptor.
** It allocates memory for to the drag descriptor and links
** it with the given canvas handle.
**
** ---------------------------------------------------------------
**
** void   DragClose( Ihandle *iup_cv )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**
** This function frees the memory that had been allocated for
** storing the internal drag specification descriptor which
** is associated with the given canvas handle.
**
** ---------------------------------------------------------------
**
** void   DragRegFunc( Ihandle *iup_cv, 
**           void (*selpk)  ( Ihandle *iup_cv, double x, double y, double tol,
**                                          int shift, int control ),
**           void (*put)    ( Ihandle *iup_cv, double x0, double y0,
**                                          double x1, double y1, double tol ), 
**           void (*prompt) ( Ihandle *iup_cv, char *msg ),
**           void (*snap)   ( Ihandle *iup_cv, double *x, double *y, 
**                            double tol, int shift, int control ),
**           void (*echo)   ( Ihandle *iup_cv, double x0, double y0,
**                                          double x1, double y1 ) )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**   selpk     - pointer to client pick selection function    (in)
**   put       - pointer to put function of draged entity     (in)
**   prompt    - pointer to prompt function during drag action(in)
**   snap      - pointer to snap function during drag action  (in)
**   echo      - pointer to echo function given a direction   (in)
**
** This function registers the functions necessary to make a drag
** action: button down action calls a select registered function 
** and button up action calls a put registered function to make
** the drag of selected entities. It also could be registered
** functions to make echo, snap and prompt during drag action.
** The "shift" parameter in the client function argument list
** indicates that the SHIFT key is pressed.
** The "control" parameter in the client function argument list
** indicates that CTRL key is pressed.
**
** ---------------------------------------------------------------
**
** void   DragSetup( Ihandle *iup_cv, 
**                   void (*end_action)( Ihandle *iup_cv, int status ), 
**                   int tolbox )
**
**   iup_cv     - IUP handle to target canvas                 (in)
**   end_action - function to be called after finishing drag  (in)
**   tolbox     - if true (=1), cursor echo is a toler. box   (in)
**
** This function gets from the user a pick point wich is interactively
** defined with the mouse to select the entities to be drawed. This 
** select action is defined by a button down action.
** The user, with this button pressed, defines a direction vector to 
** where selected entities will be drawed when button up action occurs.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal selection ==> status = DRAG_NORMAL = 1
** (2) Quit selection ==> status = DRAG_QUIT = 0
**     This is returned if user quit selection with right mouse
**     button.
** (3) Interupted selection ==> status = DRAG_INTERUPTED = -1
**     This is returned when the selection is closed by something
**     else (not by the mouse interaction).
** It the parameter "tolbox" is on (=1), the initial cursor echo
** is a tolerance box.  Even if tolbox = 0, the cursor echo can
** changed to a box by pressing the middle mouse button.
**
** ---------------------------------------------------------------
**
** TeCGraf - Grupo de Tecnologia em Computacao Grafica, PUC-Rio
**
** Version:    1.0
**
** Created:    20-Jul-95    Joaquim Bento Cavalcante Neto
**
** Modified:   15-Feb-96    LFM
**     Changed from Gra version to CD verstion.
**     Replaced float by double.
**     Replaced "add" parameter by "shift" parameter and added
**     "control" parameter.
**     Changed such that if end_action function is null, nothing
**     is done.  Before, selection was automatically reinitialized.
**
** Modified:   07-Mar-96    LFM
**     When CD changed, creating cdCreateCanvas, which returns
**     a CD handle for the canvas.
**     Changed DragOpen adding a second parameter which is the CD
**     canvas handle.
**
** Modified:   23-Apr-97    LFM
**     Added parameters "shift" and "control" to snap callback function.
**     Added return status parameter for end_action function.
**     Considered environment variable IUP_INGLES for prompt message.
**
** Modified:   19-Jan-2001  LFM
**      When call to function getenv( "IUP_INGLES" ) was replaced 
**      by call to function IupGetLanguage( ).
**      Local string variable iup_english changed to int.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
** ---------------------------------------------------------------
**
*/

/*
** ---------------------------------------------------------------
** Global variables and symbols
*/

#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "iup.h"
#include "pck.h"
#include "drag.h"

/*
** ---------------------------------------------------------------
** Local variables and symbols:
*/

#define ShiftPressed(r)      (r[0]=='S')
#define ControlPressed(r)    (r[1]=='C')

#ifndef LEN
#define LEN(p0,p1) sqrt((p1.x-p0.x)*(p1.x-p0.x)+(p1.y-p0.y)*(p1.y-p0.y))
#endif

/* Private data drag spec. data descriptor:
 */
typedef struct _dragstate {
  struct _dragstate *next;
  void   *iup_cv;                        /* IUP handle to target canvas   */
  void   *cd_cv;                         /* cd handle to corresp.canvas   */
  void   (*selpick)                      /* function to select clients    */
             ( Ihandle *iup_cv, double x, double y, double tol,
                             int shift, int control );
  void   (*put)                          /* function to put collected pts */ 
             ( Ihandle *iup_cv, double x0, double y0,
                             double x1, double y1, double tol );
  void   (*prompt)                       /* function to prompt messages   */
             ( Ihandle *iup_cv, char *msg );
  void   (*snap)                         /* function to snap collect. pts */
             ( Ihandle *iup_cv, double *x, double *y, double tol, 
                             int shift, int control );
  void   (*echo)                         /* function to echo cursor point */
             ( Ihandle *iup_cv, double x0, double y0, double x1, double y1 );
  void   (*end_action)                   /* end action callback           */
             ( Ihandle *iup_cv, int status ); 
  int    tolbox;                         /* if true (1), tol box is on    */
 } DragState;

/* Head of drag specification descriptor list:
 */
static  DragState  *drag_head = 0L;

/* Drag echo status.
 */
static  int drag_status = 0;

static int  iup_english;
static char *prompt_msg;
static char *prompt_msgp = { "Arraste mouse com botao esquerdo precionado; "
                             "Botao da direita => Termina." };
static char *prompt_msge = { "Drag mouse with left button pressed; "
                             "Right button => Discard." };

/*
** ---------------------------------------------------------------
** Local functions:
*/
static DragState *GetDragState( Ihandle *iup_cv );

/* ========================  GetDragState  ===================== */
 
static DragState *GetDragState( Ihandle *iup_cv )
{
 DragState   *drag;
 
 for( drag = drag_head; drag != NULL; drag = drag->next )
 {
  if( drag->iup_cv == iup_cv ) break;
 }
 return( drag );
}

/*
** ---------------------------------------------------------------
** Public functions:
*/
 
/* ==========================  DragOpen  ====================== */
 
int DragOpen( Ihandle *iup_cv, void *cd_cv )
{
 DragState  *drag;
 
 drag = GetDragState( iup_cv );
 if( drag != NULL )
  DragClose( iup_cv );
 
 drag = (DragState *)malloc( sizeof( DragState ) );
 if( drag == NULL )
  return( 0 );
 
 drag->next = drag_head;
 drag->iup_cv = iup_cv;
 drag->cd_cv = cd_cv;
 drag_head = drag;
 
 return( 1 );
}

/* =========================  DragClose  ====================== */
 
void DragClose( Ihandle *iup_cv )
{
 DragState  *drag;
 DragState  *prev;
 
 drag = GetDragState( iup_cv );
 if( drag == NULL )
  return;
 
 if( drag_head == drag )
 {
  drag_head = drag->next;
 }
 else
 {
  prev = drag_head;
  while( ( prev->next != drag ) && ( prev->next != NULL ) )
  {
   prev = prev->next;
  }
 
  if( prev->next != NULL )
  {
   prev->next = drag->next;
  }
 }
 
 free( drag );
}

/* ========================  DragRegFunc  ====================== */

void DragRegFunc( Ihandle *iup_cv, 
              void (*selpk)  ( Ihandle *iup_cv, double x, double y, 
                                       double tol, int shift, int control ),
              void (*put)    ( Ihandle *iup_cv, double x0, double y0,
                                       double x1, double y1, double tol ),
              void (*prompt) ( Ihandle *iup_cv, char *msg ),
              void (*snap)   ( Ihandle *iup_cv, double *x, double *y, 
                                       double tol, int shift, int control ),
              void (*echo)   ( Ihandle *iup_cv, double x0, double y0,
                                       double x1, double y1 ) )
{
 DragState  *drag;
 
 drag = GetDragState( iup_cv );
 if( drag == NULL )
  return;
 
 drag->selpick = selpk;
 drag->put     = put;
 drag->prompt  = prompt;
 drag->snap    = snap;
 drag->echo    = echo;
}

/* ========================  DragSetup  ======================= */

static void _DragBtDwnCB( PckDesc *pk );
static void _DragBtUpCB( PckDesc *pk );
static void _DragEcho( PckDesc *pk, double x, double y );
static void _DragSnap( PckDesc *pk, double *x, double *y, double tol );

void DragSetup( Ihandle *iup_cv, 
                void (*end_action)( Ihandle *iup_cv, int status ), 
                int tolbox ) 
{
 DragState  *drag;
 PckDesc     pk;

 drag = GetDragState( iup_cv );
 if( drag == NULL )
  return;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
  prompt_msg = prompt_msge;
 else
  prompt_msg = prompt_msgp;

 drag->end_action = end_action;
 drag->tolbox = tolbox;

/* Set drag status off.
 */
 drag_status = 0;

/* Setup pick descriptor.  Register the appropriate 
 * callback functions for drag interaction.
 * Register the given end action in the data field.
 */
 if( drag->prompt != NULL )
  (*drag->prompt )( drag->iup_cv, prompt_msg ) ; 
 pk.iup_cv = drag->iup_cv;
 pk.gra_cv = drag->cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.mode = ButtonUp;
 if( drag->tolbox )
  pk.cursorecho = BoxCursor;
 else
  pk.cursorecho = NoEchoCursor;
 pk.button = 0;    /* accept both mouse buttons 1 or 2 */
 pk.toler = 0.0;
 pk.btdwn_callback = _DragBtDwnCB;
 pk.btup_callback = _DragBtUpCB;
 pk.echofunc = _DragEcho;
 pk.snapfunc = _DragSnap;
 pk.data = drag;
 pk.status = 1;
 PckSetup( &pk );
}

/* ----------------------  DragBtDwnCB  ----------------------- */

static void _DragBtDwnCB( PckDesc *pk )
{
 DragState  *drag;
 int        shift;              /* is true if SHIFT key is pressed */
 int        control;            /* is true if CRTL key is pressed  */

/* If left mouse button was pressed (which means drag), 
 * change cursor echo type to no echo and make a selection pick.
 * If middle button was pressed, the user wants to display the
 * tolerance box.  This is taken care at button up.
 */
 if( pk->button == 1 )
  pk->cursorecho = NoEchoCursor;

/* Restore registered drag spec. from the data field.
 */
 drag = (DragState *)pk->data;

/* Set drag status on.
 */
 drag_status = 1;

/* Check to see if SHIFT and CRTL keys are pressed.
 */
 shift   = ShiftPressed( pk->mousecfg );
 control = ControlPressed( pk->mousecfg );

/* Call appropriate client function to make a 
 * pick selection.
 */
 if( drag->selpick != NULL )
    (*drag->selpick)( drag->iup_cv, pk->p0.x, pk->p0.y,
                                    pk->toler, shift, control );
}

/* ----------------------  DragBtUpCB  ------------------------- */

static void _DragBtUpCB( PckDesc *pk )
{
 DragState  *drag;

/* Restore registered drag spec. from the data field.
 */
 drag = (DragState *)pk->data;

/* Set drag status off.
 */
 drag_status = 0;

/* Check to see if drag selection is valid.
 * If not valid, check to see if an end action callback was 
 * declared and, if that is the case, call it and return.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = DRAG_QUIT;        break;
   case PCK_INTERUPTED:  status = DRAG_INTERUPTED;  break;
  }
  if( drag->end_action != NULL )
   (*drag->end_action)( drag->iup_cv, status );
  return;
 }

/* If middle button was pressed, the user wants to display the 
 * tolerance box.  In this case, no selection is done 
 * and the selection is initialized with tolerance box on.
 */
 if( pk->button == 2 )
 {
  DragSetup( drag->iup_cv, drag->end_action, 1 );
  return;
 }

/* Check to see if drag line is valid (greater then tolerance).
 * If not valid, check to see if an end action callback was
 * declared and, if that is the case, call it and return.
 */
 if( LEN(pk->p0,pk->p1) < pk->toler )
 {
  if( drag->end_action )
   (*drag->end_action)( drag->iup_cv, DRAG_QUIT );
  return;
 }

/* Call the client function to put the collected drag points, 
 * Check to see if an end action callback was declared and, if 
 * that is the case, call it and return.
 */
 if( drag->put != NULL )
  (*drag->put )( drag->iup_cv, pk->p0.x, pk->p0.y, pk->p1.x, pk->p1.y, 
                               pk->toler );
 if( drag->end_action )
  (*drag->end_action)( drag->iup_cv, DRAG_NORMAL );
}

/* ---------------------  DragEcho  ------------------------------ */

static void _DragEcho( PckDesc *pk, double x, double y )
{
/* Restore registered drag spec. from the data field.
 */
 DragState  *drag = (DragState *)pk->data;

/* Check to see whether client had registered an echo function.
 * If that is the case call it passing the given mouse position if
 * a pick selection has already done (BtDwn callback called).
 */
 if( drag->echo != NULL )
 {
  if( drag_status )
   (*drag->echo )( drag->iup_cv, pk->p0.x, pk->p0.y, x, y );
 }
}

/* ---------------------  DragSnap  ------------------------------ */

static void _DragSnap( PckDesc *pk, double *x, double *y, double tol )
{
/* Restore registered drag spec. from the data field.
 */
 DragState  *drag = (DragState *)pk->data;

/* Check to see whether client had registered a snap function and
 * call it passing the given mouse position and parameters.
 */
 if( drag->snap != 0L )
  (*drag->snap )( drag->iup_cv, x, y, tol,
                    ShiftPressed(pk->mousecfg), ControlPressed(pk->mousecfg) );
}


