char *RCSpck="$Id: pck.c,v 1.9 2001/01/19 13:18:22 lfm Exp $";
/*
** ---------------------------------------------------------------
** pck.c  -  Pck: module to manage mouse events in a canvas 
**           window.
**
** ---------------------------------------------------------------
**
** Description:
** ---------------------------------------------------------------
** This package creates a simplified layer of abstraction for 
** mouse input interaction in a canvas window.
** It uses the 2D graphics package CD for displaying cursor 
** echo and translating from device to word coordinates.
** Please, refer to specific IUP and CD documentation
** for additional information.
**
** It is assumed that there can only be one current pick 
** interaction at a time and that this interaction can only be
** associated to one canvas.
**
** Currently, the following cursor echo types are implemented:
** NoEchoCursor (default IUP arrow cursor), CrossCursor,
** CrossHairCursor, BoxCursor (adjustable tolerance box), 
** RubberBandCursor, and RectangleCursor.
**
** It is assumed that the right mouse button (button 3) is the
** ESC key used for finishing an interaction.
** It is assumed the a combination of the CONTROL key and the
** middle mouse button (button 2) will ajust the size of the 
** box (tolerance) cursor echo.
**
** The size of the tolerance box is defined in device coordinates
** (pixels).  There are five fixed tolerance box sizes which are
** scrolled using the middle mouse button when the CONTROL key 
** is pressed.  The package always translates tolerance values 
** from device to world (client) coordinates.  Clients will only 
** get tolerance information in world coordinates (wc).  Note that 
** this will automatically decrease the tolerance value in wc if
** the client window sizes are decreased (zooming for example).
**
** In the header file "pck.h" there are macro and type definitions
** for usage of Pck package, as well as the function prototypes 
** which constitute the package API.
**
** ---------------------------------------------------------------
**
** Usage:
** ---------------------------------------------------------------
** For user input interaction this package interfaces with IUP 
** user interface system.  This is done through a set of two Pck 
** functions, PckMouseBt and PckMouseMv, which replicate IUP 
** canvas mouse input callback functions and three other control 
** functions (PckSetup, PckClose, and PckEchoOff).
**
** Clients of this package communicate with it through the four
** control functions (PckDefFunc, PckSetup, PckClose, and
** PckEchoOff):
** - PckDefFunc registers with the package mouse event client
**   callback functions that should be called when there is no
**   pick registered in the package.
** - PckSetup sets up the package for a next mouse input.
** - PckClose closes the current pick interaction.
** - PckEchoOff turns off the current cursor echo (if any).
**
** The package communicates with clients through callback 
** functions which are registered when the pick is set up.  There
** are four client callback functions:
** - btdwn_callback is called when a mouse button is pressed down.
** - btup_callback is called when a mouse button is released.
** - echofunc is called when there is a mouse movement, passing 
**   the current mouse position so that it can do additional echo.
** - snapfunc is called to snap the current mouse position to 
**   a client grid or to any client entity.
** The client need not register all these callback functions.  If
** a NULL value is registered, no callback function is called.
**
** Basically, there is only one function parameter for this two-way 
** communication between the package and its clients: the 
** descriptor structure PckDesc, which is defined in header file
** "pck.h".  The functions PckSetup, btdwn_callback, btup_callback,
** echofunc, and snapfunc receive as a parameter a PckDesc structure 
** passed by reference (a pointer to this structure).  The members of 
** this descriptor structure are explained below:
**
**typedef struct _pckdesc {      -* Pick point descriptor:     *-
** void     *iup_cv;             -* IUP handle of canvas  (I)  *-
** void     *gra_cv;             -* CD/G3d hdle of canvas (I)  *-
** PckPnt   p0;                  -* Initial locator point (I/O)*-
** PckPnt   p1;                  -* Final locator point   (O)  *-
** PckPnt   p0_raw;              -* p0 not snapped        (O)  *-
** PckPnt   p1_raw;              -* p1 not snapped        (O)  *-
** int      mode;                -* Call mode (bt up/dwn) (I)  *-
** int      cursorecho;          -* Cursor echo type      (I)  *-
** int      button;              -* Button selected       (I/O)*-
** char     mousecfg[6];         -* Mouse configuration   (O)  *-
** double   toler;               -* Current pick tolerance(O)  *-
**                               -* Bt-down callback func.(I)  *-
** void     (*btdwn_callback)(struct _pckdesc *);
**                               -* Bt-up callback func.  (I)  *-
** void     (*btup_callback)(struct _pckdesc *);
**                               -* Client echo function  (I)  *-
** void     (*echofunc)(struct _pckdesc *, double x, double y);
**                               -* Client snap function  (I)  *-
** void     (*snapfunc)(struct _pckdesc *, double *x, double *y,
**                                         double tol);
** void     *data;               -* Client data           (I)  *-
** int      status;              -* Pick status           (O)  *-
** } PckDesc;
**
** In this description (I) means input data to PckSetup function 
** and (O) means data returned to client callback function.
**
** iup_cv    - As a PckSetup parameter, indicates to the Pck
**             package the target IUP canvas for pick.
** gra_cv    - As a PckSetup parameter, indicates to the Pck
**             package the associated CD canvas for pick.
** p0        - As a PckSetup parameter, gives the first point 
**             location for rubber-band and rectangle cursor echo.
**             As a btdwn_callback and btup_callback parameter, 
**             is the point location of mouse button press.
** p1        - As a btdwn_callback and btup_callback parameter,
**             is the final pick point location.
** p0_raw    - The same as p0 for output, but before processed by 
**             client snap function.
** p1_raw    - The same as p1, but before processed by client 
**             snap function.
** x0dc,y0dc - Initial (raw) pick point location in raster (CD)
**             coordinates.
** x1dc,y1dc - Final (raw) pick point location in raster (CD)
**             coordinates.
** mode      - As a PckSetup parameter, indicates to the Pck 
**             package whether the pick should end at button down 
**             (mode = ButtonDown) or button release (mode = 
**             ButtonUp).
** cursoecho - As a PckSetup parameter, indicates to the Pck
**             package the type of cursor echo.  Currently the
**             following options are available (there can be
**             unions of these options):
**             NoEchoCursor     =   0
**             CrossCursor      =   1
**             CrossHairCursor  =   2
**             BoxCursor        =   4
**             RubberBandCursor =   8
**             RectangleCursor  =  16
**             VertLineCursor   =  32
**             HorizLineCursor  =  64
**             BallCursor       = 128
** button    - As a PckSetup parameter, indicates to the Pck
**             package the target mouse button for selection.
**             In this case, there are three options:
**             button = 0 => either button 1 (left) or 2 (middle) 
**                           is valid for picking.
**             button = 1 => only button 1 (left) is of interest 
**                           for picking.
**             button = 2 => only button 2 (middle) is of interest 
**                           for picking.
**             As a btdwn_callback and btup_callback parameter,
**             indicates the actual button that was pressed.
** mousecfg  - As a btdwn_callback and btup_callback parameter,
**             indicates the mouse and keybord configuration that
**             was passed by the IUP interface to the Pck package
**             (see IUP documentation).
** toler     - As a btdwn_callback and btup_callback parameter,
**             indicates the current tolerance value in world 
**             (client) coordinates.  The tolerance value 
**             corresponds to the current cursor box size which
**             is maintained by the Pck package in device (pixel)
**             coordinates.
**             For computing the tolerance, the box size is mapped 
**             from device to world coordinates.  Since the 
**             world-to-device coordinates mapping need not be the 
**             same for the horizontal and vertical directions,
**             it is necessary to define which is the direction 
**             used for computing the tolerance.  Function PckTolDir
**             may be used to set the leading direction: when 
**             the argument of this function d = 'x' (default),
**             the mapping is performed in the horizontal direction;
**             when d = 'y', the mapping is in the vertical direction.
** btdwn_callback
** btup_callback
** echofunc
** snapfunc  - As PckSetup parameters, these members of the 
**             PckDesc structure are used to register the client
**             callback functions.  If a NULL value is registered, 
**             no callback function is called.
**
** Basically, there are only two function parameters for this two-way 
** communication:
** data      - As a PckSetup parameter, this pointer is used to
**             hook client data that is passed back to it when
**             a callback function is activated.
** status    - As a btdwn_callback and btup_callback parameter,
**             this indicates whether the given pick descriptor
**             is valid (status = PCK_VALID = 1), or not.  It is 
**             not valid in two situations:
**             (a) the right (ESC) button was pressed 
**                 (status = PCK_QUIT = 0);
**             (b) the function PckClose was called from outside
**                 the package (status = PCK_INTERUPTED = -1).
**             
**
** ---------------------------------------------------------------
**
** Public functions:
** ---------------------------------------------------------------
** void PckDefFunc  ( void (*mousebt)( Ihandle *iup_cv, int b, int mode,
**                                     int x, int y, char *mousecfg ),
**                    void (*mousemv)( Ihandle *iup_cv, int x, int y,
**                                     char *mousecfg ) );
** void PckDefCursor( char *cursor );
** void PckTolDir   ( char d );
** void PckSetup    ( PckDesc *pk );
** void PckMouseBt  ( Ihandle *iup_cv, int b, int mode, 
**                    int x, int y, char *mousecfg );
** void PckMouseMv  ( Ihandle *iup_cv, int x, int y, 
**                    char *mousecfg );
** void PckClose    ( void );
** void PckEchoOff  ( void );
**
** ---------------------------------------------------------------
**
** Version:    1-000
**
** Created:    29-May-95   Luiz Fernando Martha
**      Stolen from previous version that used the Gra package.
**      Created public function PckDefFunc: now when there is no
**      pick registered in the package (pickon = 0), registered 
**      default mouse event callback functions are called.
**      Replaced real by double.
**      Made some modifications such that pick tolerance in 
**      application coordinates is not computed in function that 
**      displays cursor echo.
**      Changed cursor echo display such that it is now displayed 
**      in device (pixel) coordinates.
**      Changed parameters of client functions echofunc and 
**      snapfunc.  The parameters now are the pick point desc.,
**      and the x and y coordinates of the cursor point.
**      Replaced macro RetangleCursor by RectangleCursor in pck.h.
**
** Modified:   23-Aug-95   LFM
**      When created "wd.h" and replaced cdActivate by wdActivate.
**      Added field "context" in PckDesc struct.
**      Created VertLineCursor and HorizLineCursor cursor types.
**
** Modified:   20-Dec-95   LFM
**      Calling snap function in any case (no need for control
**      key pressed).  The application will decide about the
**      mouse configuration based on the configuration string
**      passed in the pick descriptor structure of the first
**      parameter of the snap function.
**
** Modified:   26-Jan-96   LFM
**      Changed local function _PckDoCursorEcho such that it 
**      restores previous defined CD attributes at end.
**
** Modified:   06-Mar-96    LFM
**      When CD changed, creating cdCreateCanvas, which returns
**      a CD handle for the canvas.  Now to activate a canvas with
**      CD one needs only its CD handle.  Before it was needed the
**      IUP canvas handle and the CD driver context. Therefore,
**      the driver context field in PckDesc structure was replaced
**      by the CD canvas handle.
**
** Modified:   07-Sep-96    LFM
**      Added fields p0_raw and p1_raw to PckDesc structure.
**
** Modified:   27-Feb-97    LFM
**      Created local variable prev_btmode to manage occurence of
**      two mouse button events of the same type (e.g., two button up 
**      events).  PckSetup resets this variable to button up.
**
** Modified:   04-Mar-97    LFM
**      Changed local function _PckBtDown such that point p0 is 
**      always button down location.
**
** Modified:   09-Mar-97    LFM
**      Fix problem with local variable prev_btmode that was not
**      updated in all the cases in function PckMouseBt.
**
** Modified:   16-Apr-97    LFM
**      Created macro definitions for pick return status, adding
**      the interupted status.  Created local function _PckQuit.
**
** Modified:   01-May-97    LFM
**      Modifications such that pick interations always end only
**      at button release (even when target is button pressed 
**      point):
**      Renamed local function _PckQuit to _PckExit and modified 
**      function PckMouseBt to call it at button release.
**      Modified functions _PckBtDown and _PckBtUp such that they
**      do not call their corresponding callback functions.
**      These callback functions are called by PckMouseBt at
**      button release through _PckExit.  Function _PckBtDown only
**      calls its callback function if pick interaction mode is
**      ButtonUp so that the client can get the button down pick.
**      Modified PckMouseMv to disregard mouse positions after 
**      button down if target interaction ends at button down.
**
** Modified:   08-Sep-97    LFM
**      Modified PckSetup such that it processes cursor echo 
**      according to the current mouse position.
**      Modified _PckBtDown and PckBtUp such that they update 
**      variable that holds current mouse position in raster 
**      coordinates.
**      Modified PckMouseMv such that it updates variable that
**      current mouse position in raster coordinates even when
**      there is pick requested from a client.
**      Modified PckEchoOff such that it restores default IUP 
**      cursor type (arrow).
**      Renamed local variable first_echopt to echopt_off.
**
** Modified:   09-Sep-97    LFM
**      Modified PckSetup such that is does not processes first 
**      cursor echo.
**
** Modified:   27-Feb-98    LFM
**      Added x0dc, y0dc, x1dc, and y1dc fields to PckDesc structure.
**      Created function PckTolDir.
**      Changed box cursor echo such that the box is distorted 
**      according to the current world-to-raster mapping (which
**      may be different in the horizontal and vertical directions)
**      and to the current leading tolerance direction.
**      Created BallCursor type, which is distorted as the box
**      cursor is.
**
** Modified:   02-Mar-98    LFM
**      Fixed display of tol box, making tolx = toly if they
**      are almost equal (difference of one pixel).
**
** Modified:   15-Oct-98    LFM
**      Created function PckDefCursor to set default Iup cursor.
**      Before default cursor (when no cursor echo was requested)
**      was the arrow cursor.  Now clients may specify the type
**      of cursor they want when there is no echo displayed by
**      the pck module.
**
** Modified:   17-Feb-99    LFM
**      Modified PckSetup such that tolerance value in world coords.
**      is updated when setting up pick.
**
** Modified:   19-Jan-2001    LFM
**      Added calls to cdFlush in functions _PckDoEcho and 
**      PckMouseBt.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

/*
** ---------------------------------------------------------------
** Global variables and symbols
*/
#include <string.h>     /* memcpy */
#include "iup.h"
#include "cd.h"
#include "cdiup.h"
#include "wd.h"
#include "pck.h"


/*
** ---------------------------------------------------------------
** Local definitions and data structures:
*/

#ifndef ABS
#define ABS(x) (((x) < 0.0)? -(x): (x))
#endif

#define ControlPressed(r)  (r[1]=='C')

typedef struct _pntdc {           /* device coordinates point             */
  int    x, y;
 } PntDc;

static int     pickon = 0;        /* current pick status                  */

static PckDesc pickpnt;           /* current pick point descriptor struct.*/

static PckPnt  echopt;            /* last cursor echo point               */

static PntDc   mpt0, mpt1, mptcur;/* init., final and current cursor pos. */

static int     echopt_off = 1;    /* flag for turned off echo point       */

static int     pick_tol = 8;      /* current pick toler. in device coord. */

static char    tol_dir = 'x';     /* leading direction for tolerance      */

static double  tol_ratio;         /* tolerance box aspect ratio           */

static int     cross_size = 8;    /* cursor cross hair half size in dc    */

static int     prev_btmode = ButtonUp; /* previous button mode */

static char    *def_cursor = "ARROW";

/* Mouse event callback functions that should be called when
 * there is no pick registered in the package (pickon = 0).
 */
static void (*mousebt)( Ihandle *iup_cv, int b, int mode,
                        int x, int y, char *mousecfg ) = 0L;
static void (*mousemv)( Ihandle *iup_cv, int x, int y, char *mousecfg ) = 0L;


/*
** ---------------------------------------------------------------
** Local functions:
*/
static void _PckDcWcCursorSize( int size_dc, double *size_wc );
static void _PckUpdatePickTol( void );
static void _PckDoCursorEcho( PntDc *p0, PntDc *p1, int echo_type );
static void _PckDoEcho( void );
static void _PckExit( int status );

/* ===================== PckDcWcCursorSize ==================== */

static void _PckDcWcCursorSize( int size_dc, double *size_wc )
{
 PckPnt  org, end;

 wdCanvas2World( 0, 0, &org.x, &org.y );
 wdCanvas2World( size_dc, size_dc, &end.x, &end.y );
 if( tol_dir == 'y' )
  *size_wc = end.y - org.y;
 else
  *size_wc = end.x - org.x;

 tol_ratio = (end.y - org.y) / (end.x - org.x);
}

/* ===================== PckUpdatePickTol ===================== */

static void _PckUpdatePickTol( void )
{
 if( pick_tol == 4 )
  pick_tol = 20;
 else
  pick_tol -= 4;

 _PckDcWcCursorSize( pick_tol, &pickpnt.toler );
}

/* ===================== PckDoCursorEcho ====================== */

static void _PckDoCursorEcho( PntDc *p0, PntDc *p1, int echo_type )
{
 int  tolx, toly;
 int  fore_color;
 int  write_mode;
 int  line_style;
 int  line_width;

 if( echo_type == NoEchoCursor )
  return;

 if( tol_dir == 'y' )
 {
  toly = pick_tol;
  tolx = (int)((double)toly * tol_ratio);
  if( ABS( tolx - toly ) < 2 ) tolx = toly;
 }
 else
 {
  tolx = pick_tol;
  toly = (int)((double)tolx / tol_ratio);
  if( ABS( tolx - toly ) < 2 ) toly = tolx;
 }

 fore_color = cdForeground( CD_RED );

#ifdef _UNIX_
 write_mode = cdWriteMode( CD_XOR );
#else
 write_mode = cdWriteMode( CD_NOT_XOR );
#endif

 line_style = cdLineStyle( CD_CONTINUOUS );
 line_width = cdLineWidth( 1 );

 if( echo_type & CrossHairCursor )
 {
  int  width, height;
  cdGetCanvasSize( &width, &height, 0L, 0L );
  cdLine( 0, p1->y, width-1, p1->y );
  cdLine( p1->x, 0, p1->x, height-1 );
 }
 else if( echo_type & CrossCursor )
 {
  if( (echo_type & BoxCursor) || (echo_type & BallCursor) )
  {
   if( ! (echo_type & HorizLineCursor) )
    cdLine( p1->x - tolx, p1->y, p1->x + tolx, p1->y );
   if( ! (echo_type & VertLineCursor) )
    cdLine( p1->x, p1->y - toly, p1->x, p1->y + toly ); 
  }
  else
  {
   if( ! (echo_type & HorizLineCursor) )
    cdLine( p1->x - cross_size, p1->y, p1->x + cross_size, p1->y );
   if( ! (echo_type & VertLineCursor) )
    cdLine( p1->x, p1->y - cross_size, p1->x, p1->y + cross_size );
  }
 }

 if( echo_type & RubberBandCursor )
 {
  cdLine( p0->x, p0->y, p1->x, p1->y );
 }

 if( echo_type & BoxCursor )
 {
  cdLine( p1->x-tolx, p1->y-toly, p1->x+tolx, p1->y-toly );
  cdLine( p1->x+tolx, p1->y-toly, p1->x+tolx, p1->y+toly );
  cdLine( p1->x+tolx, p1->y+toly, p1->x-tolx, p1->y+toly );
  cdLine( p1->x-tolx, p1->y+toly, p1->x-tolx, p1->y-toly );
 }

 if( echo_type & RectangleCursor )
 {
  if( echo_type & CrossHairCursor )
  {
   cdLine( p0->x, p0->y, p1->x, p0->y );
   cdLine( p0->x, p1->y, p0->x, p0->y );
  }
  else if( echo_type & VertLineCursor )
  {
   cdLine( p0->x, p0->y, p1->x, p0->y );
   cdLine( p1->x, p1->y, p0->x, p1->y );
   cdLine( p0->x, p1->y, p0->x, p0->y );
  }
  else if( echo_type & HorizLineCursor )
  {
   cdLine( p0->x, p0->y, p1->x, p0->y );
   cdLine( p1->x, p0->y, p1->x, p1->y );
   cdLine( p0->x, p1->y, p0->x, p0->y );
  }
  else
  {
   cdLine( p0->x, p0->y, p1->x, p0->y );
   cdLine( p1->x, p0->y, p1->x, p1->y );
   cdLine( p1->x, p1->y, p0->x, p1->y );
   cdLine( p0->x, p1->y, p0->x, p0->y );
  }
 }

 if( echo_type & VertLineCursor )
 {
  if( ! (echo_type & CrossHairCursor) )
  {
   int  height;
   cdGetCanvasSize( 0L, &height, 0L, 0L );
   cdLine( p1->x, 0, p1->x, height-1 );
  }
 }

 if( echo_type & HorizLineCursor )
 {
  if( ! (echo_type & CrossHairCursor) )
  {
   int  width;
   cdGetCanvasSize( &width, 0L, 0L, 0L );
   cdLine( 0, p1->y, width-1, p1->y );
  }
 }

 if( echo_type & BallCursor )
 {
  cdArc( p1->x, p1->y, 2*tolx, 2*toly, 0.0, 360.0 );
 }

 cdForeground( fore_color );
 cdWriteMode( write_mode );
 cdLineStyle( line_style );
 cdLineWidth( line_width );
}

/* ========================= PckDoEcho ======================== */

static void _PckDoEcho( void )
{
/* Update pick point tolerance value for any cursor type.
 */
 _PckDcWcCursorSize( pick_tol, &pickpnt.toler );

/* Draw cursor echo.
 */
 _PckDoCursorEcho( &mpt0, &mptcur, pickpnt.cursorecho );

/* Check to see if client wants to do its echo; 
 * call echo function if that is the case.
 */
 if( pickpnt.echofunc != 0L )
  pickpnt.echofunc( &pickpnt, echopt.x, echopt.y );

 cdFlush( );
}

/* ========================== PckExit ========================= */
/*
 * Close current pick interaction returning the given status.
 */
static void _PckExit( int status )
{
 if( !pickon ) return;

/* Check to see if echo point is on
 * (in which case, process last cursor echo point).
 */
 if( !echopt_off )
 {
  _PckDoEcho( );
 }

/* Restore default cursor type.
 */
 IupSetAttribute( (Ihandle *)pickpnt.iup_cv, IUP_CURSOR, def_cursor );

/* Close pick interaction, calling client back with the given 
 * status and with the collected pick information.
 */
 pickon = 0;
 pickpnt.status = status;
 if( (pickpnt.mode == ButtonDown) && 
     (pickpnt.btdwn_callback != 0L) )
 {
  (*pickpnt.btdwn_callback)( &pickpnt );
 }
 else if( (pickpnt.mode == ButtonUp) && 
          (pickpnt.btup_callback != 0L) )
 {
  (*pickpnt.btup_callback)( &pickpnt );
 }
}


/*
** ---------------------------------------------------------------
** Public functions:
*/

/* ======================== PckDefFunc ======================== */

void PckDefFunc( void (*mbt)( Ihandle *iup_cv, int b, int mode,
                              int x, int y, char *mousecfg ),
                 void (*mmv)( Ihandle *iup_cv, int x, int y,
                              char *mousecfg ) )
{
 mousebt = mbt;
 mousemv = mmv;
}

/* ======================= PckDefCursor ======================= */

void PckDefCursor( char *cursor )
{
 def_cursor = cursor;
 if( !pickon ) return;
 IupSetAttribute( (Ihandle *)pickpnt.iup_cv, IUP_CURSOR, def_cursor );
}

/* ========================= PckTolDir ======================== */

void PckTolDir( char d )
{
 PckEchoOff( );
 if( (d == 'x') || (d == 'y') )
  tol_dir = d;
}

/* ========================= PckSetup ========================= */
/*
 * Setup pick point descriptor.
 */
void PckSetup( PckDesc *pk )
{
/* Set up things for next pick.
 * Set flag for first echo point.  This will avoid clearing
 * previous cursor echo (since it does not exist).
 */
 pickpnt = *pk;
 if( wdActivate( (cdCanvas *)pickpnt.gra_cv ) == CD_ERROR )
  return;
 wdWorld2Canvas( pickpnt.p0.x, pickpnt.p0.y, &mpt0.x, &mpt0.y );
 pickon = 1;
 prev_btmode = ButtonUp;

/* If there is a cursor echo, erase IUP cursor.
 */
 if( pickpnt.cursorecho == NoEchoCursor )
  IupSetAttribute( (Ihandle *)pickpnt.iup_cv, IUP_CURSOR, def_cursor );
 else
  IupSetAttribute( (Ihandle *)pickpnt.iup_cv, IUP_CURSOR, "NONE" );

/* Update pick point tolerance value for any cursor type.
 */
 _PckDcWcCursorSize( pick_tol, &pickpnt.toler );

#if 0
/* Check to see whether the client had registered a snap function.
 * If that is the case, call it giving the current mouse point.
 */
 if( pickpnt.snapfunc != 0L )
 {
  wdCanvas2World( mptcur.x, mptcur.y, &echopt.x, &echopt.y );
  (*pickpnt.snapfunc )( &pickpnt, &echopt.x, &echopt.y, pickpnt.toler );
  wdWorld2Canvas( echopt.x, echopt.y, &mptcur.x, &mptcur.y );
 }

/* Process first cursor echo: display current cursor position.
 * Check to see if client wants to do its echo; 
 * call echo function if that is the case.
 */
 _PckDoEcho( );
 echopt_off = 0;
#else
 echopt_off = 1;
#endif
}


/* ======================== PckMouseBt ======================== */
/*
 * Get pick point:
 *  _PckBtDown <====> Mouse button press event
 *  _PckBtUp   <====> Mouse button release event
 */
static void _PckBtDown( int x, int y, char *mousecfg );
static void _PckBtUp( int x, int y, char *mousecfg );

void PckMouseBt( Ihandle *iup_cv, int b, int mode, int x, int y, 
                 char *mousecfg )
{
/* If there is no pick registered, call default mouse
 * button event function (in case client had registered one).
 */
 if( !pickon )
 {
  if( mousebt != 0L )
   (*mousebt)( iup_cv, b, mode, x, y, mousecfg );
  return;
 }

/* Disregard pick in the following conditions:
 * - If given canvas is not the current canvas;
 * - If current canvas is not valid;
 * - If previous mouse button mode is equal to current button mode;
 * - If mouse condition is button2-up-with-Control-key or button3-up.
 */
 if( iup_cv != pickpnt.iup_cv ) return;
 if( wdActivate( (cdCanvas *)pickpnt.gra_cv ) == CD_ERROR )
  return;
 if( ( prev_btmode == mode ) || 
     ( (b == IUP_BUTTON2) && (mode == ButtonUp) && ControlPressed(mousecfg) && 
       ((pickpnt.cursorecho & BoxCursor) || (pickpnt.cursorecho & BallCursor)) )
  || ( (b == IUP_BUTTON3) && (mode == ButtonUp) ) )
 {
  prev_btmode = mode;
  return;
 }

/* Update previous button mode variable.
 */
 prev_btmode = mode;

/* Quit pick interaction if button3 down.
 */
 if( b == IUP_BUTTON3 )
 {
  _PckExit( PCK_QUIT );
  return;
 }

/* If target pick is button down and this pick mode is button up,
 * close interaction calling the appropriate callback function.
 */
 if( (pickpnt.mode == ButtonDown) && (mode == ButtonUp) )
 {
  _PckExit( PCK_VALID );
  return;
 }

/* Change cursor box or ball size if button2 down with Control key pressed.
 */
 if( (!echopt_off) && 
    ((pickpnt.cursorecho & BoxCursor) || (pickpnt.cursorecho & BallCursor))
     && (b == IUP_BUTTON2) && ControlPressed(mousecfg) )
 {
  _PckDoCursorEcho( &mpt0, &mptcur, pickpnt.cursorecho );
  cdFlush( );
  _PckUpdatePickTol( );
  _PckDoCursorEcho( &mpt0, &mptcur, pickpnt.cursorecho );
  cdFlush( );
  return;
 }

/* Check to see which button press the client is expecting.
 * If requested button is 1 or 2, accept only the requested one.
 * Otherwise, accept both button 1 or 2.
 */
 if( pickpnt.button == 1 )
 {
  if( b != IUP_BUTTON1 )
   return;
 }
 else if( pickpnt.button == 2 )
 {
  if( b != IUP_BUTTON2 )
   return;
 }

/* Get button that was pressed.
 */
 if( b == IUP_BUTTON1 )
  pickpnt.button = 1;
 else
  pickpnt.button = 2;

/* Update pick point tolerance value for any cursor type.
 */
 _PckDcWcCursorSize( pick_tol, &pickpnt.toler );

/* Copy mouse configuration report to pick structure.
 */
 memcpy( &pickpnt.mousecfg[0], mousecfg, 6 );

/* Reset any current cursor echo.
 */
 PckEchoOff( );

/* Check for button press or release and take the appropriate action.
 */
 if( mode == ButtonDown )
 {
  _PckBtDown( x, y, mousecfg );
 }
 else
 {
  _PckBtUp( x, y, mousecfg );
 }

/* If target pick is button up and this pick mode is button up,
 * close interaction calling the appropriate callback function.
 */
 if( (pickpnt.mode == ButtonUp) && (mode == ButtonUp) )
 {
  _PckExit( PCK_VALID );
  return;
 }
}

/* ------------------------ PckBtDown ------------------------ */

static void _PckBtDown( int x, int y, char *mousecfg )
{
/* Translate pick point from device to application (world) coordinates.
 */
 cdCanvas2Raster( &x, &y );
 wdCanvas2World( x, y, &pickpnt.p0_raw.x, &pickpnt.p0_raw.y );
 pickpnt.p0 = pickpnt.p0_raw;
 pickpnt.x0dc = x;
 pickpnt.y0dc = y;

/* Check to see whether the client had registered a snap function.
 * If that is the case, call it giving the current point.
 * Also get the cursor corresponding device coordinates (snaped or not).
 */
 if( pickpnt.snapfunc != 0L )
 {
  (*pickpnt.snapfunc )( &pickpnt, &pickpnt.p0.x, &pickpnt.p0.y, pickpnt.toler );
  wdWorld2Canvas( pickpnt.p0.x, pickpnt.p0.y, &mpt0.x, &mpt0.y );
 }
 else
 {
  mpt0.x = x;
  mpt0.y = y;
 }

/* Update variable that holds current mouse position in raster 
 * coordinates.
 */
 mptcur.x = mpt0.x;
 mptcur.y = mpt0.y;

/* If pick is in button down mode, set button down location as 
 * final locator point.  Otherwise, this point will be the origin 
 * of cursor echo until button up.
 */
 if( pickpnt.mode == ButtonDown )
 {
  pickpnt.p1 = pickpnt.p0;
 }

/* If target pick interaction mode is ButtonUp, call application function 
 * button down callback with a valid status and with the collected button 
 * down pick information.
 */
 if( pickpnt.mode == ButtonUp )
 {
  pickpnt.status = PCK_VALID;
  if( pickpnt.btdwn_callback != 0L )
  {
   (*pickpnt.btdwn_callback)( &pickpnt );
  }
 }
}

/* ------------------------- PckBtUp ------------------------- */

static void _PckBtUp( int x, int y, char *mousecfg )
{
/* Translate button release pick point from device to application 
 * coordinates.
 */
 cdCanvas2Raster( &x, &y );
 wdCanvas2World( x, y, &pickpnt.p1_raw.x, &pickpnt.p1_raw.y );
 pickpnt.p1 = pickpnt.p1_raw;
 pickpnt.x1dc = x;
 pickpnt.y1dc = y;

/* Check to see whether the client had registered a snap function.
 * If that is the case, call it giving the current point.
 * Also get the cursor corresponding device coordinates (snaped or not).
 */
 if( pickpnt.snapfunc != 0L )
 {
  (*pickpnt.snapfunc )( &pickpnt, &pickpnt.p1.x, &pickpnt.p1.y, pickpnt.toler );
  wdWorld2Canvas( pickpnt.p1.x, pickpnt.p1.y, &mpt1.x, &mpt1.y );
 }
 else
 {
  mpt1.x = x;
  mpt1.y = y;
 }

/* Update variable that holds current mouse position in raster 
 * coordinates.
 */
 mptcur.x = mpt1.x;
 mptcur.y = mpt1.y;
}


/* ======================== PckMouseMv ======================== */
/*
 * Get mouse movement
 */
void PckMouseMv( Ihandle *iup_cv, int x, int y, char *mousecfg )
{
/* If there is no pick registered, keep updated variable that holds 
 * current mouse position in raster coordinates and call default mouse
 * movement event function (in case client had registered one).
 */
 if( !pickon )
 {
  mptcur.x = x;
  mptcur.y = y;
  cdCanvas2Raster( &mptcur.x, &mptcur.y );
  if( mousemv != 0L )
   (*mousemv)( iup_cv, x, y, mousecfg );
  return;
 }

/* Disregard mouse movement in the following conditions:
 * - If given canvas is not the current canvas;
 * - If current canvas is not valid;
 * - If previous pick point was in button down mode and target mode is 
 *   also button down.
 */
 if( iup_cv != pickpnt.iup_cv )
  return;
 if( wdActivate( (cdCanvas *)pickpnt.gra_cv ) == CD_ERROR )
  return;
 if( (prev_btmode == ButtonDown) && (pickpnt.mode == ButtonDown) )
  return;

/* Copy mouse configuration report to pick structure.
 */
 memcpy( &pickpnt.mousecfg[0], mousecfg, 6 );

/* Update initial echo point in case there was a window change.
 */
 wdWorld2Canvas( pickpnt.p0.x, pickpnt.p0.y, &mpt0.x, &mpt0.y );

/* Process cursor echo:
 * If echo point is on, erase previous echo position.
 */
 if( echopt_off )
 {
  echopt_off = 0;

/* If there is a cursor echo, erase IUP cursor.
 */
  if( pickpnt.cursorecho == NoEchoCursor )
   IupSetAttribute( (Ihandle *)pickpnt.iup_cv, IUP_CURSOR, def_cursor );
  else
   IupSetAttribute( (Ihandle *)pickpnt.iup_cv, IUP_CURSOR, "NONE" );
 }
 else
 {
  _PckDoEcho( );
 }

/* Translate cursor position point from device to application 
 * coordinates.
 */
 cdCanvas2Raster( &x, &y );
 wdCanvas2World( x, y, &echopt.x, &echopt.y );

/* Check to see whether the client had registered a snap function.
 * If that is the case, call it giving the current point.
 * Also get the cursor corresponding device coordinates (snaped or not).
 */
 if( pickpnt.snapfunc != 0L )
 {
  (*pickpnt.snapfunc )( &pickpnt, &echopt.x, &echopt.y, pickpnt.toler );
  wdWorld2Canvas( echopt.x, echopt.y, &mptcur.x, &mptcur.y );
 }
 else
 {
  mptcur.x = x;
  mptcur.y = y;
 }

/* Process cursor echo: display current cursor position.
 * Check to see if client wants to do its echo; 
 * call echo function if that is the case.
 */
 _PckDoEcho( );
}


/* ========================= PckClose ========================= */
/*
 * Close pick interaction, calling client back with a non-valid 
 * interupted status (pick was closed by somebody else) and with the 
 * collected pick information.
 */
void PckClose( void )
{
 _PckExit( PCK_INTERUPTED );
}


/* ======================== PckEchoOff ======================== */
/*
 * Reset any current cursor echo.
 */
void PckEchoOff( void )
{
/* If there is pick request, just return.
 */
 if( !pickon ) return;

/* Check to see if echo point is on
 * (in which case, process last cursor echo point).
 */
 if( !echopt_off )
 {
  _PckDoEcho( );
 }

/* Set flag for turned off echo point.
 */
 echopt_off = 1;

/* Restore default cursor type.
 */
 IupSetAttribute( (Ihandle *)pickpnt.iup_cv, IUP_CURSOR, def_cursor );
}


