char *RCSsel="$Id: sel.c,v 1.4 1998/02/27 02:23:27 lfm Exp $";
/*
** ---------------------------------------------------------------
** sel.c - Generic selection functions for use with CD graphics 
**         package and Iup graphics interface.
**
** ---------------------------------------------------------------
** Purpose:
** Allows generic selection of client entities on a canvas window.
**
** ---------------------------------------------------------------
** Description:
**
** This module drives the interactive dialog with the user to
** provide for generic selection of client entities on a canvas
** window.
**
** Each canvas is associated with a selection specification
** descriptor.  It is possible to have a number of these
** descriptors.  To use this package, clients should first call
** function SelOpen to create a descriptor.  This function
** requires two arguments: one is the IUP canvas handle and the
** other is the CD canvas handle (see IUP and CD documentation).
** The IUP canvas handle will be used as an identifier for the
** viewing specification descriptor.  All calls of functions of
** this module have the IUP canvas handle as the first parameter
** (argument). This will set the target viewing descriptor as the
** current active one.
**
** There is basically two types of selection:
**   pick point selection
**   fence selection
**
** Clients of this module need to implement and register with
** the Sel package two functions:
**   void (*selpick)( Ihandle *iup_cv, double x, double y, 
**                    double tol, int shift, int control )
**   Selects client entities based on a given pick point.  
**
**   void (*selfence)( Ihandle *iup_cv, double xmin, double xmax, 
**                     double ymin, double ymax, int shift, int control )
**   Selects client entities based on a given fence.
**
** Besides these callback functions, clients may register two other
** auxiliar functions:
**
**   void (*snap)( Ihandle *iup_cv, double *x, double *y, double tol,
**                               int shift, int control )
**   Client snap cursor point function.
**
**   void (*echo)( Ihandle *iup_cv, double x, double y )
**   Client echo cursor point position function.
**
** To register these four callback functions, the client must call 
** the SelRegFunc function, passing the pointers to its functions.
**
** ---------------------------------------------------------------
** Public functions:
** ---------------------------------------------------------------
**
** int    SelOpen( Ihandle *iup_cv, void *cd_cv )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**   cd_cv     - CD handle to target canvas                   (in)
**
** This function initializes a selection specification descriptor.
** It allocates memory for to the selection descriptor and links
** it with the given canvas handle.
**
** ---------------------------------------------------------------
**
** void   SelClose( Ihandle *iup_cv )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**
** This function frees the memory that had been allocated for
** storing the internal selection specification descriptor which
** is associated with the given canvas handle.
**
** ---------------------------------------------------------------
**
** void   SelRegFunc( Ihandle *iup_cv, 
**                    void (*selpick)( Ihandle *iup_cv, double x, double y, 
**                                     double tol, int shift, int control ),
**                    void (*selfence)( Ihandle *iup_cv, 
**                                      double xmin, double xmax, 
**                                      double ymin, double ymax, 
**                                      int shift, int control ),
**                    void (*snap)( Ihandle *iup_cv, double *x, double *y,
**                                      double tol, int shift, int control ),
**                    void (*echo)( Ihandle *iup_cv, double x, double y ) )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**   selpick   - pointer to client pick selection function    (in)
**   selfence  - pointer to client fence selection function   (in)
**   snap      - ptr. to function to snap cursor points       (in)
**   echo      - ptr. to function to echo cursor position     (in)
**
** This function registers with the selection and inquire 
** specification descriptor which is associated with the given 
** canvas handle two callback functions: one to select and inquire 
** client entities based on a given pick point and another to 
** select client entities based on a given fence.
** In addition it registers two auxiliary functions to snap 
** cursor points and to echo cursor position.
** The "shift" parameter in the client function argument list 
** indicates that the SHIFT key is pressed.
** The "control" parameter in the client function argument list
** indicates that CTRL key is pressed.
**
** ---------------------------------------------------------------
**
** void   SelSetup( Ihandle *iup_cv, 
**                  void (*end_action)( Ihandle *iup_cv, int status ), 
**                  int tolbox )
**
**   iup_cv     - IUP handle to target canvas                 (in)
**   end_action - function to be called after finishing selec.(in)
**   tolbox     - if true (=1), cursor echo is a toler. box   (in)
**
** This function gets from the user a pick point or a fence which 
** is interactively defined with the mouse.  The user indicates 
** two points on the target canvas which defines the fence.
** If the fence size is smaller than a portion of the canvas,
** it is assumed as a pick point.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal selection ==> status = SEL_NORMAL = 1
** (2) Quit selection ==> status = SEL_QUIT = 0
**     This is returned if user quit selection with right mouse
**     button.
** (3) Interupted selection ==> status = SEL_INTERUPTED = -1
**     This is returned when the selection is closed by something
**     else (not by the mouse interaction).
** It the parameter "tolbox" is on (=1), the initial cursor echo
** is a tolerance box.  Even if tolbox = 0, the cursor echo can
** changed to a box by pressing the middle mouse button.
**
** ---------------------------------------------------------------
**
** Version:    1.0
**
** Created:    09-Feb-94    Luiz F. Martha
**
** Modified:   28-Mar-94    LFM
**     Added "reinit" and "tolbox" parameters to SelSetup.
**
** Modified:   31-Mar-94    LFM
**     When created Pck package: change old Gra mouse input (pick)
**     calls to Pck calls.
**
** Modified:   11-Apr-94    LFM
**     Eliminated "reinit" parameter in SelSetup and added
**     "end_action" parameter.
**
** Modified:   23-Sep-94    Joao Luiz Campos ( Acai )
**     Added ControlPressed macro and inquire callback function.
**
** Modified:   29-May-95    LFM
**     Changed from Gra version to CD verstion.
**     Replaced float by double.
**     Replaced macro RetangleCursor by RectangleCursor in pck.h.
**
** Modified:   22-Dec-95    LFM
**     Changed such that cursor is not changed in button down 
**     callback if no selection fence function is defined.
**
** Modified:   15-Feb-96    LFM
**     Changed such that if end_action function is null, nothing
**     is done.  Before, selection was automatically reinitialized.
**
** Modified:   08-Mar-96    LFM
**     When CD changed, creating cdCreateCanvas, which returns
**     a CD handle for the canvas.
**     Changed SelOpen adding a second parameter which is the CD
**     canvas handle.
**
** Modified:   17-Apr-97    LFM
**      Added return status parameter for end action function.
**
** Modified:   07-Sep-97    LFM
**     Added snap and echo callback functions.
**
** Modified:   22-Dec-97    LFM
**     Added cross cursor to box cursor when tolerance box is on.
**
** Modified:   26-Feb-98    LFM
**     When x0dc, y0dc, x1dc, and y1dc fields were added to the Pck 
**     pick descriptor structure.  Changed control of fence and pick 
**     selection accordingly.
**
** Modified:   21-Apr-2004    William Lira
**     Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

/*
** ---------------------------------------------------------------
** Global variables and symbols
*/
#include <stdlib.h>
#include <math.h>

#include "pck.h"
#include "sel.h"

/*
** ---------------------------------------------------------------
** Local variables and symbols:
*/

#define ShiftPressed(r)      (r[0]=='S')
#define ControlPressed(r)    (r[1]=='C')

#ifndef MAX
#define MAX(a,b) (((a)>(b))?(a):(b))
#endif

#ifndef MIN
#define MIN(a,b) (((a)<(b))?(a):(b))
#endif

#ifndef ABS
#define ABS(a) (((a)<0)?-(a):(a))
#endif

/* Private data selection spec. data descriptor:
 */
typedef struct _selstate {
  struct _selstate *next;
  Ihandle *iup_cv;                      /* IUP handle to target canvas */
  void   *cd_cv;                        /* cd handle to corresp.canvas */
  void   (*selpick)( Ihandle *iup_cv, double x, double y, double tol, 
                                   int shift, int control );
  void   (*selfence)( Ihandle *iup_cv, double xmin, double xmax, 
                                    double ymin, double ymax,
                                    int shift, int control );
  void   (*snap)( Ihandle *iup_cv, double *x, double *y, double tol, 
                                int shift, int control );
  void   (*echo)( Ihandle *iup_cv, double x, double y );
  void   (*end_action)( Ihandle *iup_cv, int status ); /* end action callback */
  int    tolbox;                        /* if true (1), tol box is on  */
 } SelState;

/* Head of selection specification descriptor list:
 */
static  SelState  *sel_head = 0L;


/*
** ---------------------------------------------------------------
** Local functions:
*/
static SelState *GetSelState( Ihandle *iup_cv );
static void SelSnap( PckDesc *pk, double *x, double *y, double tol );
static void SelEcho( PckDesc *pk, double x, double y );

/* ========================  GetSelState  ====================== */
 
static SelState *GetSelState( Ihandle *iup_cv )
{
 SelState   *sel;
 
 for( sel = sel_head; sel != NULL; sel = sel->next )
 {
  if( sel->iup_cv == iup_cv ) break;
 }
 return( sel );
}

/* ==========================  SelSnap  ======================== */

static void SelSnap( PckDesc *pk, double *x, double *y, double tol )
{
/* Restore registered selection spec. from the data field.
 */
 SelState  *sel = (SelState *)pk->data;

/* Check to see whether client had registered a snap function.
 * If that is the case call it passing the given mouse parameters.
 */
 if( sel->snap != NULL )
 {
  (*sel->snap )( sel->iup_cv, x, y, tol,
                 ShiftPressed(pk->mousecfg), ControlPressed(pk->mousecfg) );
 }
}

/* ==========================  SelEcho  ======================== */

static void SelEcho( PckDesc *pk, double x, double y )
{
/* Restore registered selection spec. from the data field.
 */
 SelState  *sel = (SelState *)pk->data;

/* Check to see whether client had registered an echo function.
 * If that is the case call it passing the given mouse position.
 */
 if( sel->echo != NULL )
 {
  (*sel->echo )( sel->iup_cv, x, y );
 }
}


/*
** ---------------------------------------------------------------
** Public functions:
*/
 
/* ==========================  SelOpen  ======================= */
 
int SelOpen( Ihandle *iup_cv, void *cd_cv )
{
 SelState  *sel;
 
 sel = GetSelState( iup_cv );
 if( sel != NULL )
  SelClose( iup_cv );
 
 sel = (SelState *)malloc( sizeof( SelState ) );
 if( sel == NULL )
  return( 0 );
 
 sel->next = sel_head;
 sel->iup_cv = iup_cv;
 sel->cd_cv = cd_cv;
 sel_head = sel;
 
 return( 1 );
}


/* =========================  SelClose  ======================= */
 
void SelClose( Ihandle *iup_cv )
{
 SelState  *sel;
 SelState  *prev;
 
 sel = GetSelState( iup_cv );
 if( sel == NULL )
  return;
 
 if( sel_head == sel )
 {
  sel_head = sel->next;
 }
 else
 {
  prev = sel_head;
  while( ( prev->next != sel ) && ( prev->next != NULL ) )
  {
   prev = prev->next;
  }
 
  if( prev->next != NULL )
  {
   prev->next = sel->next;
  }
 }
 
 free( sel );
}


/* ========================  SelRegFunc  ====================== */

void SelRegFunc( Ihandle *iup_cv, 
                 void (*selpk)( Ihandle *iup_cv, double x, double y, 
                                double tol, int shift, int control ),
                 void (*selfc)( Ihandle *iup_cv, double xmn, double xmx, 
                                double ymn,double ymx,int shift,int control ),
                 void (*snap)( Ihandle *iup_cv, double *x, double *y, double tol,
                               int shift, int control ),
                 void (*echo)( Ihandle *iup_cv, double x, double y ) )
{
 SelState  *sel;
 
 sel = GetSelState( iup_cv );
 if( sel == NULL )
  return;
 
 sel->selpick = selpk;
 sel->selfence = selfc;
 sel->snap = snap;
 sel->echo = echo;
}


/* ========================  SelSetup  ======================== */

static void _SelBtDwnCB( PckDesc *pk );
static void _SelBtUpCB( PckDesc *pk );

void SelSetup( Ihandle *iup_cv, 
               void (*end_action)( Ihandle *iup_cv, int status ), 
               int tolbox ) 
{
 SelState  *sel;
 PckDesc   pk;

 sel = GetSelState( iup_cv );
 if( sel == NULL )
  return;

 sel->end_action = end_action;
 sel->tolbox = tolbox;

/* Setup pick descriptor.  Register the appropriate 
 * callback functions for selection interaction.
 * Register the given end action in the data field.
 */
 pk.iup_cv = sel->iup_cv;
 pk.gra_cv = sel->cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.mode = ButtonUp;
 if( sel->tolbox )
  pk.cursorecho = BoxCursor | CrossCursor;
 else
  pk.cursorecho = NoEchoCursor;
 pk.button = 0;    /* accept both mouse buttons 1 or 2 */
 pk.toler = 0.0;
 pk.btdwn_callback = _SelBtDwnCB;
 pk.btup_callback = _SelBtUpCB;
 pk.echofunc = SelEcho;
 pk.snapfunc = SelSnap;
 pk.data = sel;
 pk.status = PCK_VALID;
 PckSetup( &pk );
}

/* ----------------------  SelBtDwnCB  ------------------------ */

static void _SelBtDwnCB( PckDesc *pk )
{
 SelState  *sel;

/* Restore registered selection spec. from the data field.
 */
 sel = (SelState *)pk->data;

/* If left mouse button was pressed (which means selection), 
 * change cursor echo type to retangle.
 * If middle button was pressed, the user wants to display the
 * tolerance box.  This is taken care at button up.
 */
 if( (sel->selfence != NULL) && (pk->button == 1) )
  pk->cursorecho = RectangleCursor;
}

/* ----------------------  SelBtUpCB  ------------------------- */

static void _SelBtUpCB( PckDesc *pk )
{
 SelState  *sel;
 double    xmin, xmax,
           ymin, ymax;         /* selection fence                 */
 int       shift;              /* is true if SHIFT key is pressed */
 int       control;            /* is true if CRTL key is pressed  */

/* Restore registered selection spec. from the data field.
 */
 sel = (SelState *)pk->data;

/* Check to see if selected fence is valid.
 * If not valid, check to see if an end action callback was 
 * declared and, if that is the case, call it and return.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = SEL_QUIT;        break;
   case PCK_INTERUPTED:  status = SEL_INTERUPTED;  break;
  }
  if( sel->end_action != NULL )
   (*sel->end_action)( sel->iup_cv, status );
  return;
 }

/* If middle button was pressed, the user wants to display the 
 * tolerance box.  In this case, no selection is done 
 * and the selection is initialized with tolerance box on.
 */
 if( pk->button == 2 )
 {
  SelSetup( sel->iup_cv, sel->end_action, 1 );
  return;
 }

/* Check to see if SHIFT and CRTL keys are pressed.
 */
 shift   = ShiftPressed( pk->mousecfg );
 control = ControlPressed( pk->mousecfg );

/* Check to see if selection mode is pick or fence.
 * It is pick mode if the fence window is small.
 * And call the appropriate client function.
 */
 if( (ABS(pk->x0dc - pk->x1dc) < 5) && (ABS(pk->y0dc - pk->y1dc) < 5) )
 {
  if( sel->selpick != NULL )
   (*sel->selpick)( sel->iup_cv, pk->p0.x, pk->p0.y,
                                 pk->toler, shift, control );
 }
 else
 {
/* Get fence sizes
 */
  xmin = MIN( pk->p0.x, pk->p1.x );
  xmax = MAX( pk->p0.x, pk->p1.x );
  ymin = MIN( pk->p0.y, pk->p1.y );
  ymax = MAX( pk->p0.y, pk->p1.y );

  if( sel->selfence != NULL )
   (*sel->selfence)( sel->iup_cv, xmin, xmax, ymin, ymax, shift, control );
 }

/* Check to see if an end action callback was declared and, 
 * if that is the case, call it and return.
 */
 if( sel->end_action != NULL )
  (*sel->end_action)( sel->iup_cv, SEL_NORMAL );
}


