/*
** t2d.c  -  Package to manage 2D matrix transformations.
**
** remarks:
** ---------------------------------------------------------------
** Functionality:
**
** This package maintains a 2D transformation matrix which can
** be initialized as the identity matrix, and which accumulates 
** the usual 2D transformations.
**
** There is an entry point to transform a given 2D point 
** according to the current transformation matrix.
**
** The 2D transformations considered are scaling, translation, 
** and rotation.  There is an entry point for each one of these
** transformations.  The effect of these functions is to 
** accumulate the corresponding transformation in the current
** transformation matrix.
**
** ---------------------------------------------------------------
**
** entry pts:
** ---------------------------------------------------------------
**
** void T2dTransform( double *x, double *y )
**
**    x, y       - coordinates of transformed point       (in/out)
**
** This function transforms the given coordinates according
** to the current transformation matrix.
**
** ---------------------------------------------------------------
**
** void T2dGetTransform( Matrix2d mtx )
**
**    mtx    - current transformation matrix                 (out)
**
** This function returns the current transformation matrix.
**
** ---------------------------------------------------------------
**
** void T2dSetTransform( Matrix2d mtx )
**
**    mtx    - given transformation matrix                    (in)
**
** This function sets the given matrix as the current 
** transformation matrix.
**
** ---------------------------------------------------------------
**
** void T2dIdentity( void )
**
** This function sets the current transformation matrix as 
** the identity matrix.
**
** ---------------------------------------------------------------
**
** void T2dInverse( void )
**
** This function sets the current transformation matrix as 
** its inverse matrix.
**
** ---------------------------------------------------------------
**
** void T2dTranslate( double dx, double dy )
**
**    dx, dy   - translation parameters                       (in)
**
** This function accumulates a translation transformation to 
** the current transformation matrix.
**
** ---------------------------------------------------------------
**
** void T2dScale( double sx, double sy )
**
**    sx, sy   - scaling parameters                           (in)
**
** This function accumulates a scaling transformation to the
** current transformation matrix.  The scaling is performed 
** about the origin (center of scaling).
**
** ---------------------------------------------------------------
**
** void T2dScaleAbout( double cx, double cy, double sx, double sy )
**
**    cx, cy   - center of scaling                            (in)
**    sx, sy   - scaling parameters                           (in)
**
** This function accumulates a scaling transformation to the
** current transformation matrix.  The scaling is performed 
** about the given point (center of scaling).
**
** ---------------------------------------------------------------
**
** void T2dRotate( double angle )
**
**    angle    - rotation angle (degrees)                     (in)
**
** This function accumulates a rotation transformation to the
** current transformation matrix.  The rotation is performed 
** about the origin (center of rotation).
** The rotation angle in assumed given in degrees.
**
** ---------------------------------------------------------------
**
** void T2dRotateAbout( double cx, double cy, double angle )
**
**    cx, cy   - center of rotation                           (in)
**    angle    - rotation angle (degrees)                     (in)
**
** This function accumulates a rotation transformation to the
** current transformation matrix.  The rotation is performed 
** about the given point (center of rotation).
** The rotation angle in assumed given in degrees.
**
** ---------------------------------------------------------------
**
** void T2dShear( double shx, double shy )
**
**    shx, shy - shear parameters                             (in)
**
** This function accumulates a shear transformation to the
** current transformation matrix.  The shearing is performed 
** about the origin (center of shearing).
**
** ---------------------------------------------------------------
**
** void T2dShearAbout( double cx, double cy, double shx, double shy )
**
**    cx, cy   - center of shearing                           (in)
**    shx, shy - shear parameters                             (in)
**
** This function accumulates a shear transformation to the
** current transformation matrix.  The shearing is performed 
** about the origin (center of shearing).
**
** ---------------------------------------------------------------
**
** void T2dMirrorX( void )
**
** This function accumulates a mirror (X - axis) transformation
** to the current transformation matrix.
**
** ---------------------------------------------------------------
**
** void T2dMirrorY( void )
**
** This function accumulates a mirror (Y - axis) transformation
** to the current transformation matrix.
**
** ---------------------------------------------------------------
**
** void T2dMirrorVec( double vx, double vy )
**
**    vx, vy   - given vector direction                       (in)
**
** This function accumulates a mirror transformation to the 
** current transformation matrix.  The mirroring transformation
** is performed about a line that passes at the origin which has 
** the given vector direction.
**
** ---------------------------------------------------------------
**
** void T2dMirrorVecAbout( double x0, double y0, double x1, double y1 )
**
**    x0, y0   - first mirror line point                      (in)
**    x1, y1   - second mirror line point                     (in)
**
** This function accumulates a mirror transformation to the 
** current transformation matrix.  The mirroring transformation
** is performed about the line which is defined through the given 
** points.
**
** ---------------------------------------------------------------
**
** Version:    0-001
**
** Created:    05-Apr-97    Luiz Fernando Martha
**
*/

/*
** ---------------------------------------------------------------
** Global variables and symbols
*/
#include <math.h>
#include "t2d.h"

/*
** ---------------------------------------------------------------
** Local variables and symbols:
*/
#ifndef rad
#define rad(a)  ((a)*(double)0.01745329252)
#endif

static Matrix2d m =      /* Current transformation matrix */
 {
  { 1.0, 0.0, 0.0 },
  { 0.0, 1.0, 0.0 }
 };

/*
** ---------------------------------------------------------------
** Private functions:
*/

/*
** ---------------------------------------------------------------
** Public entry points:
*/

/*======================  T2dTransform  ========================*/

void T2dTransform( double *x, double *y )
{
 double t;
  t = (*x)*m[0][0]+(*y)*m[0][1]+m[0][2];
 *y = (*x)*m[1][0]+(*y)*m[1][1]+m[1][2];
 *x = t;
}

/*=====================  T2dGetTransform  ======================*/

void T2dGetTransform( Matrix2d mtx )
{
 mtx[0][0] = m[0][0];  mtx[0][1] = m[0][1];  mtx[0][2] = m[0][2];
 mtx[1][0] = m[1][0];  mtx[1][1] = m[1][1];  mtx[1][2] = m[1][2];
}

/*=====================  T2dSetTransform  ======================*/

void T2dSetTransform( Matrix2d mtx )
{
 m[0][0] = mtx[0][0];  m[0][1] = mtx[0][1];  m[0][2] = mtx[0][2];
 m[1][0] = mtx[1][0];  m[1][1] = mtx[1][1];  m[1][2] = mtx[1][2];
}

/*======================  T2dIdentity  =========================*/

void T2dIdentity( void )
{
 m[0][0] = 1.0; m[0][1] = 0.0; m[0][2] = 0.0;
 m[1][0] = 0.0; m[1][1] = 1.0; m[1][2] = 0.0;
}

/*=======================  T2dInverse  =========================*/

void T2dInverse( void )
{
 Matrix2d inv;
 double   det = m[0][0] * m[1][1] - m[0][1] * m[1][0];

 inv[0][0] =  m[1][1] / det;
 inv[0][1] = -m[0][1] / det;
 inv[0][2] =  (m[0][1] * m[1][2] - m[0][2] * m[1][1]) / det;
 inv[1][0] = -m[1][0] / det;
 inv[1][1] =  m[0][0] / det;
 inv[1][2] = -(m[0][0] * m[1][2] - m[0][2] * m[1][0]) / det;

 m[0][0] = inv[0][0];  m[0][1] = inv[0][1];  m[0][2] = inv[0][2];
 m[1][0] = inv[1][0];  m[1][1] = inv[1][1];  m[1][2] = inv[1][2];
}

/*=======================  T2dTranslate  =======================*/

void T2dTranslate( double dx, double dy )
{
 m[0][2] += dx;
 m[1][2] += dy;
}

/*=========================  T2dScale  =========================*/

void T2dScale( double sx, double sy )
{
 m[0][0] *= sx; m[0][1] *= sx; m[0][2] *= sx;
 m[1][0] *= sy; m[1][1] *= sy; m[1][2] *= sy;
}

/*=======================  T2dScaleAbout  ======================*/

void T2dScaleAbout( double cx, double cy, double sx, double sy )
{
 T2dTranslate( -cx, -cy );
 T2dScale( sx, sy );
 T2dTranslate( cx, cy );
}

/*=========================  T2dRotate  ========================*/

void T2dRotate( double angle )
{
 double cost = cos(rad(angle));
 double sint = sin(rad(angle));
 double t;

 t       = m[0][0]*cost-m[1][0]*sint;
 m[1][0] = m[0][0]*sint+m[1][0]*cost;
 m[0][0] = t;
 t       = m[0][1]*cost-m[1][1]*sint;
 m[1][1] = m[0][1]*sint+m[1][1]*cost;
 m[0][1] = t;
 t       = m[0][2]*cost-m[1][2]*sint;
 m[1][2] = m[0][2]*sint+m[1][2]*cost;
 m[0][2] = t;
}

/*======================  T2dRotateAbout  ======================*/

void T2dRotateAbout( double cx, double cy, double angle )
{
 T2dTranslate( -cx, -cy );
 T2dRotate( angle );
 T2dTranslate( cx, cy );
}

/*=========================  T2dShear  =========================*/

void T2dShear( double shx, double shy )
{
 double t;

 t       = m[0][0] + m[1][0]*shx;
 m[1][0] = m[0][0]*shy + m[1][0];
 m[0][0] = t;
 t       = m[0][1] + m[1][1]*shx;
 m[1][1] = m[0][1]*shy + m[1][1];
 m[0][1] = t;
 t       = m[0][2] + m[1][2]*shx;
 m[1][2] = m[0][2]*shy + m[1][2];
 m[0][2] = t;
}

/*=======================  T2dShearAbout  ======================*/

void T2dShearAbout( double cx, double cy, double shx, double shy )
{
 T2dTranslate( -cx, -cy );
 T2dShear( shx, shy );
 T2dTranslate( cx, cy );
}

/*========================  T2dMirrorX  ========================*/

void T2dMirrorX( void )
{
 m[1][0] = -m[1][0];
 m[1][1] = -m[1][1];
 m[1][2] = -m[1][2];
}
/*========================  T2dMirrorY  ========================*/

void T2dMirrorY( void )
{
 m[0][0] = -m[0][0];
 m[0][1] = -m[0][1];
 m[0][2] = -m[0][2];
}

/*=======================  T2dMirrorVec  =======================*/

void T2dMirrorVec( double vx, double vy )
{
 double  a, a2, p, q, t;

 if( fabs(vx) == 0.0 )
 {
  T2dMirrorY( );
  return;
 }
 if( fabs(vy) == 0.0 )
 {
  T2dMirrorX( );
  return;
 }

 if( fabs(vx) > fabs(vy) )
 {
  a = vy / vx;
  q =  1.0 - a*a;
 }
 else
 {
  a = vx / vy;
  q = -1.0 + a*a;
 }
 a2 = 2.0*a;
 p = 1.0 + a*a;

 t       = (m[0][0]*q  + m[1][0]*a2) / p;
 m[1][0] = (m[0][0]*a2 - m[1][0]*q ) / p;
 m[0][0] = t;
 t       = (m[0][1]*q  + m[1][1]*a2) / p;
 m[1][1] = (m[0][1]*a2 - m[1][1]*q ) / p;
 m[0][1] = t;
 t       = (m[0][2]*q  + m[1][2]*a2) / p;
 m[1][2] = (m[0][2]*a2 - m[1][2]*q ) / p;
 m[0][2] = t;
}

/*====================  T2dMirrorVecAbout  =====================*/

void T2dMirrorVecAbout( double x0, double y0, double x1, double y1 )
{
 T2dTranslate( -x0, -y0 );
 T2dMirrorVec( x1-x0, y1-y0 );
 T2dTranslate( x0, y0 );
}

