char *RCStrf="$Id: trf.c,v 1.1 1997/04/23 19:22:02 lfm Exp $";
/*
** trf.c - Public functions for collection and generation of 
**         transformation matrices.
**
** Public functions:
** --------------------------------------------------------------------
**
** void TrfCollect( TrfType type,
**                  Ihandle *iup_cv, void *cd_cv,
**                  void (*end_action)( Ihandle *iup_cv, int status ),
**                  void (*put)( Ihandle *iup_cv, void *trf, double tol ),
**                  void (*prompt)( Ihandle *iup_cv, char *msg ),
**                  void (*snap)( Ihandle *iup_cv, double *x, double *y,
**                                double tol, int shift, int control ),
**                  void (*echo)( Ihandle *iup_cv, double x, double y,
**                                int status, double mat[2][3] ) )
**
** type       - type of the transformation to collect              (IN)
** iup_cv     - IUP handle of canvas where transform. is collected (IN)
** cd_cv      - CD handle of canvas                                (IN)
** end_action - function to be called after finishing collection   (IN)
** put        - ptr. to function to put transf. after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect.  (IN)
** snap       - ptr. to function to snap points during collection  (IN)
** echo       - ptr. to func. to echo cursor pos. and transf.mat.  (IN)
**
** This function interactively collects from the user data necessary to
** generate a transformation of the given type on a given canvas.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal transformation collection ==> status = TRF_NORMAL = 1
** (2) Quit transformation collection ==> status = TRF_QUIT = 0
**     This is returned if user quit transformation collection with
**     mouse right button.
** (3) Interupted transf. collection ==> status = TRF_INTERUPTED = -1
**     This is returned when the transformation collection is closed 
**     by something else (not by the mouse interaction).
**
** --------------------------------------------------------------------
**
** int  TrfKbdCollect( TrfType type,
**                     Ihandle *iup_cv, 
**                     void (*put)( Ihandle *iup_cv, void *trf, double tol ),
**                     void (*prompt)( Ihandle *iup_cv, char *msg ) )
**
** type       - type of the transformation to collect              (IN)
** iup_cv     - IUP handle of canvas for client reference          (IN)
** put        - ptr. to function to put transf. after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect.  (IN)
**
** This function interactively collects from the keyboard the 
** parameters that generate a transformation of the given type.
** If the collection is successfull the function returns a true (1)
** status.  Otherwise, it returns a false (0) status.
**
** --------------------------------------------------------------------
**
** Version:      1-000
**
** Created:    17-Apr-97    Luiz Fernando Martha
**
** Modified:   21-Apr-2004    William Lira
**      Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <math.h>

#include "trf.h"
#include "trfmove.h"
#include "trfmirr.h"
#include "trfrotat.h"
#include "trfscale.h"


/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/

/*
** --------------------------------------------------------------------
** Local functions:
*/

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* ==========================  TrfCreate  ========================= */

void TrfCollect( TrfType type,
                 Ihandle *iup_cv, void *cd_cv,
                 void (*end_action)( Ihandle *iup_cv, int status ),
                 void (*put)( Ihandle *iup_cv, void *trf, double tol ),
                 void (*prompt)( Ihandle *iup_cv, char *msg ),
                 void (*snap)( Ihandle *iup_cv, double *x, double *y,
                               double tol, int shift, int control ),
                 void (*echo)( Ihandle *iup_cv, double x, double y,
                               int status, double mat[2][3] ) )
{
 switch( type )
 {
  case TRF_MOVE:
   TrfMove( iup_cv, cd_cv, end_action, put, prompt, snap, echo );
   break;

  case TRF_MIRROR:
   TrfMirror( iup_cv, cd_cv, end_action, put, prompt, snap, echo );
   break;

  case TRF_ROTATE:
   TrfRotate( iup_cv, cd_cv, end_action, put, prompt, snap, echo );
   break;

  case TRF_SCALE:
   TrfScale( iup_cv, cd_cv, end_action, put, prompt, snap, echo );
   break;

  default:
   break;
 }
}

/* ========================  TrfKbdCreate  ======================== */

int  TrfKbdCollect( TrfType type,
                    Ihandle *iup_cv, 
                    void (*put)( Ihandle *iup_cv, void *trf, double tol ),
                    void (*prompt)( Ihandle *iup_cv, char *msg ) )
{
 int  status = 0;

 switch( type )
 {
  case TRF_MOVE:
   status = TrfKbdMove( iup_cv, put, prompt );
   break;

  case TRF_MIRROR:
   status = TrfKbdMirror( iup_cv, put, prompt );
   break;

  case TRF_ROTATE:
   status = TrfKbdRotate( iup_cv, put, prompt );
   break;

  case TRF_SCALE:
   status = TrfKbdScale( iup_cv, put, prompt );
   break;

  default:
   break;
 }

 return( status );
}

/* ========================  TrfShiftSnap  ======================== */

void TrfShiftSnap( double x0, double y0, double *x1, double *y1 )
{
 double  vx, vy, a;

 vx = *x1 - x0;
 vy = *y1 - y0;

 if( (fabs(vx) == 0.0) || (fabs(vy) == 0.0) )
  return;

 if( fabs(vx) > fabs(vy) )
 {
  a = vy / vx;
  if( fabs(a) > 0.75 )
  {
   if( a > 0 )
    *y1 = y0 + (*x1-x0);
   else
    *y1 = y0 - (*x1-x0);
  }
  else
  {
   *y1 = y0;
  }
 }
 else
 {
  a = vx / vy;
  if( fabs(a) > 0.75 )
  {
   if( a > 0 )
    *x1 = x0 + (*y1-y0);
   else
    *x1 = x0 - (*y1-y0);
  }
  else
  {
   *x1 = x0;
  }
 }
}


