char *RCStrfmove="$Id: trfmove.c,v 1.5 2001/01/19 13:29:52 lfm Exp $";
/*
** trfmove.c - Functions to generate a matrix transformation for
**             translating objects.
**
** Public functions:
** -------------------------------------------------------------------
**
** void TrfMove( Ihandle *iup_cv, void *cd_cv,
**               void (*end_action)( Ihandle *iup_cv, int status ), 
**               void (*put)( Ihandle *iup_cv, void *m, double tol ),
**               void (*prompt)( Ihandle *iup_cv, char *msg ),
**               void (*snap)( Ihandle *iup_cv, double *x, double *y,
**                             double tol, int shift, int control ),
**               void (*echo)( Ihandle *iup_cv, double x, double y,
**                             int status, double mat[2][3] ) )
**
**    iup_cv     - IUP handle of canvas where object is selected     (IN)
**    cd_cv      - CD handle of canvas                               (IN)
**    end_action - function to be called after finishing operation   (IN)
**    put        - ptr. to function to put matrix after collection   (IN)
**    prompt     - ptr. to function to prompt messg. during collect. (IN)
**    snap       - ptr. to function to snap points during collection (IN)
**    echo       - ptr. to func. to echo cursor pos. and transf.mat. (IN)
**
** This function interactively collects from the user a straight
** line on a given canvas for constructing a translation transformation 
** matrix.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal transformation collection ==> status = TRF_NORMAL = 1
** (2) Quit transformation collection ==> status = TRF_QUIT = 0
**     This is returned if user quit transformation collection with
**     mouse right button.
** (3) Interupted transf. collection ==> status = TRF_INTERUPTED = -1
**     This is returned when the transformation collection is closed 
**     by something else (not by the mouse interaction).
** 
** -------------------------------------------------------------------
**
** int  TrfKbdMove( Ihandle *iup_cv,
**                  void (*put)( Ihandle *iup_cv, void *m, double tol ),
**                  void (*prompt)( Ihandle *iup_cv, char *msg ) )
**
** iup_cv     - IUP handle of canvas for client reference         (IN)
** put        - ptr. to function to put matrix after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
**
** This function interactively collects from the keyboard a 
** translation vector.
** If the collection is successfull the function returns a true (1)
** status.  Otherwise, it returns a false (0) status.
**
** --------------------------------------------------------------------
**
** Version:      1-000
** 
** Created:    23-Apr-97    Luiz Fernando Martha
**
** Modified:   18-Feb-98    LFM
**      Modification to avoid displaying echo messages on the canvas.
**      Instead, echo messages are send to prompt message UI object.
**
** Modified:   19-Jan-2001  LFM
**      When call to function getenv( "IUP_INGLES" ) was replaced 
**      by call to function IupGetLanguage( ).
**      Local string variable iup_english changed to int.
**      Modified IupScanf floating point format.
**
** Modified:   08-Aug-2002    Marcos C. de Arruda
**      Replaced call to IupScanf by call to corresponding dialog
**      (Dlg) function.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

/*
** -------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "iup.h"
#include "cd.h"
#include "pck.h"
#include "trf.h"
#include "trfmove.h"
#include "dlgmove.h"

/*
** -------------------------------------------------------------------
** Local variables and definitions:
*/
/* Move data structure:
 */
typedef struct _trfmovep
{
 TrfMv  m;
 PckPnt p0, p1;            /* translation vector points     */
 int    fst;               /* if true, first point given    */
 void   *iup_cv;           /* IUP handle to target canvas   */
 void   *cd_cv;            /* cd handle to corresp.canvas   */
 void   (*end_action)( Ihandle *iup_cv, int status );
 void   (*put)( Ihandle *iup_cv, void *m, double tol );
 void   (*prompt)( Ihandle *iup_cv, char *msg );
 void   (*snap)( Ihandle *iup_cv, double *x, double *y, double tol,
                               int shift, int control );
 void   (*echo)( Ihandle *iup_cv, double x, double y, 
                               int status, double mat[2][3] );
} TrfMoveP;

#ifndef LEN
#define LEN(p0,p1) sqrt((p1.x-p0.x)*(p1.x-p0.x)+(p1.y-p0.y)*(p1.y-p0.y))
#endif

static int  iup_english;
static char *prompt_msg1;
static char *prompt_msg2;

static char *prompt_msg1p ={ "Entre primeiro ponto com botao esquerdo do mouse;"                             " Botao do meio => Teclado." };
static char *prompt_msg2p = { "Entre com ponto final; "
                              "Botao do meio => Teclado; "
                              "Botao da direita => Cancela." };
static char *prompt_msg1e = { "Enter first point with mouse left button; "
                              "Mid button => Keyboard." };
static char *prompt_msg2e = { "Enter end point; "
                              "Mid button => Keyboard; "
                              "Right button => Cancel." };

/*
** ------------------------------------------------------------------
** Local functions
*/

/* ==========================  TrfMoveDel  ======================== */
 
static void TrfMoveDel( TrfMoveP *move )
{
 if( move == 0L ) return;
 
/*  Release memory used for collection.
 */
 free( move );
}

/* ========================  TrfMoveBuildMat  ===================== */
 
static void TrfMoveBuildMat( TrfMoveP *move )
{
 move->m.mat[0][0] = move->m.mat[1][1] = 1.0;
 move->m.mat[0][1] = move->m.mat[1][0] = 0.0;
 move->m.mat[0][2] = move->m.dx;
 move->m.mat[1][2] = move->m.dy;
}

/* ========================  TrfMoveSnap  ========================= */

static void TrfMoveSnap( PckDesc *pk, double *x, double *y, double tol )
{
/* Restore registered collection spec. from the data field.
 */
 TrfMoveP  *move = (TrfMoveP *)pk->data;

/* If shift key is pressed and first move line point has been given,
 * snap move line to the horizontal axis, to the vertical axis, or
 * the bi-sector lines.
 */
 if( (move->fst) && isshift(pk->mousecfg) )
 {
  TrfShiftSnap( move->p0.x, move->p0.y, x, y );
 }

/* Check to see whether client had registered a snap function and
 * call it passing the given mouse position and parameters.
 */
 if( move->snap != 0L)
  (*move->snap )( move->iup_cv, x, y, tol,
                             isshift(pk->mousecfg), iscontrol(pk->mousecfg) );
}

/* ========================  TrfMoveEcho  ========================= */
 
static void TrfMoveEcho( PckDesc *pk, double x, double y )
{
 char   echotext[60];
 int    status = 0;

/* Restore registered collection spec. from the data field.
 */
 TrfMoveP  *move = (TrfMoveP *)pk->data;
 
/* If first point was already selected, compute current
 * transformation matrix and echo translation parameters.
 */
 if( move->fst )
 {
  status = 1;
  move->m.dx = x - move->p0.x;
  move->m.dy = y - move->p0.y;
  TrfMoveBuildMat( move );

  sprintf( echotext, "DX = %10.3f;   DY = %10.3f", move->m.dx, move->m.dy );
  if( move->prompt != 0L )
   (*move->prompt)( move->iup_cv, echotext ) ;
 }

/* Check to see whether client had registered an echo function.
 * If that is the case call it passing the given mouse position and
 * the current transformation matrix.
 */
 if( move->echo != 0L )
  (*move->echo )( move->iup_cv, x, y, status, move->m.mat );
}


/*
** ------------------------------------------------------------------
** Public functions
*/

static void TrfMove1stPt( PckDesc * );
static void TrfMove2ndPt( PckDesc * );

/* ==========================  TrfMove  ========================== */

void TrfMove( Ihandle *iup_cv, void *cd_cv,
              void (*end_action)( Ihandle *iup_cv, int status ), 
              void (*put)( Ihandle *iup_cv, void *m, double tol ),
              void (*prompt)( Ihandle *iup_cv, char *msg ),
              void (*snap)( Ihandle *iup_cv, double *x, double *y, 
                            double tol, int shift, int control ),
              void (*echo)( Ihandle *iup_cv, double x, double y,
                            int status, double mat[2][3] ) )
{
 TrfMoveP  *move;
 PckDesc    pk;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  prompt_msg1 = prompt_msg1e;
  prompt_msg2 = prompt_msg2e;
 }
 else
 {
  prompt_msg1 = prompt_msg1p;
  prompt_msg2 = prompt_msg2p;
 }

/* Allocate a move collection descriptor
 */
 move = (TrfMoveP *)malloc( sizeof(TrfMoveP) );

/* Initialize collection
 */
 move->fst = 0;
 move->iup_cv = iup_cv;
 move->cd_cv = cd_cv;
 move->end_action = end_action;
 move->put = put;
 move->prompt = prompt;
 move->snap = snap;
 move->echo = echo;
 move->m.dx = move->m.dy = 0.0;
 TrfMoveBuildMat( move );

/* Setup pick descriptor.  Register the appropriate translation 
 * collector function as the callback for pick interaction.
 */
 if( move->prompt != 0L )
  (*move->prompt )( move->iup_cv, prompt_msg1 ) ;
 pk.iup_cv = move->iup_cv;
 pk.gra_cv = move->cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.mode = ButtonDown;
 pk.cursorecho = CrossCursor;
 pk.button = 0;    /* accept both mouse buttons 1 or 2 */
 pk.toler = 0.0;
 pk.btdwn_callback = TrfMove1stPt;
 pk.btup_callback = 0L;
 pk.echofunc = TrfMoveEcho;
 pk.snapfunc = TrfMoveSnap;
 pk.data = move;
 pk.status = PCK_VALID;
 PckSetup( &pk );
}

/* -------------------------  TrfMove1stPt  ---------------------- */

static void TrfMove1stPt( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 TrfMoveP  *move = (TrfMoveP *)pk->data;
 void      *iup_cv = move->iup_cv;
 void      (*end_action)( Ihandle *iup_cv, int status ) = move->end_action;
 
/* Check to see if collected vector is valid (greater then tolerance).
 * If not valid, call end action function and return.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = TRF_QUIT;        break;
   case PCK_INTERUPTED:  status = TRF_INTERUPTED;  break;
  }
  TrfMoveDel( move );
  if( end_action != 0L ) (*end_action)( iup_cv, status );
  return;
 }
 
/* Get button down point as first translation line point.
 */
 move->p0.x = pk->p0.x;
 move->p0.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter the
 * first point using the keyboard.  In this case, prompt the user
 * for point coordinates.
 */
 if( pk->button == 2 )
 {
  float x0, y0;
  x0 = move->p0.x;
  y0 = move->p0.y;
  if( DlgCaptureFMove( &x0, &y0 ) == 1 )
  {
   move->p0.x = pk->p0.x = x0;
   move->p0.y = pk->p0.y = y0;
  }
  else
  {
   TrfMoveDel( move );
   if( end_action != 0L ) (*end_action)( iup_cv, TRF_QUIT );
   return;
  }
 }

/* Set flag to indicate that first translation vector point has been 
 * selected.
 */
 move->fst = 1;

/* Change cursor echo for rubber band line (first point
 * is initial point of rubber band line) and get pick from user
 * for end translation line point.
 */
 if( move->prompt != NULL )
  (*move->prompt )( iup_cv, prompt_msg2 ) ;
 pk->cursorecho = CrossCursor | RubberBandCursor;
 pk->button = 0;       /* accept both mouse buttons 1 or 2 */
 pk->btdwn_callback = TrfMove2ndPt;
 PckSetup( pk );
}

/* -------------------------  TrfMove2ndPt  ---------------------- */

static void TrfMove2ndPt( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 TrfMoveP  *move = (TrfMoveP *)pk->data;
 void      *iup_cv = move->iup_cv;
 void      (*end_action)( Ihandle *iup_cv, int status ) = move->end_action;
 
/* Get button down point as end translation line point.
 */
 move->p1.x = pk->p0.x;
 move->p1.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter the
 * translation line using the keyboard. In this case, prompt 
 * the user for line coordinates, build the translation data,
 * and return.
 */
 if( (pk->status == PCK_VALID) && (pk->button == 2) )
 {
  float x0, y0, x1, y1;
  x0 = move->p0.x;  y0 = move->p0.y;
  x1 = move->p1.x;  y1 = move->p1.y;
  if( DlgCaptureSMove( &x0, &y0, &x1, &y1 ) == 1 )
  {
   move->p0.x = x0;  move->p0.y = y0;
   move->p1.x = x1;  move->p1.y = y1;
   if( LEN(move->p0,move->p1) >= pk->toler )
   {
    move->m.dx = move->p1.x - move->p0.x;
    move->m.dy = move->p1.y - move->p0.y;
    TrfMoveBuildMat( move );
    if( move->put != 0L )
     (*move->put )( move->iup_cv, &move->m, pk->toler );
   }
   TrfMoveDel( move );
   if( end_action ) (*end_action)( iup_cv, TRF_NORMAL );
  }
  else
  {
   TrfMoveDel( move );
   if( end_action ) (*end_action)( iup_cv, TRF_QUIT );
  }
  return;
 }

/* Check to see if collected vector is valid (greater then tolerance).
 * If not valid, call end action function and return.
 */
 if( (pk->status != PCK_VALID) || (LEN(move->p0,move->p1) < pk->toler) )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_VALID:
   case PCK_QUIT:        status = TRF_QUIT;        break;
   case PCK_INTERUPTED:  status = TRF_INTERUPTED;  break;
  }
  TrfMoveDel( move );
  if( end_action != 0L ) (*end_action)( iup_cv, status );
  return;
 }
 
/* Call the client function to put the collected transformation matrix,
 * call end action function, and return.
 */
 move->m.dx = move->p1.x - move->p0.x;
 move->m.dy = move->p1.y - move->p0.y;
 TrfMoveBuildMat( move );
 if( move->put != 0L ) 
  (*move->put )( move->iup_cv, &move->m, pk->toler );
 TrfMoveDel( move );
 if( end_action != 0L ) (*end_action)( iup_cv, TRF_NORMAL );
}


/* ==========================  TrfKbdMove  ======================= */

static double   kbddx, kbddy;
static double   kbdtol = 0.001;

static char *kbdprompt_msg;

static char *kbdprompt_msgp = { "Digite dados da translacao. " };
static char *kbdprompt_msge = { "Use the keyboard to enter translation data. "};

int  TrfKbdMove( Ihandle *iup_cv,
                 void (*put)( Ihandle *iup_cv, void *m, double tol ),
                 void (*prompt)( Ihandle *iup_cv, char *msg ) )
{
 TrfMv     move;
 float     dxf, dyf;
 float     toler;
 int       status = TRF_QUIT;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  kbdprompt_msg = kbdprompt_msge;
 }
 else
 {
  kbdprompt_msg = kbdprompt_msgp;
 }

/* Collect data from user with the keyboard.
 */
 if( prompt != 0L )
  (*prompt)( iup_cv, kbdprompt_msg );

 dxf = kbddx;
 dyf = kbddy;
 toler = kbdtol;
 if( DlgCaptureMove( &dxf, &dyf, &toler ) == 1 )
 {
  move.dx = (double)dxf;
  move.dy = (double)dyf;
  if( (fabs(move.dx) > 0.0) || (fabs(move.dy) > 0.0) )
  {
   kbddx = move.dx;
   kbddy = move.dy;
   kbdtol = toler;
   move.mat[0][0] = move.mat[1][1] = 1.0;
   move.mat[0][1] = move.mat[1][0] = 0.0;
   move.mat[0][2] = move.dx;
   move.mat[1][2] = move.dy;
   if( put != 0L )
    (*put)( iup_cv, &move, kbdtol );
   status = TRF_NORMAL;
  }
 }

 return( status );
}


