char *RCStrfrotat="$Id: trfrotat.c,v 1.7 2001/01/19 13:30:31 lfm Exp $";
/*
** trfrotat.c - Functions to generate a matrix transformation for
**              rotating objects.
**
** Public functions:
** --------------------------------------------------------------------
**
** void TrfRotate( Ihandle *iup_cv, void *cd_cv,
**                 void (*end_action)( Ihandle *iup_cv, int status ),
**                 void (*put)( Ihandle *iup_cv, void *r, double tol ), 
**                 void (*prompt)( Ihandle *iup_cv, char *msg ), 
**                 void (*snap)( Ihandle *iup_cv, double *x, double *y,
**                               double tol, int shift, int control ),
**                 void (*echo)( Ihandle *iup_cv, double x, double y,
**                               int status, double mat[2][3] ) )
**
** iup_cv     - IUP handle of canvas where object is selected (IN)
** cd_cv      - CD handle of canvas                               (IN)
** end_action - funct. to be called after finishing collection    (IN)
** put        - ptr. to function to put matrix after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
** snap       - ptr. to function to snap points during collection (IN)
** echo       - ptr. to func. to echo cursor pos. and transf.mat. (IN)
**
** This function interactively collects from the user a circle arc 
** on a given canvas for constructing a rotation transformation
** matrix.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal transformation collection ==> status = TRF_NORMAL = 1
** (2) Quit transformation collection ==> status = TRF_QUIT = 0
**     This is returned if user quit transformation collection with
**     mouse right button.
** (3) Interupted transf. collection ==> status = TRF_INTERUPTED = -1
**     This is returned when the transformation collection is closed 
**     by something else (not by the mouse interaction).
**
** --------------------------------------------------------------------
**
** int  TrfKbdRotate( Ihandle *iup_cv,
**                    void (*put)( Ihandle *iup_cv, void *r, double tol ),
**                    void (*prompt)( Ihandle *iup_cv, char *msg ) )
** 
** iup_cv     - IUP handle of canvas for client reference         (IN)
** put        - ptr. to function to put matrix after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
**
** This function interactively collects from the keyboard the center
** of rotation and an angle for constructing a rotation transformation
** matrix.
** If the collection is successfull the function returns a true (1)
** status.  Otherwise, it returns a false (0) status.
**
** --------------------------------------------------------------------
**
** Version:      0-001
**
** Created:    23-Apr-97    Luiz Fernando Martha
**
** Modified:   26-Aug-97    LFM
**      Fixed problem of accessing collected transformation object after 
**      deleting it for calling end action function.
**
** Modified:   18-Feb-98    LFM
**      Modification to avoid displaying echo messages on the canvas.
**      Instead, echo messages are send to prompt message UI object.
**
** Modified:   19-Jan-2001  LFM
**      When call to function getenv( "IUP_INGLES" ) was replaced 
**      by call to function IupGetLanguage( ).
**      Local string variable iup_english changed to int.
**      Modified IupScanf floating point format.
**
** Modified:   08-Aug-2002    Marcos C. de Arruda
**      Replaced call to IupScanf by call to corresponding dialog
**      (Dlg) function.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "iup.h"
#include "cd.h"
#include "wd.h"
#include "pck.h"
#include "t2d.h"
#include "trf.h"
#include "trfrotat.h"
#include "dlgrotat.h"


/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/
/*  Arc data structure:
 */
typedef struct _trfrotatep  {   /* Collected arc private spec.descrip.: */
        TrfRt    r;                 /* collected rotation            */
        PckPnt   center;            /* rotation arc center           */
        PckPnt   p0;                /* first arc point               */
        PckPnt   p1;                /* second arc point              */
        double   radius;            /* arc radius                    */
        double   ang0;              /* angle at first point          */
        double   ang1;              /* angle at end point            */
        int      ctr;               /* if true, center already given */
        int      fst;               /* if true, first arc pt. given  */
        void     *iup_cv;           /* IUP handle to target canvas   */
        void     *cd_cv;            /* cd handle to corresp.canvas   */
        void     (*end_action)      /* end action callback           */
                       ( Ihandle *iup_cv, int status );
        void     (*put)             /* function to put collect rotat.*/
                       ( Ihandle *iup_cv, void *r, double tol );
        void     (*prompt)          /* function to prompt messages   */
                       ( Ihandle *iup_cv, char *msg );
        void     (*snap)            /* function to snap collect. pt. */
                       ( Ihandle *iup_cv, double *x, double *y, 
                                       double tol, int shift, int control );
        void     (*echo)            /* function to echo cursor point */
                       ( Ihandle *iup_cv, double x, double y, 
                                       int status, double mat[2][3] );
        } TrfRotateP;

#ifndef LEN
#define LEN(p0,p1) sqrt((p1.x-p0.x)*(p1.x-p0.x)+(p1.y-p0.y)*(p1.y-p0.y))
#endif
#ifndef PI
#define PI           3.141592653589793
#endif
#ifndef RAD
#define RAD(a)       ((a)*(PI/180.0))
#endif
#ifndef DEG
#define DEG(a)       ((a)*(180.0/PI))
#endif

static int  iup_english;
static char *prompt_msg1;
static char *prompt_msg2;
static char *prompt_msg3;

static char *prompt_msg1p = { "Com botao esquerdo do mouse entre "
                              "com centro de rotacao; "
                              "Botao do meio => Teclado." };
static char *prompt_msg2p = { "Entre com primeiro ponto do arco "
                              "(sentido anti-horario); "
                              "Botao do meio => Teclado." };
static char *prompt_msg3p = { "Entre com segundo ponto do arco; "
                              "Botao do meio => Teclado; "
                              "Botao da direita => Cancela." };
static char *prompt_msg1e = { "With mouse left button enter rotation center; "
                              "Mid button => Keyboard." };
static char *prompt_msg2e = { "Enter arc first point "
                              "(counter-clockwise direction); "
                              "Mid button => Keyboard." };
static char *prompt_msg3e = { "Enter arc second point; "
                              "Mid button => Keyboard; "
                              "Right button => Cancel." };

/*
** --------------------------------------------------------------------
** Local functions:
*/

/* ========================  TrfRotateSnapTo  ======================= */

static int TrfRotateSnapTo( double xc, double yc, double r, 
                            double *x, double *y )
{
 double   s, t;

 s = sqrt( (*x-xc) * (*x-xc) + (*y-yc) * (*y-yc) );
 if( s == 0.0 )
  return( 0 );

 t = r / s;

 *x = xc + t * (*x-xc);
 *y = yc + t * (*y-yc);

 return( 1 );
}

/* ========================  TrfRotateAngle  ======================== */

static int TrfRotateAngle( double xc, double yc, double x, double y, 
                           double *a )
{
 double   s;

 s = sqrt( (x-xc) * (x-xc) + (y-yc) * (y-yc) );
 if( s == 0.0 )
  return( 0 );

 *a = atan2( y-yc, x-xc );
 *a = DEG( *a );

 if( *a == -180.0 )
  *a = 180.0;

 return( 1 );
}

/* ==========================  TrfRotateDel  ======================== */

static void TrfRotateDel( TrfRotateP *rot )
{
 if( rot == 0L ) return;

/*  Release memory used for collection.
 */    
 free( rot );
}

/* ========================  TrfRotateBuildMat  ===================== */

static void TrfRotateBuildMat( TrfRotateP *rot )
{
 rot->r.cx = rot->center.x;
 rot->r.cy = rot->center.y;
 rot->r.angle = rot->ang1 - rot->ang0;

/* Set up rotation angle between -180 and +180.
 */
 if( rot->r.angle > 180.0 )
 {
  rot->r.angle -= 360.0;
 }
 else if( rot->r.angle <= -180.0 )
 {
  rot->r.angle += 360.0;
 }

 T2dIdentity( );
 T2dRotateAbout( rot->r.cx, rot->r.cy, rot->r.angle );
 T2dGetTransform( rot->r.mat );
}

/* ========================  TrfRotateSnap  ========================= */

static void TrfRotateSnap( PckDesc *pk, double *x, double *y, double tol )
{
/* Restore registered collection spec. from the data field.
 */
 TrfRotateP  *rot = (TrfRotateP *)pk->data;

/* If shift key is pressed and rotation has been given, snap radius to 
 * the horizontal axis, to the vertical axis, or the bi-sector lines.
 */
 if( (rot->ctr) && isshift(pk->mousecfg) )
 {
  TrfShiftSnap( rot->center.x, rot->center.y, x, y );
 }

/* Check to see whether client had registered a snap function and 
 * call it passing the given mouse position and parameters.
 */
 if( rot->snap != 0L )
  (*rot->snap )( rot->iup_cv, x, y, tol, 
                              isshift(pk->mousecfg), iscontrol(pk->mousecfg) );

/* If first rotation arc points has already been collected, 
 * snap current point to arc.
 */
 if( (rot->fst) && (rot->radius > 0.0) )
 {
  TrfRotateSnapTo( rot->center.x, rot->center.y, rot->radius, x, y );
 }
}

/* ========================  TrfRotateEcho  ========================= */
 
static void TrfRotateEcho( PckDesc *pk, double x, double y )
{
 int    write_mode;
 int    fore_color;
 int    mark_type;
 int    mark_size;
 int    line_style;
 int    line_width;
 double d;
 char   echotext[60];
 int    status = 0;

/* Restore registered collection spec. from the data field.
 */
 TrfRotateP  *rot = (TrfRotateP *)pk->data;
 
/* Check to see if center has been given.
 * If that is the case, draw a mark at the center in Xor mode.
 */
 if( rot->ctr )
 {
#ifdef _UNIX_
  write_mode = cdWriteMode( CD_XOR );
#else
  write_mode = cdWriteMode( CD_NOT_XOR );
#endif
  fore_color = cdForeground( CD_RED );
  mark_type = cdMarkType( CD_BOX );
  mark_size = cdMarkSize( 4 );

  wdMark( rot->center.x, rot->center.y );

  cdMarkSize( mark_size );
  cdMarkType( mark_type );
  cdForeground( fore_color );
  cdWriteMode( write_mode );
 }

/* Check to see if first point has been given.
 * If that is the case, draw an arc in Xor mode.
 */
 if( (rot->fst) && (rot->radius > 0.0) )
 {
  d = 2.0 * rot->radius;

  if( TrfRotateAngle( rot->center.x, rot->center.y, x, y, &rot->ang1 ) )
  {
   status = 1;

/* Compute current transformation matrix.
 */
   TrfRotateBuildMat( rot );

#ifdef _UNIX_
   write_mode = cdWriteMode( CD_XOR );
#else
   write_mode = cdWriteMode( CD_NOT_XOR );
#endif
   fore_color = cdForeground( CD_RED );
   mark_type = cdMarkType( CD_BOX );
   mark_size = cdMarkSize( 4 );
   line_style = cdLineStyle( CD_CONTINUOUS );
   line_width = cdLineWidth( 1 );

   if( rot->r.angle < 0.0 )
    wdArc( rot->center.x, rot->center.y, d, d, rot->ang1, rot->ang0 );
   else
    wdArc( rot->center.x, rot->center.y, d, d, rot->ang0, rot->ang1 );

   wdMark( rot->p0.x, rot->p0.y );

   if( iup_english )
    sprintf( echotext, "Angle = %7.2f", rot->r.angle );
   else
    sprintf( echotext, "Angulo = %7.2f", rot->r.angle );
   if( rot->prompt != 0L )
    (*rot->prompt)( rot->iup_cv, echotext );

   cdLineWidth( line_width );
   cdLineStyle( line_style );
   cdMarkSize( mark_size );
   cdMarkType( mark_type );
   cdForeground( fore_color );
   cdWriteMode( write_mode );
  }
 }

/* Check to see whether client had registered an echo function.
 * If that is the case call it passing the given mouse position and
 * the current transformation matrix.
 */
 if( rot->echo != 0L )
  (*rot->echo )( rot->iup_cv, x, y, status, rot->r.mat ) ;
}
 

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* ============================  TrfRotate  ========================= */

static void TrfRotateCenter( PckDesc * );

void TrfRotate( Ihandle *iup_cv, void *cd_cv,
                void (*end_action)( Ihandle *iup_cv, int status ),
                void (*put)( Ihandle *iup_cv, void *r, double tol ),
                void (*prompt)( Ihandle *iup_cv, char *msg ),
                void (*snap)( Ihandle *iup_cv, double *x, double *y, 
                              double tol, int shift, int control ),
                void (*echo)( Ihandle *iup_cv, double x, double y,
                              int status, double mat[2][3] ) )
{
 TrfRotateP  *rot;
 PckDesc     pk;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  prompt_msg1 = prompt_msg1e;
  prompt_msg2 = prompt_msg2e;
  prompt_msg3 = prompt_msg3e;
 }
 else
 {
  prompt_msg1 = prompt_msg1p;
  prompt_msg2 = prompt_msg2p;
  prompt_msg3 = prompt_msg3p;
 }

/* Allocate a rotation collection descriptor
 */
 rot = (TrfRotateP *)malloc( sizeof( TrfRotateP ) );

/* Initialize collection.
 */
 rot->ctr = 0;
 rot->fst = 0;
 rot->iup_cv = iup_cv;
 rot->cd_cv = cd_cv;
 rot->end_action = end_action; 
 rot->put = put; 
 rot->prompt = prompt; 
 rot->snap = snap; 
 rot->echo = echo; 
 rot->center.x = rot->center.y = 0.0;
 rot->ang0 = rot->ang1 = 0.0;
 rot->p0.x = 1.0;
 rot->p0.y = 0.0;
 rot->p1.x = 1.0;
 rot->p1.y = 0.0;
 rot->radius = 1.0;
 TrfRotateBuildMat( rot );

/* Setup pick descriptor for getting rotation center at button down.  
 */
 if( rot->prompt != 0L )
  (*rot->prompt )( rot->iup_cv, prompt_msg1 );
 pk.iup_cv = rot->iup_cv;
 pk.gra_cv = rot->cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.mode = ButtonDown;
 pk.cursorecho = CrossCursor;
 pk.button = 0;    /* accept both mouse buttons 1 or 2 */
 pk.toler = 0.0;
 pk.btdwn_callback  = TrfRotateCenter;
 pk.btup_callback = 0L;
 pk.echofunc = TrfRotateEcho;
 pk.snapfunc = TrfRotateSnap;
 pk.data = rot;
 pk.status = PCK_VALID;
 PckSetup( &pk );
}

/* -------------------------  TrfRotateCenter  ------------------------ */

static void TrfRotate1stPoint( PckDesc * );

static void TrfRotateCenter( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 TrfRotateP  *rot = (TrfRotateP *)pk->data;
 void        *iup_cv = rot->iup_cv;
 void        (*end_action)( Ihandle *iup_cv, int status ) = rot->end_action;

/* Check to see if collected point is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = TRF_QUIT;        break;
   case PCK_INTERUPTED:  status = TRF_INTERUPTED;  break;
  }
  TrfRotateDel( rot );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Get button down point as center point.
 */
 rot->center.x = pk->p0.x;
 rot->center.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter the
 * rotation center using the keyboard. In this case, prompt the user 
 * for center coordinates, and continue collection.
 */
 if( pk->button == 2 )
 {
  float xc, yc;
  xc = rot->center.x;
  yc = rot->center.y;
  if( DlgCaptureFRotate( &xc, &yc ) == 1 )
  {
   rot->center.x = xc;
   rot->center.y = yc;
  }
  else
  {
   TrfRotateDel( rot );
   if( end_action ) (*end_action)( iup_cv, TRF_QUIT );
   return;
  }
 }

/* Set flag to indicate that center has been selected.
 */
 rot->ctr = 1;

/* Setup pick descriptor to collect first rotation arc point.  Register the 
 * appropriate collector function as the callback for pick interaction.
 */
 if( rot->prompt != 0L )
  (*rot->prompt )( rot->iup_cv, prompt_msg2 );
 pk->p0.x = rot->center.x;
 pk->p0.y = rot->center.y;
 pk->p1.x = pk->p1.y = 0.0;
 pk->mode = ButtonDown;
 pk->cursorecho = CrossCursor | RubberBandCursor;
 pk->button = 0;    /* accept both mouse buttons 1 or 2 */
 pk->btdwn_callback = TrfRotate1stPoint;
 pk->btup_callback  = 0L;
 PckSetup( pk );
}

/* -------------------------  TrfRotate1stPoint  ---------------------- */

static void TrfRotate2ndPoint( PckDesc * );

static void TrfRotate1stPoint( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 TrfRotateP  *rot = (TrfRotateP *)pk->data;
 void        *iup_cv = rot->iup_cv;
 void        (*end_action)( Ihandle *iup_cv, int status ) = rot->end_action;

/* Check to see if collected point is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = TRF_QUIT;        break;
   case PCK_INTERUPTED:  status = TRF_INTERUPTED;  break;
  }
  TrfRotateDel( rot );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Get button down point as first rotation arc point.
 */
 rot->p0.x = pk->p0.x;
 rot->p0.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter first
 * rotation arc point using the keyboard. In this case, prompt the user 
 * for point coordinates, and continue collection.
 */
 if( pk->button == 2 )
 {
  float x0, y0;
  x0 = rot->p0.x;
  y0 = rot->p0.y;
  if( DlgCaptureSRotate( &x0, &y0 ) == 1 )
  {
   rot->p0.x = x0;
   rot->p0.y = y0;
  }
  else
  {
   TrfRotateDel( rot );
   if( end_action ) (*end_action)( iup_cv, TRF_QUIT );
   return;
  }
 }

/* Compute rotation arc radius.
 */
 rot->radius = LEN( rot->center, rot->p0 );

/* Set flag to indicate that first rotation arc point has been selected and
 * compute corresponding angle.
 */
 rot->fst = 1;
 TrfRotateAngle( rot->center.x, rot->center.y, 
                 rot->p0.x, rot->p0.y, &rot->ang0 );

/* Setup pick descriptor to collect final rotation arc point.  Register the 
 * appropriate collector function as the callback for pick interaction.
 */
 if( rot->prompt != 0L )
  (*rot->prompt )( rot->iup_cv, prompt_msg3 );
 pk->p0.x = rot->p0.x;
 pk->p0.y = rot->p0.y;
 pk->p1.x = pk->p1.y = 0.0;
 pk->mode = ButtonDown;
 pk->cursorecho = CrossCursor;
 pk->button = 0;    /* accept both mouse buttons 1 or 2 */
 pk->btdwn_callback = TrfRotate2ndPoint;
 pk->btup_callback  = 0L;
 PckSetup( pk );
}

/* -------------------------  TrfRotate2ndPoint  ---------------------- */

static void TrfRotate2ndPoint( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 TrfRotateP  *rot = (TrfRotateP *)pk->data;
 void        *iup_cv = rot->iup_cv;
 void        (*end_action)( Ihandle *iup_cv, int status ) = rot->end_action;

/* Check to see if collected point is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = TRF_QUIT;        break;
   case PCK_INTERUPTED:  status = TRF_INTERUPTED;  break;
  }
  TrfRotateDel( rot );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Get button down point as final rotation arc point.
 */
 rot->p1.x = pk->p0.x;
 rot->p1.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter final
 * rotation arc point using the keyboard. In this case, prompt the user 
 * for all arc point coordinates and finish collection.
 */
 if( pk->button == 2 )
 {
  float xc, yc, x0, y0, x1, y1;
  xc = rot->center.x;
  yc = rot->center.y;
  x0 = rot->p0.x;
  y0 = rot->p0.y;
  x1 = rot->p1.x;
  y1 = rot->p1.y;
  if( DlgCaptureTRotate( &xc, &yc, &x0, &y0, &x1, &y1 ) == 1 )
  {
   rot->center.x = xc;
   rot->center.y = xc;
   rot->p0.x = x0;
   rot->p0.y = y0;
   rot->p1.x = x1;
   rot->p1.y = y1;
   rot->radius = LEN( rot->center, rot->p0 );
   if( rot->radius > 0.0 )
   {
    if( TrfRotateSnapTo( rot->center.x, rot->center.y, rot->radius, 
                         &rot->p1.x, &rot->p1.y ) )
    {
     if( LEN( rot->p0, rot->p1 ) >= pk->toler )
     {
      TrfRotateAngle( rot->center.x, rot->center.y, 
                      rot->p1.x, rot->p1.y, &rot->ang1 );
      TrfRotateBuildMat( rot );
      if( rot->put != 0L )
       (*rot->put )( rot->iup_cv, &rot->r, pk->toler );
     }
    }
   }
   TrfRotateDel( rot );
   if( end_action ) (*end_action)( iup_cv, TRF_NORMAL );
  }
  else
  {
   TrfRotateDel( rot );
   if( end_action ) (*end_action)( iup_cv, TRF_QUIT );
  }
  return;
 }

/* Call the client function to put the collected rotation transformation, 
 * and finish collection calling end action function.
 */
 if( (rot->radius > 0.0) && (LEN(rot->p0,rot->p1) >= pk->toler) )
 {
  TrfRotateAngle( rot->center.x, rot->center.y, 
                  rot->p1.x, rot->p1.y, &rot->ang1 );
  TrfRotateBuildMat( rot );
  if( rot->put != 0L )
   (*rot->put )( rot->iup_cv, &rot->r, pk->toler );
 }
 TrfRotateDel( rot );
 if( end_action ) (*end_action)( iup_cv, TRF_NORMAL );
}


/* ======================  TrfKbdArcCreate  ====================== */

static TrfRt    kbdrot;
static double   kbdtol = 0.001;

static char *kbdprompt_msg;

static char *kbdprompt_msgp = { "Digite dados da rotacao. " };
static char *kbdprompt_msge = { "Use the keyboard to enter rotation data. " };

int  TrfKbdRotate( Ihandle *iup_cv, 
                   void (*put)( Ihandle *iup_cv, void *r, double tol ),
                   void (*prompt)( Ihandle *iup_cv, char *msg ) )
{
 TrfRt     rot;
 float     cx, cy;
 float     ang;
 float     toler;
 int       status = TRF_QUIT;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  kbdprompt_msg = kbdprompt_msge;
 }
 else
 {
  kbdprompt_msg = kbdprompt_msgp;
 }

/* Initialize collection
 */
 rot = kbdrot;

/* Collect data from user with the keyboard.
 */
 if( prompt != 0L )
  (*prompt)( iup_cv, kbdprompt_msg );

 cx = rot.cx;
 cy = rot.cy;
 ang = rot.angle;
 toler = kbdtol;
 if( DlgCaptureRotate( &cx, &cy, &ang, &toler ) == 1 )
 {
  rot.cx = cx;
  rot.cy = cy;
  rot.angle = ang;
  kbdrot = rot;
  kbdtol = toler;
  T2dIdentity( );
  T2dRotateAbout( rot.cx, rot.cy, rot.angle );
  T2dGetTransform( rot.mat );
  if( put != 0L )
   (*put)( iup_cv, &rot, kbdtol );
  status = TRF_NORMAL;
 }

 return( status );
}


