char *RCStrfscale="$Id: trfscale.c,v 1.7 2001/01/19 13:31:07 lfm Exp $";
/*
** trfscale.c - Functions to generate a matrix transformation for
**              scaling objects.
**
** Public functions:
** --------------------------------------------------------------------
**
** void TrfScale( Ihandle *iup_cv, void *cd_cv,
**                void (*end_action)( Ihandle *iup_cv, int status ),
**                void (*put)( Ihandle *iup_cv, void *s, double tol ), 
**                void (*prompt)( Ihandle *iup_cv, char *msg ), 
**                void (*snap)( Ihandle *iup_cv, double *x, double *y,
**                              double tol, int shift, int control ),
**                void (*echo)( Ihandle *iup_cv, double x, double y,
**                              int status, double mat[2][3] ) )
**
** iup_cv     - IUP handle of canvas where object is selected     (IN)
** cd_cv      - CD handle of canvas                               (IN)
** end_action - funct. to be called after finishing collection    (IN)
** put        - ptr. to function to put matrix after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
** snap       - ptr. to function to snap points during collection (IN)
** echo       - ptr. to func. to echo cursor pos. and transf.mat. (IN)
**
** This function interactively collects from the user a circle arc 
** on a given canvas for constructing a scaling transformation
** matrix.  Only uniform scaling is performed (x and y scaling 
** factors are always the same).
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal transformation collection ==> status = TRF_NORMAL = 1
** (2) Quit transformation collection ==> status = TRF_QUIT = 0
**     This is returned if user quit transformation collection with
**     mouse right button.
** (3) Interupted transf. collection ==> status = TRF_INTERUPTED = -1
**     This is returned when the transformation collection is closed 
**     by something else (not by the mouse interaction).
**
** --------------------------------------------------------------------
**
** int  TrfKbdScale( Ihandle *iup_cv,
**                    void (*put)( Ihandle *iup_cv, void *s, double tol ),
**                    void (*prompt)( Ihandle *iup_cv, char *msg ) )
** 
** iup_cv     - IUP handle of canvas for client reference         (IN)
** put        - ptr. to function to put matrix after collection   (IN)
** prompt     - ptr. to function to prompt messg. during collect. (IN)
**
** This function interactively collects from the keyboard the center
** of scale and a line for constructing a scaling transformation
** matrix.  Only uniform scaling is performed (x and y scaling 
** factors are always the same).
** If the collection is successfull the function returns a true (1)
** status.  Otherwise, it returns a false (0) status.
**
** --------------------------------------------------------------------
**
** Version:      0-001
**
** Created:    23-Apr-97    Luiz Fernando Martha
**
** Modified:   26-Aug-97    LFM
**      Fixed problem of accessing collected transformation object after
**      deleting it for calling end action function.
**
** Modified:   18-Feb-98    LFM
**      Modification to avoid displaying echo messages on the canvas.
**      Instead, echo messages are send to prompt message UI object.
**
** Modified:   19-Jan-2001  LFM
**      When call to function getenv( "IUP_INGLES" ) was replaced 
**      by call to function IupGetLanguage( ).
**      Local string variable iup_english changed to int.
**      Modified IupScanf floating point format.
**
** Modified:   08-Aug-2002    Marcos C. de Arruda
**      Replaced call to IupScanf by call to corresponding dialog
**      (Dlg) function.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

/*
** --------------------------------------------------------------------
** Global variables and definitions:
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "iup.h"
#include "cd.h"
#include "wd.h"
#include "pck.h"
#include "t2d.h"
#include "trf.h"
#include "trfscale.h"
#include "dlgscale.h"


/*
** --------------------------------------------------------------------
** Local variables and definitions:
*/
/*  Arc data structure:
 */
typedef struct _trfscalep  {   /* Collected arc private spec.descrip.: */
        TrfSc    s;                 /* collected scale               */
        PckPnt   center;            /* scaling center                */
        PckPnt   p0;                /* first scaling line point      */
        PckPnt   p1;                /* second scaling line point     */
        int      ctr;               /* if true, center already given */
        int      fst;               /* if true, first point given    */
        void     *iup_cv;           /* IUP handle to target canvas   */
        void     *cd_cv;            /* cd handle to corresp.canvas   */
        void     (*end_action)      /* end action callback           */
                       ( Ihandle *iup_cv, int status );
        void     (*put)             /* function to put collect transf*/
                       ( Ihandle *iup_cv, void *r, double tol );
        void     (*prompt)          /* function to prompt messages   */
                       ( Ihandle *iup_cv, char *msg );
        void     (*snap)            /* function to snap collect. pt. */
                       ( Ihandle *iup_cv, double *x, double *y, 
                                       double tol, int shift, int control );
        void     (*echo)            /* function to echo cursor point */
                       ( Ihandle *iup_cv, double x, double y, 
                                       int status, double mat[2][3] );
        } TrfScaleP;

#ifndef LEN
#define LEN(p0,p1) sqrt((p1.x-p0.x)*(p1.x-p0.x)+(p1.y-p0.y)*(p1.y-p0.y))
#endif

static int  iup_english;
static char *prompt_msg1;
static char *prompt_msg2;
static char *prompt_msg3;

static char *prompt_msg1p = { "Com botao esquerdo do mouse entre "
                              "com centro de escalamento; "
                              "Botao do meio => Teclado." };
static char *prompt_msg2p ={ "Entre com primeiro ponto da linha de escalamento."
                             " Botao do meio => Teclado." };
static char *prompt_msg3p = { "Entre com segundo ponto; "
                              "Botao do meio => Teclado; "
                              "Botao da direita => Cancela." };
static char *prompt_msg1e = { "With mouse left button enter scaling center; "
                              "Mid button => Keyboard." };
static char *prompt_msg2e = { "Enter first scaling line point "
                              "Mid button => Keyboard." };
static char *prompt_msg3e = { "Enter second point; "
                              "Mid button => Keyboard; "
                              "Right button => Cancel." };

/*
** --------------------------------------------------------------------
** Local functions:
*/

/* ========================  TrfScaleSnapTo  ======================= */

static int TrfScaleSnapTo( double xc, double yc, double x0, double y0, 
                           double *x, double *y )
{
 double   len, vx, vy, t;

 len = sqrt( (x0-xc) * (x0-xc) + (y0-yc) * (y0-yc) );
 if( len == 0.0 )
  return( 0 );

 vx = (x0-xc) / len;
 vy = (y0-yc) / len;

 t = (*x-xc)*vx + (*y-yc)*vy;

 *x = xc + t * vx;
 *y = yc + t * vy;

 return( 1 );
}

/* ========================  TrfScaleFactor  ======================= */

static int TrfScaleFactor( double xc, double yc, double x0, double y0, 
                           double x1, double y1, double *f )
{
 double   len, vx, vy, t;

 len = sqrt( (x0-xc) * (x0-xc) + (y0-yc) * (y0-yc) );
 if( len == 0.0 )
  return( 0 );

 vx = (x0-xc) / len;
 vy = (y0-yc) / len;

 t = (x1-xc)*vx + (y1-yc)*vy;

 if( t == 0.0 )
  return( 0 );

 *f = t / len;

 return( 1 );
}

/* ==========================  TrfScaleDel  ======================== */

static void TrfScaleDel( TrfScaleP *scl )
{
 if( scl == 0L ) return;

/*  Release memory used for collection.
 */    
 free( scl );
}

/* ========================  TrfScaleBuildMat  ===================== */

static void TrfScaleBuildMat( TrfScaleP *scl )
{
 scl->s.cx = scl->center.x;
 scl->s.cy = scl->center.y;
 scl->s.sy = scl->s.sx;

 T2dIdentity( );
 T2dScaleAbout( scl->s.cx, scl->s.cy, scl->s.sx, scl->s.sy );
 T2dGetTransform( scl->s.mat );
}

/* ========================  TrfScaleSnap  ========================= */

static void TrfScaleSnap( PckDesc *pk, double *x, double *y, double tol )
{
/* Restore registered collection spec. from the data field.
 */
 TrfScaleP  *scl = (TrfScaleP *)pk->data;

/* If shift key is pressed, the center of scaling has been selected and
 * and first scaling line point has not been given, snap scaling line 
 * to the horizontal axis, to the vertical axis, or the bi-sector lines.
 */
 if( (scl->ctr) && (!scl->fst) && isshift(pk->mousecfg) )
 {
  TrfShiftSnap( scl->center.x, scl->center.y, x, y );
 }

/* Check to see whether client had registered a snap function and 
 * call it passing the given mouse position and parameters.
 */
 if( scl->snap != 0L )
  (*scl->snap )( scl->iup_cv, x, y, tol, 
                              isshift(pk->mousecfg), iscontrol(pk->mousecfg) );

/* If center and first scaling point have already been collected, 
 * snap current point to scaling line.
 */
 if( scl->fst )
 {
  TrfScaleSnapTo( scl->center.x, scl->center.y, scl->p0.x, scl->p0.y, x, y );
 }
}

/* ========================  TrfScaleEcho  ========================= */
 
static void TrfScaleEcho( PckDesc *pk, double x, double y )
{
 int    write_mode;
 int    fore_color;
 int    mark_type;
 int    mark_size;
 int    line_style;
 int    line_width;
 char   echotext[60];
 int    status = 0;

/* Restore registered collection spec. from the data field.
 */
 TrfScaleP  *scl = (TrfScaleP *)pk->data;
 
/* Check to see if center has been given.
 * If that is the case, draw a mark at the center in Xor mode.
 */
 if( scl->ctr )
 {
#ifdef _UNIX_
  write_mode = cdWriteMode( CD_XOR );
#else
  write_mode = cdWriteMode( CD_NOT_XOR );
#endif
  fore_color = cdForeground( CD_RED );
  mark_type = cdMarkType( CD_BOX );
  mark_size = cdMarkSize( 4 );

  wdMark( scl->center.x, scl->center.y );

  cdMarkSize( mark_size );
  cdMarkType( mark_type );
  cdForeground( fore_color );
  cdWriteMode( write_mode );
 }

/* Check to see if first point has been given.
 * If that is the case, draw current scaling line in Xor mode.
 */
 if( scl->fst )
 {
  if( TrfScaleFactor( scl->center.x, scl->center.y, scl->p0.x, scl->p0.y,
                      x, y, &scl->s.sx ) )
  {
   status = 1;

/* Compute current transformation matrix.
 */
   TrfScaleBuildMat( scl );

#ifdef _UNIX_
   write_mode = cdWriteMode( CD_XOR );
#else
   write_mode = cdWriteMode( CD_NOT_XOR );
#endif
   fore_color = cdForeground( CD_RED );
   mark_type = cdMarkType( CD_BOX );
   mark_size = cdMarkSize( 4 );
   line_style = cdLineStyle( CD_CONTINUOUS );
   line_width = cdLineWidth( 1 );

   wdLine( scl->p0.x, scl->p0.y, x, y );

   wdMark( scl->p0.x, scl->p0.y );

   if( iup_english )
    sprintf( echotext, "Scaling factor = %7.2f", scl->s.sx );
   else
    sprintf( echotext, "Fator de escala = %7.2f", scl->s.sx );
   if( scl->prompt != 0L )
    (*scl->prompt)( scl->iup_cv, echotext );

   cdLineWidth( line_width );
   cdLineStyle( line_style );
   cdMarkSize( mark_size );
   cdMarkType( mark_type );
   cdForeground( fore_color );
   cdWriteMode( write_mode );
  }
 }

/* Check to see whether client had registered an echo function.
 * If that is the case call it passing the given mouse position and
 * the current transformation matrix.
 */
 if( scl->echo != 0L )
  (*scl->echo )( scl->iup_cv, x, y, status, scl->s.mat ) ;
}
 

/*
** --------------------------------------------------------------------
** Public functions:
*/

/* ============================  TrfScale  ========================= */

static void TrfScaleCenter( PckDesc * );

void TrfScale( Ihandle *iup_cv, void *cd_cv,
               void (*end_action)( Ihandle *iup_cv, int status ),
               void (*put)( Ihandle *iup_cv, void *s, double tol ),
               void (*prompt)( Ihandle *iup_cv, char *msg ),
               void (*snap)( Ihandle *iup_cv, double *x, double *y, 
                             double tol, int shift, int control ),
               void (*echo)( Ihandle *iup_cv, double x, double y,
                             int status, double mat[2][3] ) )
{
 TrfScaleP   *scl;
 PckDesc     pk;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  prompt_msg1 = prompt_msg1e;
  prompt_msg2 = prompt_msg2e;
  prompt_msg3 = prompt_msg3e;
 }
 else
 {
  prompt_msg1 = prompt_msg1p;
  prompt_msg2 = prompt_msg2p;
  prompt_msg3 = prompt_msg3p;
 }

/* Allocate a scaling collection descriptor
 */
 scl = (TrfScaleP *)malloc( sizeof( TrfScaleP ) );

/* Initialize collection.
 */
 scl->ctr = 0;
 scl->fst = 0;
 scl->iup_cv = iup_cv;
 scl->cd_cv = cd_cv;
 scl->end_action = end_action; 
 scl->put = put; 
 scl->prompt = prompt; 
 scl->snap = snap; 
 scl->echo = echo; 
 scl->center.x = scl->center.y = 0.0;
 scl->p0.x = 1.0;
 scl->p0.y = 0.0;
 scl->p1.x = 1.0;
 scl->p1.y = 0.0;
 scl->s.sx = 1.0;
 TrfScaleBuildMat( scl );

/* Setup pick descriptor for getting scaling center at button down.  
 */
 if( scl->prompt != 0L )
  (*scl->prompt )( scl->iup_cv, prompt_msg1 );
 pk.iup_cv = scl->iup_cv;
 pk.gra_cv = scl->cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.mode = ButtonDown;
 pk.cursorecho = CrossCursor;
 pk.button = 0;    /* accept both mouse buttons 1 or 2 */
 pk.toler = 0.0;
 pk.btdwn_callback  = TrfScaleCenter;
 pk.btup_callback = 0L;
 pk.echofunc = TrfScaleEcho;
 pk.snapfunc = TrfScaleSnap;
 pk.data = scl;
 pk.status = PCK_VALID;
 PckSetup( &pk );
}

/* -------------------------  TrfScaleCenter  ------------------------ */

static void TrfScale1stPoint( PckDesc * );

static void TrfScaleCenter( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 TrfScaleP  *scl = (TrfScaleP *)pk->data;
 void       *iup_cv = scl->iup_cv;
 void       (*end_action)( Ihandle *iup_cv, int status ) = scl->end_action;

/* Check to see if collected point is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = TRF_QUIT;        break;
   case PCK_INTERUPTED:  status = TRF_INTERUPTED;  break;
  }
  TrfScaleDel( scl );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Get button down point as center point.
 */
 scl->center.x = pk->p0.x;
 scl->center.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter the
 * scaling center using the keyboard. In this case, prompt the user 
 * for center coordinates, and continue collection.
 */
 if( pk->button == 2 )
 {
  float xc, yc;
  xc = scl->center.x;
  yc = scl->center.y;
  if( DlgCaptureFScale( &xc, &yc ) == 1 )
  {
   scl->center.x = xc;
   scl->center.y = yc;
  }
  else
  {
   TrfScaleDel( scl );
   if( end_action ) (*end_action)( iup_cv, TRF_QUIT );
   return;
  }
 }

/* Set flag to indicate that center has been selected.
 */
 scl->ctr = 1;

/* Setup pick descriptor to collect first scaling line point.  Register the 
 * appropriate collector function as the callback for pick interaction.
 */
 if( scl->prompt != 0L )
  (*scl->prompt )( scl->iup_cv, prompt_msg2 );
 pk->p0.x = scl->center.x;
 pk->p0.y = scl->center.y;
 pk->p1.x = pk->p1.y = 0.0;
 pk->mode = ButtonDown;
 pk->cursorecho = CrossCursor | RubberBandCursor;
 pk->button = 0;    /* accept both mouse buttons 1 or 2 */
 pk->btdwn_callback = TrfScale1stPoint;
 pk->btup_callback  = 0L;
 PckSetup( pk );
}

/* -------------------------  TrfScale1stPoint  ---------------------- */

static void TrfScale2ndPoint( PckDesc * );

static void TrfScale1stPoint( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 TrfScaleP  *scl = (TrfScaleP *)pk->data;
 void       *iup_cv = scl->iup_cv;
 void       (*end_action)( Ihandle *iup_cv, int status ) = scl->end_action;

/* Check to see if collected point is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = TRF_QUIT;        break;
   case PCK_INTERUPTED:  status = TRF_INTERUPTED;  break;
  }
  TrfScaleDel( scl );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Get button down point as first scaling line point.
 */
 scl->p0.x = pk->p0.x;
 scl->p0.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter first
 * scaling line point using the keyboard. In this case, prompt the user 
 * for point coordinates, and continue collection.
 */
 if( pk->button == 2 )
 {
  float x0, y0;
  x0 = scl->p0.x;
  y0 = scl->p0.y;
  if( DlgCaptureSScale( &x0, &y0 ) == 1 )
  {
   scl->p0.x = x0;
   scl->p0.y = y0;
  }
  else
  {
   TrfScaleDel( scl );
   if( end_action ) (*end_action)( iup_cv, TRF_QUIT );
   return;
  }
 }

/* Set flag to indicate that first scaling line point has been selected.
 */
 scl->fst = 1;

/* Setup pick descriptor to collect final scaling line point.  Register the 
 * appropriate collector function as the callback for pick interaction.
 */
 if( scl->prompt != 0L )
  (*scl->prompt )( scl->iup_cv, prompt_msg3 );
 pk->p0.x = scl->p0.x;
 pk->p0.y = scl->p0.y;
 pk->p1.x = pk->p1.y = 0.0;
 pk->mode = ButtonDown;
 pk->cursorecho = CrossCursor;
 pk->button = 0;    /* accept both mouse buttons 1 or 2 */
 pk->btdwn_callback = TrfScale2ndPoint;
 pk->btup_callback  = 0L;
 PckSetup( pk );
}

/* -------------------------  TrfScale2ndPoint  ---------------------- */

static void TrfScale2ndPoint( PckDesc *pk )
{
/* Restore registered collection spec. from the data field.
 */
 TrfScaleP  *scl = (TrfScaleP *)pk->data;
 void       *iup_cv = scl->iup_cv;
 void       (*end_action)( Ihandle *iup_cv, int status ) = scl->end_action;

/* Check to see if collected point is valid.
 * If not valid, just quit the collection, calling end action function.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = TRF_QUIT;        break;
   case PCK_INTERUPTED:  status = TRF_INTERUPTED;  break;
  }
  TrfScaleDel( scl );
  if( end_action ) (*end_action)( iup_cv, status );
  return;
 }

/* Get button down point as final scaling line point.
 */
 scl->p1.x = pk->p0.x;
 scl->p1.y = pk->p0.y;

/* If middle button was pressed, the user wants to enter final
 * scaling line point using the keyboard. In this case, prompt the user 
 * for all scaling line point coordinates and finish collection.
 */
 if( pk->button == 2 )
 {
  float xc, yc, x0, y0, x1, y1;
  xc = scl->center.x;
  yc = scl->center.y;
  x0 = scl->p0.x;
  y0 = scl->p0.y;
  x1 = scl->p1.x;
  y1 = scl->p1.y;
  if( DlgCaptureTScale( &xc, &yc, &x0, &y0, &x1, &y1 ) == 1 )
  {
   scl->center.x = xc;
   scl->center.y = xc;
   scl->p0.x = x0;
   scl->p0.y = y0;
   scl->p1.x = x1;
   scl->p1.y = y1;
   if( TrfScaleSnapTo( scl->center.x, scl->center.y, scl->p0.x, scl->p0.y, 
                       &scl->p1.x, &scl->p1.y ) )
   {
    TrfScaleFactor( scl->center.x, scl->center.y, scl->p0.x, scl->p0.y, 
                    scl->p1.x, scl->p1.y, &scl->s.sx );
    TrfScaleBuildMat( scl );
    if( scl->put != 0L )
     (*scl->put )( scl->iup_cv, &scl->s, pk->toler );
   }
   TrfScaleDel( scl );
   if( end_action ) (*end_action)( iup_cv, TRF_NORMAL );
  }
  else
  {
   TrfScaleDel( scl );
   if( end_action ) (*end_action)( iup_cv, TRF_QUIT );
  }
  return;
 }

/* Call the client function to put the collected scaling transformation, 
 * and finish collection calling end action function.
 */
 TrfScaleFactor( scl->center.x, scl->center.y, scl->p0.x, scl->p0.y, 
                 scl->p1.x, scl->p1.y, &scl->s.sx );
 TrfScaleBuildMat( scl );
 if( scl->put != 0L )
  (*scl->put )( scl->iup_cv, &scl->s, pk->toler );
 TrfScaleDel( scl );
 if( end_action ) (*end_action)( iup_cv, TRF_NORMAL );
}


/* ======================  CllKbdArcCreate  ====================== */

static TrfSc    kbdscl;
static double   kbdtol = 0.001;

static char *kbdprompt_msg;

static char *kbdprompt_msgp = { "Digite dados do escalamento. " };
static char *kbdprompt_msge = { "Use the keyboard to enter scaling data. " };

int  TrfKbdScale( Ihandle *iup_cv, 
                  void (*put)( Ihandle *iup_cv, void *s, double tol ),
                  void (*prompt)( Ihandle *iup_cv, char *msg ) )
{
 TrfSc     scl;
 float     cx, cy;
 float     fac;
 float     toler;
 int       status = TRF_QUIT;

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  kbdprompt_msg = kbdprompt_msge;
 }
 else
 {
  kbdprompt_msg = kbdprompt_msgp;
 }

/* Initialize collection
 */
 scl = kbdscl;

/* Collect data from user with the keyboard.
 */
 if( prompt != 0L )
  (*prompt)( iup_cv, kbdprompt_msg );

 cx = scl.cx;
 cy = scl.cy;
 fac = scl.sx;
 toler = kbdtol;
 if( DlgCaptureScale( &cx, &cy, &fac, &toler ) == 1 )
 {
  scl.cx = cx;
  scl.cy = cy;
  scl.sx = scl.sy = fac;
  kbdscl = scl;
  kbdtol = toler;
  T2dIdentity( );
  T2dScaleAbout( scl.cx, scl.cy, scl.sx, scl.sy );
  T2dGetTransform( scl.mat );
  if( put != 0L )
   (*put)( iup_cv, &scl, kbdtol );
  status = TRF_NORMAL;
 }

 return( status );
}


