char *RCSvis="$Id: vis.c,v 1.6 1999/12/16 18:25:01 lfm Exp $";
/*
** ---------------------------------------------------------------
** vis.c - 2D View Specification Package.  Generic visualization 
**         functions for use with CD graphics package and Iup 
**         graphics interface.
**
** ---------------------------------------------------------------
** Purpose:
** Allows updating of the view of a model on a canvas window.
**
** ---------------------------------------------------------------
** Description:
**
** This module manipulates the visualization parameters of a
** bidimensional environment (object) on a canvas window.
** Each canvas is associated with a 2D view specification
** descriptor.  It is possible to have a number of these 
** descriptors.  To use this package, clients should first call
** function VisOpen to create a descriptor.  This function 
** requires two arguments: one is the IUP canvas handle and the
** other is the CD canvas handle (see IUP and CD documentation).
** The IUP canvas handle will be used as an identifier for the
** viewing specification descriptor.  All calls of functions of
** this module have the IUP canvas handle as the first parameter
** (argument). This will set the target viewing descriptor as the
** current active one.
**
** ---------------------------------------------------------------
** Public functions:
** ---------------------------------------------------------------
**
** int    VisOpen( Ihandle *iup_cv, void *cd_cv )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**   cd_cv     - CD handle to target canvas                   (in)
**
** This function initializes a viewing specification descriptor.
** It allocates memory for to the viewing descriptor and links 
** it with the given IUP canvas handle.
**
** ---------------------------------------------------------------
**
** void   VisClose( Ihandle *iup_cv )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**
** This function frees the memory that had been allocated for 
** storing the internal viewing specification descriptor which
** is associated with the given canvas handle.
**
** ---------------------------------------------------------------
**
** void   VisRegFunc( Ihandle *iup_cv,
**                    void (*rdsp)(Ihandle *iup_cv, double xmn, double xmx,
**                                               double ymn, double ymx),
**                    int  (*fmnmx)(Ihandle *iup_cv, double *xmn,double *xmx,
**                                                double *ymn,double *ymx) )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**   rdsp      - pointer to client redisplay function         (in)
**   fmnmx     - pointer to client findminmax function        (in)
**
** This function registers with the viewing specification 
** descriptor which is associated with the given canvas handle two
** callback functions: one to redisplay the client model and
** another to get the bounding box of the client model.
**
** ---------------------------------------------------------------
**
** void   VisDefault( Ihandle *iup_cv )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**
** This function inquires the client, through a registerd call
** back function, the limiting coordinates of its model and 
** computes the canvas window so that the model adjusts exactly
** in the canvas.  
** If the client had not defined a findminmax callback function 
** or its return value is false (a model has not been defined),
** this function sets the window of the viewing specification 
** descriptor which is associated with the given canvas handle 
** as a normalized window.
** This function does not call the client redisplay callback
** function.
**
** ---------------------------------------------------------------
**
** void   VisFit( Ihandle *iup_cv )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**
** This function calls function VisDefault above and then it 
** calls the registered call back function to display the 
** client's model.
**
** ---------------------------------------------------------------
**
** void   VisResize( Ihandle *iup_cv, int width, int height )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**   width     - new horizontal canvas width in pixels        (in)
**   height    - new vertical canvas height in pixels         (in)
**
** This function should be called when the canvas whose handle is
** given is resized.  It takes the appropriate actions with the
** CD package to reflect the canvas resize.
**
** ---------------------------------------------------------------
**
** void   VisRedraw( Ihandle *iup_cv, double posx, double posy )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**   posx,posy - thumb positions in the scroll bars of canvas (in)
**
** This function should be called when the canvas whose handle is
** given receives a redraw event.  It takes the appropriate actions 
** with the Gra package to reflect the updated canvas scroll bar
** thumb positions.  It computes the window limits based on 
** current positions:  posx is the position of the horizontal 
** thumb from the left side of the canvas and posy is the position 
** of the vertical thumb from the top side of the canvas.
**
** ---------------------------------------------------------------
**
** void   VisRedisplay( Ihandle *iup_cv )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**
** This function calls the registered client redisplay function
** to display its data on the canvas.
**
** ---------------------------------------------------------------
**
** void   VisZoom( Ihandle *iup_cv, 
**                 void (*end_action)( Ihandle *iup_cv, int status ) )
**
**   iup_cv     - IUP handle to target canvas                 (in)
**   end_action - function to be called after finishing zoom  (in)
**
** This function gets from the user a new window which is 
** interactively defined with the mouse.  The user indicates two
** points on the target canvas which defines the new window.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal zoom collection ==> status = VIS_NORMAL = 1
** (2) Quit zoom collection ==> status = VIS_QUIT = 0
**     This is returned if user quit zoom collection with 
**     the right mouse button.
** (3) Interupted zoom collection ==> status = VIS_INTERUPTED = -1
**     This is returned when the zoom collection is closed by 
**     something else (not by the mouse interaction).
**
** ---------------------------------------------------------------
**
** void   VisZoomIn( Ihandle *iup_cv )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**
** This function adjusts the view window of the targed canvas 
** based on the current zoom factor.  It decreases the window 
** sizes, maitaining its current center.
**
** ---------------------------------------------------------------
**
** void   VisZoomOut( Ihandle *iup_cv )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**
** This function adjusts the view window of the targed canvas 
** based on the current zoom factor.  It increases the window 
** sizes, maitaining its current center.
**
** ---------------------------------------------------------------
**
** void   VisPrevZoom( Ihandle *iup_cv )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**
** This function restore the previous window (wc sizes of a
** window).
**
** ---------------------------------------------------------------
**
** void   VisPan( Ihandle *iup_cv, 
**               void (*end_action)( Ihandle *iup_cv, int status ) )
**
**   iup_cv     - IUP handle to target canvas                 (in)
**   end_action - function to be called after finishing pan   (in)
**
** This function gets from the user a mouse movement which is
** used to define translation factor for the current window.
** The window is translated in the opposite direction of the
** mouse movement and the result is that the client model 
** is translated in the mouse move direction.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal pan collection ==> status = VIS_NORMAL = 1
** (2) Quit pan collection ==> status = VIS_QUIT = 0
**     This is returned if user pan scale collection with 
**     the right mouse button.
** (3) Interupted scale collection ==> status = VIS_INTERUPTED = -1
**     This is returned when the pan collection is closed by 
**     something else (not by the mouse interaction).
**
** ---------------------------------------------------------------
**
** void   VisPanX( Ihandle *iup_cv, double fac )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**   fac       - pan factor                                   (in)
**
** This function translates lthe world coordinate window for 
** the target canvas in the horizontal direction based on the 
** given pan factor.
** This factor is a percentage of the current window horizontal 
** size.  A negative factor indicates a window translation to 
** the right (the client model will move left), and a positive 
** factor indicates a window translation to the left (the client
** model will move right).
** The scroll bar limits are verified and expanded automatically.
**
** ---------------------------------------------------------------
**
** void   VisPanY( Ihandle *iup_cv, double fac )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**   fac       - pan factor                                   (in)
**
** This function translates lthe world coordinate window for 
** the target canvas in the vertical direction based on the 
** given pan factor.
** This factor is a percentage of the current window vertical 
** size.  A negative factor indicates a upward window translation
** (the client model will downward), and a positive factor 
** indicates a downward window translation (the client model will 
** move upward).
** The scroll bar limits are verified and expanded automatically.
**
** ---------------------------------------------------------------
**
** void   VisScale( Ihandle *iup_cv, 
**                  void (*end_action)( Ihandle *iup_cv, int status ) )
**
**   iup_cv     - IUP handle to target canvas                 (in)
**   end_action - function to be called after finishing scale (in)
**
** This function gets from the user a scaling factor which is
** interactively define with the mouse:
** mouse movement to right ==> scale up;
** mouse movement to left  ==> scale down.
** The scaling factor will be used to scale a model on the 
** target canvas and this function computes a new canvas window 
** to reflect the scaling.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal scale collection ==> status = VIS_NORMAL = 1
** (2) Quit scale collection ==> status = VIS_QUIT = 0
**     This is returned if user quit scale collection with 
**     the right mouse button.
** (3) Interupted scale collection ==> status = VIS_INTERUPTED = -1
**     This is returned when the scale collection is closed by 
**     something else (not by the mouse interaction).
**
** ---------------------------------------------------------------
**
** void   VisScaleFac( Ihandle *iup_cv, double fac )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**   fac       - scaling factor                               (in)
**
** This function computes a new world coordinate window for 
** the target canvas based on the given scaling factor.
** This factor is a percentage of decrease (positive factor)
** or increase (negative factor) of the window sizes.  The 
** result of this inversion (positive ==> decrease, negative ==>
** increase) is that the size of the client's enviroment 
** decreases with a negative factor and increases with a positive
** factor.
**
** ---------------------------------------------------------------
**
** void   VisDistortFac( Ihandle *iup_cv, double vscalefac )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**   vscalefac - window vertical distortion factor            (in)
**
** This function adjusts the model to fit in the canvas taking
** into account the given vertical distortion factor.
**
** ---------------------------------------------------------------
**
** void   VisGetDistortFac( Ihandle *iup_cv, double *vscalefac )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**   vscalefac - window vertical distortion factor           (out)
**
** This function returns the current factor for window vertical
** distortion.
**
** ---------------------------------------------------------------
**
** void   VisSetWindow( Ihandle *iup_cv, double xmin, double xmax,
**                                    double ymin, double ymax )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**   xmin, xmax,
**   ymin, ymax - new window limits                           (in)
**
** This function sets the limits of the world coordinate window 
** for the target canvas.
** The limits are adjusted to be consistent with the canvas
** viewport aspect ratio and with the canvas vertical distortion 
** factor.  The adjustment is such that the center of the given
** window is maintained.
**
** ---------------------------------------------------------------
**
** void   VisGetWindow( Ihandle *iup_cv, double *xmin, double *xmax,
**                                    double *ymin, double *ymax )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**   xmin, xmax,
**   ymin, ymax - returned window limits                     (out)
**
** This function returns the limits of the current world 
** coordinate window of the target canvas.
**
** ---------------------------------------------------------------
**
** void   VisSetSizes( Ihandle *iup_cv, double width, double height )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**   width     - new window horizontal size                   (in)
**   height    - new window vertical size                     (in)
**
** This function sets the limits of the world coordinate window 
** for the target canvas based on the given window sizes.
** The limits are adjusted to be consistent with the canvas
** viewport aspect ratio and with the canvas vertical distortion 
** factor.  The adjustment is such that the center of the previous
** canvas window is maintained.
**
** ---------------------------------------------------------------
**
** void   VisSetWidth( Ihandle *iup_cv, double width, double *height )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**   width     - new window horizontal size                   (in)
**   height    - returned window vertical size               (out)
**
** This function sets the limits of the world coordinate window 
** for the target canvas based on the given window horizontal size.
** The limits are adjusted to be consistent with the canvas
** viewport aspect ratio and with the canvas vertical distortion 
** factor.  The adjustment is such that the center of the previous
** canvas window and the given window width are maintained.
**
** ---------------------------------------------------------------
**
** void   VisSetHeight( Ihandle *iup_cv, double *width, double height )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**   width     - returned window horizontal size             (out)
**   height    - new window vertical size                     (in)
**
** This function sets the limits of the world coordinate window 
** for the target canvas based on the given window vertical size.
** The limits are adjusted to be consistent with the canvas
** viewport aspect ratio and with the canvas vertical distortion 
** factor.  The adjustment is such that the center of the previous
** canvas window and the given window height are maintained.
**
** ---------------------------------------------------------------
**
** void   VisGetSizes( Ihandle *iup_cv, double *width, double *height )
**
**   iup_cv    - IUP handle to target canvas                  (in)
**   width     - returned window horizontal size             (out)
**   height    - returned window vertical size               (out)
**
** This function returns the sizes of the current world coordinate 
** window of the target canvas.
**
** ---------------------------------------------------------------
**
** Version:    2.0
**
** Created:    13-Jan-94    Luiz F. Martha
**
** Modified:   31-Mar-94    LFM
**   When created Pck package: change old Gra mouse input (pick)
**   calls to Pck calls.
**
** Modified:   11-Apr-94    Joao Luiz E. Campos (Acai)
**   Created function VisPrevZoom.
**
** Modified:   06-May-94    LFM
**   When changed GraSetWindow such that it does no adjust 
**   automatically the given wc window dimensions to have the
**   same aspect ratio of the viewport.  Also added window
**   distortion factor field the the visualization state 
**   structure.
**   Added call to function GraResizeCanvas in VisResize.
**   Eliminated calls to PckEchoOff.
**
** Modified:   06-Jul-94    LFM
**   Modified VisReset to adjust canvas window limits on the
**   basis of a client window, if that is defined.
**
** Modified:   29-May-95    LFM
**   Version 2.0:  Version to use with CD graphics package.
**   Added width and height parameters in function VisResize.
**   Changed float to double. 
**   Eliminated function VisFitWindow.
**   Renamed function VisAdjustWindow to VisFitLimits.
**   Replaced macro RetangleCursor by RectangleCursor in pck.h.
**   Fixed problem in function VisRedraw when the canvas had 
**   no scroll bar.
**
** Modified:   16-Aug-95    LFM
**   When created "wd.h" and replaced cdActivate by wdActivate.
**
** Modified:   25-Aug-95    LFM
**   Created prompt messages in Portuguese.
**
** Modified:   26-Jan-96    LFM
**   When created ZOOM_FACTOR definition.
**   Created functions VisZoonIn and VisZoomOut.
**   Changed function _VisZoomBtUpCB such that, if a small zoom
**   fence is defined, it adjusts window based on the current zoom 
**   factor.  The center of the new window is the pick point.
**   Created local function ChkScreenLimits.
**
** Modified:   07-Mar-96    LFM
**   When CD changed, creating cdCreateCanvas, which returns
**   a CD handle for the canvas.
**   Changed VisOpen adding a second parameter which is the CD
**   canvas handle.
**   Changed calls to wdActivate such that it now has only
**   one parameter: the CD canvas handle.
**
** Modified:   12-Apr-96    LFM
**   Added functions VisScaleFac, VisSetLimits, VisGetLimits, 
**   VisSetSizes, VisSetWidth, VisSetHeight, and VisGetSizes.
**
** Modified:   03-May-96    LFM
**   Changed function VisScaleFac such that it does not allow 
**   windows large compared to obj/environment bounding box.
**
** Modified:   18-Aug-96    LFM
**   Added four window parameters to client redisplay callback function.
**   Got rid of public function VisLimits.
**   Renamed VisSetLimits to VisSetWindow.
**   Renamed VisGetLimits to VisGetWindow.
**   Replaced public function VisDistort by VisDistortFac.
**   Created function VisGetDistortFac.
**   Created window limit fields (xmin,xmax,ymin,ymax) in VisState structure.
**   Got rid of local function ScaleScreenLimits replacing calls to 
**   it by calls to SetScreenLimits.
**   Modified local function SetScreeLimits such that it tries to 
**   get from Iup attributes for limit factors of window.
**   Modified public function VisRedraw such that it check to see if
**   horizontal and vertical scroll bars are specified.
**
** Modified:   10-Sep-96    LFM
**   Added non-adjusted window sizes fields (rawxmin, rawxmax,
**   rawymin, rawymax) in VisState structure.  These fields are
**   always stored in local function AdjustWindow before any window
**   adjustment to match the viewport aspect ratio.  Function
**   VisRedraw uses these fields to restore the window.
**   Created local variable "doing_resize" for treating scroll bar
**   thumb positions during resize.
**   Changed first parameter of local function AdjustWindow from
**   Iup canvas handle to pointer to view state descriptor, and 
**   eliminated window vertical distortion paramater.
**
** Modified:   10-Sep-96    LFM
**   Created string fields in VisState structure to hold scroll bar
**   attributes.  Modified API of local functions due to this.
**
** Modified:   22-Mar-97    LFM
**   Modified function VisScaleFac such that it scales window sizes
**   even when client does not provide an environment (object)
**   bounding box.
**
** Modified:   04-Apr-97    LFM
**   Created function VisDefault and modified function VisReset
**   (actually the new VisDefault) such that it does not call 
**   VisFitLimits to reset the window and scroll bar limits 
**   (this was done because the client bounding box function 
**   was being called twice).
**   Fixed function VisGetDistortFac eliminating the erroneous
**   call to VisFit.
**
** Modified:   15-Apr-97    LFM
**   Created Zoom snap function to modify current mouse position 
**   during mouse collection such that the cursor echo shows a 
**   new window that keeps the same aspect ratio of the canvas
**   viewport.
**   Created non-drag Zoom fence mode.  Renamed function 
**   _VisZoomBtDwnCB to _VisZoomDrag1stPtCB, created function 
**   _VisZoom1stPtCB, and renamed function _VisZoomBtUpCB to
**   _VisZoom2ndPtCB (with modifications).
**
** Modified:   17-Apr-97    LFM
**   Added return status parameter for end action function.
**
** Modified:   18-May-97    LFM
**   Eliminated calls to wdWindow in this package: clients shall
**   call themselves wdWindow in the redisplay callback function
**   using the window limit function paramaters.
**   Added (xmin_sav, xmax_sav, ymin_sav, ymax_sav) fields to
**   VisState struct such that functions that temporarely set 
**   the window with wd are able to restore client defined windows.
**   Modified all computations to viewport ratio such that this 
**   ratio is based on the raster (pixel) sizes of the canvas 
**   (before the ratio was based on the sizes in milimiters).
**   Created local function AdjustZoomFence and added call to in 
**   in function _VisZoom2ndPtCB.
**   Changed VisFit such that if the client had not defined a 
**   findminmax callback function, the function sets a default 
**   window instead of doing nothing as before.  With this 
**   modification VisFit became exactly equal to function 
**   VisReset which was eliminated.  Function VisFitLimits was
**   also eliminated for the same reason (it became exactly 
**   equal to VisDefault).
**   Created functions VisPan, VisPanX, and VisPanY.
**   Modified function VisSetWindown such that is does not 
**   changes automatically the limits of the scroll bar (now
**   it only performs a check to see if the limits have
**   been exceeded).
**   Created WINDOWRESIZE_ADJUST mode for doing resize (added
**   windowresize_adj, hsize, and vsize fields to VisState struct).
**
** Modified:   19-Jan-98    LFM
**   Fixed mistake in updating window in functions _VisPan2ndPtCB,
**   VisPanX, and VisPanY.
**
** Modified:   26-Feb-98    LFM
**   When x0dc, y0dc, x1dc, and y1dc fields were added to the Pck 
**   pick descriptor structure.  Changed control of zoom fence and 
**   zoom pick accordingly.  Added x0dc and y0dc to local VisState
**   structure.
**
** Modified:   10-Feb-99    LFM
**   Changed function VisScaleFac to check for scale factor too
**   large against maximum object (bounding box) size instead of
**   checking for each direction independently.
**
** Modified:   16-Dec-99    LFM
**   Modified fence zoom such that it does not reset the end 
**   action callback to null after second fence point.
**
** Modified:   21-Apr-2004    William Lira
**   Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

/*
** ---------------------------------------------------------------
** Global variables and symbols
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "iup.h"
#include "cd.h"
#include "cdiup.h"
#include "wd.h"
#include "pck.h"
#include "vis.h"

/*
** ---------------------------------------------------------------
** Local variables and symbols:
*/

#ifndef MAX
#define MAX(a,b) (((a)>(b))?(a):(b))
#endif

#ifndef MIN
#define MIN(a,b) (((a)<(b))?(a):(b))
#endif

#ifndef ABS
#define ABS(a) (((a)<0)?-(a):(a))
#endif

#define  LIMIT_FACTOR  3.00
#define  FIT_FACTOR    1.15
#define  ZOOM_FACTOR   1.20

/* Struct of zoom list
 */
typedef struct _zoom_list_ {
  struct _zoom_list_ *prev;
  double             xmin, xmax, ymin, ymax;
 } ZoomList;

/* Private data view spec. data descriptor:
 */
typedef struct _visstate {
  struct _visstate *next;
  void      *iup_cv;                        /* IUP handle to target canvas  */
  void      *cd_cv;                         /* cd handle to corresp.canvas  */
  int       hsize;                          /* canvas width in pixels       */
  int       vsize;                          /* canvas height in pixels      */
  double    windistort;                     /* window distortion factor     */
  int       windowresize_adj;               /* flag to chng window in resize*/
  int       zoomfence_adj;                  /* if true, adjust zoom fence   */
  int       zoomfence_drag;                 /* if true, drag mode zoom fence*/
  int       panline_drag;                   /* if true, drag mode pan line  */
  PckPnt    p0;                             /* first zoom fence point       */
  int       x0dc, y0dc;                     /* p0 in raster coordinates     */
  ZoomList  *prev_zoom;                     /* handle to zoom list          */
  double    xmin, xmax, ymin, ymax;         /* current window               */
  double    rawxmin,rawxmax,rawymin,rawymax; /* non-adjusted windon version */
  double    xmin_sav, xmax_sav;             /* saved client window          */
  double    ymin_sav, ymax_sav;
  char      sb_xmin[20];                    /* scroll bar xmin value        */
  char      sb_xmax[20];                    /* scroll bar xmax value        */
  char      sb_ymin[20];                    /* scroll bar ymin value        */
  char      sb_ymax[20];                    /* scroll bar ymax value        */
  char      sb_dx[20];                      /* scroll bar dx value          */
  char      sb_dy[20];                      /* scroll bar dy value          */
  char      sb_posx[20];                    /* scroll bar posx value        */
  char      sb_posy[20];                    /* scroll bar posy value        */
  void      (*redisplay)( Ihandle *iup_cv, double xmin, double xmax,
                                        double ymin, double ymax );
  int       (*findminmax)( Ihandle *iup_cv, double *xmin, double *xmax, 
                                         double *ymin, double *ymax );
  void      (*end_action)( Ihandle *iup_cv, int status ); /* end action callback*/
 } VisState;


/* Head of view specification descriptor list:
 */
static  VisState  *view_head = 0L;

/* Doing resize flag:
 */
static int doing_resize = 0;

/*
** ---------------------------------------------------------------
** Local functions:
*/
static VisState *GetViewState( Ihandle *iup_cv );
static void AdjustWindow( VisState *view, double *lt, double *rt, 
                                          double *bt, double *tp );
static void SetScreenLimits( VisState *view, double lt, double rt,
                                             double bt, double tp );
static void SetScreenWindow( VisState *view, double lt, double rt,
                                             double bt, double tp );
static void ChkScreenLimits( VisState *view, double wlt, double wrt,
                                             double wbt, double wtp );
static void AddToZoomList( VisState *view, double xmin, double xmax,
                                           double ymin, double ymax );
static void FreeZoomList( VisState *view );
static void AdjustZoomFence( VisState *view, double x0, double y0, 
                                             double *x1, double *y1 );


/* =======================  GetViewState  ====================== */

static VisState *GetViewState( Ihandle *iup_cv )
{
 VisState   *view;

 for( view = view_head; view != NULL; view = view->next )
 {
  if( view->iup_cv == iup_cv ) break;
 }
 return( view );
}


/* ========================  AdjustWindow  ======================= */

static void AdjustWindow( VisState *view, double *lt, double *rt, 
                                          double *bt, double *tp )
{
 double  vpr;               /* viewport distortion ratio        */
 double  xc, yc;            /* center of window in world coords.*/
 double  hsize, vsize;      /* window sizes in world coords.    */
 double  vscalefac;         /* window vertical distortion factor*/

/* Compute canvas viewport ratio.
 */
 vpr = (double)view->vsize / (double)view->hsize;

/* Save raw window.
 */
 view->rawxmin = *lt;
 view->rawxmax = *rt;
 view->rawymin = *bt;
 view->rawymax = *tp;

/* Find window center and sizes.
 */
 xc = (*rt + *lt) * 0.5;
 yc = (*tp + *bt) * 0.5;
 hsize = (*rt - *lt);
 vsize = (*tp - *bt);

/* Adjust window to keep the same aspect ratio of the viewport.
 * Also taking into account given window vertical scale factor.
 */
 vscalefac = view->windistort;
 if( (vsize*vscalefac) > (vpr*hsize) )
  hsize = (vsize*vscalefac) / vpr;
 else
  vsize = (hsize/vscalefac) * vpr;

 hsize *= 0.5;
 vsize *= 0.5;

 *lt = xc - hsize;
 *rt = xc + hsize;
 *bt = yc - vsize;
 *tp = yc + vsize;
}


/* ======================  SetScreenLimits  ==================== */
 
static void SetScreenLimits( VisState *view, double lt, double rt,
                                             double bt, double tp )
{
 char      *hlimitfactor = IupGetAttribute( view->iup_cv, "HLIMITFACTOR" );
 char      *vlimitfactor = IupGetAttribute( view->iup_cv, "VLIMITFACTOR" );
 double    h_factor, v_factor;
 double    cx, cy;
 double    hsize, vsize;

/* Check to see if there are limit factors hooked to canvas.
 */
 if( hlimitfactor != NULL )
  h_factor = IupGetFloat( view->iup_cv, "HLIMITFACTOR" );
 else
  h_factor = LIMIT_FACTOR;
 if( vlimitfactor != NULL )
  v_factor = IupGetFloat( view->iup_cv, "VLIMITFACTOR" );
 else
  v_factor = LIMIT_FACTOR;

/* Set canvas window limit sizes as "limit factor" times
 * the sizes of the given window sizes.
 */
 cx = (rt + lt) / 2.0;
 cy = (tp + bt) / 2.0;
 hsize = (rt - lt) / 2.0;
 vsize = (tp - bt) / 2.0;

 lt = cx - (hsize * h_factor);
 rt = cx + (hsize * h_factor);
 bt = cy - (vsize * v_factor);
 tp = cy + (vsize * v_factor);

 sprintf( view->sb_xmin, "%f", lt );
 IupSetAttribute( (Ihandle *)view->iup_cv, IUP_XMIN, view->sb_xmin );
 sprintf( view->sb_xmax, "%f", rt );
 IupSetAttribute( (Ihandle *)view->iup_cv, IUP_XMAX, view->sb_xmax );
 sprintf( view->sb_ymin, "%f", bt );
 IupSetAttribute( (Ihandle *)view->iup_cv, IUP_YMIN, view->sb_ymin );
 sprintf( view->sb_ymax, "%f", tp );
 IupSetAttribute( (Ihandle *)view->iup_cv, IUP_YMAX, view->sb_ymax );
}
 
 
/* =====================  SetScreenWindow  ===================== */
 
static void SetScreenWindow( VisState *view, double lt, double rt,
                                             double bt, double tp )
{
 double posx, posy, dx, dy;
 double ymin = IupGetFloat( (Ihandle *)view->iup_cv, IUP_YMIN );
 double ymax = IupGetFloat( (Ihandle *)view->iup_cv, IUP_YMAX );

/* posx and posy are the thumb position in the scroll bars
 * of the canvas.  Note that posx is the position of the
 * horizontal thumb from the left side of the canvas and
 * posy is the position of the vertical thumb from the
 * top side of the canvas.
 */
 dx = rt - lt;
 dy = tp - bt;
 posx = lt;
 posy = ymax - bt + ymin - dy;

 sprintf( view->sb_dx, "%f", dx );
 IupSetAttribute( (Ihandle *)view->iup_cv, IUP_DX, view->sb_dx );
 sprintf( view->sb_dy, "%f", dy );
 IupSetAttribute( (Ihandle *)view->iup_cv, IUP_DY, view->sb_dy );
 sprintf( view->sb_posx, "%f", posx );
 IupSetAttribute( (Ihandle *)view->iup_cv, IUP_POSX, view->sb_posx );
 sprintf( view->sb_posy, "%f", posy );
 IupSetAttribute( (Ihandle *)view->iup_cv, IUP_POSY, view->sb_posy );
}


/* ======================  ChkScreenLimits  ==================== */
 
static void ChkScreenLimits( VisState *view, double wlt, double wrt,
                                             double wbt, double wtp )
{
 double    lt, rt, bt, tp;

/* Get current canvas window limit sizes.
 */
 lt = IupGetFloat( (Ihandle *)view->iup_cv, IUP_XMIN );
 rt = IupGetFloat( (Ihandle *)view->iup_cv, IUP_XMAX );
 bt = IupGetFloat( (Ihandle *)view->iup_cv, IUP_YMIN );
 tp = IupGetFloat( (Ihandle *)view->iup_cv, IUP_YMAX );

/* Set new canvas window limit sizes based on given window sizes.
 */
 if( lt > wlt ) lt = wlt;
 if( rt < wrt ) rt = wrt;
 if( bt > wbt ) bt = wbt;
 if( tp < wtp ) tp = wtp;

 sprintf( view->sb_xmin, "%f", lt );
 IupSetAttribute( (Ihandle *)view->iup_cv, IUP_XMIN, view->sb_xmin );
 sprintf( view->sb_xmax, "%f", rt );
 IupSetAttribute( (Ihandle *)view->iup_cv, IUP_XMAX, view->sb_xmax );
 sprintf( view->sb_ymin, "%f", bt );
 IupSetAttribute( (Ihandle *)view->iup_cv, IUP_YMIN, view->sb_ymin );
 sprintf( view->sb_ymax, "%f", tp );
 IupSetAttribute( (Ihandle *)view->iup_cv, IUP_YMAX, view->sb_ymax );
}
 

/* ======================  AddToZoomList  ====================== */
 
static void AddToZoomList( VisState *view, double xmin, double xmax,
                                           double ymin, double ymax )
{
 ZoomList  *new;

/* Insert given window in previous zoom list.
 */
 new = (ZoomList *)malloc( sizeof(ZoomList) );
 new->xmin = xmin;
 new->xmax = xmax;
 new->ymin = ymin;
 new->ymax = ymax;
 if( view->prev_zoom == 0L ) new->prev = 0L;
 else                        new->prev = view->prev_zoom;
 view->prev_zoom = new;
}
 

/* =======================  FreeZoomList  ====================== */
 
static void FreeZoomList( VisState *view )
{
 ZoomList  *prev_zoom;

/* Free list of previous zoom windows.
 */
 while( view->prev_zoom != 0L )
 {
  prev_zoom = view->prev_zoom;
  view->prev_zoom = prev_zoom->prev;
  free( prev_zoom );
 }
}
 

/* =====================  AdjustZoomFence  ===================== */
 
static void AdjustZoomFence( VisState *view, double x0, double y0, 
                                             double *x1, double *y1 )
{
 double    vpr;               /* viewport distortion ratio            */
 double    hsize, vsize;      /* new window sizes in world coords.    */
 double    vscalefac;         /* new window vertical distortion factor*/

/* Find fence sizes.
 */
 hsize = fabs( *x1 - x0 );
 vsize = fabs( *y1 - y0 );

 if( (hsize > 0.0) && (vsize > 0.0) )
 {
/* Compute canvas viewport ratio.
 */
  vpr = (double)view->vsize / (double)view->hsize;

/* Adjust fence to keep the same aspect ratio of the viewport.
 * Also taking into account given window vertical scale factor.
 */
  vscalefac = view->windistort;
  if( (vsize*vscalefac) > (vpr*hsize) )
  {
   hsize = (vsize*vscalefac) / vpr;
   if( *x1 > x0 )
    *x1 = x0 + hsize;
   else
    *x1 = x0 - hsize;
  }
  else
  {
   vsize = (hsize/vscalefac) * vpr;
   if( *y1 > y0 )
    *y1 = y0 + vsize;
   else
    *y1 = y0 - vsize;
  }
 }
}


/*
** ---------------------------------------------------------------
** Public functions:
*/

/* ==========================  VisOpen  ======================= */
 
int VisOpen( Ihandle *iup_cv, void *cd_cv )
{
 VisState  *view;

 if( wdActivate( (cdCanvas *)cd_cv ) == CD_ERROR )
  return( 0 );

 view = GetViewState( iup_cv );
 if( view != NULL )
  VisClose( iup_cv );

 view = (VisState *)malloc( sizeof( VisState ) );
 if( view == NULL )
  return( 0 );

/* Get initial canvas sizes and set its viewport as its entire
 * surface.
 */
 cdGetCanvasSize( &view->hsize, &view->vsize, 0L, 0L );
 wdViewport( 0, view->hsize-1, 0, view->vsize-1 );

/* Set up default parameter values.
 */
 view->next           = view_head;
 view->windistort     = 1.0;
 view->zoomfence_adj  = 0;
 view->zoomfence_drag = 0;
 view->panline_drag   = 0;
 view->p0.x           = 0.0;
 view->p0.y           = 0.0;
 view->x0dc           = 0;
 view->y0dc           = 0;
 view->iup_cv         = iup_cv;
 view->cd_cv          = cd_cv;
 view->prev_zoom      = 0L;
 view_head            = view;
 view->xmin = view->rawxmin = view->xmin_sav = -1.0;
 view->xmax = view->rawxmax = view->xmax_sav = 11.0;
 view->ymin = view->rawymin = view->ymin_sav = -1.0;
 view->ymax = view->rawymax = view->ymax_sav = 11.0;

 return( 1 );
}
 

/* =========================  VisClose  ======================= */
 
void VisClose( Ihandle *iup_cv )
{
 VisState  *view;
 VisState  *prev;

 view = GetViewState( iup_cv );
 if( view == NULL )
  return;

 if( view_head == view )
 {
  view_head = view->next;
 }
 else
 {
  prev = view_head;
  while( ( prev->next != view ) && ( prev->next != NULL ) )
  {
   prev = prev->next;
  }
 
  if( prev->next != NULL )
  {
   prev->next = view->next;
  }
 }

 FreeZoomList( view );

 free( view );
}
 

/* =========================  VisRegFunc  ===================== */
 
void VisRegFunc( Ihandle *iup_cv,
                 void (*rdsp)(Ihandle *iup_cv, double xmn, double xmx,
                                            double ymn, double ymx),
                 int  (*fmnmx)(Ihandle *iup_cv, double *xmn,double *xmx,
                                             double *ymn,double *ymx) )
{
 VisState  *view;

 view = GetViewState( iup_cv );
 if( view == NULL )
  return;

 view->redisplay = rdsp;
 view->findminmax = fmnmx;
}
 
 
/* ========================  VisDefault  ====================== */
 
void VisDefault( Ihandle *iup_cv )
{
 VisState  *view;
 double    center_x, center_y; /* bounding box center            */
 double    sizex, sizey;       /* bounding box sizes             */
 char      *fit_status = IupGetAttribute( iup_cv, "FITFACTOR" );
 double    fit_factor;

 view = GetViewState( iup_cv );
 if( view == NULL )
  return;

 if( wdActivate( (cdCanvas *)view->cd_cv ) == CD_ERROR )
  return;

 if( (view->findminmax == NULL ) || 
     (!(*view->findminmax)( view->iup_cv, 
                      &view->xmin, &view->xmax, &view->ymin, &view->ymax ) ) )
 {
  view->xmin =  0.0;
  view->xmax = 10.0;
  view->ymin =  0.0;
  view->ymax = 10.0;
 }

 if( fit_status != NULL )
  fit_factor = IupGetFloat( iup_cv, "FITFACTOR" );
 else
  fit_factor = FIT_FACTOR;

 center_x = (view->xmin + view->xmax) / 2.0;
 center_y = (view->ymin + view->ymax) / 2.0;
 sizex = (view->xmax - view->xmin) / 2.0;
 sizey = (view->ymax - view->ymin) / 2.0;
 
 view->xmin = center_x - (sizex * fit_factor);
 view->xmax = center_x + (sizex * fit_factor);
 view->ymin = center_y - (sizey * fit_factor);
 view->ymax = center_y + (sizey * fit_factor);

 AdjustWindow( view, &view->xmin, &view->xmax, &view->ymin, &view->ymax );
 SetScreenLimits( view, view->xmin, view->xmax, view->ymin, view->ymax );
 SetScreenWindow( view, view->xmin, view->xmax, view->ymin, view->ymax );

 FreeZoomList( view );
}


/* ==========================  VisFit  ======================== */
 
void VisFit( Ihandle *iup_cv )
{
 VisState  *view;

 view = GetViewState( iup_cv );
 if( view == NULL )
  return;

 if( wdActivate( (cdCanvas *)view->cd_cv ) == CD_ERROR )
  return;

 VisDefault( iup_cv );

 if( view->redisplay != NULL )
  (*view->redisplay)(view->iup_cv,view->xmin,view->xmax,view->ymin,view->ymax);
 cdFlush( );
}


/* =========================  VisResize  ====================== */
 
void VisResize( Ihandle *iup_cv, int width, int height )
{
 VisState  *view;
 char      *windowresize_adj = IupGetAttribute( iup_cv, "WINDOWRESIZE_ADJUST" );

 view = GetViewState( iup_cv );
 if( view == NULL )
  return;

/* Get flag for window resize and, if it is true, set raw window 
 * according to new sizes (window origin is maintained).
 */
 view->windowresize_adj = (windowresize_adj && 
                           !strcmp( windowresize_adj, "YES" ));
 if( view->windowresize_adj )
 {
  double  sx, sy;
  sx = (double)width/(double)view->hsize;
  sy = (double)height/(double)view->vsize;
  view->xmax = view->xmin + (view->xmax - view->xmin) * sx;
  view->ymax = view->ymin + (view->ymax - view->ymin) * sy;
  view->rawxmin = view->xmin;
  view->rawxmax = view->xmax;
  view->rawymin = view->ymin;
  view->rawymax = view->ymax;
 }

/* Update canvas sizes in pixels and set flag for doing resize.
 */
 view->hsize = width;
 view->vsize = height;
 doing_resize = 1;

/* Set canvas viewport according to new canvas sizes.
 */
 if( wdActivate( (cdCanvas *)view->cd_cv ) == CD_ERROR )
  return;
 wdViewport( 0, view->hsize-1, 0, view->vsize-1 );
}
 
 
/* =========================  VisRedraw  ====================== */
 
void VisRedraw( Ihandle *iup_cv, double posx, double posy )
{
 VisState  *view;
 int       hscroll_status, vscroll_status;
 char      *scrollbar = IupGetAttribute( iup_cv, IUP_SCROLLBAR );
 char      *horizontal = IupGetAttribute( iup_cv, IUP_HORIZONTAL );
 char      *sbh = IupGetAttribute( iup_cv, IUP_SBH );
 char      *vertical = IupGetAttribute( iup_cv, IUP_VERTICAL );
 char      *sbv = IupGetAttribute( iup_cv, IUP_SBV );

 view = GetViewState( iup_cv );
 if( view == NULL )
  return;

 if( wdActivate( (cdCanvas *)view->cd_cv ) == CD_ERROR )
  return;

/* Check for active scroll bars.
 */
 hscroll_status = (scrollbar && !strcmp( scrollbar, "YES" )) ||
                  (horizontal && !strcmp( horizontal, "YES" )) ||
                  (sbh && !strcmp( sbh, "YES" ));
 vscroll_status = (scrollbar && !strcmp( scrollbar, "YES" )) ||
                  (vertical && !strcmp( vertical, "YES" )) ||
                  (sbv && !strcmp( sbv, "YES" ));

/* Compute window limits based on current canvas data.
 */
 if( !doing_resize && hscroll_status )
 {
  view->xmax = posx + IupGetFloat( (Ihandle *)view->iup_cv, IUP_DX );
  view->xmin = posx;
 }
 else
 {
  view->xmin = view->rawxmin;
  view->xmax = view->rawxmax;
 }
 if( !doing_resize && vscroll_status )
 {
/* Note that we have to revert the direction of the y axis because
 * Iup thinks that Ymin is at the top of the canvas and Ymax is
 * at the bottom.
 */
  view->ymax = IupGetFloat( (Ihandle *)view->iup_cv, IUP_YMAX ) - 
               posy + IupGetFloat( (Ihandle *)view->iup_cv, IUP_YMIN );
  view->ymin = view->ymax - IupGetFloat( (Ihandle *)view->iup_cv, IUP_DY );
 }
 else
 {
  view->ymin = view->rawymin;
  view->ymax = view->rawymax;
 }
 
 if( !doing_resize )
 {
/* If not doing resize, adjust window to current viewport ratio.
 */
  AdjustWindow( view, &view->xmin, &view->xmax, &view->ymin, &view->ymax );
 }
 else
 {
/* If doing resize, set scroll bar thumb positions according to
 * restored window.
 */
  SetScreenWindow( view, view->xmin, view->xmax, view->ymin, view->ymax );
  ChkScreenLimits( view, view->xmin, view->xmax, view->ymin, view->ymax );

/* If not resizing window after canvas resize, adjust window to 
 * current viewport ratio.
 */
  if( !view->windowresize_adj )
  {
   AdjustWindow( view, &view->xmin, &view->xmax, &view->ymin, &view->ymax );
  }
  doing_resize = 0;
 }

 if( view->redisplay != NULL ) 
  (*view->redisplay)(view->iup_cv,view->xmin,view->xmax,view->ymin,view->ymax);
 cdFlush( );
}


/* ========================  VisRedisplay  ==================== */
 
void VisRedisplay( Ihandle *iup_cv )
{
 VisState  *view;

 view = GetViewState( iup_cv );
 if( view == NULL )
  return;

 if( wdActivate( (cdCanvas *)view->cd_cv ) == CD_ERROR )
  return;

 if( view->redisplay != NULL ) 
  (*view->redisplay)(view->iup_cv,view->xmin,view->xmax,view->ymin,view->ymax);
 cdFlush( );
}
 
 
/* ==========================  VisZoom  ======================= */
 
static void _VisZoom1stPtCB( PckDesc *pk );
static void _VisZoomDrag1stPtCB( PckDesc *pk );
static void _VisZoomSnapCB( PckDesc *pk, double *x, double *y, double tol );
static void _VisZoom2ndPtCB( PckDesc *pk );
 
void VisZoom( Ihandle *iup_cv, 
              void (*end_action)( Ihandle *iup_cv, int status ) )
{
 VisState  *view;
 PckDesc   pk;
 char      *zoomfence_adj  = IupGetAttribute( iup_cv, "ZOOMFENCE_ADJUST" );
 char      *zoomfence_drag = IupGetAttribute( iup_cv, "ZOOMFENCE_DRAG" );

 view = GetViewState( iup_cv );
 if( view == NULL )
  return;
 
 if( wdActivate( (cdCanvas *)view->cd_cv ) == CD_ERROR )
  return;

/* Save current window in wd module (to restore later) and set window
 * according to current local window parameters.
 */
 wdGetWindow( &view->xmin_sav,&view->xmax_sav,&view->ymin_sav,&view->ymax_sav );
 wdWindow( view->xmin, view->xmax, view->ymin, view->ymax );

/* Get flag for adjusting zoom fence according to canvas viewport ratio and
 * for flag for drag mode zoom.
 */
 view->zoomfence_adj = (zoomfence_adj && !strcmp( zoomfence_adj, "YES" ));
 view->zoomfence_drag = (zoomfence_drag && !strcmp( zoomfence_drag, "YES" ));

/* Store the given end action callback function.
 */
 view->end_action = end_action;

/* Setup pick descriptor.  Register the appropriate
 * callback function for zoom interaction.
 * Register the active viewing spec. in the data field.
 */
 pk.iup_cv = view->iup_cv;
 pk.gra_cv = view->cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.cursorecho = NoEchoCursor;
 pk.toler = 0.0;
 pk.button = 1;
 if( view->zoomfence_drag )
 {
  pk.mode = ButtonUp;
  pk.btdwn_callback = _VisZoomDrag1stPtCB;
  pk.btup_callback = _VisZoom2ndPtCB;
 }
 else
 {
  pk.mode = ButtonDown;
  pk.btdwn_callback = _VisZoom1stPtCB;
  pk.btup_callback = NULL;
 }
 pk.echofunc = NULL;
 pk.snapfunc = NULL;
 pk.data = view;
 pk.status = PCK_VALID;
 PckSetup( &pk );
}
 
/* ----------------------  VisZoom1stPtCB  -------------------- */
 
static void _VisZoom1stPtCB( PckDesc *pk )
{
 VisState  *view;

/* Restore registered viewing spec. from the data field.
 */
 view = (VisState *)pk->data;
 
/* Check to see if selected window is valid.
 * If not valid, check to see if an end action callback was
 * declared and, if that is the case, call it and return.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = VIS_QUIT;        break;
   case PCK_INTERUPTED:  status = VIS_INTERUPTED;  break;
  }
  wdWindow( view->xmin_sav, view->xmax_sav, view->ymin_sav, view->ymax_sav );
  if( view->end_action != NULL ) (*view->end_action)( view->iup_cv, status );
  view->end_action = NULL;
  return;
 }
 
/* Get button down point as first zoom fence point.
 */
 view->p0.x = pk->p0.x;
 view->p0.y = pk->p0.y;
 view->x0dc = pk->x0dc;
 view->y0dc = pk->y0dc;

/* Register appropriate snap callback function for adjusting
 * rectangle cursor according to viewport.
 */
 if( view->zoomfence_adj )
  pk->snapfunc = _VisZoomSnapCB;

/* Change cursor echo type to rectangle (first point is initial
 * point in initial rectangle corner) and get pick from user 
 * for opposite corner.
 */
 pk->cursorecho = RectangleCursor;
 pk->btdwn_callback = _VisZoom2ndPtCB;
 PckSetup( pk );
}

/* --------------------  VisZoomDrag1stPtCB  ------------------ */
 
static void _VisZoomDrag1stPtCB( PckDesc *pk )
{
 VisState  *view;

/* Restore registered viewing spec. from the data field.
 */
 view = (VisState *)pk->data;
 
/* Get button down point as first zoom fence point.
 */
 view->p0.x = pk->p0.x;
 view->p0.y = pk->p0.y;
 view->x0dc = pk->x0dc;
 view->y0dc = pk->y0dc;

/* Register appropriate snap callback function for adjusting
 * rectangle cursor according to viewport.
 */
 if( view->zoomfence_adj )
  pk->snapfunc = _VisZoomSnapCB;

/* Change cursor echo type to retangle and return.
 */
 pk->cursorecho = RectangleCursor;
}

/* -----------------------  VisZoomSnapCB  -------------------- */
 
static void _VisZoomSnapCB( PckDesc *pk, double *x, double *y, double tol )
{
 VisState  *view;

/* Restore registered viewing spec. from the data field.
 */
 view = (VisState *)pk->data;
 
/* If adjusting zoom fence, snap opposite fence point 
 * to rectangle adjusted to viewport ratio.
 */
 if( view->zoomfence_adj )
  AdjustZoomFence( view, view->p0.x, view->p0.y, x, y );
}

/* ----------------------  VisZoom2ndPtCB  -------------------- */
 
static void _VisZoom2ndPtCB( PckDesc *pk )
{
 VisState  *view;
 PckPnt    p1;           /* opposite zoom fence corner point */
 int       x1dc, y1dc;

/* Restore registered viewing spec. from the data field.
 */
 view = (VisState *)pk->data;
 
/* Check to see if selected window is valid.
 * If not valid, check to see if an end action callback was
 * declared and, if that is the case, call it and return.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = VIS_QUIT;        break;
   case PCK_INTERUPTED:  status = VIS_INTERUPTED;  break;
  }
  wdWindow( view->xmin_sav, view->xmax_sav, view->ymin_sav, view->ymax_sav );
  if( view->end_action != NULL ) (*view->end_action)( view->iup_cv, status );
  view->end_action = NULL;
  return;
 }
 
 if( view->zoomfence_drag )
 {
/* Get button up point as opposite zoom fence point.
 */
  p1.x = pk->p1.x;
  p1.y = pk->p1.y;
  x1dc = pk->x1dc;
  y1dc = pk->y1dc;
 }
 else
 {
/* Get button down point as opposite zoom fence point.
 */
  p1.x = pk->p0.x;
  p1.y = pk->p0.y;
  x1dc = pk->x0dc;
  y1dc = pk->y0dc;
 }

/* If adjusting zoom fence, snap opposite fence point 
 * to rectangle adjusted to viewport ratio.
 */
 if( view->zoomfence_adj )
  AdjustZoomFence( view, view->p0.x, view->p0.y, &p1.x, &p1.y );

/* Insert old window in zoom list
 */
 AddToZoomList( view, view->xmin,view->xmax,view->ymin,view->ymax );

/* Check to see if user defined a pick or a zoom fence
 */
 if( (ABS(view->x0dc - x1dc) < 8) && (ABS(view->y0dc - y1dc) < 8) )
 {
/* In the pick case, adjust window based on the current zoom factor.
 * The center of the new window is the pick point.
 */
  double    center_x, center_y;
  double    sizex, sizey;
  char      *zoom_status = IupGetAttribute( view->iup_cv, "ZOOMFACTOR" );
  double    zoomfac;

  if( zoom_status != NULL )
   zoomfac = IupGetFloat( view->iup_cv, "ZOOMFACTOR" );
  else
   zoomfac = ZOOM_FACTOR;

  center_x = view->p0.x;
  center_y = view->p0.y;
  sizex = (view->xmax - view->xmin) / 2.0;
  sizey = (view->ymax - view->ymin) / 2.0;
  view->xmin = center_x - (sizex / zoomfac);
  view->xmax = center_x + (sizex / zoomfac);
  view->ymin = center_y - (sizey / zoomfac);
  view->ymax = center_y + (sizey / zoomfac);
 }
 else
 {
/* Set new window sizes based on zoom fence.
 */
  view->xmin = MIN( view->p0.x, p1.x );
  view->xmax = MAX( view->p0.x, p1.x );
  view->ymin = MIN( view->p0.y, p1.y );
  view->ymax = MAX( view->p0.y, p1.y );
 }

/* Adjust new window to canvas viewport.
 */
 AdjustWindow( view, &view->xmin, &view->xmax, &view->ymin, &view->ymax );
 SetScreenWindow( view, view->xmin, view->xmax, view->ymin, view->ymax );

/* Ask client to redisplay.
 */
 wdWindow( view->xmin_sav, view->xmax_sav, view->ymin_sav, view->ymax_sav );
 if( view->redisplay != NULL ) 
  (*view->redisplay)(view->iup_cv,view->xmin,view->xmax,view->ymin,view->ymax);
 cdFlush( );
 
/* If an end action callback was declared, call it and return.
 */
 if( view->end_action != NULL ) (*view->end_action)( view->iup_cv, VIS_NORMAL );
#if 0
 view->end_action = NULL;
#endif
}
 

/* =========================  VisZoomIn  ====================== */

static void _VisZoomInOutGen( Ihandle *iup_cv, int in );

void VisZoomIn( Ihandle *iup_cv )
{
 _VisZoomInOutGen( iup_cv, 1 );
}


/* =========================  VisZoomOut  ===================== */

void VisZoomOut( Ihandle *iup_cv )
{
 _VisZoomInOutGen( iup_cv, 0 );
}


/* ----------------------  VisZoomInOutGen  ------------------- */

static void _VisZoomInOutGen( Ihandle *iup_cv, int in )
{
 VisState  *view;
 char      *zoom_status;
 double    zoomfac;
 double    center_x, center_y;
 double    sizex, sizey;

 view = GetViewState( iup_cv );
 if( view == NULL ) return;
 
 if( wdActivate( (cdCanvas *)view->cd_cv ) == CD_ERROR )
  return;

/* Get zoom factor
 */
 zoom_status = IupGetAttribute( view->iup_cv, "ZOOMFACTOR" );
 if( zoom_status != NULL )
  zoomfac = IupGetFloat( view->iup_cv, "ZOOMFACTOR" );
 else
  zoomfac = ZOOM_FACTOR;

/* Insert old window in zoom list
 */
 AddToZoomList( view, view->xmin,view->xmax,view->ymin,view->ymax );

/* Adjust window based on the current zoom factor.
 * The center of the new window is maintained.
 */
 center_x = (view->xmin + view->xmax) / 2.0;
 center_y = (view->ymin + view->ymax) / 2.0;
 sizex = (view->xmax - view->xmin) / 2.0;
 sizey = (view->ymax - view->ymin) / 2.0;
 if( in )
 {
  sizex /= zoomfac;
  sizey /= zoomfac;
 }
 else
 {
  sizex *= zoomfac;
  sizey *= zoomfac;
 }
 view->xmin = center_x - sizex;
 view->xmax = center_x + sizex;
 view->ymin = center_y - sizey;
 view->ymax = center_y + sizey;

/* Set new window.
 */
 AdjustWindow( view, &view->xmin, &view->xmax, &view->ymin, &view->ymax );
 SetScreenWindow( view, view->xmin, view->xmax, view->ymin, view->ymax );

/* If zooming out, check to see if canvas window limits are
 * consistent with new window.
 */
 if( !in )
 {
  ChkScreenLimits( view, view->xmin, view->xmax, view->ymin, view->ymax );
 }

/* Ask client to redisplay.
 */
 if( view->redisplay != NULL ) 
  (*view->redisplay)(view->iup_cv,view->xmin,view->xmax,view->ymin,view->ymax);
 cdFlush( );
}


/* ========================  VisPrevZoom  ===================== */

void VisPrevZoom( Ihandle *iup_cv )
{
 VisState  *view;
 ZoomList  *prev;

 view = GetViewState( iup_cv );
 if( view == NULL ) return;
 
 if( wdActivate( (cdCanvas *)view->cd_cv ) == CD_ERROR )
  return;

/* Verify if zoom list exist.
 */
 if( view->prev_zoom == 0L ) 
  return;

/* Process previous zoom.
 */
 view->xmin = view->prev_zoom->xmin;
 view->xmax = view->prev_zoom->xmax;
 view->ymin = view->prev_zoom->ymin;
 view->ymax = view->prev_zoom->ymax;
 prev = view->prev_zoom->prev;
 free( view->prev_zoom );
 view->prev_zoom = prev;

 AdjustWindow( view, &view->xmin, &view->xmax, &view->ymin, &view->ymax );
 SetScreenWindow( view, view->xmin, view->xmax, view->ymin, view->ymax );

/* Ask client to redisplay.
 */
 if( view->redisplay != NULL ) 
  (*view->redisplay)(view->iup_cv,view->xmin,view->xmax,view->ymin,view->ymax);
 cdFlush( );
}


/* ===========================  VisPan  ======================= */
 
static void _VisPan1stPtCB( PckDesc *pk );
static void _VisPanDrag1stPtCB( PckDesc *pk );
static void _VisPan2ndPtCB( PckDesc *pk );

void VisPan( Ihandle *iup_cv, 
             void (*end_action)( Ihandle *iup_cv, int status ) )
{
 VisState  *view;
 PckDesc   pk;
 char      *panline_drag = IupGetAttribute( iup_cv, "PANLINE_DRAG" );

 view = GetViewState( iup_cv );
 if( view == NULL )
  return;
 
 if( wdActivate( (cdCanvas *)view->cd_cv ) == CD_ERROR )
  return;

/* Save current window in wd module (to restore later) and set window
 * according to current local window parameters.
 */
 wdGetWindow( &view->xmin_sav,&view->xmax_sav,&view->ymin_sav,&view->ymax_sav );
 wdWindow( view->xmin, view->xmax, view->ymin, view->ymax );

/* Get flag for flag for drag mode pan.
 */
 view->panline_drag = (panline_drag && !strcmp( panline_drag, "YES" ));

/* Store the given end action callback function.
 */
 view->end_action = end_action;

/* Setup pick descriptor.  Register the appropriate
 * callback function for scale interaction.
 * Register the active viewing spec. in the data field.
 */
 pk.iup_cv = view->iup_cv;
 pk.gra_cv = view->cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.cursorecho = NoEchoCursor;
 pk.button = 1;
 pk.toler = 0.0;
 if( view->panline_drag )
 {
  pk.mode = ButtonUp;
  pk.btdwn_callback = _VisPanDrag1stPtCB;
  pk.btup_callback = _VisPan2ndPtCB;
 }
 else
 {
  pk.mode = ButtonDown;
  pk.btdwn_callback = _VisPan1stPtCB;
  pk.btup_callback = NULL;
 }
 pk.echofunc = NULL;
 pk.snapfunc = NULL;
 pk.data = view;
 pk.status = PCK_VALID;
 PckSetup( &pk );
}

/* ----------------------  VisPan1stPtCB  --------------------- */

static void _VisPan1stPtCB( PckDesc *pk )
{
 VisState  *view;

/* Restore registered viewing spec. from the data field.
 */
 view = (VisState *)pk->data;

/* Check to see if selected window is valid.
 * If not valid, check to see if an end action callback was
 * declared and, if that is the case, call it and return.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = VIS_QUIT;        break;
   case PCK_INTERUPTED:  status = VIS_INTERUPTED;  break;
  }
  wdWindow( view->xmin_sav, view->xmax_sav, view->ymin_sav, view->ymax_sav );
  if( view->end_action != NULL ) (*view->end_action)( view->iup_cv, status );
  view->end_action = NULL;
  return;
 }

/* Get button down point as first pan line point.
 */
 view->p0.x = pk->p0.x;
 view->p0.y = pk->p0.y;

/* Change cursor echo type to rubben band line and get pick from user
 * for the other line point.
 */
 pk->cursorecho = RubberBandCursor;
 pk->btdwn_callback = _VisPan2ndPtCB;
 PckSetup( pk );
}

/* --------------------  VisPanDrag1stPtCB  ------------------- */

static void _VisPanDrag1stPtCB( PckDesc *pk )
{
 VisState  *view;

/* Restore registered viewing spec. from the data field.
 */
 view = (VisState *)pk->data;

/* Get button down point as first pan line point.
 */
 view->p0.x = pk->p0.x;
 view->p0.y = pk->p0.y;

/* Change cursor echo type to rubben band line.
 */
 pk->cursorecho = RubberBandCursor;
}

/* ----------------------  VisPan2ndPtCB  --------------------- */

static void _VisPan2ndPtCB( PckDesc *pk )
{
 VisState  *view;
 PckPnt    p1;                            /* other pan line point */
 double    translx, transly;              /* translaction sizes   */
 
/* Restore registered viewing spec. from the data field.
 */
 view = (VisState *)pk->data;
 
/* Check to see if selected window is valid.
 * If not valid, check to see if an end action callback was
 * declared and, if that is the case, call it and return.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = VIS_QUIT;        break;
   case PCK_INTERUPTED:  status = VIS_INTERUPTED;  break;
  }
  wdWindow( view->xmin_sav, view->xmax_sav, view->ymin_sav, view->ymax_sav );
  if( view->end_action != NULL ) (*view->end_action)( view->iup_cv, status );
  view->end_action = NULL;
  return;
 }

 if( view->panline_drag )
 {
/* Get button up point as other pan line point.
 */
  p1.x = pk->p1.x;
  p1.y = pk->p1.y;
 }
 else
 {
/* Get button down point as other pan line point.
 */
  p1.x = pk->p0.x;
  p1.y = pk->p0.y;
 }
 
/* Compute translation sizes from mouse movement.
 */
 translx = p1.x - view->p0.x;
 transly = p1.y - view->p0.y;

/* Translate window based on translation factors.
 */
 view->xmin -= translx;
 view->xmax -= translx;
 view->rawxmin -= translx;
 view->rawxmax -= translx;
 view->ymin -= transly;
 view->ymax -= transly;
 view->rawymin -= transly;
 view->rawymax -= transly;

/* Set new window with the user interface and check to see if 
 * scroll bar limits have been exceeded.
 */
 SetScreenWindow( view, view->xmin, view->xmax, view->ymin, view->ymax );
 ChkScreenLimits( view, view->xmin, view->xmax, view->ymin, view->ymax );

 FreeZoomList( view );

/* Ask client to redisplay.
 */
 wdWindow( view->xmin_sav, view->xmax_sav, view->ymin_sav, view->ymax_sav );
 if( view->redisplay != NULL ) 
  (*view->redisplay)(view->iup_cv,view->xmin,view->xmax,view->ymin,view->ymax);
 cdFlush( );
 
/* If an end action callback was declared, call it and return.
 */
 if( view->end_action != NULL ) (*view->end_action)( view->iup_cv, VIS_NORMAL );
 view->end_action = NULL;
}


/* ==========================  VisPanX  ======================= */
 
void VisPanX( Ihandle *iup_cv, double fac )
{
 VisState  *view;
 double    translation;               /* window translaction     */
 
 view = GetViewState( iup_cv );
 if( view == NULL )
  return;
 
 if( wdActivate( (cdCanvas *)view->cd_cv ) == CD_ERROR )
  return;

/* Translate window in horizontal direction based on given factor.
 */
 translation = (view->xmax - view->xmin) * fac;
 
/* Translate window based on pan factor.
 */
 view->xmin -= translation;
 view->xmax -= translation;
 view->rawxmin -= translation;
 view->rawxmax -= translation;

/* Set new window with the user interface and check to see if 
 * scroll bar limits have been exceeded.
 */
 SetScreenWindow( view, view->xmin, view->xmax, view->ymin, view->ymax );
 ChkScreenLimits( view, view->xmin, view->xmax, view->ymin, view->ymax );

 FreeZoomList( view );

/* Ask client to redisplay.
 */
 if( view->redisplay != NULL ) 
  (*view->redisplay)(view->iup_cv,view->xmin,view->xmax,view->ymin,view->ymax);
 cdFlush( );
}


/* ==========================  VisPanY  ======================= */
 
void VisPanY( Ihandle *iup_cv, double fac )
{
 VisState  *view;
 double    translation;               /* window translaction     */
 
 view = GetViewState( iup_cv );
 if( view == NULL )
  return;
 
 if( wdActivate( (cdCanvas *)view->cd_cv ) == CD_ERROR )
  return;

/* Translate window in vertical direction based on given factor.
 */
 translation = (view->ymax - view->ymin) * fac;
 
/* Translate window based on pan factor.
 */
 view->ymin -= translation;
 view->ymax -= translation;
 view->rawymin -= translation;
 view->rawymax -= translation;

/* Set new window with the user interface and check to see if 
 * scroll bar limits have been exceeded.
 */
 SetScreenWindow( view, view->xmin, view->xmax, view->ymin, view->ymax );
 ChkScreenLimits( view, view->xmin, view->xmax, view->ymin, view->ymax );

 FreeZoomList( view );

/* Ask client to redisplay.
 */
 if( view->redisplay != NULL ) 
  (*view->redisplay)(view->iup_cv,view->xmin,view->xmax,view->ymin,view->ymax);
 cdFlush( );
}


/* ==========================  VisScale  ====================== */
 
static void _VisScaleBtUpCB( PckDesc *pk );

void VisScale( Ihandle *iup_cv, 
               void (*end_action)( Ihandle *iup_cv, int status ) )
{
 VisState  *view;
 PckDesc    pk;

 view = GetViewState( iup_cv );
 if( view == NULL )
  return;
 
 if( wdActivate( (cdCanvas *)view->cd_cv ) == CD_ERROR )
  return;

/* Save current window in wd module (to restore later) and set window
 * according to current local window parameters.
 */
 wdGetWindow( &view->xmin_sav,&view->xmax_sav,&view->ymin_sav,&view->ymax_sav );
 wdWindow( view->xmin, view->xmax, view->ymin, view->ymax );

/* Store the given end action callback function.
 */
 view->end_action = end_action;

/* Setup pick descriptor.  Register the appropriate
 * callback function for scale interaction.
 * Register the active viewing spec. in the data field.
 */
 pk.iup_cv = view->iup_cv;
 pk.gra_cv = view->cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.mode = ButtonUp;
 pk.cursorecho = NoEchoCursor;
 pk.button = 1;
 pk.toler = 0.0;
 pk.btdwn_callback = NULL;
 pk.btup_callback = _VisScaleBtUpCB;
 pk.echofunc = NULL;
 pk.snapfunc = NULL;
 pk.data = view;
 pk.status = PCK_VALID;
 PckSetup( &pk );
}

/* ---------------------  VisScaleBtUpCB  --------------------- */

static void _VisScaleBtUpCB( PckDesc *pk )
{
 VisState  *view;
 double    center_x, center_y;              /* window center     */
 double    sizex, sizey;                    /* window sizes      */
 double    oldsizex, deltax;
 double    scalefac = 1.0;
 
/* Restore registered viewing spec. from the data field.
 */
 view = (VisState *)pk->data;
 
/* Check to see if selected window is valid.
 * If not valid, check to see if an end action callback was
 * declared and, if that is the case, call it and return.
 */
 if( pk->status != PCK_VALID )
 {
  int status;
  switch( pk->status )
  {
   default:
   case PCK_QUIT:        status = VIS_QUIT;        break;
   case PCK_INTERUPTED:  status = VIS_INTERUPTED;  break;
  }
  wdWindow( view->xmin_sav, view->xmax_sav, view->ymin_sav, view->ymax_sav );
  if( view->end_action != NULL ) (*view->end_action)( view->iup_cv, status );
  view->end_action = NULL;
  return;
 }
 
/* Compute current window (x) size.
 */
 oldsizex = view->xmax - view->xmin;
 
/* Compute scaling factor from mouse movement:
 * Mouse to right ==> scale up;
 * Mouse to left  ==> scale down.
 */
 deltax = pk->p1.x - pk->p0.x;
 if( deltax >= 0.0 )
  scalefac = 1.0 + (deltax / oldsizex);
 else
  scalefac = 1.0 + 0.5*(deltax / oldsizex);

/* Set new window sizes based on scaling factor.
 */
 center_x = (view->xmin + view->xmax) / 2.0;
 center_y = (view->ymin + view->ymax) / 2.0;
 sizex = (view->xmax - view->xmin) / 2.0;
 sizey = (view->ymax - view->ymin) / 2.0;
 
 view->xmin = center_x - (sizex / scalefac);
 view->xmax = center_x + (sizex / scalefac);
 view->ymin = center_y - (sizey / scalefac);
 view->ymax = center_y + (sizey / scalefac);

 AdjustWindow( view, &view->xmin, &view->xmax, &view->ymin, &view->ymax );
 SetScreenLimits( view, view->xmin, view->xmax, view->ymin, view->ymax );
 SetScreenWindow( view, view->xmin, view->xmax, view->ymin, view->ymax );

 FreeZoomList( view );

/* Ask client to redisplay.
 */
 wdWindow( view->xmin_sav, view->xmax_sav, view->ymin_sav, view->ymax_sav );
 if( view->redisplay != NULL ) 
  (*view->redisplay)(view->iup_cv,view->xmin,view->xmax,view->ymin,view->ymax);
 cdFlush( );
 
/* If an end action callback was declared, call it and return.
 */
 if( view->end_action != NULL ) (*view->end_action)( view->iup_cv, VIS_NORMAL );
 view->end_action = NULL;
}


/* ========================  VisScaleFac  ===================== */
 
void VisScaleFac( Ihandle *iup_cv, double fac )
{
 VisState  *view;
 double    xmin, xmax,
           ymin, ymax;                /* object min max sizes    */
 double    objsizex, objsizey;        /* obj bounding box sizes  */
 double    maxobjsize;                /* maximum obj size        */
 double    center_x, center_y;        /* window center           */
 double    sizex, sizey;              /* window sizes            */
 double    scalefac;
 
 view = GetViewState( iup_cv );
 if( view == NULL )
  return;
 
 if( wdActivate( (cdCanvas *)view->cd_cv ) == CD_ERROR )
  return;

/* Compute scaling factor from given percentage factor. 
 */
 scalefac = 1.0 + fac;

/* Get current window center and new sizes.
 */
 center_x = (view->xmin + view->xmax) / 2.0;
 center_y = (view->ymin + view->ymax) / 2.0;
 sizex = (view->xmax - view->xmin) / 2.0;
 sizey = (view->ymax - view->ymin) / 2.0;
 
/* Check to see if new window is too large, in 
 * which case just return.
 */
 if( fac < 0.0 )
 {
/* Get client environment bounding box sizes.
 */
  if( (view->findminmax != NULL ) && 
      ((*view->findminmax)( view->iup_cv, &xmin, &xmax, &ymin, &ymax ) ) )
  {
   objsizex = (xmax - xmin) / 2.0;
   objsizey = (ymax - ymin) / 2.0;
   maxobjsize = MAX( objsizex, objsizey );
   if( sizex > 50.0*maxobjsize )
    return;
   if( sizey > 50.0*maxobjsize )
    return;
  }
 }

/* Set new window sizes based on scaling factor.
 */
 view->xmin = center_x - (sizex / scalefac);
 view->xmax = center_x + (sizex / scalefac);
 view->ymin = center_y - (sizey / scalefac);
 view->ymax = center_y + (sizey / scalefac);

 AdjustWindow( view, &view->xmin, &view->xmax, &view->ymin, &view->ymax );
 SetScreenLimits( view, view->xmin, view->xmax, view->ymin, view->ymax );
 SetScreenWindow( view, view->xmin, view->xmax, view->ymin, view->ymax );

 FreeZoomList( view );

/* Ask client to redisplay.
 */
 if( view->redisplay != NULL ) 
  (*view->redisplay)(view->iup_cv,view->xmin,view->xmax,view->ymin,view->ymax);
 cdFlush( );
}


/* =======================  VisDistortFac  ==================== */
 
void VisDistortFac( Ihandle *iup_cv, double vscalefac )
{
 VisState  *view;
 
 view = GetViewState( iup_cv );
 if( view == NULL )
  return;
 
 view->windistort = vscalefac;

 VisFit( view->iup_cv );
}


/* =====================  VisGetDistortFac  =================== */
 
void VisGetDistortFac( Ihandle *iup_cv, double *vscalefac )
{
 VisState  *view;
 
 view = GetViewState( iup_cv );
 if( view == NULL )
  return;
 
 *vscalefac = view->windistort;
}


/* ========================  VisSetWindow  ==================== */
 
void VisSetWindow( Ihandle *iup_cv, double xmin, double xmax, 
                                 double ymin, double ymax )
{
 VisState  *view;
 
 view = GetViewState( iup_cv );
 if( view == NULL )
  return;
 
 if( xmax <= xmin )
  return;
 if( ymax <= ymin )
  return;

 if( wdActivate( (cdCanvas *)view->cd_cv ) == CD_ERROR )
  return;

 view->xmin = xmin;
 view->xmax = xmax;
 view->ymin = ymin;
 view->ymax = ymax;

/* Set new window with the user interface and check to see if 
 * scroll bar limits have been exceeded.
 */
 AdjustWindow( view, &view->xmin, &view->xmax, &view->ymin, &view->ymax );
 SetScreenWindow( view, view->xmin, view->xmax, view->ymin, view->ymax );
 ChkScreenLimits( view, view->xmin, view->xmax, view->ymin, view->ymax );
 
 FreeZoomList( view );

 if( view->redisplay != NULL ) 
  (*view->redisplay)(view->iup_cv,view->xmin,view->xmax,view->ymin,view->ymax);
 cdFlush( );
}


/* ========================  VisGetWindow  ==================== */
 
void VisGetWindow( Ihandle *iup_cv, double *xmin, double *xmax, 
                                 double *ymin, double *ymax )
{
 VisState  *view;
 
 view = GetViewState( iup_cv );
 if( view == NULL )
  return;
 
 if( xmin ) *xmin = view->xmin;
 if( xmax ) *xmax = view->xmax;
 if( ymin ) *ymin = view->ymin;
 if( ymax ) *ymax = view->ymax;
}


/* ========================  VisSetSizes  ===================== */
 
void VisSetSizes( Ihandle *iup_cv, double width, double height )
{
 VisState  *view;
 double    center_x, center_y; /* bounding box center            */
 
 view = GetViewState( iup_cv );
 if( view == NULL )
  return;
 
 width = fabs( width );
 if( width <= 0.0 )
  return;
 height = fabs( height );
 if( height <= 0.0 )
  return;

 if( wdActivate( (cdCanvas *)view->cd_cv ) == CD_ERROR )
  return;

/* Compute new window limits keeping the center fixed.
 */
 center_x = (view->xmin + view->xmax) / 2.0;
 center_y = (view->ymin + view->ymax) / 2.0;
 
 view->xmin = center_x - width * 0.5;
 view->xmax = center_x + width * 0.5;
 view->ymin = center_y - height * 0.5;
 view->ymax = center_y + height * 0.5;

 AdjustWindow( view, &view->xmin, &view->xmax, &view->ymin, &view->ymax );
 SetScreenLimits( view, view->xmin, view->xmax, view->ymin, view->ymax );
 SetScreenWindow( view, view->xmin, view->xmax, view->ymin, view->ymax );

 FreeZoomList( view );

 if( view->redisplay != NULL ) 
  (*view->redisplay)(view->iup_cv,view->xmin,view->xmax,view->ymin,view->ymax);
 cdFlush( );
}


/* ========================  VisSetWidth  ===================== */
 
void VisSetWidth( Ihandle *iup_cv, double width, double *height )
{
 VisState  *view;
 double    vpr;                /* viewport distortion ratio      */
 double    center_x, center_y; /* bounding box center            */

 view = GetViewState( iup_cv );
 if( view == NULL )
  return;
 
 width = fabs( width );
 if( width <= 0.0 )
  return;

 if( wdActivate( (cdCanvas *)view->cd_cv ) == CD_ERROR )
  return;

/* Compute canvas viewport ratio.
 */
 vpr = (double)view->vsize / (double)view->hsize;

/* Find window center.
 */
 center_x = (view->xmin + view->xmax) / 2.0;
 center_y = (view->ymin + view->ymax) / 2.0;

/* Adjust window according to given width, keeping the same aspect 
 * ratio of the viewport.  Also take into account the window 
 * vertical scale factor.
 * Compute new window limits keeping the center fixed.
 */
 *height = (width/view->windistort) * vpr;
 view->xmin = center_x - width * 0.5;
 view->xmax = center_x + width * 0.5;
 view->ymin = center_y - (*height) * 0.5;
 view->ymax = center_y + (*height) * 0.5;

 AdjustWindow( view, &view->xmin, &view->xmax, &view->ymin, &view->ymax );
 SetScreenLimits( view, view->xmin, view->xmax, view->ymin, view->ymax );
 SetScreenWindow( view, view->xmin, view->xmax, view->ymin, view->ymax );

 FreeZoomList( view );

 if( view->redisplay != NULL ) 
  (*view->redisplay)(view->iup_cv,view->xmin,view->xmax,view->ymin,view->ymax);
 cdFlush( );
}


/* =======================  VisSetHeight  ===================== */
 
void VisSetHeight( Ihandle *iup_cv, double *width, double height )
{
 VisState  *view;
 double    vpr;                /* viewport distortion ratio      */
 double    center_x, center_y; /* bounding box center            */

 view = GetViewState( iup_cv );
 if( view == NULL )
  return;
 
 height = fabs( height );
 if( height <= 0.0 )
  return;

 if( wdActivate( (cdCanvas *)view->cd_cv ) == CD_ERROR )
  return;

/* Compute canvas viewport ratio.
 */
 vpr = (double)view->vsize / (double)view->hsize;

/* Find window center.
 */
 center_x = (view->xmin + view->xmax) / 2.0;
 center_y = (view->ymin + view->ymax) / 2.0;

/* Adjust window according to given height, keeping the same aspect 
 * ratio of the viewport.  Also take into account the window 
 * vertical scale factor.
 * Compute new window limits keeping the center fixed.
 */
 *width = (height*view->windistort) / vpr;
 view->xmin = center_x - (*width) * 0.5;
 view->xmax = center_x + (*width) * 0.5;
 view->ymin = center_y - height * 0.5;
 view->ymax = center_y + height * 0.5;

 AdjustWindow( view, &view->xmin, &view->xmax, &view->ymin, &view->ymax );
 SetScreenLimits( view, view->xmin, view->xmax, view->ymin, view->ymax );
 SetScreenWindow( view, view->xmin, view->xmax, view->ymin, view->ymax );

 FreeZoomList( view );

 if( view->redisplay != NULL ) 
  (*view->redisplay)(view->iup_cv,view->xmin,view->xmax,view->ymin,view->ymax);
 cdFlush( );
}


/* ========================  VisGetSizes  ===================== */
 
void VisGetSizes( Ihandle *iup_cv, double *width, double *height )
{
 VisState  *view;

 view = GetViewState( iup_cv );
 if( view == NULL )
  return;
 
 *width  = view->xmax - view->xmin;
 *height = view->ymax - view->ymin;
}


