char *RCSwdclip="$Id: wdclip.c,v 1.1 1998/02/27 00:18:03 lfm Exp $";
/* 
** --------------------------------------------------------------
** wdclip.c - Module for performing polygon and polyline clipping.
**
** --------------------------------------------------------------
**
** void wdClipRegFunc( void (*put_vertex)( double x, double y ) )
**
** This function registers a client function that is used to
** put the vertices resulting from a polygon clipping.
** to perform polygon clipping.
**
**      put_vertex - (in) client function to register
**
** --------------------------------------------------------------
**
** void wdClipVertex( double x, double y )
**
** This function treats every vertex of a polygon in sequence
** to perform polygon clipping.
**
**      x, y - (in) vertex coordinates
**
** --------------------------------------------------------------
**
** void wdClipClose( void )
**
** This function finishes up clipping of a polygon.
**
** --------------------------------------------------------------
**
** void wdClipCloseLine( void )
**
** This function finishes up clipping of a polyline.
**
** --------------------------------------------------------------
**
** Version: 1.0
**
** Created:   26-Feb-98  Luiz Fernando Martha
**      Stolen from g3dclip.c and modified.
**
*/

/*
** -------------------------------------------------------------------
** Global symbols:
*/
#include "cd.h"
#include "wd.h"
#include "wdclip.h"

/*
** --------------------------------------------------------------
** Local prototypes
*/
static void   _wdSaveVertex( double, double );
static void   _wdClipAtPlane( double, double, int );
static double _wdDistance( double, double, int );
static void   _wdIntersect( double, double, int, double );
static void   _wdClipClose( int plane );

/*
** --------------------------------------------------------------
** Local variables
*/
static void (*wdVertexClipped)( double x, double y );

/*
** --------------------------------------------------------------
** Local data and definitions
*/
#define LAST_PLANE 3

/* First flag for each clipping plane
 */
static int first[4] = { 1, 1, 1, 1 };

/* Previous point and distance for clipping planes
 */
static double sx[4], sy[4], sd[4];

/* First point and distance for clipping planes
 */
static double fx[4], fy[4], fd[4];


/* ======================== _wdSaveVertex ======================= */

static void _wdSaveVertex( double x, double y )
{
 if( !wdVertexClipped )
  return;

  (*wdVertexClipped)( x, y );
}

/* ======================  wdClipAtPlane  ====================
**
** This function clips line segment from s[edge] to (x,y)
** against plane.
*/
static void _wdClipAtPlane( double x, double y, int plane )
{
 double d = _wdDistance(x, y, plane);

 /* Check whether it is the first point */
 if( first[plane] )
 {
  fx[plane] = x;
  fy[plane] = y;
  fd[plane] = d;
  first[plane] = 0;
 }
 else if ((sd[plane] < 0) ^ (d < 0)) /* We have an intersection to compute */
  _wdIntersect( x, y, plane, d );

 /* Save as previous */
 sx[plane] = x;
 sy[plane] = y;
 sd[plane] = d;
 
 /* Check whether it is a visible point */
 if(d <= 0)
 {
  if( plane == LAST_PLANE )
   _wdSaveVertex( x, y );
  else
   _wdClipAtPlane( x, y, plane+1 );
 }
}

/* =======================  wdDistance  ======================
**
** This function computes and returns the distance between a
** point and a plane. Normal points toward out.
*/
static double _wdDistance( double x, double y, int plane )
{
 switch( plane )
 {
  case 0:
  {
    double xmin;
    wdGetWindow(&xmin, 0, 0, 0);
    return( xmin - x );
  }
  case 1:
  {
    double xmax;
    wdGetWindow(0, &xmax, 0, 0);
    return( -xmax + x );
  }
  case 2:
  {
    double ymin;
    wdGetWindow(0, 0, &ymin, 0);
    return( ymin - y );
  }
  case 3:
  {
    double ymax;
    wdGetWindow(0, 0, 0, &ymax);
    return( -ymax + y );
  }
 }
 return( 0.0 );
}

/* =======================  wdIntersect  =====================
**
** This function computes intersection between given point and
** the previous with plane.
** Using well-known names, d1 refers to the previous point and
** d2 to the current one.
*/
static void _wdIntersect( double x, double y, int plane, double d2 )
{
 double d1 = sd[plane];

 double t  = d1 / (d1 - d2);
 double xi = sx[plane] + t * (x - sx[plane]);
 double yi = sy[plane] + t * (y - sy[plane]);

 if( plane == LAST_PLANE )
  _wdSaveVertex( xi, yi );
 else
  _wdClipAtPlane( xi, yi, plane+1 );
}

static void _wdClipClose( int plane )
{
 if ((sd[plane] < 0) ^ (fd[plane] < 0))
  _wdIntersect(fx[plane], fy[plane], plane, fd[plane]);

 first[plane] = 1;

 if( plane < LAST_PLANE )
  _wdClipClose( plane+1 );
}

/* 
** ---------------------------------------------------------------
** Public function
*/

/* ======================= wdClipRegFunc ===================== */

void wdClipRegFunc( void (*put_vertex)( double x, double y ) )
{
 wdVertexClipped = put_vertex;
}

/* ======================== wdClipVertex ===================== */

void wdClipVertex( double x, double y )
{
 _wdClipAtPlane( x, y, 0 );
}

/* =======================  wdClipClose  ===================== */

void wdClipClose( void )
{
 _wdClipClose( 0 );
}

/* =====================  wdClipCloseLine  =================== */

void wdClipCloseLine( void )
{
 int i;
 for (i=0; i<=LAST_PLANE; first[i++]=1);
}

