char *RCSwdhdcpy="$Id: wdhdcpy.c,v 1.2 1998/02/25 16:20:18 lfm Exp $";
/*
**   wdhdcpy.c - WD Hardcopy Client functions.
**
**   Description:
**   ---------------------------------------------------------------
**
**   This file contains functions to manage hardcopy redisplays of
**   CD canvas primitives define in world coordinates (WD).
**
**   ---------------------------------------------------------------
**
**   entry pts:
**   ---------------------------------------------------------------
**
**   void wdHdcpyPS(cdCanvas *cnv, void *data, void(*dsp)(void))
**
**      cnv       - target visualization surface (canvas)       (in)
**      data      - PS canvas information                       (in)
**      dsp       - pointer to display function                 (in)
**
**   This function calls a given display function to send primitives 
**   to a post-script file.
**
**   ---------------------------------------------------------------
**
**   void wdHdcpyPrinter(cdCanvas *cnv, void *data, void(*dsp)(void))
**
**      cnv       - target visualization surface (canvas)       (in)
**      data      - Printer canvas information                  (in)
**      dsp       - pointer to display function                 (in)
**
**   This function calls a given display function to send primitives 
**   to the current active printer.
**   (Only works in MS-Windows environment).
**
**   ---------------------------------------------------------------
**
**   void wdHdcpyClipboard(cdCanvas *cnv, void *data, void(*dsp)(void))
**
**      cnv       - target visualization surface (canvas)       (in)
**      data      - Clipboard canvas information                (in)
**      dsp       - pointer to display function                 (in)
**
**   This function calls a given display function to send primitives 
**   to the clipboard.
**   (Only works in MS-Windows environment).
**
**   ---------------------------------------------------------------
**
**   void wdHdcpyCGM(cdCanvas *cnv, void *data, void(*dsp)(void))
**
**      cnv       - target visualization surface (canvas)       (in)
**      data      - CGM file canvas information                 (in)
**      dsp       - pointer to display function                 (in)
**
**   This function calls a given display function to send primitives 
**   to a CGM (computer graphics metafile) file.
**
**   ---------------------------------------------------------------
**
**   void wdHdcpyDXF(cdCanvas *cnv, void *data, void(*dsp)(void))
**
**      cnv       - target visualization surface (canvas)       (in)
**      data      - DXF file canvas information                 (in)
**      dsp       - pointer to display function                 (in)
**
**   This function calls a given display function to send primitives 
**   to a DXF (AutoCAD transferring format) file.
**
**   ---------------------------------------------------------------
**
**   Version:      0-001
**
**   Created:    08-May-96    Luiz Fernando Martha
**       Stolen from g3dhdcpy.c and modified.
**
**   Modified:   25-Feb-98    Luiz Fernando Martha
**       Created function wdHdcpyDXF.
**
*/

/*
**   ---------------------------------------------------------------
**   Global variables and symbols
*/
#include <stdlib.h>
#include <stdio.h>
#include "cd.h"
#include "cdps.h"
#include "cdprint.h"
#include "cdclipbd.h"
#include "cdcgm.h"
#include "cddxf.h"
#include "wd.h"
#include "wdhdcpy.h"

/*
**   ---------------------------------------------------------------
**   Local variables and symbols:
*/

/*
**   ---------------------------------------------------------------
**   Private functions:
*/
static void _wdHdcpyDoit(cdCanvas *cnv, cdCanvas *hcp, void(*dsp)(void));

/*========================  _wdHdcpyDoit  =======================*/

static void _wdHdcpyDoit(cdCanvas *cnv, cdCanvas *hcp, void(*dsp)(void))
{
  double  left, right, bottom, top;   /* canvas visualization window       */
  int     canvas_hsize, canvas_vsize; /* canvas sizes in pixels            */
  int     hdcpy_hsize, hdcpy_vsize;   /* paper sizes in points             */
  double  canvas_vpr;                 /* canvas viewport distortion ratio  */
  double  hdcpy_vpr;                  /* paper viewport distortion ratio   */
  int     xc, yc;                     /* paper center in pixels            */
  int     xmin, xmax, ymin, ymax;     /* paper viewport                    */

  /* Activate canvas visualization surface. */
  if (wdActivate(cnv) != CD_OK) return;

  /* Get current canvas window parameters and sizes. */
  wdGetWindow( &left, &right, &bottom, &top );
  cdGetCanvasSize( &canvas_hsize, &canvas_vsize, 0L, 0L );

  /* Activate hardcopy visualization surface. */
  if (wdActivate(hcp) != CD_OK) return;

  /* Set window parameters on hardcopy surface. */
  wdWindow( left, right, bottom, top );

  /* Adjust paper viewport, centralized, matching canvas viewport. */
  canvas_vpr = (double)canvas_vsize / (double)canvas_hsize;
  cdGetCanvasSize( &hdcpy_hsize, &hdcpy_vsize, 0L, 0L );
  hdcpy_vpr = (double)hdcpy_vsize / (double)hdcpy_hsize;
  xc = (int)((double)hdcpy_hsize/2.0);
  yc = (int)((double)hdcpy_vsize/2.0);
  if (canvas_vpr < hdcpy_vpr)
  {
    xmin = 0;
    xmax = hdcpy_hsize;
    ymin = yc - (int)((double)hdcpy_hsize*(double)canvas_vpr/2.0);
    ymax = yc + (int)((double)hdcpy_hsize*(double)canvas_vpr/2.0);
  }
  else
  {
    xmin = xc - (int)((double)hdcpy_vsize/(double)canvas_vpr/2.0);
    xmax = xc + (int)((double)hdcpy_vsize/(double)canvas_vpr/2.0);
    ymin = 0;
    ymax = hdcpy_vsize;
  }
  cdClipArea(xmin, xmax, ymin, ymax);
  cdClip(CD_CLIPON);
  wdViewport(xmin, xmax, ymin, ymax);

  /* Display on hardcopy surface.  */
  dsp();

  /* Reactivate given canvas. */
  wdActivate(cnv);
}

/*
**   ---------------------------------------------------------------
**   Entry points begin here:
*/

/*===========================  wdHdcpyPS  =======================*/

void wdHdcpyPS( cdCanvas *cnv, void *data, void(*dsp)(void) )
{
 cdCanvas *ps;

 if (!cnv) return;

/* Create a postscript visualization surface.
 */
 ps = cdCreateCanvas(CD_PS, data);
 if (!ps) return;

/* Do hardcopy.
 */
 _wdHdcpyDoit(cnv, ps, dsp);

/* Destroy postscript visualization surface.
 */
 cdKillCanvas(ps);
}

/*========================  wdHdcpyPrinter  =====================*/

void wdHdcpyPrinter( cdCanvas *cnv, void *data, void(*dsp)(void) )
{
 cdCanvas *prn;

 if (!cnv) return;

/* Create a printer visualization surface.
 */
 prn = cdCreateCanvas(CD_PRINTER, data);
 if (!prn) return;

/* Do hardcopy.
 */
 _wdHdcpyDoit(cnv, prn, dsp);

/* Destroy printer visualization surface.
 */
 cdKillCanvas(prn);
}

/*======================  wdHdcpyClipboard  =====================*/

void wdHdcpyClipboard( cdCanvas *cnv, void *data, void(*dsp)(void) )
{
 cdCanvas *cbd;

 if (!cnv) return;

/* Create a clipboard visualization surface.
 */
 cbd = cdCreateCanvas(CD_CLIPBOARD, data);
 if (!cbd) return;

/* Do hardcopy.
 */
 _wdHdcpyDoit(cnv, cbd, dsp);

/* Destroy clipboard visualization surface.
 */
 cdKillCanvas(cbd);
}

/*=========================  wdHdcpyCGM  ========================*/

void wdHdcpyCGM( cdCanvas *cnv, void *data, void(*dsp)(void) )
{
 cdCanvas *cgm;

 if (!cnv) return;

/* Create a CGM visualization surface.
 */
 cgm = cdCreateCanvas(CD_CGM, data);
 if (!cgm) return;

/* Do hardcopy.
 */
 _wdHdcpyDoit(cnv, cgm, dsp);

/* Destroy CGM visualization surface.
 */
 cdKillCanvas(cgm);
}

/*=========================  wdHdcpyDXF  ========================*/

void wdHdcpyDXF( cdCanvas *cnv, void *data, void(*dsp)(void) )
{
 cdCanvas *dxf;

 if (!cnv) return;

/* Create a CGM visualization surface.
 */
 dxf = cdCreateCanvas(CD_DXF, data);
 if (!dxf) return;

/* Do hardcopy.
 */
 _wdHdcpyDoit(cnv, dxf, dsp);

/* Destroy DXF visualization surface.
 */
 cdKillCanvas(dxf);
}

