char *RCSwim="$Id: wim.c,v 1.3 1999/03/29 15:07:48 lfm Exp $";
/*
**   wim.c - World (coordinates) image package.
**           Functions to manage an image for applications
**           that work with WD/CD world coordinates.
**
**   Remarks:
**   ---------------------------------------------------------------
**
**   *  The module uses the IM package to treat different image
**      file formats.
**   *  It works with only one image at a time.
**   *  It considers two image modes: free from world coordinate (WC)
**      space and glued to a WC space.
**   *  When the image is not glued to a WC space, it is always
**      displayed with it original sizes and centered in the canvas.
**   *  When the image is glued to a WC space, it is displayed
**      according to the window that was defined at the moment
**      that it was glued.  This means that the image may be
**      reduced or expanded.
**   *  The redifinition of a WC scale (zoom in/out) might take 
**      some time becase the scaled image has to be recreated in         
**      memory.
**   *  Once a WC scale is defined to a glued image, the image can
**      be translated (pan) very efficiently because the scaled
**      image is already created in memory.
**   *  It assumes that the viewport defined in the WD package
**      always takes the entire canvas.
**
**   ---------------------------------------------------------------
**
**   entry pts:
**   ---------------------------------------------------------------
**
**   int  wimRead( char *filename )
**
**      filename  - image file name                             (in)
**
**   This function reads an image from a file and stores it in a 
**   local data structure.
**
**   ---------------------------------------------------------------
**
**   int  wimRebuild( void )
**
**   This function rebuilds the current loaded image with the current
**   parameters defined in the local data structure.
**
**   ---------------------------------------------------------------
**
**   int  wimGlueToCurrent( void )
**
**   This function attaches current image to current world 
**   coordinates space.  If the image is already glued to world 
**   coordinates space, it redefines the gluing parameters.
**   It assumes that the unglued image (even the one that is being
**   reglued) is centered in the canvas.
**
**   ---------------------------------------------------------------
**
**   int  wimGlueToFence( int vxmin, int vxmax, 
**                        int vymin, int vymax, 
**                        double wxmin, double wxmax, 
**                        double wymin, double wymax )
**
**      vxmin, vxmax, vymin, vymax 
**                - fence in canvas (raster) coordinates        (in)
**      wxmin, wxmax, wymin, wymax 
**                - fence in world coordinates                  (in)
**
**   This function attaches current image to a given world 
**   coordinates space.  It defines an afine transformation from
**   image (raster) space to world coordinates space based on the
**   given fence limiting coordinates.
**   It defines a transformation from canvas space to image space 
**   considering that the unglued image is centered in the canvas.
**   If the image is already glued to world coordinates space, 
**   it unglues the image before attaching it to the given WC space.
**
**   ---------------------------------------------------------------
**
**   int  wimGlueToBoundBox( double wxmin, double wxmax, 
**                           double wymin, double wymax )
**
**      wxmin, wxmax, wymin, wymax 
**                - bounding box in world coordinates           (in)
**
**   This function attaches current image to a given world 
**   coordinates space.  It defines an afine transformation from
**   image (raster) space to world coordinates space based on the
**   image sizes and on the given image WC box limiting coordinates.
**   It defines a transformation from canvas space to image space 
**   considering that the unglued image is centered in the canvas.
**   If the image is already glued to world coordinates space, 
**   it unglues the image before attaching it to the given WC space.
**   This function can be used to restore previously defined WC
**   bounding box parameters of an image which were obtained through
**   function wimGetBoundBox (see below).
**
**   ---------------------------------------------------------------
**
**   int  wimUnglue( void )
**
**   This function deattaches the current image from current world 
**   coordinates space.  When it is unglued, it is defined in canvas 
**   (raster) coordinates (in which case it is drawn centered in the 
**   canvas).
**
**   ---------------------------------------------------------------
**
**   int  wimGetRasterSizes( int *iw, int *ih )
**
**      iw, ih    - returned image raster sizes                (out)
**
**   This function returns the current image raster sizes.  If there 
**   is no image defined in the module, it returns an error status.
**
**   ---------------------------------------------------------------
**
**   int  wimGetBoundBox( double *wxmin, double *wxmax, 
**                        double *wymin, double *wymax )
**
**      wxmin, wxmax, wymin, wymax 
**                - returned bounding box in world coordinates (out)
**
**   This function returns the image limiting coordinates in WC
**   space.  If the image is not attached to a WC space, it returns 
**   an error status.
**
**   ---------------------------------------------------------------
**
**   int  wimDisplay( void )
**
**   This function displays the current image in the current active
**   canvas.  If the image is glued to the world coordinate space 
**   it is displayed used the window specification that was was 
**   defined at the time that it was glued (function wimGlue).
**   The stored image position and sizes in world coordinates will be
**   transformed into canvas (raster) world coordinates according to
**   the current Window-Viewport relationship defined in WD package
**   for the current active canvas.  This might reduce or expand the
**   drawn image.
**   If the image is not glued to the world coordinate space, it 
**   is defined in canvas (raster) coordinates, in which case it is 
**   drawn centered in the canvas.
**
**   ---------------------------------------------------------------
**
**   int  wimVisibility( int on_off )
**
**      on_off    - paramater to turn on or off image           (in)
**
**   This function change the visibility status of an image.
**   If the parameter on_off = 1, the image will be displayed 
**   when function wimDisplay gets called.  Otherwise, it will not
**   be displayed.
**
**   ---------------------------------------------------------------
**
**   int  wimFree( void )
**
**   This function releases from memory the current image that is 
**   stored it in a local data structure.
**
**   ---------------------------------------------------------------
**
**   int  wimCheck( void )
**
**   This function checks for a current registered image stored in 
**   the local package data structure.  If there is an image in the
**   local datastructure, it returns a WIM_ERR_NONE status.  
**   Otherwise, it returns a WIM_ERR_NOIMAGE status.
**
**   ---------------------------------------------------------------
**
**   Version:      0-001
**
**   Created:    04-Aug-97    Luiz Fernando Martha
**       Stolen from wdhdcpy.c and modified.
**
**   Modified:   07-Sep-97    LFM
**       Renamed function wimWCBox to wimGetBoundBox.
**       Created functions wimGlueToBoundBox and wimGetRarterSizes.
**
*/

/*
**   ---------------------------------------------------------------
**   Global variables and symbols
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "im.h"
#include "cd.h"
#include "cdimage.h"
#include "wd.h"
#include "wim.h"

/*
**   ---------------------------------------------------------------
**   Local variables and symbols:
*/

#ifndef ABS
#define ABS(a) (((a)<0)?-(a):(a))
#endif

typedef struct _wim {
   int     iw, ih;       /* image sizes in pixels                         */
   double  x, y;         /* image origin position in glued WC             */
   double  w, h;         /* image sizes in glued WC                       */
   int     ox, oy;       /* offscreen image origin position w.r.t. canvas */
   int     ow, oh;       /* offscreen image sizes                         */
   int     glued;        /* flag for image glued to WC                    */
   int     visible;      /* flag for visible image                        */
   char    filename[80]; /* image file name                               */
   int     format;       /* image file format                             */
   int     compressed;   /* flag for image with compression               */
   int     type;         /* image type (RGB or Map)                       */
   int     pal_size;     /* image palette size (if Map)                   */
   void    *offscreen;   /* handle to offscreen image                     */
   int     wcv, hcv;     /* current canvas sizes in pixels                */
   unsigned char *red;   /* image color component matrices (r,g,b)        */
   unsigned char *green;
   unsigned char *blue;
   unsigned char *map;   /* matrix of image color indices (if Map)        */
   long int      *color; /* image color palette (if Map)                  */
  } Wim;

static Wim wim;  /* current active image */
static Wim *curr_wim = NULL;

/*
**   ---------------------------------------------------------------
**   Private functions:
*/

static void _wimImage2World( int ix, int iy, int iw, int ih,
                             double *x, double *y, double *w, double *h );
static void _wimWorld2Image( double x, double y, double w, double h,
                             int *ix, int *iy, int *iw, int *ih );
static void _wimImage2WorldFence( int vxmin, int vxmax, 
                                  int vymin, int vymax, 
                                  double wxmin, double wxmax, 
                                  double wymin, double wymax, 
                                  int ix, int iy, int iw, int ih, 
                                  double *x, double *y, double *w, double *h );
static int  _wimCreateOffscreen( void );

/*=======================  wimImage2World  ======================*/

static void _wimImage2World( int ix, int iy, int iw, int ih,
                             double *x, double *y, double *w, double *h )
{
 double   xw, yw;
 wdCanvas2World( ix, iy, x, y );
 ix += iw;
 iy += ih;
 wdCanvas2World( ix, iy, &xw, &yw );
 *w = xw - *x;
 *h = yw - *y;
}

/*=======================  wimWorld2Image  ======================*/

static void _wimWorld2Image( double x, double y, double w, double h,
                             int *ix, int *iy, int *iw, int *ih )
{
 int      xr, yr;
 wdWorld2Canvas( x, y, ix, iy );
 x += w;
 y += h;
 wdWorld2Canvas( x, y, &xr, &yr );
 *iw = xr - *ix;
 *ih = yr - *iy;
}

/*====================  wimImage2WorldFence  ====================*/

static void _wimImage2WorldFence( int vxmin, int vxmax, 
                                  int vymin, int vymax, 
                                  double wxmin, double wxmax, 
                                  double wymin, double wymax, 
                                  int ix, int iy, int iw, int ih, 
                                  double *x, double *y, double *w, double *h )
{
 double   sx, sy, tx, ty;    /* image to WC afine transf. parameters */
 sx = (wxmax - wxmin) / (double)(vxmax - vxmin);
 sy = (wymax - wymin) / (double)(vymax - vymin);
 tx = wxmin - sx * (double)vxmin;
 ty = wymin - sy * (double)vymin;
 *x = sx * (double)ix + tx;
 *y = sy * (double)iy + ty;
 *w = sx * (double)(ix + iw) + tx - *x;
 *h = sy * (double)(iy + ih) + ty - *y;
}

/*=====================  wimCreateOffscreen  ====================*/

static int  _wimCreateOffscreen( void )
{
 cdCanvas *cnv;                 /* current active canvas */
 cdCanvas *off_cnv;             /* auxiliary offscreen canvas */

/* Get current active canvas and its sizes.
 */
 cnv = cdActiveCanvas( );

/* If there is a defined offscreen image, delete it.
 */
 if( wim.offscreen )
  cdKillImage( wim.offscreen );

 if( wim.glued )
 {
/* Compute offscreen image sizes based on current window-viewport relationship.
 * Define offscreen image origin position in relation to canvas also based on 
 * this relationship.
 */
  _wimWorld2Image( wim.x, wim.y, wim.w, wim.h, 
                   &wim.ox, &wim.oy, &wim.ow, &wim.oh );
 }
 else
 {
/* Define offscreen image with given image sizes and origin such that 
 * image is centered in canvas.
 */
  cdGetCanvasSize( &wim.wcv, &wim.hcv, NULL, NULL );
  wim.ow = wim.iw;
  wim.oh = wim.ih;
  wim.ox = wim.wcv/2 - wim.iw/2;
  wim.oy = wim.hcv/2 - wim.ih/2;
 }

/* Create an offscreen image and an auxiliary canvas 
 * and display original image on offscreen.
 */
 wim.offscreen = cdCreateImage( wim.ow, wim.oh );
 if( wim.offscreen == NULL )
  return( WIM_ERR_NOMEMORY );
 off_cnv = cdCreateCanvas( CD_IMAGE, wim.offscreen );
 if( off_cnv == NULL )
  return( WIM_ERR_NOMEMORY );
 cdActivate( off_cnv );
 if( wim.type == IM_RGB )
  cdPutImageRGB( wim.iw, wim.ih, wim.red, wim.green, wim.blue,
                 0, 0, wim.ow, wim.oh );
 else
  cdPutImageMap( wim.iw, wim.ih, wim.map, wim.color, 
                 0, 0, wim.ow, wim.oh );

/* Activate original canvas and kill auxiliary canvas.
 */
 cdActivate( cnv );
 cdKillCanvas( off_cnv );

 return( WIM_ERR_NONE ); 
}


/*
**   ---------------------------------------------------------------
**   Entry points begin here:
*/

/*===========================  wimRead  =========================*/

int  wimRead( char *filename )
{
 int      err, f;
 long int size;
 int      status;

 wimFree( );

 err = imFileFormat( filename, &f );
 if( err )
  return( err );

 strcpy( wim.filename, filename );
 wim.format = (int)((unsigned char)f);
 if( f >> 8 )
  wim.compressed = 1;
 else
  wim.compressed = 0;

 err = imImageInfo( wim.filename, &wim.iw, &wim.ih, &wim.type, &wim.pal_size );
 if( err )
  return( err );

 size = (long)wim.iw * (long)wim.ih;

 if( wim.type == IM_RGB )
 {
  wim.red   = (unsigned char *)malloc( size );
  wim.green = (unsigned char *)malloc( size );
  wim.blue  = (unsigned char *)malloc( size );
  if( !wim.red || !wim.green || !wim.blue )
   return( WIM_ERR_NOMEMORY );
  err = imLoadRGB( wim.filename, wim.red, wim.green, wim.blue );
  if( err )
  {
   free( wim.red );
   free( wim.green );
   free( wim.blue );
   return( err );
  }
 }
 else
 {
  wim.map   = (unsigned char *)malloc( size );
  wim.color = (long int*)calloc( wim.pal_size, 4 );
  if( !wim.map || !wim.color )
   return( WIM_ERR_NOMEMORY );
  err = imLoadMap( wim.filename, wim.map, wim.color );
  if( err )
  {
   free( wim.map );
   free( wim.color );
   return( err );
  }
 }

/* Create initial offscreen image, considering that it is 
 * not glued on canvas (keeping its original sizes).
 */
 wim.glued = 0;
 wim.visible = 1;
 status = _wimCreateOffscreen( );
 if( status != WIM_ERR_NONE )
  return( status );

 curr_wim = &wim;

 return( WIM_ERR_NONE ); 
}

/*=========================  wimRebuild  ========================*/

int  wimRebuild( void )
{
 return( _wimCreateOffscreen( ) ); 
}

/*======================  wimGlueToCurrent  =====================*/

int  wimGlueToCurrent( void )
{
 int      ix, iy;
 int      status;

 if( !curr_wim )
  return( WIM_ERR_NOIMAGE ); 

 wim.glued = 0;

/* Define image origin and sizes in glued WC based on current 
 * window-viewport relationship.  Consider that original image 
 * is centered in canvas.
 */
 cdGetCanvasSize( &wim.wcv, &wim.hcv, NULL, NULL );
 ix = wim.wcv/2 - wim.iw/2;
 iy = wim.hcv/2 - wim.ih/2;
 _wimImage2World( ix, iy, wim.iw, wim.ih, 
                  &wim.x, &wim.y, &wim.w, &wim.h );

/* Recreate offscreen image.
 */
 status = _wimCreateOffscreen( );
 if( status != WIM_ERR_NONE )
  return( status );

 wim.glued = 1;

 return( WIM_ERR_NONE ); 
}

/*=======================  wimGlueToFence  ======================*/

int  wimGlueToFence( int vxmin, int vxmax, int vymin, int vymax,
                     double wxmin, double wxmax, double wymin, double wymax )
{
 int      ix, iy;
 int      status;

 if( !curr_wim )
  return( WIM_ERR_NOIMAGE ); 

 wim.glued = 0;

/* Get non glued image origin raster coordinates
 * considering that original image is centered in canvas.
 */
 cdGetCanvasSize( &wim.wcv, &wim.hcv, NULL, NULL );
 ix = wim.wcv/2 - wim.iw/2;
 iy = wim.hcv/2 - wim.ih/2;

/* Define image origin and sizes in glued WC based on given 
 * raster-WC fence relationship.
 */
 _wimImage2WorldFence( vxmin, vxmax, vymin, vymax, 
                       wxmin, wxmax, wymin, wymax, 
                       ix, iy, wim.iw, wim.ih, 
                       &wim.x, &wim.y, &wim.w, &wim.h );

/* Recreate offscreen image.
 */
 status = _wimCreateOffscreen( );
 if( status != WIM_ERR_NONE )
  return( status );

 wim.glued = 1;

 return( WIM_ERR_NONE ); 
}

/*=====================  wimGlueToBoundBox  =====================*/

int  wimGlueToBoundBox( double wxmin, double wxmax, 
                        double wymin, double wymax )
{
 int      status;

 if( !curr_wim )
  return( WIM_ERR_NOIMAGE ); 

 wim.glued = 0;

/* Define image origin and sizes in glued WC based on given 
 * bounding box sizes.
 */
 wim.x = wxmin;
 wim.y = wymin;
 wim.w = wxmax - wxmin;
 wim.h = wymax - wymin;

/* Recreate offscreen image.
 */
 status = _wimCreateOffscreen( );
 if( status != WIM_ERR_NONE )
  return( status );

 wim.glued = 1;

 return( WIM_ERR_NONE ); 
}

/*==========================  wimUnglue  ========================*/

int  wimUnglue( void )
{
 int      status;

 if( !curr_wim )
  return( WIM_ERR_NOIMAGE ); 

 if( !wim.glued )
  return( WIM_ERR_NOTGLUED );

 wim.glued = 0;

/* Recreate offscreen image.
 */
 status = _wimCreateOffscreen( );
 if( status != WIM_ERR_NONE )
  return( status );

 return( WIM_ERR_NONE ); 
}

/*=====================  wimGetRasterSizes  =====================*/

int  wimGetRarterSizes( int *iw, int *ih )
{
 if( !curr_wim )
  return( WIM_ERR_NOIMAGE ); 

 *iw = wim.iw;
 *ih = wim.ih;

 return( WIM_ERR_NONE ); 
}

/*=======================  wimGetBoundBox  ======================*/

int  wimGetBoundBox( double *wxmin, double *wxmax, 
                     double *wymin, double *wymax )
{
 if( !curr_wim )
  return( WIM_ERR_NOIMAGE ); 

 if( !wim.glued )
  return( WIM_ERR_NOTGLUED );

 *wxmin = wim.x;
 *wymin = wim.y;
 *wxmax = wim.x + wim.w;
 *wymax = wim.y + wim.h;

 return( WIM_ERR_NONE ); 
}

/*=========================  wimDisplay  ========================*/

int  wimDisplay( void )
{
 int      status;

 if( !curr_wim )
  return( WIM_ERR_NOIMAGE ); 

 if( !wim.visible )
  return( WIM_ERR_NONE ); 

 if( wim.glued )
 {
/* Compute offscreen image sizes based on current window-viewport relationship
 * (which potentially is different than the glued WC relationship).
 * Define offscreen image origin position in relation to canvas also based on 
 * current relationship.
 */
  int      ow, oh;
  _wimWorld2Image( wim.x, wim.y, wim.w, wim.h, &wim.ox, &wim.oy, &ow, &oh );

/* If offscreen image sizes have changed, create another offscreen image.
 * Otherwise, just use the updated offscreen image origin position.
 */
  if( (ABS(ow - wim.ow) > 2) || (ABS(oh - wim.oh) > 2) )
  {
   status = _wimCreateOffscreen( );
   if( status != WIM_ERR_NONE )
    return( status );
  }
 }
 else
 {
/* Define image origin such that image is centered in canvas.
 */
  cdGetCanvasSize( &wim.wcv, &wim.hcv, NULL, NULL );
  wim.ox = wim.wcv/2 - wim.iw/2;
  wim.oy = wim.hcv/2 - wim.ih/2;
 }

 cdPutImage( wim.offscreen, wim.ox, wim.oy );

 return( WIM_ERR_NONE ); 
}

/*========================  wimVisibility  ======================*/

int  wimVisibility( int on_off )
{
 if( !curr_wim )
  return( WIM_ERR_NOIMAGE ); 

 wim.visible = on_off;

 return( WIM_ERR_NONE ); 
}

/*===========================  wimFree  =========================*/

int  wimFree( void )
{
 if( !curr_wim )
  return( WIM_ERR_NOIMAGE ); 

 if( wim.type == IM_RGB )
 {
  free( wim.red );
  free( wim.green );
  free( wim.blue );
 }
 else
 {
  free( wim.map );
  free( wim.color );
 }

 if( wim.offscreen )
  cdKillImage( wim.offscreen );
 wim.offscreen = NULL;

 curr_wim = NULL;

 return( WIM_ERR_NONE ); 
}

/*==========================  wimCheck  =========================*/

int  wimCheck( void )
{
 if( !curr_wim )
  return( WIM_ERR_NOIMAGE ); 

 return( WIM_ERR_NONE ); 
}

