char *RCSwimfence="$Id: wimfence.c,v 1.2 2001/01/19 13:19:04 lfm Exp $";
/*
** ---------------------------------------------------------------
** wimfence.c - Functions to interactively get a fence from the
**              user to map a background image from raster to world 
**              coordinate space (see wim.c module).
**
** ---------------------------------------------------------------
** Entry points:
** ---------------------------------------------------------------
**
** void   wimGetFence( Ihandle *iup_cv, void *cd_cv, 
**                     void (*end_action)( Ihandle *iup_cv, int status ),
**                     void (*prompt)( Ihandle *iup_cv, char *msg ),
**                     int (*put)( int vxmin, int vxmax, 
**                                 int vymin, int vymax,
**                                 double wxmin, double wxmax, 
**                                 double wymin, double wymax ) )
**
**   iup_cv     - IUP handle to target canvas                 (in)
**   cd_cv      - CD handle to target canvas (vis.surface)    (in)
**   end_action - function to be called after interaction     (in)
**   prompt     - prompt message callback function            (in)
**   put        - function to put collected fence             (in)
**
** This function gets from the user a fence which is interactively 
** defined with the mouse.  The user indicates two points on the 
** target canvas which defines the fence.  Then the user indicates 
** the desired world coordinates of these points.  This information
** is used to map from a background image space, which is assumed
** centered in the canvas, to world coordinates.
** If the parameter "end_action" is not null, after the user
** interaction the function pointed to by the parameter is
** called, passing the given canvas and a return status as arguments.
** The possible return status are:
** (1) Normal fence collection ==> status = WIM_ERR_NONE
** (2) Non valid fence collection ==> status = WIM_ERR_NOTGLUED
**
** ---------------------------------------------------------------
**
** Version:    1.0
**
** Created:    05-Aug-97    Luiz F. Martha
**      Stolen from VisZoom and modified.
**
** Modified:   19-Jan-2001  LFM
**      When call to function getenv( "IUP_INGLES" ) was replaced 
**      by call to function IupGetLanguage( ).
**      Local string variable iup_english changed to int.
**
** Modified:   08-Aug-2002    Marcos C. de Arruda
**      Replaced call to IupScanf by call to corresponding dialog
**      (Dlg) function.
**
** Modified:   21-Apr-2004    William Lira
**      Replaced "void *" by "Ihandle *" in declarations of IUP handles.
**
*/

/*
** ---------------------------------------------------------------
** Global variables and symbols
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "iup.h"
#include "cd.h"
#include "cdiup.h"
#include "wd.h"
#include "pck.h"
#include "wim.h"
#include "wimfence.h"
#include "dlgwimfence.h"

/*
** ---------------------------------------------------------------
** Local variables and symbols:
*/

#ifndef LEN
#define LEN(p0,p1) sqrt((p1.x-p0.x)*(p1.x-p0.x)+(p1.y-p0.y)*(p1.y-p0.y))
#endif

#ifndef MAX
#define MAX(a,b) (((a)>(b))?(a):(b))
#endif

#ifndef MIN
#define MIN(a,b) (((a)<(b))?(a):(b))
#endif

/* Private data structure descriptor:
 */
typedef struct _wimfence {
  void      *iup_cv;                        /* IUP handle to target canvas  */
  void      *cd_cv;                         /* CD handle to corresp.canvas  */
  PckPnt    p0;                             /* first fence point            */
  int       vxmin, vxmax, vymin, vymax;     /* fence in raster coordinates  */
  double    wxmin, wxmax, wymin, wymax;     /* fence in world coordinates   */
  void      (*end_action)( Ihandle *iup_cv, int status );
  void      (*prompt)( Ihandle *iup_cv, char *msg );
  int       (*put)( int vxmin, int vxmax, int vymin, int vymax, 
                    double wxmin, double wxmax, double wymin, double wymax );
 } WimFence;

static WimFence fencedsc;

static int  iup_english;
static char *prompt_msg1;
static char *prompt_msg2;
static char *form;

static char *prompt_msg1p ={ "Entre primeiro ponto com botao esquerdo do mouse;"
                             "Botao da direita => Cancela." };
static char *prompt_msg2p = { "Entre outro ponto do retangulo; "
                              "Botao da direita => Cancela." };
static char *formp = { "Entre com as coordenadas WC do retangulo :\n"
                       "X min = %10.10%10.3f\n"
                       "Y min = %10.10%10.3f\n"
                       "X max = %10.10%10.3f\n" };

static char *prompt_msg1e = { "Enter first point with mouse left button; "
                              "Right button => Cancel." };
static char *prompt_msg2e = { "Enter other rectangle point; "
                              "Right button => Cancel." };
static char *forme = { "Enter with rectangle world coordinates:\n"
                       "X min = %10.10%10.3f\n"
                       "Y min = %10.10%10.3f\n"
                       "X max = %10.10%10.3f\n" };

/*
** ---------------------------------------------------------------
** Local functions:
*/
static int _wimFenceGetWorld( WimFence *fence );

/* ====================  wimFenceGetWorld  ===================== */
 
static int _wimFenceGetWorld( WimFence *fence )
{
 double    vpr;               /* viewport aspect ratio  */
 float     x0, y0, x1;
 long int  color_sav = cdForeground( CD_RED );

/* Display collected fence.
 */
 cdLine( fence->vxmin, fence->vymin, fence->vxmax, fence->vymin );
 cdLine( fence->vxmin, fence->vymax, fence->vxmax, fence->vymax );
 cdLine( fence->vxmin, fence->vymin, fence->vxmin, fence->vymax );
 cdLine( fence->vxmax, fence->vymin, fence->vxmax, fence->vymax );
 cdForeground( color_sav );

/* Compute viewport aspect ratio.
 */
 vpr = (double)(fence->vymax - fence->vymin) / 
       (double)(fence->vxmax - fence->vxmin);

/* Prompt user for fence world coordinates.
 * Force fence to keep same viewport aspect ratio.
 */
 x0 = fence->wxmin;
 y0 = fence->wymin;
 x1 = fence->wxmax;
 if( DlgCaptureWimFence( &x0, &y0, &x1 ) == 1 )
 {
  fence->wxmin = x0;
  fence->wymin = y0;
  fence->wxmax = x1;
  fence->wymax = fence->wymin + (fence->wxmax - fence->wxmin) * vpr;
 }
 else
 {
  return( WIM_ERR_NOTGLUED );
 }

 return( WIM_ERR_NONE );
}


/*
** ---------------------------------------------------------------
** Public functions:
*/

/* ========================  wimGetFence  ===================== */
 
static void _wimGetFence1stPtCB( PckDesc *pk );
static void _wimGetFence2ndPtCB( PckDesc *pk );
 
void wimGetFence( Ihandle *iup_cv, void *cd_cv,
                  void (*end_action)( Ihandle *iup_cv, int status ),
                  void (*prompt)( Ihandle *iup_cv, char *msg ),
                  int (*put)( int vxmin, int vxmax,
                              int vymin, int vymax,
                              double wxmin, double wxmax,
                              double wymin, double wymax ) )
{
 PckDesc   pk;

 if( (!put) || (wdActivate( (cdCanvas *)cd_cv ) == CD_ERROR) )
 {
  if( end_action != NULL ) 
    (*end_action)( iup_cv, WIM_ERR_NOTGLUED );
  return;
 }

 if( strcmp( IupGetLanguage( ), IUP_ENGLISH ) == 0 )
  iup_english = 1;
 else
  iup_english = 0;

 if( iup_english )
 {
  prompt_msg1 = prompt_msg1e;
  prompt_msg2 = prompt_msg2e;
  form = forme;
 }
 else
 {
  prompt_msg1 = prompt_msg1p;
  prompt_msg2 = prompt_msg2p;
  form = formp;
 }

/* Store given data in local collection descriptor.
 */
 fencedsc.iup_cv = iup_cv;
 fencedsc.cd_cv = cd_cv;
 fencedsc.end_action = end_action;
 fencedsc.prompt = prompt;
 fencedsc.put = put;

/* Setup pick descriptor.  Register the appropriate
 * callback function for fence interaction.
 * Register the active collection spec. in the data field.
 */
 if( fencedsc.prompt != 0L )
  (*fencedsc.prompt )( iup_cv, prompt_msg1 );
 pk.iup_cv = iup_cv;
 pk.gra_cv = cd_cv;
 pk.p0.x = pk.p0.y = 0.0;
 pk.p1.x = pk.p1.y = 0.0;
 pk.cursorecho = NoEchoCursor;
 pk.toler = 0.0;
 pk.button = 1;
 pk.mode = ButtonDown;
 pk.btdwn_callback = _wimGetFence1stPtCB;
 pk.btup_callback = NULL;
 pk.echofunc = NULL;
 pk.snapfunc = NULL;
 pk.data = &fencedsc;
 pk.status = PCK_VALID;
 PckSetup( &pk );
}
 
/* --------------------  wimGetFence1stPtCB  ------------------ */
 
static void _wimGetFence1stPtCB( PckDesc *pk )
{
 WimFence  *fence;

/* Restore registered collection spec. from the data field.
 */
 fence = (WimFence *)pk->data;
 
/* Check to see if selected pick is valid.
 * If not valid, check to see if an end action callback was
 * declared and, if that is the case, call it and return.
 */
 if( pk->status != PCK_VALID )
 {
  if( fence->end_action != NULL ) 
    (*fence->end_action)( fence->iup_cv, WIM_ERR_NOTGLUED );
  fence->end_action = NULL;
  fence->prompt = NULL;
  fence->put = NULL;
  return;
 }
 
/* Get button down point as first fence point.
 */
 fence->p0.x = pk->p0.x;
 fence->p0.y = pk->p0.y;

/* Change cursor echo type to rectangle (first point is initial
 * point in initial rectangle corner) and get pick from user 
 * for opposite corner.
 */
 if( fence->prompt != NULL )
  (*fence->prompt )( fence->iup_cv, prompt_msg2 );
 pk->cursorecho = RectangleCursor;
 pk->btdwn_callback = _wimGetFence2ndPtCB;
 PckSetup( pk );
}

/* --------------------  wimGetFence2ndPtCB  ------------------ */
 
static void _wimGetFence2ndPtCB( PckDesc *pk )
{
 WimFence  *fence;
 PckPnt    p1;           /* opposite fence corner point */
 int       status = 0;

/* Restore registered collection spec. from the data field.
 */
 fence = (WimFence *)pk->data;
 
/* Clear prompt message.
 */
 if( fence->prompt != NULL )
  (*fence->prompt )( fence->iup_cv, " " );

/* Check to see if selected pick is valid.
 */
 if( pk->status != PCK_VALID )
 {
  if( fence->end_action != NULL ) 
    (*fence->end_action)( fence->iup_cv, WIM_ERR_NOTGLUED );
  fence->end_action = NULL;
  fence->prompt = NULL;
  fence->put = NULL;
  return;
 }
 
/* Get button down point as opposite fence point.
 */
  p1.x = pk->p0.x;
  p1.y = pk->p0.y;

/* Check to see if user defined a valid fence
 */
 if( LEN(fence->p0,p1) > pk->toler )
 {
  fence->wxmin = MIN( fence->p0.x, p1.x );
  fence->wxmax = MAX( fence->p0.x, p1.x );
  fence->wymin = MIN( fence->p0.y, p1.y );
  fence->wymax = MAX( fence->p0.y, p1.y );
 }
 else
 {
  if( fence->end_action != NULL ) 
    (*fence->end_action)( fence->iup_cv, WIM_ERR_NOTGLUED );
  fence->end_action = NULL;
  fence->prompt = NULL;
  fence->put = NULL;
  return;
 }

/* Transform collected fence from current world to canvas (raster) coords.
 */
 wdWorld2Canvas( fence->wxmin, fence->wymin, &fence->vxmin, &fence->vymin );
 wdWorld2Canvas( fence->wxmax, fence->wymax, &fence->vxmax, &fence->vymax );

/* Get new fence world coordinates from user.
 */
 status = _wimFenceGetWorld( fence );

/* If this is a valid fence, call client callback function to put it.
 */
 if( status == WIM_ERR_NONE )
 {
  status = (*fence->put)(fence->vxmin,fence->vxmax,fence->vymin,fence->vymax, 
                         fence->wxmin,fence->wxmax,fence->wymin,fence->wymax);
 }

/* If an end action callback was declared, call it and return.
 */
 if( fence->end_action != NULL ) (*fence->end_action)( fence->iup_cv, status );
 fence->end_action = NULL;
 fence->prompt = NULL;
 fence->put = NULL;
}
 
